Package: omophub
Title: R Client for the 'OMOPHub' Medical Vocabulary API
Version: 1.8.1
Authors@R: c(
    person("Alex", "Chen", email = "alex@omophub.com", role = c("aut", "cre", "cph")),
    person("Observational Health Data Science and Informatics", role = c("cph"))
  )
Description: Provides an R interface to the 'OMOPHub' API for accessing
    'OHDSI ATHENA' standardized medical vocabularies. Supports concept search,
    semantic search using neural embeddings, concept similarity, vocabulary
    exploration, hierarchy navigation, relationship queries, concept
    mappings, and FHIR-to-OMOP concept resolution with automatic pagination.
License: MIT + file LICENSE
URL: https://github.com/omopHub/omophub-R, https://docs.omophub.com,
        https://omophub.github.io/omophub-R/, https://omophub.com
BugReports: https://github.com/omopHub/omophub-R/issues
Depends: R (>= 4.1)
Imports: httr2 (>= 1.0.0), R6, rlang (>= 1.0.0), cli, tibble, purrr,
        glue, checkmate
Suggests: testthat (>= 3.0.0), httptest2, webmockr, knitr, rmarkdown,
        keyring, withr
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.2
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-06-01 15:32:27 UTC; konstantin
Author: Alex Chen [aut, cre, cph],
  Observational Health Data Science and Informatics [cph]
Maintainer: Alex Chen <alex@omophub.com>
Repository: CRAN
Date/Publication: 2026-06-01 18:20:02 UTC
Built: R 4.5.2; ; 2026-06-01 21:02:48 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,omophub_concepts)
S3method(as.data.frame,omophub_hierarchy)
S3method(as.data.frame,omophub_mappings)
S3method(as.data.frame,omophub_relationships)
S3method(as.data.frame,omophub_vocabularies)
S3method(print,omophub_concepts)
S3method(print,omophub_hierarchy)
S3method(print,omophub_mappings)
S3method(print,omophub_relationships)
S3method(print,omophub_vocabularies)
export(OMOPHubClient)
export(fhir_resolve)
export(fhir_resolve_batch)
export(fhir_resolve_codeable_concept)
export(get_api_key)
export(has_api_key)
export(omophub_fhir_url)
export(set_api_key)
importFrom(R6,R6Class)
importFrom(cli,cli_abort)
importFrom(cli,cli_inform)
importFrom(cli,cli_warn)
importFrom(glue,glue)
importFrom(httr2,req_body_json)
importFrom(httr2,req_error)
importFrom(httr2,req_headers)
importFrom(httr2,req_perform)
importFrom(httr2,req_retry)
importFrom(httr2,req_throttle)
importFrom(httr2,req_timeout)
importFrom(httr2,req_url_path_append)
importFrom(httr2,req_url_query)
importFrom(httr2,req_user_agent)
importFrom(httr2,request)
importFrom(httr2,resp_body_json)
importFrom(httr2,resp_status)
importFrom(purrr,map)
importFrom(purrr,map_chr)
importFrom(purrr,map_dbl)
importFrom(purrr,map_int)
importFrom(purrr,map_lgl)
importFrom(rlang,abort)
importFrom(rlang,caller_env)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)

# OMOPHub R SDK

**Query millions of standardized medical concepts from R**

Access SNOMED CT, ICD-10, RxNorm, LOINC, and 90+ OHDSI ATHENA vocabularies without downloading, installing, or maintaining local databases.

[![CRAN status](https://www.r-pkg.org/badges/version/omophub)](https://CRAN.R-project.org/package=omophub)
[![CRAN downloads](https://cranlogs.r-pkg.org/badges/omophub)](https://cran.r-project.org/package=omophub)
[![R-CMD-check](https://github.com/omopHub/omophub-R/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/omopHub/omophub-R/actions/workflows/R-CMD-check.yaml)
[![Codecov](https://codecov.io/gh/omopHub/omophub-R/branch/main/graph/badge.svg)](https://app.codecov.io/gh/omopHub/omophub-R?branch=main)
[![License: MIT](https://img.shields.io/badge/License-MIT-blue.svg)](https://opensource.org/licenses/MIT)

**[Documentation](https://docs.omophub.com/sdks/r/overview)** ·
**[API Reference](https://docs.omophub.com/api-reference)** ·
**[Examples](https://github.com/omopHub/omophub-R/tree/main/inst/examples)**

---

## Why OMOPHub?
Working with OHDSI ATHENA vocabularies traditionally requires downloading multi-gigabyte files, setting up a database, and writing complex SQL queries. **OMOPHub eliminates this friction.**

| Traditional Approach | With OMOPHub |
|---------------------|--------------|
| Download 5GB+ ATHENA vocabulary files | `install.packages("omophub")` |
| Set up and maintain database instance | One API call |
| Write complex SQL with multiple JOINs | Simple R functions |
| Manually update vocabularies quarterly | Always current data |
| Local infrastructure required | Works anywhere R runs |

## Installation

```r
# Install from CRAN
install.packages("omophub")

# Or install development version from GitHub
# install.packages("pak")
pak::pak("omopHub/omophub-R")
```

## Authentication

```r
library(omophub)

# Option 1: Environment variable (recommended)
Sys.setenv(OMOPHUB_API_KEY = "oh_xxxxxxxxx")

# Option 2: Use helper function
set_api_key("oh_xxxxxxxxx")

# Option 3: Store securely in system keyring
set_api_key("oh_xxxxxxxxx", store = "keyring")
```

Get your API key from the [OMOPHub Dashboard](https://dashboard.omophub.com/api-keys).

## Quick Start

```r
library(omophub)

# Create client
client <- OMOPHubClient$new()

# Get a concept by ID
concept <- client$concepts$get(201826)
concept$concept_name
# [1] "Type 2 diabetes mellitus"

# Search for concepts
results <- client$search$basic("metformin", vocabulary_ids = "RxNorm")
results$data

# Get concept by vocabulary code
snomed_concept <- client$concepts$get_by_code("SNOMED", "44054006")

# Map to another vocabulary
mappings <- client$mappings$get(201826, target_vocabulary = "ICD10CM")

# Navigate hierarchy
ancestors <- client$hierarchy$ancestors(201826, max_levels = 3)
```

## Semantic Search

Use natural language queries to find concepts using neural embeddings:

```r
# Natural language search - understands clinical intent
results <- client$search$semantic("high blood sugar levels")
for (r in results$data$results) {
  cat(sprintf("%s (similarity: %.2f)\n", r$concept_name, r$similarity_score))
}

# Filter by vocabulary and set minimum similarity threshold
results <- client$search$semantic(
  "heart attack",
  vocabulary_ids = "SNOMED",
  domain_ids = "Condition",
  threshold = 0.5
)

# Fetch all results with auto-pagination
all_results <- client$search$semantic_all("chronic kidney disease", page_size = 50)

# Find concepts similar to a reference concept
similar <- client$search$similar(concept_id = 201826, algorithm = "hybrid")
for (s in similar$similar_concepts) {
  cat(sprintf("%s (score: %.2f)\n", s$concept_name, s$similarity_score))
}
```

### Bulk Search

Search for multiple terms in a single API call:

```r
# Bulk lexical search (up to 50 queries) - returns a flat list of
# per-query result objects; iterate it directly.
results <- client$search$bulk_basic(list(
  list(search_id = "q1", query = "diabetes mellitus"),
  list(search_id = "q2", query = "hypertension"),
  list(search_id = "q3", query = "aspirin")
), defaults = list(vocabulary_ids = list("SNOMED"), page_size = 5))

for (item in results) {
  cat(sprintf("%s: %d results\n", item$search_id, length(item$results)))
}

# Bulk semantic search (up to 25 queries) - returns a dict with
# `results`, `total_searches`, `completed_count`, and `total_duration`.
response <- client$search$bulk_semantic(list(
  list(search_id = "s1", query = "heart failure treatment options"),
  list(search_id = "s2", query = "type 2 diabetes medication")
), defaults = list(threshold = 0.5, page_size = 10))

for (item in response$results) {
  cat(sprintf("%s: %d results\n", item$search_id, length(item$results)))
}
```

## FHIR-to-OMOP Resolution

Resolve FHIR coded values to OMOP standard concepts in one call:

```r
# Single FHIR Coding -> OMOP concept + CDM target table
result <- client$fhir$resolve(
  system = "http://snomed.info/sct",
  code = "44054006",
  resource_type = "Condition"
)
result$resolution$target_table
# [1] "condition_occurrence"

# ICD-10-CM -> traverses 'Maps to' automatically
result <- client$fhir$resolve(
  system = "http://hl7.org/fhir/sid/icd-10-cm",
  code = "E11.9"
)
result$resolution$standard_concept$vocabulary_id
# [1] "SNOMED"

# Batch resolve up to 100 codings
batch <- client$fhir$resolve_batch(list(
  list(system = "http://snomed.info/sct", code = "44054006"),
  list(system = "http://loinc.org", code = "2339-0"),
  list(system = "http://www.nlm.nih.gov/research/umls/rxnorm", code = "197696")
))
cat(sprintf("Resolved %d/%d\n", batch$summary$resolved, batch$summary$total))

# CodeableConcept with vocabulary preference (SNOMED wins over ICD-10)
result <- client$fhir$resolve_codeable_concept(
  coding = list(
    list(system = "http://snomed.info/sct", code = "44054006"),
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9")
  ),
  resource_type = "Condition"
)
result$best_match$resolution$source_concept$vocabulary_id
# [1] "SNOMED"
```

### Tibble Output for Batch Resolution

Pass `as_tibble = TRUE` to get a flat [`tibble`](https://tibble.tidyverse.org/) with one row per input coding - ready to pipe into `dplyr` / `tidyr`:

```r
library(dplyr)

tbl <- client$fhir$resolve_batch(
  list(
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9"),
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "I10"),
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "J45.909")
  ),
  as_tibble = TRUE
)

tbl |>
  filter(status == "resolved") |>
  select(source_code, standard_concept_name, target_table)
#> # A tibble: 3 x 3
#>   source_code standard_concept_name       target_table
#>   <chr>       <chr>                       <chr>
#> 1 E11.9       Type 2 diabetes mellitus    condition_occurrence
#> 2 I10         Essential hypertension      condition_occurrence
#> 3 J45.909     Asthma                      condition_occurrence
```

The tibble columns are `source_system`, `source_code`, `source_concept_id`, `source_concept_name`, `standard_concept_id`, `standard_concept_name`, `standard_vocabulary_id`, `domain_id`, `target_table`, `mapping_type`, `similarity_score`, `status`, and `status_detail`. Failed rows stay in-place with `status = "failed"` and the API error text in `status_detail`. The batch summary (`total` / `resolved` / `failed`) is attached as `attr(tbl, "summary")`.

Default `as_tibble = FALSE` still returns the legacy `list(results, summary)` shape.

### Standalone Wrapper Functions

The R6 interface is always available:

```r
client$fhir$resolve(system = "http://snomed.info/sct", code = "44054006")
```

For pipe-friendly workflows, three standalone wrappers forward to the same R6 methods and take the client as their first argument:

```r
# Equivalent to client$fhir$resolve()
client |>
  fhir_resolve(
    system = "http://snomed.info/sct",
    code = "44054006",
    resource_type = "Condition"
  )

# Tibble-shaped batch in a pipe
tbl <- client |>
  fhir_resolve_batch(
    codings = list(
      list(system = "http://snomed.info/sct", code = "44054006"),
      list(system = "http://loinc.org", code = "2339-0")
    ),
    as_tibble = TRUE
  )

client |>
  fhir_resolve_codeable_concept(
    coding = list(
      list(system = "http://snomed.info/sct", code = "44054006"),
      list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9")
    ),
    resource_type = "Condition"
  )
```

Both forms are fully supported - pick whichever reads better for the surrounding code.

### FHIR Client Interop

When you need raw FHIR `Parameters` / `Bundle` responses instead of the Concept Resolver envelope, `omophub_fhir_url()` returns the OMOPHub FHIR Terminology Service base URL for a given FHIR version (`"r4"` default, plus `"r4b"`, `"r5"`, `"r6"`). Use it with [`httr2`](https://httr2.r-lib.org/) or [`fhircrackr`](https://cran.r-project.org/package=fhircrackr) to talk directly to OMOPHub's FHIR endpoint.

```r
library(httr2)

# Call CodeSystem/$lookup directly against OMOPHub's FHIR endpoint
resp <- request(omophub_fhir_url()) |>
  req_url_path_append("CodeSystem/$lookup") |>
  req_url_query(
    system = "http://snomed.info/sct",
    code = "44054006"
  ) |>
  req_headers(Authorization = paste("Bearer", Sys.getenv("OMOPHUB_API_KEY"))) |>
  req_perform()

params <- resp_body_json(resp)
# Raw FHIR Parameters resource with the concept display and designations.

# R5 / R6 endpoints work the same way
omophub_fhir_url("r5")
#> "https://fhir.omophub.com/fhir/r5"
```

**When to use which**: Use `client$fhir$resolve()` (or `fhir_resolve()`) when you want OMOP-enriched answers (standard concept, CDM target table, mapping quality). Use `omophub_fhir_url()` + `httr2` when you need raw FHIR responses for FHIR-native tooling.

## Use Cases

### ETL & Data Pipelines

Validate and map clinical codes during OMOP CDM transformations:

```r
# Validate source codes and find standard equivalents
validate_and_map <- function(source_vocab, source_code) {
  concept <- client$concepts$get_by_code(source_vocab, source_code)
  
  if (concept$standard_concept != "S") {
    mappings <- client$mappings$get(
      concept$concept_id,
      target_vocabulary = "SNOMED"
    )
    return(mappings$mappings[[1]]$target_concept_id)
  }
  
  concept$concept_id
}

# Example: Map ICD-10 to SNOMED
standard_id <- validate_and_map("ICD10CM", "E11.9")
```

### Data Quality Checks

Verify codes exist and are valid:

```r
# Check if condition codes are valid. HTTP 404 responses come through
# as httr2's `httr2_http_404` condition class.
condition_codes <- c("E11.9", "I10", "J44.9")

for (code in condition_codes) {
  tryCatch({
    concept <- client$concepts$get_by_code("ICD10CM", code)
    message(sprintf("OK %s: %s", code, concept$concept_name))
  }, httr2_http_404 = function(e) {
    message(sprintf("ERROR %s: Invalid code!", code))
  })
}
```

### Phenotype Development

Explore hierarchies to build comprehensive concept sets:

```r
# Get all descendants for phenotype definition. `descendants()` returns
# `list(data = list(descendants = [...], concept_id, concept_name, ...), meta)`
descendants <- client$hierarchy$descendants(
  201826,  # Type 2 diabetes mellitus
  max_levels = 5
)

descendant_list <- descendants$data$descendants
concept_set <- vapply(descendant_list, function(x) x$concept_id, integer(1))
message(sprintf("Found %d concepts for T2DM phenotype", length(concept_set)))
```

### Integration with tidyverse

```r
library(dplyr)
library(purrr)

# Search and convert to tibble
results <- client$search$basic("hypertension", page_size = 100)

concepts_df <- results$data %>%
  map_dfr(~ tibble(
    concept_id = .x$concept_id,
    concept_name = .x$concept_name,
    vocabulary_id = .x$vocabulary_id,
    domain_id = .x$domain_id
  ))

# Filter and analyze
concepts_df %>%
  filter(vocabulary_id == "SNOMED") %>%
  count(domain_id)
```

## API Resources

| Resource | Description | Key Methods |
|----------|-------------|-------------|
| `concepts` | Concept lookup and batch operations | `get()`, `get_by_code()`, `batch()`, `suggest()` |
| `search` | Full-text and semantic search | `basic()`, `advanced()`, `semantic()`, `similar()`, `bulk_basic()`, `bulk_semantic()` |
| `hierarchy` | Navigate concept relationships | `ancestors()`, `descendants()` |
| `mappings` | Cross-vocabulary mappings | `get()`, `map()` |
| `vocabularies` | Vocabulary metadata | `list()`, `get()`, `stats()` |
| `domains` | Domain information | `list()`, `get()`, `concepts()` |
| `fhir` | FHIR-to-OMOP resolution | `resolve()`, `resolve_batch()`, `resolve_codeable_concept()` |

## Pagination

```r
# Automatic pagination - fetch all results
all_results <- client$search$basic_all("diabetes", page_size = 100)

# Manual pagination
page1 <- client$search$basic("diabetes", page = 1, page_size = 20)
page2 <- client$search$basic("diabetes", page = 2, page_size = 20)
```

## Configuration

```r
# Specify vocabulary version
client <- OMOPHubClient$new(vocab_version = "2025.2")

# Custom configuration
client <- OMOPHubClient$new(
  api_key = "oh_xxx",
  base_url = "https://api.omophub.com/v1",
  timeout = 30
)
```

## Error Handling

HTTP errors are raised as [httr2 condition classes](https://httr2.r-lib.org/reference/req_error.html) (`httr2_http_404`, `httr2_http_401`, `httr2_http_429`, `httr2_http_403`, etc.). Pre-request input-validation errors use the SDK's `omophub_validation_error` class.

```r
tryCatch({
  result <- client$concepts$get(999999999)
}, httr2_http_404 = function(e) {
  message("Concept not found: ", conditionMessage(e)[[1]])
}, httr2_http_401 = function(e) {
  message("Unauthorized - check your API key")
}, httr2_http_403 = function(e) {
  message("Forbidden - API key lacks permission or vocabulary restricted")
}, httr2_http_429 = function(e) {
  # The SDK already auto-retries 429 via httr2::req_retry() with
  # exponential backoff; handle here only for custom logging.
  message("Rate limited (", e$retry_after %||% "?", "s)")
}, httr2_http = function(e) {
  # Generic HTTP error fallback (any 4xx/5xx not caught above)
  message("HTTP error: ", conditionMessage(e)[[1]])
}, omophub_validation_error = function(e) {
  # Pre-request validation (bad concept_id type, empty query, etc.)
  message("Validation error: ", conditionMessage(e)[[1]])
})
```

See [`inst/examples/error_handling.R`](inst/examples/error_handling.R) for the full set of patterns including graceful degradation and batch error collection.

## Compared to Alternatives

| Feature | OMOPHub SDK | ATHENA Download | OHDSI WebAPI |
|---------|-------------|-----------------|--------------|
| Setup time | 1 minute | Hours | Hours |
| Infrastructure | None | PostgreSQL required | Full OHDSI stack |
| Updates | Automatic | Manual download | Manual |
| Programmatic access | Native R | SQL/DatabaseConnector | REST API |

**Best for:** R users who need quick, programmatic access to OMOP vocabularies without infrastructure overhead.

## Examples

The package includes comprehensive examples in `inst/examples/`:

| Example | Description |
|---------|-------------|
| `basic_usage.R` | Getting started - client setup, concept lookup, search |
| `search_concepts.R` | Search capabilities - filters, autocomplete, pagination |
| `navigate_hierarchy.R` | Hierarchy navigation - ancestors, descendants |
| `map_between_vocabularies.R` | Cross-vocabulary mapping |
| `error_handling.R` | Error handling patterns |
| `fhir_resolver.R` | FHIR Concept Resolver - single / batch / CodeableConcept, quality, recommendations |
| `fhir_interop.R` | 1.7.0 interop - tibble batch output, standalone wrappers, `omophub_fhir_url()` |

Run an example:

```r
example_path <- system.file("examples", "basic_usage.R", package = "omophub")
source(example_path)
```

## Documentation

- [Full Documentation](https://docs.omophub.com/sdks/r/overview)
- [API Reference](https://docs.omophub.com/api-reference)
- [Examples](https://github.com/omopHub/omophub-R/tree/main/inst/examples)
- [Get API Key](https://dashboard.omophub.com/api-keys)

## Contributing

We welcome contributions! 

```r
# Clone and install for development
# install.packages("devtools")
devtools::install_github("omopHub/omophub-R")

# Run tests
devtools::test()

# Check package
devtools::check()
```

## Support

- [GitHub Issues](https://github.com/omopHub/omophub-R/issues)
- Email: support@omophub.com
- Website: [omophub.com](https://omophub.com)

## License

MIT License - see [LICENSE](LICENSE) for details.

---

*Built for the OHDSI community*

# OMOPHub R SDK

**Query millions of standardized medical concepts from R**

Access SNOMED CT, ICD-10, RxNorm, LOINC, and 90+ OHDSI ATHENA vocabularies without downloading, installing, or maintaining local databases.

[![CRAN status](https://www.r-pkg.org/badges/version/omophub)](https://CRAN.R-project.org/package=omophub)
[![CRAN downloads](https://cranlogs.r-pkg.org/badges/omophub)](https://cran.r-project.org/package=omophub)
[![R-CMD-check](https://github.com/omopHub/omophub-R/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/omopHub/omophub-R/actions/workflows/R-CMD-check.yaml)
[![Codecov](https://codecov.io/gh/omopHub/omophub-R/branch/main/graph/badge.svg)](https://app.codecov.io/gh/omopHub/omophub-R?branch=main)
[![License: MIT](https://img.shields.io/badge/License-MIT-blue.svg)](https://opensource.org/licenses/MIT)

**[Documentation](https://docs.omophub.com/sdks/r/overview)** ·
**[API Reference](https://docs.omophub.com/api-reference)** ·
**[Examples](https://github.com/omopHub/omophub-R/tree/main/inst/examples)**

---

## Why OMOPHub?
Working with OHDSI ATHENA vocabularies traditionally requires downloading multi-gigabyte files, setting up a database, and writing complex SQL queries. **OMOPHub eliminates this friction.**

| Traditional Approach | With OMOPHub |
|---------------------|--------------|
| Download 5GB+ ATHENA vocabulary files | `install.packages("omophub")` |
| Set up and maintain database instance | One API call |
| Write complex SQL with multiple JOINs | Simple R functions |
| Manually update vocabularies quarterly | Always current data |
| Local infrastructure required | Works anywhere R runs |

## Installation

```r
# Install from CRAN
install.packages("omophub")

# Or install development version from GitHub
# install.packages("pak")
pak::pak("omopHub/omophub-R")
```

## Authentication

```r
library(omophub)

# Option 1: Environment variable (recommended)
Sys.setenv(OMOPHUB_API_KEY = "oh_xxxxxxxxx")

# Option 2: Use helper function
set_api_key("oh_xxxxxxxxx")

# Option 3: Store securely in system keyring
set_api_key("oh_xxxxxxxxx", store = "keyring")
```

Get your API key from the [OMOPHub Dashboard](https://dashboard.omophub.com/api-keys).

## Quick Start

```r
library(omophub)

# Create client
client <- OMOPHubClient$new()

# Get a concept by ID
concept <- client$concepts$get(201826)
concept$concept_name
# [1] "Type 2 diabetes mellitus"

# Search for concepts
results <- client$search$basic("metformin", vocabulary_ids = "RxNorm")
results$data

# Get concept by vocabulary code
snomed_concept <- client$concepts$get_by_code("SNOMED", "44054006")

# Map to another vocabulary
mappings <- client$mappings$get(201826, target_vocabulary = "ICD10CM")

# Navigate hierarchy
ancestors <- client$hierarchy$ancestors(201826, max_levels = 3)
```

## Semantic Search

Use natural language queries to find concepts using neural embeddings:

```r
# Natural language search - understands clinical intent
results <- client$search$semantic("high blood sugar levels")
for (r in results$data$results) {
  cat(sprintf("%s (similarity: %.2f)\n", r$concept_name, r$similarity_score))
}

# Filter by vocabulary and set minimum similarity threshold
results <- client$search$semantic(
  "heart attack",
  vocabulary_ids = "SNOMED",
  domain_ids = "Condition",
  threshold = 0.5
)

# Fetch all results with auto-pagination
all_results <- client$search$semantic_all("chronic kidney disease", page_size = 50)

# Find concepts similar to a reference concept
similar <- client$search$similar(concept_id = 201826, algorithm = "hybrid")
for (s in similar$similar_concepts) {
  cat(sprintf("%s (score: %.2f)\n", s$concept_name, s$similarity_score))
}
```

### Bulk Search

Search for multiple terms in a single API call:

```r
# Bulk lexical search (up to 50 queries) - returns a flat list of
# per-query result objects; iterate it directly.
results <- client$search$bulk_basic(list(
  list(search_id = "q1", query = "diabetes mellitus"),
  list(search_id = "q2", query = "hypertension"),
  list(search_id = "q3", query = "aspirin")
), defaults = list(vocabulary_ids = list("SNOMED"), page_size = 5))

for (item in results) {
  cat(sprintf("%s: %d results\n", item$search_id, length(item$results)))
}

# Bulk semantic search (up to 25 queries) - returns a dict with
# `results`, `total_searches`, `completed_count`, and `total_duration`.
response <- client$search$bulk_semantic(list(
  list(search_id = "s1", query = "heart failure treatment options"),
  list(search_id = "s2", query = "type 2 diabetes medication")
), defaults = list(threshold = 0.5, page_size = 10))

for (item in response$results) {
  cat(sprintf("%s: %d results\n", item$search_id, length(item$results)))
}
```

## FHIR-to-OMOP Resolution

Resolve FHIR coded values to OMOP standard concepts in one call:

```r
# Single FHIR Coding -> OMOP concept + CDM target table
result <- client$fhir$resolve(
  system = "http://snomed.info/sct",
  code = "44054006",
  resource_type = "Condition"
)
result$resolution$target_table
# [1] "condition_occurrence"

# ICD-10-CM -> traverses 'Maps to' automatically
result <- client$fhir$resolve(
  system = "http://hl7.org/fhir/sid/icd-10-cm",
  code = "E11.9"
)
result$resolution$standard_concept$vocabulary_id
# [1] "SNOMED"

# Batch resolve up to 100 codings
batch <- client$fhir$resolve_batch(list(
  list(system = "http://snomed.info/sct", code = "44054006"),
  list(system = "http://loinc.org", code = "2339-0"),
  list(system = "http://www.nlm.nih.gov/research/umls/rxnorm", code = "197696")
))
cat(sprintf("Resolved %d/%d\n", batch$summary$resolved, batch$summary$total))

# CodeableConcept with vocabulary preference (SNOMED wins over ICD-10)
result <- client$fhir$resolve_codeable_concept(
  coding = list(
    list(system = "http://snomed.info/sct", code = "44054006"),
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9")
  ),
  resource_type = "Condition"
)
result$best_match$resolution$source_concept$vocabulary_id
# [1] "SNOMED"
```

The resolver also follows the [HL7 FHIR-to-OMOP IG](https://hl7.org/fhir/uv/omop/INFORMATIVE1/en/): it resolves FHIR administrative codes via the IG ConceptMaps, decomposes composite concepts (`Maps to value`), honors `Coding.userSelected`, and can return a `concept_id` 0 sentinel instead of an error.

```r
# Administrative gender -> person (via IG ConceptMap)
client$fhir$resolve(system = "http://hl7.org/fhir/administrative-gender", code = "male")

# A user-selected coding wins over vocabulary preference
client$fhir$resolve_codeable_concept(coding = list(
  list(system = "http://snomed.info/sct", code = "44054006"),
  list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9", user_selected = TRUE)
))

# on_unmapped = "sentinel" -> a concept_id 0 record instead of an error (one row per input)
client$fhir$resolve(system = "http://snomed.info/sct", code = "00000000", on_unmapped = "sentinel")
```

Composite concepts (e.g. "Allergy to penicillin") additionally surface `resolution$value_as_concept` (the IG Value-as-Concept pattern). `on_unmapped` is accepted by `resolve()`, `resolve_batch()`, and `resolve_codeable_concept()`.

### Tibble Output for Batch Resolution

Pass `as_tibble = TRUE` to get a flat [`tibble`](https://tibble.tidyverse.org/) with one row per input coding - ready to pipe into `dplyr` / `tidyr`:

```r
library(dplyr)

tbl <- client$fhir$resolve_batch(
  list(
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9"),
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "I10"),
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "J45.909")
  ),
  as_tibble = TRUE
)

tbl |>
  filter(status == "resolved") |>
  select(source_code, standard_concept_name, target_table)
#> # A tibble: 3 x 3
#>   source_code standard_concept_name       target_table
#>   <chr>       <chr>                       <chr>
#> 1 E11.9       Type 2 diabetes mellitus    condition_occurrence
#> 2 I10         Essential hypertension      condition_occurrence
#> 3 J45.909     Asthma                      condition_occurrence
```

The tibble columns are `source_system`, `source_code`, `source_concept_id`, `source_concept_name`, `standard_concept_id`, `standard_concept_name`, `standard_vocabulary_id`, `domain_id`, `value_as_concept_id`, `value_as_concept_name`, `target_table`, `mapping_type`, `similarity_score`, `status`, and `status_detail`. Failed rows stay in-place with `status = "failed"` and the API error text in `status_detail`; a coding that resolves but has no standard target gets `status = "unmapped"` (`standard_concept_id = 0`). The batch summary (`total` / `resolved` / `failed`) is attached as `attr(tbl, "summary")`.

Default `as_tibble = FALSE` still returns the legacy `list(results, summary)` shape.

### Standalone Wrapper Functions

The R6 interface is always available:

```r
client$fhir$resolve(system = "http://snomed.info/sct", code = "44054006")
```

For pipe-friendly workflows, three standalone wrappers forward to the same R6 methods and take the client as their first argument:

```r
# Equivalent to client$fhir$resolve()
client |>
  fhir_resolve(
    system = "http://snomed.info/sct",
    code = "44054006",
    resource_type = "Condition"
  )

# Tibble-shaped batch in a pipe
tbl <- client |>
  fhir_resolve_batch(
    codings = list(
      list(system = "http://snomed.info/sct", code = "44054006"),
      list(system = "http://loinc.org", code = "2339-0")
    ),
    as_tibble = TRUE
  )

client |>
  fhir_resolve_codeable_concept(
    coding = list(
      list(system = "http://snomed.info/sct", code = "44054006"),
      list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9")
    ),
    resource_type = "Condition"
  )
```

Both forms are fully supported - pick whichever reads better for the surrounding code.

### FHIR Client Interop

When you need raw FHIR `Parameters` / `Bundle` responses instead of the Concept Resolver envelope, `omophub_fhir_url()` returns the OMOPHub FHIR Terminology Service base URL for a given FHIR version (`"r4"` default, plus `"r4b"`, `"r5"`, `"r6"`). Use it with [`httr2`](https://httr2.r-lib.org/) or [`fhircrackr`](https://cran.r-project.org/package=fhircrackr) to talk directly to OMOPHub's FHIR endpoint.

```r
library(httr2)

# Call CodeSystem/$lookup directly against OMOPHub's FHIR endpoint
resp <- request(omophub_fhir_url()) |>
  req_url_path_append("CodeSystem/$lookup") |>
  req_url_query(
    system = "http://snomed.info/sct",
    code = "44054006"
  ) |>
  req_headers(Authorization = paste("Bearer", Sys.getenv("OMOPHUB_API_KEY"))) |>
  req_perform()

params <- resp_body_json(resp)
# Raw FHIR Parameters resource with the concept display and designations.

# R5 / R6 endpoints work the same way
omophub_fhir_url("r5")
#> "https://fhir.omophub.com/fhir/r5"
```

**When to use which**: Use `client$fhir$resolve()` (or `fhir_resolve()`) when you want OMOP-enriched answers (standard concept, CDM target table, mapping quality). Use `omophub_fhir_url()` + `httr2` when you need raw FHIR responses for FHIR-native tooling.

## Use Cases

### ETL & Data Pipelines

Validate and map clinical codes during OMOP CDM transformations:

```r
# Validate source codes and find standard equivalents
validate_and_map <- function(source_vocab, source_code) {
  concept <- client$concepts$get_by_code(source_vocab, source_code)
  
  if (concept$standard_concept != "S") {
    mappings <- client$mappings$get(
      concept$concept_id,
      target_vocabulary = "SNOMED"
    )
    return(mappings$mappings[[1]]$target_concept_id)
  }
  
  concept$concept_id
}

# Example: Map ICD-10 to SNOMED
standard_id <- validate_and_map("ICD10CM", "E11.9")
```

### Data Quality Checks

Verify codes exist and are valid:

```r
# Check if condition codes are valid. HTTP 404 responses come through
# as httr2's `httr2_http_404` condition class.
condition_codes <- c("E11.9", "I10", "J44.9")

for (code in condition_codes) {
  tryCatch({
    concept <- client$concepts$get_by_code("ICD10CM", code)
    message(sprintf("OK %s: %s", code, concept$concept_name))
  }, httr2_http_404 = function(e) {
    message(sprintf("ERROR %s: Invalid code!", code))
  })
}
```

### Phenotype Development

Explore hierarchies to build comprehensive concept sets:

```r
# Get all descendants for phenotype definition. `descendants()` returns
# `list(data = list(descendants = [...], concept_id, concept_name, ...), meta)`
descendants <- client$hierarchy$descendants(
  201826,  # Type 2 diabetes mellitus
  max_levels = 5
)

descendant_list <- descendants$data$descendants
concept_set <- vapply(descendant_list, function(x) x$concept_id, integer(1))
message(sprintf("Found %d concepts for T2DM phenotype", length(concept_set)))
```

### Integration with tidyverse

```r
library(dplyr)
library(purrr)

# Search and convert to tibble
results <- client$search$basic("hypertension", page_size = 100)

concepts_df <- results$data %>%
  map_dfr(~ tibble(
    concept_id = .x$concept_id,
    concept_name = .x$concept_name,
    vocabulary_id = .x$vocabulary_id,
    domain_id = .x$domain_id
  ))

# Filter and analyze
concepts_df %>%
  filter(vocabulary_id == "SNOMED") %>%
  count(domain_id)
```

## API Resources

| Resource | Description | Key Methods |
|----------|-------------|-------------|
| `concepts` | Concept lookup and batch operations | `get()`, `get_by_code()`, `batch()`, `suggest()` |
| `search` | Full-text and semantic search | `basic()`, `advanced()`, `semantic()`, `similar()`, `bulk_basic()`, `bulk_semantic()` |
| `hierarchy` | Navigate concept relationships | `ancestors()`, `descendants()` |
| `mappings` | Cross-vocabulary mappings | `get()`, `map()` |
| `vocabularies` | Vocabulary metadata | `list()`, `get()`, `stats()` |
| `domains` | Domain information | `list()`, `get()`, `concepts()` |
| `fhir` | FHIR-to-OMOP resolution | `resolve()`, `resolve_batch()`, `resolve_codeable_concept()` |

## Pagination

```r
# Automatic pagination - fetch all results
all_results <- client$search$basic_all("diabetes", page_size = 100)

# Manual pagination
page1 <- client$search$basic("diabetes", page = 1, page_size = 20)
page2 <- client$search$basic("diabetes", page = 2, page_size = 20)
```

## Configuration

```r
# Specify vocabulary version
client <- OMOPHubClient$new(vocab_version = "2025.2")

# Custom configuration
client <- OMOPHubClient$new(
  api_key = "oh_xxx",
  base_url = "https://api.omophub.com/v1",
  timeout = 30
)
```

## Error Handling

HTTP errors are raised as [httr2 condition classes](https://httr2.r-lib.org/reference/req_error.html) (`httr2_http_404`, `httr2_http_401`, `httr2_http_429`, `httr2_http_403`, etc.). Pre-request input-validation errors use the SDK's `omophub_validation_error` class.

```r
tryCatch({
  result <- client$concepts$get(999999999)
}, httr2_http_404 = function(e) {
  message("Concept not found: ", conditionMessage(e)[[1]])
}, httr2_http_401 = function(e) {
  message("Unauthorized - check your API key")
}, httr2_http_403 = function(e) {
  message("Forbidden - API key lacks permission or vocabulary restricted")
}, httr2_http_429 = function(e) {
  # The SDK already auto-retries 429 via httr2::req_retry() with
  # exponential backoff; handle here only for custom logging.
  message("Rate limited (", e$retry_after %||% "?", "s)")
}, httr2_http = function(e) {
  # Generic HTTP error fallback (any 4xx/5xx not caught above)
  message("HTTP error: ", conditionMessage(e)[[1]])
}, omophub_validation_error = function(e) {
  # Pre-request validation (bad concept_id type, empty query, etc.)
  message("Validation error: ", conditionMessage(e)[[1]])
})
```

See [`inst/examples/error_handling.R`](inst/examples/error_handling.R) for the full set of patterns including graceful degradation and batch error collection.

## Compared to Alternatives

| Feature | OMOPHub SDK | ATHENA Download | OHDSI WebAPI |
|---------|-------------|-----------------|--------------|
| Setup time | 1 minute | Hours | Hours |
| Infrastructure | None | PostgreSQL required | Full OHDSI stack |
| Updates | Automatic | Manual download | Manual |
| Programmatic access | Native R | SQL/DatabaseConnector | REST API |

**Best for:** R users who need quick, programmatic access to OMOP vocabularies without infrastructure overhead.

## Examples

The package includes comprehensive examples in `inst/examples/`:

| Example | Description |
|---------|-------------|
| `basic_usage.R` | Getting started - client setup, concept lookup, search |
| `search_concepts.R` | Search capabilities - filters, autocomplete, pagination |
| `navigate_hierarchy.R` | Hierarchy navigation - ancestors, descendants |
| `map_between_vocabularies.R` | Cross-vocabulary mapping |
| `error_handling.R` | Error handling patterns |
| `fhir_resolver.R` | FHIR Concept Resolver - single / batch / CodeableConcept, quality, recommendations, administrative codes, `user_selected`, `on_unmapped` |
| `fhir_interop.R` | 1.7.0 interop - tibble batch output, standalone wrappers, `omophub_fhir_url()` |

Run an example:

```r
example_path <- system.file("examples", "basic_usage.R", package = "omophub")
source(example_path)
```

## Documentation

- [Full Documentation](https://docs.omophub.com/sdks/r/overview)
- [API Reference](https://docs.omophub.com/api-reference)
- [Examples](https://github.com/omopHub/omophub-R/tree/main/inst/examples)
- [Get API Key](https://dashboard.omophub.com/api-keys)

## Contributing

We welcome contributions! 

```r
# Clone and install for development
# install.packages("devtools")
devtools::install_github("omopHub/omophub-R")

# Run tests
devtools::test()

# Check package
devtools::check()
```

## Support

- [GitHub Issues](https://github.com/omopHub/omophub-R/issues)
- Email: support@omophub.com
- Website: [omophub.com](https://omophub.com)

## License

MIT License - see [LICENSE](LICENSE) for details.

---

*Built for the OHDSI community*

#!/usr/bin/env Rscript
#' Example: Navigate Hierarchy
#'
#' Demonstrates concept hierarchy exploration:
#' - Getting ancestors (parent concepts)
#' - Getting descendants (child concepts)
#' - Exploring concept relationships
#' - Finding related concepts
#'
#' Run with: Rscript inst/examples/navigate_hierarchy.R

library(omophub)

# Null-coalescing operator (available in base R 4.4+; define locally for
# compatibility with the R >= 4.1 package requirement).
`%||%` <- function(a, b) if (is.null(a)) b else a

# ============================================================================
# Setup
# ============================================================================

client <- OMOPHubClient$new()

cat("OMOPHub R Client - Hierarchy Navigation Examples\n")
cat("=================================================\n\n")

# We'll use "Type 2 diabetes mellitus" as our starting concept
DIABETES_CONCEPT_ID <- 201826

# ============================================================================
# Get Ancestors (Parent Concepts)
# ============================================================================

cat("1. Getting ancestors (parent concepts)\n")
cat("--------------------------------------\n")

# Get ancestors up to 3 levels, including distance information
ancestors <- client$hierarchy$ancestors(
  DIABETES_CONCEPT_ID,
  max_levels = 3,
  include_distance = TRUE
)

cat(sprintf("Ancestors of concept %d (%s):\n",
            DIABETES_CONCEPT_ID,
            ancestors$data$concept_name))

# Actual ancestor list lives at $data$ancestors
ancestor_list <- ancestors$data$ancestors %||% list()
for (a in ancestor_list) {
  level <- a$min_levels_of_separation %||% a$level %||% "?"
  cat(sprintf("  Level %s: [%d] %s\n",
              level,
              a$concept_id,
              a$concept_name))
}
cat("\n")

# ============================================================================
# Get Descendants (Child Concepts)
# ============================================================================

cat("2. Getting descendants (child concepts)\n")
cat("---------------------------------------\n")

# Get descendants up to 2 levels
descendants <- client$hierarchy$descendants(
  DIABETES_CONCEPT_ID,
  max_levels = 2,
  page_size = 10
)

cat(sprintf("Descendants of concept %d (first 10):\n", DIABETES_CONCEPT_ID))
descendant_list <- descendants$data$descendants %||% list()
for (d in descendant_list) {
  level <- d$min_levels_of_separation %||% d$level %||% "?"
  cat(sprintf("  Level %s: [%d] %s\n",
              level,
              d$concept_id,
              d$concept_name))
}
cat("\n")

# ============================================================================
# Get Ancestors with Filters
# ============================================================================

cat("3. Filtered ancestors (SNOMED only)\n")
cat("------------------------------------\n")

# Get only SNOMED ancestors
ancestors_snomed <- client$hierarchy$ancestors(
  DIABETES_CONCEPT_ID,
  vocabulary_ids = c("SNOMED"),
  max_levels = 5,
  include_invalid = FALSE
)

cat("SNOMED ancestors only:\n")
ancestor_list <- ancestors_snomed$data$ancestors %||% list()
for (a in ancestor_list) {
  cat(sprintf("  [%d] %s (%s)\n",
              a$concept_id,
              a$concept_name,
              a$concept_class_id %||% ""))
}
cat("\n")

# ============================================================================
# Explore Concept Relationships
# ============================================================================

cat("4. Exploring concept relationships\n")
cat("----------------------------------\n")

# Get all relationships for a concept
relationships <- client$concepts$relationships(DIABETES_CONCEPT_ID)

rel_list <- relationships$data$relationships %||% list()
cat(sprintf("Relationships for concept %d (%d total):\n",
            DIABETES_CONCEPT_ID, length(rel_list)))

# Group by relationship type (first 3 per type)
by_type <- list()
for (r in rel_list) {
  rel_type <- r$relationship_id %||% "Unknown"
  if (is.null(by_type[[rel_type]])) {
    by_type[[rel_type]] <- list()
  }
  by_type[[rel_type]] <- c(by_type[[rel_type]], list(r))
}

for (type_name in names(by_type)) {
  items <- by_type[[type_name]]
  cat(sprintf("\n  %s (%d):\n", type_name, length(items)))
  for (r in head(items, 3)) {
    # Prefer the nested ``concept_2`` object (full concept details) but
    # fall back to the flat ``concept_id_2`` field so deprecated or
    # invalid target concepts without a populated nested object still
    # surface an ID instead of rendering as unknown.
    target <- r$concept_2 %||% list()
    target_id <- target$concept_id %||% r$concept_id_2 %||% "?"
    target_name <- target$concept_name %||% "Unknown"
    cat(sprintf("    -> [%s] %s\n", target_id, target_name))
  }
  if (length(items) > 3) {
    cat(sprintf("    ... and %d more\n", length(items) - 3))
  }
}
cat("\n")

# ============================================================================
# Get Related Concepts (Semantic Similarity)
# ============================================================================

cat("5. Finding semantically related concepts\n")
cat("----------------------------------------\n")

# Get concepts related by semantic similarity. `related()` returns a flat
# unnamed list of concept objects (not wrapped in $data / $meta).
related_list <- client$concepts$related(
  DIABETES_CONCEPT_ID,
  page_size = 5
)

cat("Top 5 semantically related concepts:\n")
for (r in related_list) {
  score <- r$relationship_score %||% r$score %||% NA_real_
  score_str <- if (is.na(score)) "?" else sprintf("%.2f", score)
  cat(sprintf("  [%d] %s (score: %s, distance: %s)\n",
              r$concept_id,
              r$concept_name,
              score_str,
              r$relationship_distance %||% "?"))
}
cat("\n")

# ============================================================================
# Build a Concept Path
# ============================================================================

cat("6. Building concept path to root\n")
cat("---------------------------------\n")

# Get ancestors sorted by distance to show the path from the starting
# concept up through increasingly general parents.
path_result <- client$hierarchy$ancestors(
  DIABETES_CONCEPT_ID,
  max_levels = 10
)

ancestor_list <- path_result$data$ancestors %||% list()

# Sort by min_levels_of_separation (closest first)
if (length(ancestor_list) > 0) {
  distances <- sapply(
    ancestor_list,
    function(x) x$min_levels_of_separation %||% 999L
  )
  ancestor_list <- ancestor_list[order(distances)]
}

cat(sprintf("Path from '%s' to root:\n",
            path_result$data$concept_name %||% "Type 2 diabetes mellitus"))
for (i in seq_along(ancestor_list)) {
  a <- ancestor_list[[i]]
  indent <- paste(rep("  ", i), collapse = "")
  cat(sprintf("%s-> [%d] %s\n", indent, a$concept_id, a$concept_name))
}
cat("\n")

# ============================================================================
# Done
# ============================================================================

cat("Done!\n")

#!/usr/bin/env Rscript
#' Example: FHIR-to-OMOP Concept Resolver
#'
#' Demonstrates the FHIR Concept Resolver, which translates FHIR coded
#' values (system URI + code) into OMOP standard concepts, CDM target
#' tables, and optional Phoebe recommendations - all in a single API call.
#'
#' Covers:
#'   - Direct standard lookups (SNOMED, LOINC, RxNorm)
#'   - Non-standard code mapping (ICD-10-CM -> SNOMED via "Maps to")
#'   - Text-only semantic search fallback
#'   - Vocabulary ID bypass (skip URI resolution)
#'   - Phoebe recommendations
#'   - Mapping quality signal
#'   - Batch resolution
#'   - CodeableConcept resolution with OHDSI vocabulary preference
#'   - Administrative codes via the IG ConceptMap layer (gender, ...)
#'   - user_selected coding precedence
#'   - on_unmapped = "sentinel" (concept_id 0 instead of a 404)
#'   - Error handling
#'
#' For tibble-shaped batch output, standalone wrapper functions, and the
#' omophub_fhir_url() helper, see inst/examples/fhir_interop.R.
#'
#' Run with: Rscript inst/examples/fhir_resolver.R

library(omophub)

# Null-coalescing operator (available in base R 4.4+; define locally for
# compatibility with the R >= 4.1 package requirement).
`%||%` <- function(a, b) if (is.null(a)) b else a

# ============================================================================
# Setup
# ============================================================================

client <- OMOPHubClient$new()

cat("OMOPHub R Client - FHIR Resolver Examples\n")
cat("==========================================\n\n")

# ============================================================================
# 1. Direct SNOMED Resolution
# ============================================================================

cat("1. SNOMED direct resolution\n")
cat("---------------------------\n")

result <- client$fhir$resolve(
  system = "http://snomed.info/sct",
  code = "44054006",
  resource_type = "Condition"
)
res <- result$resolution
cat("  Source:      ", res$source_concept$concept_name, "\n")
cat("  Standard:    ", res$standard_concept$concept_name, "\n")
cat("  Mapping type:", res$mapping_type, "\n")
cat("  Target table:", res$target_table, "\n")
cat("  Alignment:   ", res$domain_resource_alignment, "\n\n")

# ============================================================================
# 2. ICD-10-CM -> SNOMED via Maps to
# ============================================================================

cat("2. Non-standard code with Maps-to traversal\n")
cat("-------------------------------------------\n")

result <- client$fhir$resolve(
  system = "http://hl7.org/fhir/sid/icd-10-cm",
  code = "E11.9",
  resource_type = "Condition"
)
res <- result$resolution
cat("  Source:      [", res$source_concept$vocabulary_id, "] ",
    res$source_concept$concept_name, "\n", sep = "")
cat("  Standard:    [", res$standard_concept$vocabulary_id, "] ",
    res$standard_concept$concept_name, "\n", sep = "")
cat("  Mapping type:", res$mapping_type, "\n")
cat("  Relationship:", res$relationship_id %||% "N/A", "\n")
cat("  Target table:", res$target_table, "\n\n")

# ============================================================================
# 3. LOINC -> measurement table
# ============================================================================

cat("3. LOINC lab code -> measurement table\n")
cat("--------------------------------------\n")

result <- client$fhir$resolve(
  system = "http://loinc.org",
  code = "2339-0",  # Glucose [Mass/volume] in Blood
  resource_type = "Observation"
)
res <- result$resolution
cat("  Concept:     ", res$standard_concept$concept_name, "\n")
cat("  Domain:      ", res$standard_concept$domain_id, "\n")
cat("  Target table:", res$target_table, "\n\n")

# ============================================================================
# 4. RxNorm -> drug_exposure table
# ============================================================================

cat("4. RxNorm drug code -> drug_exposure table\n")
cat("------------------------------------------\n")

result <- client$fhir$resolve(
  system = "http://www.nlm.nih.gov/research/umls/rxnorm",
  code = "197696",  # Acetaminophen 325 MG Oral Tablet
  resource_type = "MedicationRequest"
)
res <- result$resolution
cat("  Concept:     ", res$standard_concept$concept_name, "\n")
cat("  Target table:", res$target_table, "\n\n")

# ============================================================================
# 5. Text-only resolution (semantic search fallback)
# ============================================================================

cat("5. Text-only semantic search fallback\n")
cat("-------------------------------------\n")

result <- client$fhir$resolve(
  display = "Blood Sugar",
  resource_type = "Observation"
)
res <- result$resolution
cat("  Matched:     ", res$standard_concept$concept_name, "\n")
cat("  Mapping type:", res$mapping_type, "\n")
if (!is.null(res$similarity_score)) {
  cat("  Similarity:  ", sprintf("%.2f", res$similarity_score), "\n")
}
cat("  Target table:", res$target_table, "\n\n")

# ============================================================================
# 6. Vocabulary ID bypass (skip URI resolution)
# ============================================================================

cat("6. Vocabulary ID bypass\n")
cat("-----------------------\n")

result <- client$fhir$resolve(
  vocabulary_id = "ICD10CM",
  code = "E11.9"
)
res <- result$resolution
cat("  Vocabulary:  ", res$vocabulary_id, "\n")
cat("  Standard:    ", res$standard_concept$concept_name, "\n\n")

# ============================================================================
# 7. Include Phoebe recommendations
# ============================================================================

cat("7. With Phoebe recommendations\n")
cat("------------------------------\n")

result <- client$fhir$resolve(
  system = "http://snomed.info/sct",
  code = "44054006",
  include_recommendations = TRUE,
  recommendations_limit = 5L
)
res <- result$resolution
cat("  Resolved:    ", res$standard_concept$concept_name, "\n")
recs <- res$recommendations %||% list()
cat("  Recommendations (", length(recs), "):\n", sep = "")
for (rec in recs) {
  cat(sprintf("    - %s (%s) via %s\n",
              rec$concept_name, rec$domain_id, rec$relationship_id))
}
cat("\n")

# ============================================================================
# 8. Mapping quality signal
# ============================================================================

cat("8. With mapping quality signal\n")
cat("------------------------------\n")

result <- client$fhir$resolve(
  system = "http://snomed.info/sct",
  code = "44054006",
  include_quality = TRUE
)
cat("  SNOMED direct -> quality:",
    result$resolution$mapping_quality %||% "N/A", "\n")

result <- client$fhir$resolve(
  display = "heart attack",
  resource_type = "Condition",
  include_quality = TRUE
)
cat("  Text semantic -> quality:",
    result$resolution$mapping_quality %||% "N/A", "\n\n")

# ============================================================================
# 9. Batch resolution
# ============================================================================

cat("9. Batch resolution\n")
cat("-------------------\n")

batch <- client$fhir$resolve_batch(
  list(
    list(system = "http://snomed.info/sct", code = "44054006"),
    list(system = "http://loinc.org", code = "2339-0"),
    list(system = "http://www.nlm.nih.gov/research/umls/rxnorm", code = "197696"),
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9")
  ),
  resource_type = "Condition",
  include_quality = TRUE
)

cat(sprintf("  Total: %d, Resolved: %d, Failed: %d\n",
            batch$summary$total, batch$summary$resolved, batch$summary$failed))

for (i in seq_along(batch$results)) {
  item <- batch$results[[i]]
  if (!is.null(item$resolution)) {
    res <- item$resolution
    cat(sprintf("  [%d] %s -> %s (quality: %s)\n",
                i, res$standard_concept$concept_name, res$target_table,
                res$mapping_quality %||% "N/A"))
  } else {
    cat(sprintf("  [%d] FAILED: %s\n", i, item$error$code %||% "unknown"))
  }
}
cat("\n")

# ============================================================================
# 10. CodeableConcept with vocabulary preference
# ============================================================================

cat("10. CodeableConcept resolution (OHDSI preference)\n")
cat("-------------------------------------------------\n")

result <- client$fhir$resolve_codeable_concept(
  coding = list(
    list(
      system = "http://hl7.org/fhir/sid/icd-10-cm",
      code = "E11.9",
      display = "Type 2 diabetes mellitus without complications"
    ),
    list(
      system = "http://snomed.info/sct",
      code = "44054006",
      display = "Type 2 diabetes mellitus"
    )
  ),
  resource_type = "Condition",
  include_recommendations = TRUE,
  recommendations_limit = 3L
)

best <- result$best_match
if (!is.null(best)) {
  res <- best$resolution
  cat(sprintf("  Best match: [%s] %s\n",
              res$source_concept$vocabulary_id,
              res$standard_concept$concept_name))
  cat("  Target table:", res$target_table, "\n")
}
cat("  Alternatives:", length(result$alternatives), "\n")
cat("  Unresolved:  ", length(result$unresolved), "\n\n")

# ============================================================================
# 11. CodeableConcept text fallback
# ============================================================================

cat("11. CodeableConcept text fallback\n")
cat("---------------------------------\n")

result <- client$fhir$resolve_codeable_concept(
  coding = list(
    # Bogus code - will fail; text fallback should pick it up
    list(system = "http://loinc.org", code = "99999-9")
  ),
  text = "Type 2 diabetes mellitus",
  resource_type = "Condition"
)

best <- result$best_match
if (!is.null(best)) {
  res <- best$resolution
  cat("  Resolved via:", res$mapping_type, "\n")
  cat("  Concept:    ", res$standard_concept$concept_name, "\n")
}
cat("  Failed codings:", length(result$unresolved), "\n\n")

# ============================================================================
# 12. Administrative code via the IG ConceptMap layer (gender, encounter, ...)
# ============================================================================

cat("12. Administrative gender (IG ConceptMap)\n")
cat("-----------------------------------------\n")

result <- client$fhir$resolve(
  system = "http://hl7.org/fhir/administrative-gender",
  code = "male",
  resource_type = "Patient"
)

res <- result$resolution
cat(sprintf("  male -> %s (%s)\n",
            res$standard_concept$concept_name,
            res$standard_concept$concept_id))
cat("  Target table:     ", res$target_table, "\n")
cat("  Via IG ConceptMap:", res$concept_map_id %||% "N/A", "\n")
# Composite source concepts also surface value_as_concept (IG 'Maps to value').
if (!is.null(res$value_as_concept)) {
  cat("  Value concept:    ", res$value_as_concept$concept_name, "\n")
}
cat("\n")

# ============================================================================
# 13. CodeableConcept with user_selected (overrides vocabulary preference)
# ============================================================================

cat("13. CodeableConcept with user_selected\n")
cat("--------------------------------------\n")

result <- client$fhir$resolve_codeable_concept(
  coding = list(
    # SNOMED would normally win; the user-selected ICD-10-CM coding wins.
    list(system = "http://snomed.info/sct", code = "44054006"),
    list(
      system = "http://hl7.org/fhir/sid/icd-10-cm",
      code = "E11.9",
      user_selected = TRUE
    )
  ),
  resource_type = "Condition"
)

best <- result$best_match
if (!is.null(best)) {
  cat("  best_match source:", best$resolution$source_concept$vocabulary_id, "\n")
}
cat("\n")

# ============================================================================
# 14. on_unmapped = "sentinel" (concept_id 0 record instead of a 404)
# ============================================================================

cat("14. on_unmapped = 'sentinel'\n")
cat("----------------------------\n")

result <- client$fhir$resolve(
  system = "http://snomed.info/sct",
  code = "00000000",
  on_unmapped = "sentinel"
)

res <- result$resolution
cat("  Mapping type:       ", res$mapping_type, "\n")
cat("  Standard concept_id:", res$standard_concept$concept_id, "\n\n")

# ============================================================================
# 15. Error handling
# ============================================================================

cat("15. Error handling\n")
cat("------------------\n")

# Restricted vocabulary (CPT4) -> 403
tryCatch(
  client$fhir$resolve(
    system = "http://www.ama-assn.org/go/cpt",
    code = "99213"
  ),
  omophub_api_error = function(e) {
    cat(sprintf("  CPT4 restricted -> %s: %s\n",
                e$status_code %||% "?", e$message %||% e$parent$message))
  },
  error = function(e) {
    cat("  CPT4 restricted -> error:", conditionMessage(e), "\n")
  }
)

# Non-existent code -> 404
tryCatch(
  client$fhir$resolve(
    system = "http://snomed.info/sct",
    code = "00000000"
  ),
  omophub_not_found = function(e) {
    cat("  Non-existent code -> not found:", e$message, "\n")
  },
  omophub_api_error = function(e) {
    cat(sprintf("  Non-existent code -> %s: %s\n",
                e$status_code %||% "?", e$message %||% e$parent$message))
  },
  error = function(e) {
    cat("  Non-existent code -> error:", conditionMessage(e), "\n")
  }
)
cat("\n")

cat("Done!\n")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## ----install------------------------------------------------------------------
# # install.packages("devtools")
# devtools::install_github("omophub/omophub-R")

## ----auth-env-----------------------------------------------------------------
# Sys.setenv(OMOPHUB_API_KEY = "your_api_key_here")

## ----client-------------------------------------------------------------------
# library(omophub)
# 
# # Create client (reads API key from environment)
# client <- OMOPHubClient$new()
# 
# # Or provide API key explicitly
# client <- OMOPHubClient$new(api_key = "your_api_key")
# 
# # With additional options
# client <- OMOPHubClient$new(
#   api_key = "your_api_key",
#   timeout = 30,
#   max_retries = 3,
#   vocab_version = "2025.1"
# )

## ----get-concept--------------------------------------------------------------
# concept <- client$concepts$get(201826)
# print(concept$concept_name)
# # [1] "Type 2 diabetes mellitus"

## ----get-by-code--------------------------------------------------------------
# concept <- client$concepts$get_by_code("SNOMED", "44054006")
# print(concept$concept_name)
# # [1] "Type 2 diabetes mellitus"

## ----batch--------------------------------------------------------------------
# result <- client$concepts$batch(c(201826, 4329847, 1112807))
# for (concept in result$concepts) {
#   cat(sprintf("%s: %s\n", concept$concept_id, concept$concept_name))
# }

## ----search-basic-------------------------------------------------------------
# results <- client$search$basic("diabetes mellitus", page_size = 10)
# for (concept in results$data) {
#   cat(sprintf("%s: %s\n", concept$concept_id, concept$concept_name))
# }

## ----search-filters-----------------------------------------------------------
# results <- client$search$basic(
#   "heart attack",
#   vocabulary_ids = "SNOMED",
#   domain_ids = "Condition",
#   page_size = 20
# )

## ----semantic-search----------------------------------------------------------
# # Natural language search - understands clinical intent
# results <- client$search$semantic("high blood sugar levels")
# for (r in results$data$results) {
#   cat(sprintf("%s (similarity: %.2f)\n", r$concept_name, r$similarity_score))
# }

## ----semantic-filtered--------------------------------------------------------
# results <- client$search$semantic(
#   "heart attack",
#   vocabulary_ids = "SNOMED",
#   domain_ids = "Condition",
#   threshold = 0.5
# )

## ----semantic-all-------------------------------------------------------------
# all_results <- client$search$semantic_all(
#   "chronic kidney disease",
#   page_size = 50,
#   max_pages = 5,
#   progress = TRUE
# )
# print(nrow(all_results))

## ----similar-by-id------------------------------------------------------------
# # Find concepts similar to Type 2 diabetes mellitus
# similar <- client$search$similar(concept_id = 201826)
# for (s in similar$similar_concepts) {
#   cat(sprintf("%s (score: %.2f)\n", s$concept_name, s$similarity_score))
# }

## ----similar-by-query---------------------------------------------------------
# # Semantic similarity (neural embeddings)
# similar <- client$search$similar(
#   query = "high blood pressure",
#   algorithm = "semantic"
# )
# 
# # Lexical similarity (string matching)
# similar <- client$search$similar(
#   query = "high blood pressure",
#   algorithm = "lexical"
# )
# 
# # Hybrid (combined - default)
# similar <- client$search$similar(
#   query = "high blood pressure",
#   algorithm = "hybrid",
#   include_scores = TRUE,
#   include_explanations = TRUE
# )

## ----bulk-basic---------------------------------------------------------------
# results <- client$search$bulk_basic(list(
#   list(search_id = "q1", query = "diabetes mellitus"),
#   list(search_id = "q2", query = "hypertension"),
#   list(search_id = "q3", query = "aspirin")
# ), defaults = list(vocabulary_ids = list("SNOMED"), page_size = 5))
# 
# # Each result is matched by search_id
# for (item in results$results) {
#   cat(sprintf("%s: %d results\n", item$search_id, length(item$results)))
# }

## ----bulk-semantic------------------------------------------------------------
# results <- client$search$bulk_semantic(list(
#   list(search_id = "s1", query = "heart failure treatment options"),
#   list(search_id = "s2", query = "type 2 diabetes medication")
# ), defaults = list(threshold = 0.5, page_size = 10))
# 
# for (item in results$results) {
#   cat(sprintf("%s: %d results\n", item$search_id,
#               item$result_count %||% length(item$results)))
# }

## ----bulk-overrides-----------------------------------------------------------
# # Different domains per query, shared vocabulary filter
# results <- client$search$bulk_basic(list(
#   list(search_id = "cond", query = "diabetes", domain_ids = list("Condition")),
#   list(search_id = "drug", query = "metformin", domain_ids = list("Drug"))
# ), defaults = list(vocabulary_ids = list("SNOMED", "RxNorm"), page_size = 5))

## ----autocomplete-------------------------------------------------------------
# suggestions <- client$concepts$suggest("diab", page_size = 5)
# for (s in suggestions$suggestions) {
#   print(s$suggestion)
# }

## ----pagination-manual--------------------------------------------------------
# # First page
# results <- client$search$basic("diabetes", page = 1, page_size = 50)
# 
# # Check pagination info
# print(results$meta$total_items)
# print(results$meta$has_next)
# 
# # Get next page if available
# if (isTRUE(results$meta$has_next)) {
#   page2 <- client$search$basic("diabetes", page = 2, page_size = 50)
# }

## ----pagination-auto----------------------------------------------------------
# all_results <- client$search$basic_all(
#   "diabetes",
#   page_size = 100,
#   max_pages = 5,
#   progress = TRUE
# )
# 
# # Results are a tibble
# print(nrow(all_results))
# print(names(all_results))

## ----ancestors----------------------------------------------------------------
# result <- client$hierarchy$ancestors(201826, max_levels = 3)
# for (ancestor in result$ancestors) {
#   print(ancestor$concept_name)
# }

## ----descendants--------------------------------------------------------------
# result <- client$hierarchy$descendants(201826, max_levels = 2)
# for (descendant in result$descendants) {
#   print(descendant$concept_name)
# }

## ----mappings-----------------------------------------------------------------
# result <- client$mappings$get(201826)
# for (mapping in result$mappings) {
#   cat(sprintf("%s: %s\n",
#               mapping$target_vocabulary_id,
#               mapping$target_concept_name))
# }

## ----mappings-filter----------------------------------------------------------
# result <- client$mappings$get(
#   201826,
#   target_vocabulary = "ICD10CM"
# )

## ----error-handling-----------------------------------------------------------
# tryCatch(
#   {
#     concept <- client$concepts$get(999999999)
#   },
#   omophub_not_found = function(e) {
#     message("Concept not found: ", e$message)
#   },
#   omophub_api_error = function(e) {
#     message("API error: ", e$message)
#   }
# )

## ----fhir-resolve-------------------------------------------------------------
# result <- client$fhir$resolve(
#   system = "http://snomed.info/sct",
#   code = "44054006",
#   resource_type = "Condition"
# )
# cat(result$resolution$standard_concept$concept_name)
# cat(result$resolution$target_table)   # "condition_occurrence"
# cat(result$resolution$mapping_type)   # "direct"

## ----fhir-mapped--------------------------------------------------------------
# result <- client$fhir$resolve(
#   system = "http://hl7.org/fhir/sid/icd-10-cm",
#   code = "E11.9"
# )
# cat(result$resolution$mapping_type)                    # "mapped"
# cat(result$resolution$standard_concept$vocabulary_id)  # "SNOMED"

## ----fhir-batch---------------------------------------------------------------
# batch <- client$fhir$resolve_batch(list(
#   list(system = "http://snomed.info/sct", code = "44054006"),
#   list(system = "http://loinc.org", code = "2339-0")
# ))
# cat(sprintf("Resolved: %d/%d\n", batch$summary$resolved, batch$summary$total))

## ----fhir-codeable------------------------------------------------------------
# result <- client$fhir$resolve_codeable_concept(
#   coding = list(
#     list(system = "http://snomed.info/sct", code = "44054006"),
#     list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9")
#   ),
#   resource_type = "Condition"
# )
# # SNOMED wins over ICD-10-CM per OHDSI preference
# cat(result$best_match$resolution$source_concept$vocabulary_id)  # "SNOMED"

## ----fhir-batch-tibble--------------------------------------------------------
# library(dplyr)
# 
# tbl <- client$fhir$resolve_batch(
#   list(
#     list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9"),
#     list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "I10"),
#     list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "J45.909")
#   ),
#   as_tibble = TRUE
# )
# 
# tbl |>
#   filter(status == "resolved") |>
#   select(source_code, standard_concept_name, target_table)

## ----fhir-batch-summary-------------------------------------------------------
# attr(tbl, "summary")

## ----fhir-standalone-wrappers-------------------------------------------------
# # Equivalent to client$fhir$resolve()
# client |>
#   fhir_resolve(
#     system = "http://snomed.info/sct",
#     code = "44054006",
#     resource_type = "Condition"
#   )
# 
# # Tibble-shaped batch in a pipe
# tbl <- client |>
#   fhir_resolve_batch(
#     codings = list(
#       list(system = "http://snomed.info/sct", code = "44054006"),
#       list(system = "http://loinc.org", code = "2339-0")
#     ),
#     as_tibble = TRUE
#   )
# 
# client |>
#   fhir_resolve_codeable_concept(
#     coding = list(
#       list(system = "http://snomed.info/sct", code = "44054006"),
#       list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9")
#     ),
#     resource_type = "Condition"
#   )

## ----fhir-url-helper----------------------------------------------------------
# omophub_fhir_url()
# #> "https://fhir.omophub.com/fhir/r4"
# 
# omophub_fhir_url("r5")
# #> "https://fhir.omophub.com/fhir/r5"

## ----fhir-httr2---------------------------------------------------------------
# library(httr2)
# 
# resp <- request(omophub_fhir_url()) |>
#   req_url_path_append("CodeSystem/$lookup") |>
#   req_url_query(
#     system = "http://snomed.info/sct",
#     code = "44054006"
#   ) |>
#   req_headers(Authorization = paste("Bearer", Sys.getenv("OMOPHUB_API_KEY"))) |>
#   req_perform()
# 
# params <- resp_body_json(resp)
# # Raw FHIR Parameters resource with the concept display and designations.

# Redaction function for httptest2
# Removes sensitive data from recorded HTTP responses

function(response) {
  # Redact API keys from headers
  response <- gsub_response(response, "Bearer [a-zA-Z0-9_-]+", "Bearer REDACTED")
  response <- gsub_response(response, "api_key=[^&]+", "api_key=REDACTED")
  response <- gsub_response(response, '"api_key":"[^"]+"', '"api_key":"REDACTED"')

  # Redact any OMOPHUB_API_KEY patterns
  response <- gsub_response(response, "oh_[a-zA-Z0-9]+", "oh_REDACTED")

  response
}

#!/usr/bin/env Rscript
#' Example: FHIR Interop (omophub R SDK 1.7.0+)
#'
#' Demonstrates the 1.7.0 additions to the FHIR Concept Resolver:
#'
#'   1. Tibble output for batch resolution (as_tibble = TRUE)
#'   2. Standalone wrapper functions (fhir_resolve, fhir_resolve_batch,
#'      fhir_resolve_codeable_concept) for pipe-friendly workflows
#'   3. omophub_fhir_url() helper for configuring external FHIR clients
#'      (httr2, fhircrackr) to talk to the OMOPHub FHIR Terminology
#'      Service directly
#'
#' For the core Concept Resolver surface (single/batch/codeable-concept
#' resolution, recommendations, quality signals, error handling) see
#' inst/examples/fhir_resolver.R.
#'
#' Run with: Rscript inst/examples/fhir_interop.R

library(omophub)

# Null-coalescing operator (available in base R 4.4+; define locally for
# compatibility with the R >= 4.1 package requirement).
`%||%` <- function(a, b) if (is.null(a)) b else a

client <- OMOPHubClient$new()

cat("OMOPHub R Client - FHIR Interop Examples (1.7.0+)\n")
cat("==================================================\n\n")

# ============================================================================
# 1. Tibble output for batch resolution
# ============================================================================

cat("1. Tibble output for resolve_batch (as_tibble = TRUE)\n")
cat("-----------------------------------------------------\n")

tbl <- client$fhir$resolve_batch(
  list(
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9"),
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "I10"),
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "J45.909"),
    list(system = "http://snomed.info/sct", code = "bogus-code")  # will fail
  ),
  as_tibble = TRUE
)

cat("  Tibble class:", paste(class(tbl), collapse = ", "), "\n")
cat("  Rows:        ", nrow(tbl), "\n")
cat("  Columns:     ", paste(names(tbl), collapse = ", "), "\n\n")

# Print per-row summary
for (i in seq_len(nrow(tbl))) {
  row <- tbl[i, ]
  if (row$status == "resolved") {
    cat(sprintf("  [%d] %s -> %s (%s)\n",
                i, row$source_code, row$standard_concept_name, row$target_table))
  } else {
    cat(sprintf("  [%d] %s -> FAILED (%s)\n",
                i, row$source_code, row$status_detail))
  }
}
cat("\n")

# Summary attached as attribute
summary_info <- attr(tbl, "summary")
cat(sprintf("  Summary: total=%d resolved=%d failed=%d\n",
            summary_info$total, summary_info$resolved, summary_info$failed))
cat("\n")

# dplyr-friendly filtering (optional - commented out to avoid forcing dplyr)
#
# library(dplyr)
# tbl |>
#   filter(status == "resolved") |>
#   select(source_code, standard_concept_name, target_table)

# Default as_tibble = FALSE still returns the legacy list shape
list_result <- client$fhir$resolve_batch(
  list(list(system = "http://snomed.info/sct", code = "44054006"))
)
cat("  as_tibble = FALSE still returns list:",
    paste(class(list_result), collapse = ", "), "\n\n")

# ============================================================================
# 2. Standalone wrapper functions
# ============================================================================

cat("2. Standalone wrapper functions\n")
cat("-------------------------------\n")

# fhir_resolve(client, ...) is equivalent to client$fhir$resolve(...)
result <- fhir_resolve(
  client,
  system = "http://snomed.info/sct",
  code = "44054006",
  resource_type = "Condition"
)
cat("  fhir_resolve: ", result$resolution$standard_concept$concept_name, "\n")

# Pipe-friendly form
result_piped <- client |>
  fhir_resolve(
    system = "http://snomed.info/sct",
    code = "44054006",
    resource_type = "Condition"
  )
cat("  Pipe form:   ", result_piped$resolution$standard_concept$concept_name, "\n")

# Batch wrapper
tbl2 <- client |>
  fhir_resolve_batch(
    codings = list(
      list(system = "http://snomed.info/sct", code = "44054006"),
      list(system = "http://loinc.org", code = "2339-0")
    ),
    as_tibble = TRUE
  )
cat("  fhir_resolve_batch: ", nrow(tbl2), "rows\n")

# CodeableConcept wrapper
cc_result <- client |>
  fhir_resolve_codeable_concept(
    coding = list(
      list(system = "http://snomed.info/sct", code = "44054006"),
      list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9")
    ),
    resource_type = "Condition"
  )
cat(sprintf("  fhir_resolve_codeable_concept: best = [%s]\n",
            cc_result$best_match$resolution$source_concept$vocabulary_id))
cat("\n")

# ============================================================================
# 3. omophub_fhir_url() for external FHIR client interop
# ============================================================================

cat("3. omophub_fhir_url() helper\n")
cat("----------------------------\n")

# Default R4 base URL
cat("  R4:  ", omophub_fhir_url(), "\n")
cat("  R4B: ", omophub_fhir_url("r4b"), "\n")
cat("  R5:  ", omophub_fhir_url("r5"), "\n")
cat("  R6:  ", omophub_fhir_url("r6"), "\n\n")

# ============================================================================
# 4. Direct httr2 call against OMOPHub's FHIR Terminology Service
# ============================================================================

cat("4. Direct httr2 call to CodeSystem/$lookup\n")
cat("------------------------------------------\n")

# Use the Concept Resolver (fhir_resolve) when you want OMOP-enriched
# answers (standard concept, CDM target table, mapping quality).
# Use omophub_fhir_url() + httr2 when you need raw FHIR Parameters /
# Bundle responses for FHIR-native tooling.

api_key <- Sys.getenv("OMOPHUB_API_KEY")
if (nzchar(api_key)) {
  resp <- httr2::request(omophub_fhir_url()) |>
    httr2::req_url_path_append("CodeSystem/$lookup") |>
    httr2::req_url_query(
      system = "http://snomed.info/sct",
      code = "44054006"
    ) |>
    httr2::req_headers(Authorization = paste("Bearer", api_key)) |>
    httr2::req_perform()

  params <- httr2::resp_body_json(resp)

  # Extract the display from the Parameters resource
  display <- NULL
  for (p in params$parameter %||% list()) {
    if (!is.null(p$name) && p$name == "display") {
      display <- p$valueString
      break
    }
  }
  cat("  $lookup display:", display %||% "(not found)", "\n")
} else {
  cat("  Skipped: OMOPHUB_API_KEY not set\n")
}
cat("\n")

cat("Done!\n")

#!/usr/bin/env Rscript
#' Example: Basic Usage
#'
#' Demonstrates basic OMOPHub client usage including:
#' - Client initialization
#' - Getting a single concept
#' - Basic search
#' - Listing vocabularies
#'
#' Run with: Rscript inst/examples/basic_usage.R

library(omophub)

# Null-coalescing operator (available in base R 4.4+; define locally for
# compatibility with the R >= 4.1 package requirement).
`%||%` <- function(a, b) if (is.null(a)) b else a

# ============================================================================
# Setup
# ============================================================================

# Initialize client (uses OMOPHUB_API_KEY environment variable)
# You can also pass the API key directly:
#   client <- OMOPHubClient$new(api_key = "oh_your_api_key")
client <- OMOPHubClient$new()

cat("OMOPHub R Client - Basic Usage Example\n")
cat("======================================\n\n")

# ============================================================================
# Get a Single Concept
# ============================================================================

cat("1. Getting a single concept by ID\n")
cat("---------------------------------\n")

# Get Type 2 diabetes mellitus (SNOMED concept)
concept <- client$concepts$get(201826)

cat("Concept ID:", concept$concept_id, "\n")
cat("Name:", concept$concept_name, "\n")
cat("Vocabulary:", concept$vocabulary_id, "\n")
cat("Domain:", concept$domain_id, "\n")
cat("Concept Class:", concept$concept_class_id, "\n")
cat("Concept Code:", concept$concept_code, "\n")
cat("Standard Concept:", concept$standard_concept, "\n\n")

# ============================================================================
# Basic Search
# ============================================================================

cat("2. Searching for concepts\n")
cat("-------------------------\n")

# Search for diabetes-related concepts
results <- client$search$basic("diabetes", page_size = 5)

cat("Found concepts matching 'diabetes':\n")
concepts <- results$data %||% results$concepts %||% results
for (c in concepts) {
  cat(sprintf("  [%s] %s (%s)\n",
              c$concept_id,
              c$concept_name,
              c$vocabulary_id))
}
cat("\n")

# ============================================================================
# List Vocabularies
# ============================================================================

cat("3. Listing available vocabularies\n")
cat("---------------------------------\n")

# Get first 10 vocabularies
vocabs <- client$vocabularies$list(page_size = 10)

cat("Available vocabularies:\n")
vocab_list <- vocabs$data %||% vocabs$vocabularies %||% vocabs
for (v in vocab_list) {
  cat(sprintf("  %s - %s\n",
              v$vocabulary_id,
              v$vocabulary_name))
}
cat("\n")

# ============================================================================
# Get Concept with Additional Information
# ============================================================================

cat("4. Getting a concept with relationships\n")
cat("---------------------------------------\n")

# Get concept with relationships included
concept_full <- client$concepts$get(201826, include_relationships = TRUE)

cat("Concept:", concept_full$concept_name, "\n")
if (!is.null(concept_full$relationships)) {
  cat("Number of relationships:", length(concept_full$relationships), "\n")
}
cat("\n")

# ============================================================================
# Done
# ============================================================================

cat("Done!\n")

#!/usr/bin/env Rscript
#' Example: Map Between Vocabularies
#'
#' Demonstrates vocabulary mapping operations:
#' - Getting mappings for a single concept
#' - Batch mapping multiple concepts
#' - Looking up concepts by vocabulary code
#' - Understanding mapping quality scores
#'
#' Run with: Rscript inst/examples/map_between_vocabularies.R

library(omophub)

# Null-coalescing operator (available in base R 4.4+; define locally for
# compatibility with the R >= 4.1 package requirement).
`%||%` <- function(a, b) if (is.null(a)) b else a

# ============================================================================
# Setup
# ============================================================================

client <- OMOPHubClient$new()

cat("OMOPHub R Client - Vocabulary Mapping Examples\n")
cat("===============================================\n\n")

# Type 2 diabetes mellitus (SNOMED)
DIABETES_CONCEPT_ID <- 201826

# ============================================================================
# Get Mappings for a Single Concept
# ============================================================================

cat("1. Getting mappings for a SNOMED concept\n")
cat("-----------------------------------------\n")

# One page of mappings for Type 2 diabetes. `get()` is NOT the whole set --
# read attr(mappings, "pagination") or use get_all(), shown in section 3.
mappings <- client$mappings$get(DIABETES_CONCEPT_ID)
pag <- attr(mappings, "pagination")

cat(sprintf("Mappings for concept %d (Type 2 diabetes mellitus):\n", DIABETES_CONCEPT_ID))
if (!is.null(pag)) {
  cat(sprintf("  page %s of %s, %s in total\n",
              pag$page, pag$total_pages, pag$total_items))
}
for (m in mappings$mappings) {
  # This endpoint projects each row down to source/target id + name,
  # relationship_id and confidence. Vocabulary id and concept code are not
  # included -- resolve them with client$concepts$get(target_concept_id),
  # as section 2 does. (client$mappings$map() does return them.)
  cat(sprintf("  %s: %s %s\n",
              m$relationship_id, m$target_concept_id, m$target_concept_name))
}
cat("\n")

# ============================================================================
# Mapping to a Specific Vocabulary -- Mind the Direction
# ============================================================================

cat("2. Which ICD-10-CM codes correspond to this concept\n")
cat("---------------------------------------------------\n")

# `Maps to` always points at a *standard* concept, and ICD-10-CM is
# non-standard, so filtering the default relationship to ICD10CM matches
# nothing -- an empty list, not an error. The codes that roll up INTO a
# standard concept are reached with `Mapped from`.
empty <- client$mappings$get(
  DIABETES_CONCEPT_ID,
  target_vocabulary = "ICD10CM"
)
cat(sprintf("  'Maps to' + ICD10CM:     %d rows (as expected)\n",
            length(empty$mappings)))

icd_mappings <- client$mappings$get_all(
  DIABETES_CONCEPT_ID,
  relationship_ids = "Mapped from",
  target_vocabulary = "ICD10CM",
  progress = FALSE
)
cat(sprintf("  'Mapped from' + ICD10CM: %d rows\n", nrow(icd_mappings)))

# The mapping row has the target's id and name but not its code, so resolve
# the first few. One request each -- fine for five rows, not for all 74.
for (i in seq_len(min(5, nrow(icd_mappings)))) {
  target <- client$concepts$get(icd_mappings$target_concept_id[i])
  cat(sprintf("    <- [%s] %s %s\n",
              target$vocabulary_id, target$concept_code, target$concept_name))
}
cat("\n")

# ============================================================================
# Get Every Mapping, Across All Pages
# ============================================================================

cat("3. Every mapping, not just the first page\n")
cat("------------------------------------------\n")

# Copy this one when building a code list: a partial code list is wrong in a
# way nothing in the result reveals.
all_mappings <- client$mappings$get_all(DIABETES_CONCEPT_ID, progress = FALSE)
cat(sprintf("Retrieved %d mappings in total\n\n", nrow(all_mappings)))

# ============================================================================
# Value-as-Concept: Composite Concepts Decompose Across Two Relationships
# ============================================================================

cat("4. Value-as-Concept\n")
cat("-------------------\n")

# The default returns "Maps to" only, so you learn the patient is allergic to
# *a drug* but not *which* drug.
PENICILLIN_ALLERGY_ID <- 4167462
decomposed <- client$mappings$get(
  PENICILLIN_ALLERGY_ID,
  relationship_ids = c("Maps to", "Maps to value")
)

for (m in decomposed$mappings) {
  # "Maps to" -> the OMOP concept column; "Maps to value" -> value_as_concept_id
  column <- if (identical(m$relationship_id, "Maps to value")) {
    "value_as_concept_id"
  } else {
    "concept_id"
  }
  cat(sprintf("  %s: %s -> %s\n",
              m$relationship_id, m$target_concept_name, column))
}
cat("\n")

# ============================================================================
# Excluding Invalid/Deprecated Mappings
# ============================================================================

cat("5. Dropping deprecated mappings\n")
cat("-------------------------------\n")

# Deprecated mappings are returned BY DEFAULT on this endpoint. FALSE is the
# meaningful direction; omitting the argument keeps them.
valid_only <- client$mappings$get_all(
  DIABETES_CONCEPT_ID,
  include_invalid = FALSE,
  progress = FALSE
)
cat(sprintf("  default (includes deprecated): %d\n", nrow(all_mappings)))
cat(sprintf("  include_invalid = FALSE:       %d\n\n", nrow(valid_only)))

# ============================================================================
# Batch Mapping Multiple Concepts
# ============================================================================

cat("6. Batch mapping multiple concepts\n")
cat("----------------------------------\n")

# Map multiple SNOMED concepts to ICD-10-CM
concept_ids <- c(
  201826,   # Type 2 diabetes mellitus
  320128,   # Essential hypertension
  4329847   # Myocardial infarction
)

batch_result <- client$mappings$map(
  target_vocabulary = "ICD10CM",
  source_concepts = concept_ids
)

cat("Batch mapping results:\n")
mapping_list <- batch_result$mappings %||% batch_result$data %||% batch_result
for (m in mapping_list) {
  source <- m$source_concept %||% m
  source_name <- source$concept_name %||% m$source_concept_name %||% "Unknown"

  targets <- m$target_concepts %||% m$targets %||% list(m$target_concept %||% m)

  cat(sprintf("\n  %s:\n", source_name))
  for (t in targets) {
    cat(sprintf("    -> %s (%s)\n",
                t$concept_name %||% "Unknown",
                t$concept_code %||% "?"))
  }
}

# Print coverage summary if available
if (!is.null(batch_result$summary)) {
  cat(sprintf("\nCoverage: %d/%d concepts mapped\n",
              batch_result$summary$mapped_count %||% 0,
              batch_result$summary$total_count %||% length(concept_ids)))
}
cat("\n")

# ============================================================================
# Lookup Concept by Vocabulary Code
# ============================================================================

cat("7. Looking up concepts by code\n")
cat("------------------------------\n")

# Look up ICD-10-CM code E11 (Type 2 diabetes mellitus)
icd_concept <- client$concepts$get_by_code("ICD10CM", "E11")

cat("ICD-10-CM E11:\n")
cat(sprintf("  Concept ID: %d\n", icd_concept$concept_id))
cat(sprintf("  Name: %s\n", icd_concept$concept_name))
cat(sprintf("  Domain: %s\n", icd_concept$domain_id))
cat(sprintf("  Standard: %s\n", icd_concept$standard_concept))
cat("\n")

# Look up SNOMED code
snomed_concept <- client$concepts$get_by_code("SNOMED", "44054006")

cat("SNOMED 44054006:\n")
cat(sprintf("  Concept ID: %d\n", snomed_concept$concept_id))
cat(sprintf("  Name: %s\n", snomed_concept$concept_name))
cat(sprintf("  Domain: %s\n", snomed_concept$domain_id))
cat("\n")

# ============================================================================
# Mapping with Specific Vocabulary Release
# ============================================================================

cat("8. Mapping with specific vocabulary release\n")
cat("-------------------------------------------\n")

# Get mappings from a specific vocabulary release version
# This is useful for reproducibility and working with specific data versions
versioned_mappings <- client$mappings$get(
  DIABETES_CONCEPT_ID,
  vocab_release = "2025.1"
)

cat("Mappings from vocabulary release 2025.1:\n")
cat(sprintf("  Found %d mappings\n", length(versioned_mappings$mappings)))
cat("\n")

# ============================================================================
# Done
# ============================================================================

cat("Done!\n")

#!/usr/bin/env Rscript
#' Example: Error Handling
#'
#' Demonstrates error handling strategies for the omophub R SDK:
#'
#' - HTTP errors (404 Not Found, 401 Unauthorized, 429 Rate Limited) are
#'   raised as httr2 conditions (`httr2_http_404`, `httr2_http_401`,
#'   `httr2_http_429`). Catch them with their specific classes.
#' - Input validation errors raised before the HTTP call (empty query,
#'   invalid concept ID, bad pagination) use the `omophub_validation_error`
#'   class from the SDK's `abort_validation()` helper.
#' - Any error you don't specifically handle falls through to the base
#'   `error` handler.
#'
#' Run with: Rscript inst/examples/error_handling.R

library(omophub)

# Null-coalescing operator (available in base R 4.4+; define locally for
# compatibility with the R >= 4.1 package requirement).
`%||%` <- function(a, b) if (is.null(a)) b else a

# ============================================================================
# Setup
# ============================================================================

cat("OMOPHub R Client - Error Handling Examples\n")
cat("==========================================\n\n")

client <- OMOPHubClient$new()

# ============================================================================
# 1. 404 Not Found
# ============================================================================

cat("1. Handling 404 Not Found\n")
cat("-------------------------\n")

tryCatch(
  {
    concept <- client$concepts$get(999999999)  # Non-existent ID
    cat("Concept found:", concept$concept_name, "\n")
  },
  httr2_http_404 = function(e) {
    cat("  Not found (404)\n")
    cat("  Message:", conditionMessage(e), "\n")
  },
  error = function(e) {
    cat("  Unexpected error:", conditionMessage(e), "\n")
  }
)
cat("\n")

# ============================================================================
# 2. Validation errors raised BEFORE the HTTP call
# ============================================================================

cat("2. Handling validation errors (pre-request)\n")
cat("-------------------------------------------\n")

# Empty query - caught by the SDK's argument validator before any HTTP call
cat("  Empty search query:\n")
tryCatch(
  client$search$basic(""),
  omophub_validation_error = function(e) {
    cat("    validation error:", conditionMessage(e), "\n")
  },
  error = function(e) {
    cat("    error:", conditionMessage(e)[[1]], "\n")
  }
)

# Invalid concept ID type
cat("  Invalid concept ID:\n")
tryCatch(
  client$concepts$get("not-an-integer"),
  omophub_validation_error = function(e) {
    cat("    validation error:", conditionMessage(e), "\n")
  },
  error = function(e) {
    cat("    error:", conditionMessage(e)[[1]], "\n")
  }
)
cat("\n")

# ============================================================================
# 3. Authentication errors (401 / 403)
# ============================================================================

cat("3. Handling authentication errors (401 / 403)\n")
cat("---------------------------------------------\n")

tryCatch(
  {
    bad_client <- OMOPHubClient$new(api_key = "oh_invalid_key_12345")
    concept <- bad_client$concepts$get(201826)
  },
  httr2_http_401 = function(e) {
    cat("  Unauthorized (401): invalid API key\n")
    cat("  Message:", conditionMessage(e)[[1]], "\n")
  },
  httr2_http_403 = function(e) {
    cat("  Forbidden (403): API key lacks permission\n")
    cat("  Message:", conditionMessage(e)[[1]], "\n")
  },
  error = function(e) {
    cat("  Error:", conditionMessage(e)[[1]], "\n")
  }
)
cat("\n")

# ============================================================================
# 4. Retry pattern for rate limiting (429)
# ============================================================================

cat("4. Retry pattern for rate limiting\n")
cat("----------------------------------\n")

# Note: the SDK already retries 429 responses automatically via
# httr2::req_retry() with exponential backoff. This example shows the
# user-space retry pattern for cases where you want custom handling
# (e.g. logging each attempt, falling back after a cap).
fetch_with_retry <- function(client, concept_id, max_retries = 3) {
  for (attempt in seq_len(max_retries)) {
    result <- tryCatch(
      list(ok = TRUE, value = client$concepts$get(concept_id)),
      httr2_http_429 = function(e) {
        retry_after <- e$retry_after %||% 5L
        cat(sprintf(
          "  Rate limited. Retry after %ds (attempt %d/%d)\n",
          retry_after, attempt, max_retries
        ))
        Sys.sleep(min(retry_after, 5))
        list(ok = FALSE)
      }
    )
    if (isTRUE(result$ok)) return(result$value)
  }
  stop("Max retries exceeded")
}

cat("  Fetching with retry logic...\n")
tryCatch(
  {
    concept <- fetch_with_retry(client, 201826)
    cat("    Success:", concept$concept_name, "\n")
  },
  error = function(e) {
    cat("    Error:", conditionMessage(e)[[1]], "\n")
  }
)
cat("\n")

# ============================================================================
# 5. Graceful degradation pattern
# ============================================================================

cat("5. Graceful degradation\n")
cat("-----------------------\n")

# Function that returns a structured result instead of failing. The
# caller can distinguish "not found" from other error categories so
# downstream logging / reporting is honest about what went wrong.
safe_get_concept <- function(client, concept_id) {
  tryCatch(
    list(ok = TRUE, value = client$concepts$get(concept_id), reason = NA_character_),
    httr2_http_404 = function(e) {
      list(ok = FALSE, value = NULL, reason = "not found")
    },
    httr2_http = function(e) {
      list(
        ok = FALSE,
        value = NULL,
        reason = sprintf("HTTP error: %s", conditionMessage(e)[[1]])
      )
    },
    error = function(e) {
      list(
        ok = FALSE,
        value = NULL,
        reason = sprintf("error: %s", conditionMessage(e)[[1]])
      )
    }
  )
}

concept_ids <- c(201826, 999999999, 320128)
cat("  Fetching concepts with structured results:\n")
for (id in concept_ids) {
  result <- safe_get_concept(client, id)
  if (isTRUE(result$ok)) {
    cat(sprintf("    [%d] %s\n", id, result$value$concept_name))
  } else {
    cat(sprintf("    [%d] (%s)\n", id, result$reason))
  }
}
cat("\n")

# ============================================================================
# 6. Batch operations with error collection
# ============================================================================

cat("6. Batch operations with error collection\n")
cat("-----------------------------------------\n")

process_concepts <- function(client, concept_ids) {
  results <- list()
  errors <- list()

  for (id in concept_ids) {
    tryCatch(
      {
        concept <- client$concepts$get(id)
        results[[as.character(id)]] <- concept
      },
      error = function(e) {
        errors[[as.character(id)]] <<- conditionMessage(e)[[1]]
      }
    )
  }

  list(
    results = results,
    errors = errors,
    success_count = length(results),
    error_count = length(errors)
  )
}

batch_ids <- c(201826, 999999999, 320128, 888888888, 4329847)
cat(sprintf("  Processing batch of %d concepts:\n", length(batch_ids)))

batch_result <- process_concepts(client, batch_ids)
cat(sprintf("    Successful: %d\n", batch_result$success_count))
cat(sprintf("    Failed:     %d\n", batch_result$error_count))

if (batch_result$error_count > 0) {
  cat("    Failed IDs:", paste(names(batch_result$errors), collapse = ", "), "\n")
}

# For real batch-resolving many concepts at once, use client$concepts$batch()
# which sends a single HTTP request for up to 100 concepts and has a
# significantly smaller rate-limit footprint.
cat("\n")

# ============================================================================
# 7. Extracting error details from httr2 conditions
# ============================================================================

cat("7. Extracting error details from httr2 conditions\n")
cat("-------------------------------------------------\n")

tryCatch(
  client$concepts$get(999999999),
  httr2_http = function(e) {
    cat("  Class chain:", paste(head(class(e), 3), collapse = " -> "), "\n")
    cat("  Message:    ", conditionMessage(e)[[1]], "\n")

    # httr2 attaches the response object for deeper inspection
    if (!is.null(e$resp)) {
      cat("  HTTP status:", httr2::resp_status(e$resp), "\n")
      url <- httr2::resp_url(e$resp)
      if (!is.null(url)) cat("  URL:        ", url, "\n")
    }
  }
)
cat("\n")

cat("Done!\n")

#!/usr/bin/env Rscript
#' Example: Search Concepts
#'
#' Demonstrates comprehensive search capabilities:
#' - Basic text search
#' - Filtered search (vocabulary, domain, standard_concept)
#' - Autocomplete/suggestions
#' - Pagination with basic_all()
#'
#' Run with: Rscript inst/examples/search_concepts.R

library(omophub)

# Null-coalescing operator (available in base R 4.4+; define locally for
# compatibility with the R >= 4.1 package requirement).
`%||%` <- function(a, b) if (is.null(a)) b else a

# ============================================================================
# Setup
# ============================================================================

client <- OMOPHubClient$new()

cat("OMOPHub R Client - Search Examples\n")
cat("===================================\n\n")

# ============================================================================
# Basic Search
# ============================================================================

cat("1. Basic text search\n")
cat("--------------------\n")

# Simple search for "diabetes"
results <- client$search$basic("diabetes", page_size = 5)

cat("Top 5 results for 'diabetes':\n")
concepts <- results$data %||% results$concepts %||% results
for (c in concepts) {
  cat(sprintf("  [%d] %s\n", c$concept_id, c$concept_name))
}
cat("\n")

# ============================================================================
# Filtered Search
# ============================================================================

cat("2. Filtered search\n")
cat("------------------\n")

# Search for diabetes in SNOMED vocabulary only
results <- client$search$basic(
  "diabetes",
  vocabulary_ids = c("SNOMED"),
  page_size = 5
)

cat("SNOMED diabetes concepts:\n")
concepts <- results$data %||% results$concepts %||% results
for (c in concepts) {
  cat(sprintf("  [%d] %s (%s)\n",
              c$concept_id,
              c$concept_name,
              c$concept_class_id))
}
cat("\n")

# Search for conditions only (by domain)
results <- client$search$basic(
  "hypertension",
  domain_ids = c("Condition"),
  standard_concept = "S",  # Standard concepts only
  page_size = 5
)

cat("Standard condition concepts for 'hypertension':\n")
concepts <- results$data %||% results$concepts %||% results
for (c in concepts) {
  cat(sprintf("  [%d] %s\n", c$concept_id, c$concept_name))
}
cat("\n")

# ============================================================================
# Search with Multiple Filters
# ============================================================================

cat("3. Multi-vocabulary search\n")
cat("--------------------------\n")

# Search across multiple vocabularies
results <- client$search$basic(
  "aspirin",
  vocabulary_ids = c("RxNorm", "SNOMED"),
  domain_ids = c("Drug"),
  page_size = 10
)

cat("Aspirin concepts from RxNorm and SNOMED:\n")
concepts <- results$data %||% results$concepts %||% results
for (c in concepts) {
  cat(sprintf("  [%d] %s (%s - %s)\n",
              c$concept_id,
              c$concept_name,
              c$vocabulary_id,
              c$concept_class_id))
}
cat("\n")

# ============================================================================
# Autocomplete / Suggestions
# ============================================================================

cat("4. Autocomplete suggestions\n")
cat("---------------------------\n")

# Get suggestions for partial text
suggestions <- client$concepts$suggest("diab", page_size = 5)

cat("Suggestions for 'diab':\n")
suggestion_list <- suggestions$suggestions %||% suggestions
for (s in suggestion_list) {
  cat(sprintf("  %s\n", s$suggestion %||% s$concept_name %||% s))
}
cat("\n")

# ============================================================================
# Paginated Search (Fetch All Results)
# ============================================================================

cat("5. Fetching multiple pages of results\n")
cat("-------------------------------------\n")

# Use basic_all() to automatically paginate through results
# Note: max_pages limits total pages fetched
all_results <- client$search$basic_all(
  "metformin",
  vocabulary_ids = c("RxNorm"),
  page_size = 10,
  max_pages = 3,  # Limit to 3 pages for this example
  progress = FALSE
)

cat(sprintf("Fetched %d metformin concepts from RxNorm:\n", nrow(all_results)))
if (nrow(all_results) > 0) {
  # Show first 5
  for (i in seq_len(min(5, nrow(all_results)))) {
    cat(sprintf("  [%d] %s\n",
                all_results$concept_id[i],
                all_results$concept_name[i]))
  }
  if (nrow(all_results) > 5) {
    cat(sprintf("  ... and %d more\n", nrow(all_results) - 5))
  }
}
cat("\n")

# ============================================================================
# Search with Synonyms
# ============================================================================

cat("6. Search including synonyms\n")
cat("----------------------------\n")

# Include synonyms in search
results <- client$search$basic(
  "heart attack",
  include_synonyms = TRUE,
  page_size = 5
)

cat("Results for 'heart attack' (including synonyms):\n")
concepts <- results$data %||% results$concepts %||% results
for (c in concepts) {
  cat(sprintf("  [%d] %s (%s)\n",
              c$concept_id,
              c$concept_name,
              c$vocabulary_id))
}
cat("\n")

# ============================================================================
# Semantic Search
# ============================================================================

cat("7. Semantic search (natural language)\n")
cat("-------------------------------------\n")

# Search using natural language - understands clinical intent
results <- client$search$semantic("high blood sugar levels", page_size = 5)

cat("Semantic results for 'high blood sugar levels':\n")
for (r in results$data$results) {
  cat(sprintf("  [%d] %s (similarity: %.2f)\n",
              r$concept_id, r$concept_name, r$similarity_score))
}
cat("\n")

# ============================================================================
# Semantic Search with Filters
# ============================================================================

cat("8. Semantic search with filters\n")
cat("-------------------------------\n")

results <- client$search$semantic(
  "heart attack",
  vocabulary_ids = "SNOMED",
  domain_ids = "Condition",
  threshold = 0.5,
  page_size = 5
)

cat("Filtered semantic results for 'heart attack':\n")
for (r in results$data$results) {
  cat(sprintf("  [%d] %s (similarity: %.2f)\n",
              r$concept_id, r$concept_name, r$similarity_score))
}
cat("\n")

# ============================================================================
# Auto-Paginated Semantic Search
# ============================================================================

cat("9. Auto-paginated semantic search\n")
cat("----------------------------------\n")

all_results <- client$search$semantic_all(
  "chronic kidney disease",
  page_size = 10,
  max_pages = 3,
  progress = FALSE
)

cat(sprintf("Fetched %d concepts for 'chronic kidney disease':\n", nrow(all_results)))
if (nrow(all_results) > 0) {
  for (i in seq_len(min(5, nrow(all_results)))) {
    cat(sprintf("  [%d] %s\n",
                all_results$concept_id[i],
                all_results$concept_name[i]))
  }
  if (nrow(all_results) > 5) {
    cat(sprintf("  ... and %d more\n", nrow(all_results) - 5))
  }
}
cat("\n")

# ============================================================================
# Similarity Search
# ============================================================================

cat("10. Similarity search\n")
cat("---------------------\n")

# Find concepts similar to Type 2 diabetes mellitus
similar <- client$search$similar(
  concept_id = 201826,
  algorithm = "hybrid",
  similarity_threshold = 0.6,
  page_size = 5
)

cat("Concepts similar to 'Type 2 diabetes mellitus':\n")
for (s in similar$similar_concepts) {
  cat(sprintf("  [%d] %s (score: %.2f)\n",
              s$concept_id, s$concept_name, s$similarity_score))
}
cat("\n")

# Similarity by natural language query
similar <- client$search$similar(
  query = "high blood pressure",
  algorithm = "semantic",
  page_size = 5
)

cat("Concepts similar to 'high blood pressure' (semantic):\n")
for (s in similar$similar_concepts) {
  cat(sprintf("  [%d] %s (score: %.2f)\n",
              s$concept_id, s$concept_name, s$similarity_score))
}
cat("\n")

# ============================================================================
# Bulk Lexical Search
# ============================================================================

cat("11. Bulk lexical search (multiple queries in one call)\n")
cat("------------------------------------------------------\n")

# Search for multiple terms at once (up to 50)
results <- client$search$bulk_basic(list(
  list(search_id = "q1", query = "diabetes mellitus"),
  list(search_id = "q2", query = "hypertension"),
  list(search_id = "q3", query = "aspirin")
), defaults = list(vocabulary_ids = list("SNOMED"), page_size = 3))

cat("Bulk search results:\n")
for (item in results$results) {
  cat(sprintf("  %s: %d results (%s)\n",
              item$search_id, length(item$results), item$status))
}
cat("\n")

# ============================================================================
# Bulk Semantic Search
# ============================================================================

cat("12. Bulk semantic search (multiple NLP queries)\n")
cat("------------------------------------------------\n")

# Search for multiple natural-language queries at once (up to 25)
results <- client$search$bulk_semantic(list(
  list(search_id = "s1", query = "heart failure treatment options"),
  list(search_id = "s2", query = "type 2 diabetes medication"),
  list(search_id = "s3", query = "elevated blood pressure")
), defaults = list(threshold = 0.5, page_size = 5))

cat("Bulk semantic results:\n")
for (item in results$results) {
  n_results <- item$result_count %||% length(item$results)
  cat(sprintf("  %s: %d results (%s)\n",
              item$search_id, n_results, item$status))

  # Show top result for each query
  if (length(item$results) > 0) {
    top <- item$results[[1]]
    cat(sprintf("    Top: %s (score: %.2f)\n",
                top$concept_name, top$similarity_score))
  }
}
cat("\n")

# ============================================================================
# Bulk Search with Per-Query Overrides
# ============================================================================

cat("13. Bulk search with per-query filters\n")
cat("--------------------------------------\n")

# Defaults apply to all, but individual searches can override
results <- client$search$bulk_basic(list(
  list(search_id = "conditions", query = "diabetes", domain_ids = list("Condition")),
  list(search_id = "drugs", query = "metformin", domain_ids = list("Drug"))
), defaults = list(vocabulary_ids = list("SNOMED", "RxNorm"), page_size = 3))

for (item in results$results) {
  cat(sprintf("  %s:\n", item$search_id))
  for (c in item$results) {
    cat(sprintf("    [%d] %s (%s/%s)\n",
                c$concept_id, c$concept_name,
                c$vocabulary_id, c$domain_id))
  }
}
cat("\n")

# ============================================================================
# Done
# ============================================================================

cat("Done!\n")

#' Get OMOPHub API Key
#'
#' @description
#' Retrieves the OMOPHub API key from multiple sources in priority order:
#' 1. Explicit argument
#' 2. `OMOPHUB_API_KEY` environment variable
#' 3. System keyring (if `keyring` package is installed)
#'
#' @param key Optional explicit API key. If provided, this takes precedence.
#'
#' @returns A character string containing the API key.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' # From environment variable
#' Sys.setenv(OMOPHUB_API_KEY = "your_api_key")
#' key <- get_api_key()
#'
#' # Explicit key
#' key <- get_api_key("your_api_key")
#' }
get_api_key <- function(key = NULL) {
  # 1. Explicit argument takes precedence

if (!is.null(key)) {
    if (!is.character(key) || length(key) != 1 || nchar(key) == 0) {
      cli::cli_abort(c(
        "Invalid API key provided.",
        "i" = "API key must be a non-empty string."
      ))
    }
    return(key)
  }

  # 2. Environment variable
  env_key <- Sys.getenv("OMOPHUB_API_KEY", unset = NA_character_)
  if (!is.na(env_key) && nzchar(env_key)) {
    return(env_key)
  }

  # 3. Keyring fallback (optional)
  if (requireNamespace("keyring", quietly = TRUE)) {
    tryCatch(
      {
        keyring_key <- keyring::key_get("omophub", "api_key")
        if (!is.null(keyring_key) && nzchar(keyring_key)) {
          return(keyring_key)
        }
      },
      error = function(e) {
        # Key not found in keyring, continue to error
        NULL
      }
    )
  }

  # No key found
  cli::cli_abort(c(
    "No API key found.",
    "i" = "Set the {.envvar OMOPHUB_API_KEY} environment variable,",
    "i" = "or use {.code set_api_key()} to store your key,",
    "i" = "or pass {.arg api_key} to {.fun OMOPHubClient$new}."
  ))
}

#' Set OMOPHub API Key
#'
#' @description
#' Stores the OMOPHub API key in the specified location.
#'
#' @param key The API key to store.
#' @param store Where to store the key. One of:
#'   - `"env"`: Set as environment variable for current session (default)
#'   - `"keyring"`: Store securely in system keyring (requires `keyring` package)
#'
#' @returns Invisibly returns `TRUE` on success.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' # Store in environment (current session only)
#' set_api_key("your_api_key")
#'
#' # Store securely in keyring (persistent)
#' set_api_key("your_api_key", store = "keyring")
#' }
set_api_key <- function(key, store = c("env", "keyring")) {
  store <- match.arg(store)

  if (!is.character(key) || length(key) != 1 || nchar(key) == 0) {
    cli::cli_abort(c(
      "Invalid API key.",
      "i" = "API key must be a non-empty string."
    ))
  }

  if (store == "env") {
    Sys.setenv(OMOPHUB_API_KEY = key)
    cli::cli_inform(c(
      "v" = "API key set for current R session.",
      "i" = "Use {.code store = \"keyring\"} for persistent storage."
    ))
  } else if (store == "keyring") {
    if (!requireNamespace("keyring", quietly = TRUE)) {
      cli::cli_abort(c(
        "The {.pkg keyring} package is required for secure storage.",
        "i" = "Install it with: {.code install.packages(\"keyring\")}"
      ))
    }
    keyring::key_set_with_value("omophub", "api_key", key)
    cli::cli_inform(c(
      "v" = "API key stored securely in system keyring."
    ))
  }

  invisible(TRUE)
}

#' Check if API Key is Available
#'
#' @description
#' Checks whether an OMOPHub API key is available without throwing an error.
#'
#' @returns `TRUE` if an API key is available, `FALSE` otherwise.
#'
#' @export
has_api_key <- function() {
  tryCatch(
    {
      get_api_key()
      TRUE
    },
    error = function(e) FALSE
  )
}

test_that("get_api_key returns explicit key", {
  key <- get_api_key("my_explicit_key")
  expect_equal(key, "my_explicit_key")
})

test_that("get_api_key returns environment variable", {
  withr::with_envvar(
    c(OMOPHUB_API_KEY = "env_api_key"),
    {
      key <- get_api_key()
      expect_equal(key, "env_api_key")
    }
  )
})

test_that("get_api_key errors when no key available", {
  withr::with_envvar(
    c(OMOPHUB_API_KEY = NA),
    {
      expect_error(get_api_key(), "No API key found")
    }
  )
})

test_that("get_api_key validates key format", {
  expect_error(get_api_key(""), "Invalid API key")
  expect_error(get_api_key(123), "Invalid API key")
  expect_error(get_api_key(c("a", "b")), "Invalid API key")
})

test_that("set_api_key sets environment variable", {
  withr::with_envvar(
    c(OMOPHUB_API_KEY = NA),
    {
      result <- set_api_key("new_key", store = "env")
      expect_true(result)
      expect_equal(Sys.getenv("OMOPHUB_API_KEY"), "new_key")
    }
  )
})

test_that("set_api_key validates input", {
  expect_error(set_api_key(""), "Invalid API key")
  expect_error(set_api_key(NULL), "Invalid API key")
})

test_that("has_api_key returns TRUE when key exists", {
  withr::with_envvar(
    c(OMOPHUB_API_KEY = "test_key"),
    {
      expect_true(has_api_key())
    }
  )
})

test_that("has_api_key returns FALSE when no key", {
  withr::with_envvar(
    c(OMOPHUB_API_KEY = NA),
    {
      expect_false(has_api_key())
    }
  )
})

test_that("get_api_key falls back to keyring when available", {
  skip_if_not_installed("keyring")
  withr::with_envvar(
    c(OMOPHUB_API_KEY = NA),
    {
      # Mock keyring to return a key
      local_mocked_bindings(
        key_get = function(service, username) "keyring_api_key",
        .package = "keyring"
      )
      key <- get_api_key()
      expect_equal(key, "keyring_api_key")
    }
  )
})

test_that("set_api_key with keyring store errors when keyring missing", {
  # Skip if keyring is actually not installed (the error would fire naturally)
  # but test the branch where keyring IS installed by mocking key_set_with_value
  # (the "requires keyring" error path is lines 107-111 in auth.R)
  skip_if_not_installed("keyring")
  # If keyring IS installed, set_api_key(store="keyring") should succeed
  # (covered by the next test). This test just confirms the store arg works.
  expect_error(set_api_key("", store = "keyring"), "Invalid API key")
})

test_that("set_api_key with keyring store calls key_set_with_value", {
  skip_if_not_installed("keyring")
  called_with <- NULL
  local_mocked_bindings(
    key_set_with_value = function(service, username, password) {
      called_with <<- list(service = service, username = username, password = password)
    },
    .package = "keyring"
  )

  result <- set_api_key("my_secure_key", store = "keyring")

  expect_true(result)
  expect_equal(called_with$service, "omophub")
  expect_equal(called_with$username, "api_key")
  expect_equal(called_with$password, "my_secure_key")
})

#' Package Load Hook
#'
#' @param libname Library name.
#' @param pkgname Package name.
#' @keywords internal
.onLoad <- function(libname, pkgname) {
  # Initialize package options
  op <- options()
  op_omophub <- list(
    omophub.base_url = "https://api.omophub.com/v1",
    omophub.timeout = 30,
    omophub.max_retries = 3
  )
  toset <- !(names(op_omophub) %in% names(op))
  if (any(toset)) options(op_omophub[toset])

  invisible()
}

#' Package Attach Hook
#'
#' @param libname Library name.
#' @param pkgname Package name.
#' @keywords internal
.onAttach <- function(libname, pkgname) {
  # Check for API key on attach
 if (!has_api_key()) {
    packageStartupMessage(
      "No OMOPHUB_API_KEY found. ",
      "Set it with Sys.setenv(OMOPHUB_API_KEY = 'your_key') ",
      "or use set_api_key()."
    )
  }
}

# Unit tests for zzz.R
# Tests package load and attach hooks

# ==============================================================================
# .onLoad() tests
# ==============================================================================

test_that(".onLoad sets default options when not present", {
  # Clear any existing options
  withr::with_options(
    list(
      omophub.base_url = NULL,
      omophub.timeout = NULL,
      omophub.max_retries = NULL
    ),
    {
      # Call .onLoad
      omophub:::.onLoad("", "omophub")

      # Check options were set
      expect_equal(getOption("omophub.base_url"), "https://api.omophub.com/v1")
      expect_equal(getOption("omophub.timeout"), 30)
      expect_equal(getOption("omophub.max_retries"), 3)
    }
  )
})

test_that(".onLoad preserves existing options", {
  withr::with_options(
    list(
      omophub.base_url = "https://custom.api.com/v1",
      omophub.timeout = 60,
      omophub.max_retries = 5
    ),
    {
      # Call .onLoad
      omophub:::.onLoad("", "omophub")

      # Check custom options were preserved
      expect_equal(getOption("omophub.base_url"), "https://custom.api.com/v1")
      expect_equal(getOption("omophub.timeout"), 60)
      expect_equal(getOption("omophub.max_retries"), 5)
    }
  )
})

test_that(".onLoad preserves partial existing options", {
  withr::with_options(
    list(
      omophub.base_url = "https://custom.api.com/v1",
      omophub.timeout = NULL,  # Should get default
      omophub.max_retries = 10
    ),
    {
      # Call .onLoad
      omophub:::.onLoad("", "omophub")

      # Check custom options preserved, missing got defaults
      expect_equal(getOption("omophub.base_url"), "https://custom.api.com/v1")
      expect_equal(getOption("omophub.timeout"), 30)  # Default
      expect_equal(getOption("omophub.max_retries"), 10)
    }
  )
})

# ==============================================================================
# .onAttach() tests
# ==============================================================================

test_that(".onAttach shows message when API key is missing", {
  withr::with_envvar(
    c(OMOPHUB_API_KEY = NA),  # Remove API key
    {
      # Clear any cached API key in keyring
      local_mocked_bindings(
        has_api_key = function() FALSE
      )

      expect_message(
        omophub:::.onAttach("", "omophub"),
        "No OMOPHUB_API_KEY found"
      )
    }
  )
})

test_that(".onAttach does not show message when API key is present", {
  withr::with_envvar(
    c(OMOPHUB_API_KEY = "test_key"),
    {
      # Mock has_api_key to return TRUE
      local_mocked_bindings(
        has_api_key = function() TRUE
      )

      # Should not produce any message
      expect_silent(
        suppressMessages(omophub:::.onAttach("", "omophub"))
      )
    }
  )
})

#' OMOPHub API Client
#'
#' @description
#' R6 class for interacting with the OMOPHub vocabulary service.
#' Provides access to OHDSI ATHENA standardized medical vocabularies.
#'
#' @details
#' The client provides access to these resources:
#' - `concepts`: Concept lookup and batch operations
#' - `search`: Basic and advanced concept search
#' - `vocabularies`: Vocabulary listing and details
#' - `domains`: Domain listing and concept filtering
#' - `hierarchy`: Ancestor and descendant navigation
#' - `relationships`: Concept relationships
#' - `mappings`: Concept mappings between vocabularies
#'
#' @export
#'
#' @examples
#' \dontrun{
#' # Create client (uses OMOPHUB_API_KEY env var)
#' client <- OMOPHubClient$new()
#'
#' # Or with explicit API key
#' client <- OMOPHubClient$new(api_key = "your_api_key")
#'
#' # Search for concepts
#' results <- client$search$basic("diabetes")
#'
#' # Get a specific concept
#' concept <- client$concepts$get(201826)
#'
#' # Use specific vocabulary version
#' client <- OMOPHubClient$new(vocab_version = "2025.1")
#' }
OMOPHubClient <- R6::R6Class(
  "OMOPHubClient",
  public = list(
    #' @description
    #' Create a new OMOPHub client.
    #'
    #' @param api_key API key for authentication. If not provided, reads from
    #'   `OMOPHUB_API_KEY` environment variable or system keyring.
    #' @param base_url API base URL. Defaults to `https://api.omophub.com/v1`.
    #' @param timeout Request timeout in seconds. Defaults to 30.
    #' @param max_retries Maximum retry attempts for failed requests. Defaults to 3.
    #' @param vocab_version Optional vocabulary version (e.g., "2025.1").
    #'   If not specified, uses the latest version.
    #'
    #' @returns A new `OMOPHubClient` object.
    initialize = function(api_key = NULL,
                          base_url = NULL,
                          timeout = 30,
                          max_retries = 3,
                          vocab_version = NULL) {
      private$.api_key <- get_api_key(api_key)
      private$.base_url <- base_url %||% .omophub_env$default_base_url
      private$.timeout <- timeout %||% .omophub_env$default_timeout
      private$.max_retries <- max_retries %||% .omophub_env$default_max_retries
      private$.vocab_version <- vocab_version

      # Build base request
      private$.base_req <- build_request(
        base_url = private$.base_url,
        api_key = private$.api_key,
        timeout = private$.timeout,
        max_retries = private$.max_retries,
        vocab_version = private$.vocab_version
      )

      # Initialize resource accessors (lazy loading via active bindings)
      # Resources are initialized on first access
      invisible(self)
    },

    #' @description
    #' Print client information.
    print = function() {
      cat("<OMOPHubClient>\n")
      cat("  Base URL:", private$.base_url, "\n")
      cat("  Authenticated:", !is.null(private$.api_key), "\n")
      if (!is.null(private$.vocab_version)) {
        cat("  Vocabulary Version:", private$.vocab_version, "\n")
      }
      cat("  Timeout:", private$.timeout, "seconds\n")
      cat("  Max Retries:", private$.max_retries, "\n")
      invisible(self)
    }
  ),
  active = list(
    #' @field concepts Access to concept operations.
    concepts = function() {
      if (is.null(private$.concepts)) {
        private$.concepts <- ConceptsResource$new(private$.base_req)
      }
      private$.concepts
    },

    #' @field search Access to search operations.
    search = function() {
      if (is.null(private$.search)) {
        private$.search <- SearchResource$new(private$.base_req)
      }
      private$.search
    },

    #' @field vocabularies Access to vocabulary operations.
    vocabularies = function() {
      if (is.null(private$.vocabularies)) {
        private$.vocabularies <- VocabulariesResource$new(private$.base_req)
      }
      private$.vocabularies
    },

    #' @field domains Access to domain operations.
    domains = function() {
      if (is.null(private$.domains)) {
        private$.domains <- DomainsResource$new(private$.base_req)
      }
      private$.domains
    },

    #' @field hierarchy Access to hierarchy operations.
    hierarchy = function() {
      if (is.null(private$.hierarchy)) {
        private$.hierarchy <- HierarchyResource$new(private$.base_req)
      }
      private$.hierarchy
    },

    #' @field relationships Access to relationship operations.
    relationships = function() {
      if (is.null(private$.relationships)) {
        private$.relationships <- RelationshipsResource$new(private$.base_req)
      }
      private$.relationships
    },

    #' @field mappings Access to mapping operations.
    mappings = function() {
      if (is.null(private$.mappings)) {
        private$.mappings <- MappingsResource$new(private$.base_req)
      }
      private$.mappings
    },

    #' @field fhir Access to FHIR-to-OMOP Concept Resolver operations.
    fhir = function() {
      if (is.null(private$.fhir)) {
        private$.fhir <- FhirResource$new(private$.base_req)
      }
      private$.fhir
    }
  ),
  private = list(
    .base_url = NULL,
    .api_key = NULL,
    .timeout = NULL,
    .max_retries = NULL,
    .vocab_version = NULL,
    .base_req = NULL,

    # Lazy-loaded resources
    .concepts = NULL,
    .search = NULL,
    .vocabularies = NULL,
    .domains = NULL,
    .hierarchy = NULL,
    .relationships = NULL,
    .mappings = NULL,
    .fhir = NULL
  )
)

test_that("OMOPHubClient initializes with API key", {
  withr::with_envvar(
    c(OMOPHUB_API_KEY = "test_key"),
    {
      client <- OMOPHubClient$new()
      expect_s3_class(client, "OMOPHubClient")
    }
  )
})

test_that("OMOPHubClient accepts explicit API key", {
  withr::with_envvar(
    c(OMOPHUB_API_KEY = NA),
    {
      client <- OMOPHubClient$new(api_key = "explicit_key")
      expect_s3_class(client, "OMOPHubClient")
    }
  )
})

test_that("OMOPHubClient errors without API key", {
  withr::with_envvar(
    c(OMOPHUB_API_KEY = NA),
    {
      expect_error(OMOPHubClient$new(), "No API key found")
    }
  )
})

test_that("OMOPHubClient accepts custom base_url", {
  withr::with_envvar(
    c(OMOPHUB_API_KEY = "test_key"),
    {
      client <- OMOPHubClient$new(base_url = "https://custom.api.com/v1")
      expect_s3_class(client, "OMOPHubClient")
    }
  )
})

test_that("OMOPHubClient accepts timeout and max_retries", {
  withr::with_envvar(
    c(OMOPHUB_API_KEY = "test_key"),
    {
      client <- OMOPHubClient$new(timeout = 60, max_retries = 5)
      expect_s3_class(client, "OMOPHubClient")
    }
  )
})

test_that("OMOPHubClient accepts vocab_version", {
  withr::with_envvar(
    c(OMOPHUB_API_KEY = "test_key"),
    {
      client <- OMOPHubClient$new(vocab_version = "2025.1")
      expect_s3_class(client, "OMOPHubClient")
    }
  )
})

test_that("OMOPHubClient has resource accessors", {
  withr::with_envvar(
    c(OMOPHUB_API_KEY = "test_key"),
    {
      client <- OMOPHubClient$new()

      expect_s3_class(client$concepts, "ConceptsResource")
      expect_s3_class(client$search, "SearchResource")
      expect_s3_class(client$vocabularies, "VocabulariesResource")
      expect_s3_class(client$domains, "DomainsResource")
      expect_s3_class(client$hierarchy, "HierarchyResource")
      expect_s3_class(client$relationships, "RelationshipsResource")
      expect_s3_class(client$mappings, "MappingsResource")
    }
  )
})

test_that("OMOPHubClient print method works", {
  withr::with_envvar(
    c(OMOPHUB_API_KEY = "test_key"),
    {
      client <- OMOPHubClient$new()
      expect_output(print(client), "OMOPHubClient")
      expect_output(print(client), "Base URL")
      expect_output(print(client), "Authenticated: TRUE")
    }
  )
})

test_that("OMOPHubClient resources are lazy-loaded", {
  withr::with_envvar(
    c(OMOPHUB_API_KEY = "test_key"),
    {
      client <- OMOPHubClient$new()

      # Access concepts twice - should be same object (cached)
      concepts1 <- client$concepts
      concepts2 <- client$concepts
      expect_identical(concepts1, concepts2)
    }
  )
})

#' @keywords internal
"_PACKAGE"

#' omophub: R Client for the OMOPHub Medical Vocabulary API
#'
#' Provides an R interface to the OMOPHub API for accessing OHDSI ATHENA
#' standardized medical vocabularies. Supports concept search, vocabulary
#' exploration, hierarchy navigation, relationship queries, and concept
#' mappings with automatic pagination and rate limiting.
#'
#' @section Main Client:
#' The main entry point is the [OMOPHubClient] R6 class which provides
#' access to all API resources:
#'
#' ```r
#' library(omophub)
#' client <- OMOPHubClient$new()
#'
#' # Search for concepts
#' results <- client$search$basic("diabetes")
#'
#' # Get a specific concept
#' concept <- client$concepts$get(201826)
#' ```
#'
#' @section Authentication:
#' Set your API key using one of these methods:
#' - Environment variable: `OMOPHUB_API_KEY`
#' - Explicit argument: `OMOPHubClient$new(api_key = "your_key")`
#' - Keyring: `set_api_key("your_key", store = "keyring")`
#'
#' @section Resources:
#' The client provides access to these resources:
#' - `concepts`: Concept lookup and batch operations
#' - `search`: Basic and advanced concept search
#' - `vocabularies`: Vocabulary listing and details
#' - `domains`: Domain listing and concept filtering
#' - `hierarchy`: Ancestor and descendant navigation
#' - `relationships`: Concept relationships
#' - `mappings`: Concept mappings between vocabularies
#'
#' @importFrom httr2 request req_url_path_append req_url_query req_headers
#' @importFrom httr2 req_body_json req_perform resp_body_json resp_status
#' @importFrom httr2 req_throttle req_retry req_timeout req_user_agent req_error
#' @importFrom R6 R6Class
#' @importFrom rlang abort caller_env
#' @importFrom cli cli_abort cli_warn cli_inform
#' @importFrom glue glue
#' @importFrom tibble tibble as_tibble
#' @importFrom purrr map map_chr map_int map_dbl map_lgl
#'
#' @name omophub-package
#' @aliases omophub
NULL

# Default configuration values
.omophub_env <- new.env(parent = emptyenv())
.omophub_env$default_base_url <- "https://api.omophub.com/v1"
.omophub_env$default_timeout <- 30
.omophub_env$default_max_retries <- 3
.omophub_env$rate_limit_capacity <- 100
.omophub_env$rate_limit_fill_time <- 60

#' Relationships Resource
#'
#' @description
#' R6 class providing access to relationship operations.
#'
#' @keywords internal
RelationshipsResource <- R6::R6Class(
  "RelationshipsResource",
  public = list(
    #' @description
    #' Create a new RelationshipsResource.
    #' @param base_req Base httr2 request object.
    initialize = function(base_req) {
      private$.base_req <- base_req
    },

    #' @description
    #' Get relationships for a concept.
    #'
    #' @param concept_id The concept ID.
    #' @param relationship_ids Filter by relationship type IDs (character vector or comma-separated string).
    #' @param vocabulary_ids Filter by target vocabulary IDs (character vector or comma-separated string).
    #' @param domain_ids Filter by target domain IDs (character vector or comma-separated string).
    #' @param include_invalid Include relationships to invalid concepts. Default `FALSE`.
    #' @param standard_only Only include relationships to standard concepts. Default `FALSE`.
    #' @param include_reverse Include reverse relationships. Default `FALSE`.
    #' @param page Page number. Default 1.
    #' @param page_size Results per page (max 1000). Default 100.
    #' @param vocab_release Specific vocabulary release version. Default `NULL`.
    #'
    #' @returns Relationships data with pagination metadata.
    get = function(concept_id,
                   relationship_ids = NULL,
                   vocabulary_ids = NULL,
                   domain_ids = NULL,
                   include_invalid = FALSE,
                   standard_only = FALSE,
                   include_reverse = FALSE,
                   page = 1,
                   page_size = 100,
                   vocab_release = NULL) {
      concept_id <- validate_concept_id(concept_id)
      pag <- validate_pagination(page, page_size)

      params <- list(
        page = pag$page,
        page_size = pag$page_size
      )

      if (!is.null(relationship_ids)) {
        params$relationship_ids <- if (length(relationship_ids) > 1) {
          paste(relationship_ids, collapse = ",")
        } else {
          relationship_ids
        }
      }
      if (!is.null(vocabulary_ids)) {
        params$vocabulary_ids <- if (length(vocabulary_ids) > 1) {
          paste(vocabulary_ids, collapse = ",")
        } else {
          vocabulary_ids
        }
      }
      if (!is.null(domain_ids)) {
        params$domain_ids <- if (length(domain_ids) > 1) {
          paste(domain_ids, collapse = ",")
        } else {
          domain_ids
        }
      }
      if (isTRUE(include_invalid)) {
        params$include_invalid <- "true"
      }
      if (isTRUE(standard_only)) {
        params$standard_only <- "true"
      }
      if (isTRUE(include_reverse)) {
        params$include_reverse <- "true"
      }
      if (!is.null(vocab_release)) {
        params$vocab_release <- vocab_release
      }

      perform_get(
        private$.base_req,
        paste0("concepts/", concept_id, "/relationships"),
        query = params
      )
    },

    #' @description
    #' Get available relationship types from the OMOP CDM.
    #'
    #' @param page Page number. Default 1.
    #' @param page_size Results per page (max 500). Default 100.
    #'
    #' @returns Relationship types with pagination metadata.
    types = function(page = 1,
                     page_size = 100) {
      pag <- validate_pagination(page, page_size)

      params <- list(
        page = pag$page,
        page_size = pag$page_size
      )

      perform_get(private$.base_req, "relationships/types", query = params)
    },

    #' @description
    #' Print resource information.
    print = function() {
      cat("<OMOPHub RelationshipsResource>\n")
      cat("  Methods: get, types\n")
      invisible(self)
    }
  ),
  private = list(
    .base_req = NULL
  )
)

# Unit tests for RelationshipsResource (R/relationships.R)
# Uses mocked HTTP responses to avoid API calls

# ==============================================================================
# RelationshipsResource initialization
# ==============================================================================

test_that("RelationshipsResource initializes correctly", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- RelationshipsResource$new(base_req)

  expect_s3_class(resource, "RelationshipsResource")
  expect_s3_class(resource, "R6")
})

test_that("RelationshipsResource print method works", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- RelationshipsResource$new(base_req)

  expect_output(print(resource), "<OMOPHub RelationshipsResource>")
  expect_output(print(resource), "get, types")
})

# ==============================================================================
# get() method
# ==============================================================================

test_that("relationships$get validates concept_id", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- RelationshipsResource$new(base_req)

  expect_error(resource$get("invalid"))
  expect_error(resource$get(-1))
})

test_that("relationships$get calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- RelationshipsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      list(relationships = list())
    }
  )

  resource$get(201826, page = 1, page_size = 50)

  expect_equal(called_with$path, "concepts/201826/relationships")
  expect_equal(called_with$query$page, 1L)
  expect_equal(called_with$query$page_size, 50L)
})

test_that("relationships$get includes optional filters", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- RelationshipsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(relationships = list())
    }
  )

  resource$get(
    201826,
    relationship_ids = "Is a",
    vocabulary_ids = "SNOMED",
    include_invalid = TRUE
  )

  expect_equal(called_with$query$relationship_ids, "Is a")
  expect_equal(called_with$query$vocabulary_ids, "SNOMED")
  expect_equal(called_with$query$include_invalid, "true")
})

# ==============================================================================
# types() method
# ==============================================================================

test_that("relationships$types calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- RelationshipsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      list(relationship_types = list())
    }
  )

  resource$types(page = 1, page_size = 100)

  expect_equal(called_with$path, "relationships/types")
  expect_equal(called_with$query$page, 1L)
  expect_equal(called_with$query$page_size, 100L)
})

test_that("relationships$types includes pagination options", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- RelationshipsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(relationship_types = list())
    }
  )

  resource$types(page = 2, page_size = 50)

  expect_equal(called_with$query$page, 2L)
  expect_equal(called_with$query$page_size, 50L)
})

# ==============================================================================
# get() — multi-element vectors and remaining flags
# ==============================================================================

test_that("relationships$get passes multi-element vectors as CSV", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- RelationshipsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list()
    }
  )

  resource$get(
    201826,
    relationship_ids = c("Is a", "Maps to"),
    vocabulary_ids = c("SNOMED", "ICD10CM"),
    domain_ids = c("Condition", "Drug")
  )

  expect_equal(called_with$query$relationship_ids, "Is a,Maps to")
  expect_equal(called_with$query$vocabulary_ids, "SNOMED,ICD10CM")
  expect_equal(called_with$query$domain_ids, "Condition,Drug")
})

test_that("relationships$get passes standard_only, include_reverse, and vocab_release", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- RelationshipsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list()
    }
  )

  resource$get(
    201826,
    standard_only = TRUE,
    include_reverse = TRUE,
    vocab_release = "2025.2"
  )

  expect_equal(called_with$query$standard_only, "true")
  expect_equal(called_with$query$include_reverse, "true")
  expect_equal(called_with$query$vocab_release, "2025.2")
})

#' Domains Resource
#'
#' @description
#' R6 class providing access to domain operations.
#'
#' @keywords internal
DomainsResource <- R6::R6Class(
  "DomainsResource",
  public = list(
    #' @description
    #' Create a new DomainsResource.
    #' @param base_req Base httr2 request object.
    initialize = function(base_req) {
      private$.base_req <- base_req
    },

    #' @description
    #' List all domains.
    #'
    #' @param include_stats Include concept counts and vocabulary coverage. Default `FALSE`.
    #'
    #' @returns Domain list.
    list = function(include_stats = FALSE) {
      params <- list()

      if (isTRUE(include_stats)) {
        params$include_stats <- "true"
      }

      perform_get(private$.base_req, "domains", query = params)
    },

    #' @description
    #' Get concepts in a domain.
    #'
    #' @param domain_id The domain ID.
    #' @param vocabulary_ids Filter by vocabularies.
    #' @param standard_only Only standard concepts. Default `FALSE`.
    #' @param include_invalid Include invalid/deprecated concepts. Default `FALSE`.
    #' @param page Page number. Default 1.
    #' @param page_size Results per page. Default 50.
    #'
    #' @returns Paginated concepts.
    concepts = function(domain_id,
                        vocabulary_ids = NULL,
                        standard_only = FALSE,
                        include_invalid = FALSE,
                        page = 1,
                        page_size = 50) {
      checkmate::assert_string(domain_id, min.chars = 1)
      pag <- validate_pagination(page, page_size)

      params <- list(
        page = pag$page,
        page_size = pag$page_size
      )

      if (!is.null(vocabulary_ids)) {
        params$vocabulary_ids <- join_params(vocabulary_ids)
      }
      if (isTRUE(standard_only)) {
        params$standard_only <- "true"
      }
      if (isTRUE(include_invalid)) {
        params$include_invalid <- "true"
      }

      perform_get(
        private$.base_req,
        paste0("domains/", domain_id, "/concepts"),
        query = params
      )
    },

    #' @description
    #' Print resource information.
    print = function() {
      cat("<OMOPHub DomainsResource>\n")
      cat("  Methods: list, concepts\n")
      invisible(self)
    }
  ),
  private = list(
    .base_req = NULL
  )
)

# Unit tests for DomainsResource (R/domains.R)
# Uses mocked HTTP responses to avoid API calls

# ==============================================================================
# DomainsResource initialization
# ==============================================================================

test_that("DomainsResource initializes correctly", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- DomainsResource$new(base_req)

  expect_s3_class(resource, "DomainsResource")
  expect_s3_class(resource, "R6")
})

test_that("DomainsResource print method works", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- DomainsResource$new(base_req)

  expect_output(print(resource), "<OMOPHub DomainsResource>")
  expect_output(print(resource), "list, concepts")
})

# ==============================================================================
# list() method
# ==============================================================================

test_that("domains$list calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- DomainsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      list(domains = list())
    }
  )

  resource$list()

  expect_equal(called_with$path, "domains")
  # No params when include_stats is FALSE (default)
  expect_equal(length(called_with$query), 0)
})

test_that("domains$list includes include_stats parameter", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- DomainsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(domains = list())
    }
  )

  resource$list(include_stats = TRUE)

  expect_equal(called_with$query$include_stats, "true")
})

test_that("domains$list without stats has no query params", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- DomainsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(domains = list())
    }
  )

  resource$list(include_stats = FALSE)

  # Should not have include_stats in query when FALSE
  expect_null(called_with$query$include_stats)
})

# ==============================================================================
# concepts() method
# ==============================================================================

test_that("domains$concepts validates domain_id", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- DomainsResource$new(base_req)

  expect_error(resource$concepts(""))
  expect_error(resource$concepts(123))
})

test_that("domains$concepts calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- DomainsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      list(concepts = list())
    }
  )

  resource$concepts("Condition", page = 1, page_size = 50)

  expect_equal(called_with$path, "domains/Condition/concepts")
  expect_equal(called_with$query$page, 1L)
  expect_equal(called_with$query$page_size, 50L)
})

test_that("domains$concepts includes optional filters", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- DomainsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(concepts = list())
    }
  )

  resource$concepts(
    "Condition",
    vocabulary_ids = c("SNOMED"),
    standard_only = TRUE,
    include_invalid = TRUE
  )

  expect_equal(called_with$query$vocabulary_ids, "SNOMED")
  expect_equal(called_with$query$standard_only, "true")
  expect_equal(called_with$query$include_invalid, "true")
})

#' Create OMOPHub Concepts Result
#'
#' @description
#' Creates an S3 object for concept search results.
#'
#' @param data Concept data (list or tibble).
#' @param meta Pagination metadata.
#'
#' @returns An `omophub_concepts` S3 object.
#' @keywords internal
new_omophub_concepts <- function(data, meta = list()) {
  # Convert data to tibble if it's a list
  if (is.list(data) && !inherits(data, "data.frame")) {
    data <- tryCatch(
      purrr::list_rbind(purrr::map(data, tibble::as_tibble)),
      error = function(e) tibble::tibble(data = data)
    )
  }

  structure(
    list(
      data = tibble::as_tibble(data),
      total = meta$total_items %||% meta$total %||% nrow(data),
      page = meta$page %||% 1L,
      page_size = meta$page_size %||% nrow(data),
      has_next = meta$has_next %||% FALSE
    ),
    class = c("omophub_concepts", "list")
  )
}

#' @export
print.omophub_concepts <- function(x, ...) {
  cat("<OMOPHub Concept Results>\n")
  cat("  Total:", x$total, "concepts found\n")
  cat("  Showing:", nrow(x$data), "results (page", x$page, ")\n\n")
  print(x$data, n = 10)
  if (nrow(x$data) > 10) {
    cat("# ... with", nrow(x$data) - 10, "more rows\n")
  }
  invisible(x)
}

#' @export
as.data.frame.omophub_concepts <- function(x, ...) {
  as.data.frame(x$data)
}

#' Create OMOPHub Vocabularies Result
#'
#' @param data Vocabulary data.
#' @param meta Pagination metadata.
#' @returns An `omophub_vocabularies` S3 object.
#' @keywords internal
new_omophub_vocabularies <- function(data, meta = list()) {
  if (is.list(data) && !inherits(data, "data.frame")) {
    data <- tryCatch(
      purrr::list_rbind(purrr::map(data, tibble::as_tibble)),
      error = function(e) tibble::tibble(data = data)
    )
  }

  structure(
    list(
      data = tibble::as_tibble(data),
      total = meta$total_items %||% nrow(data)
    ),
    class = c("omophub_vocabularies", "list")
  )
}

#' @export
print.omophub_vocabularies <- function(x, ...) {
  cat("<OMOPHub Vocabularies>\n")
  cat("  Total:", x$total, "vocabularies\n\n")
  print(x$data, n = 10)
  invisible(x)
}

#' @export
as.data.frame.omophub_vocabularies <- function(x, ...) {
  as.data.frame(x$data)
}

#' Create OMOPHub Hierarchy Result
#'
#' @param data Hierarchy data (ancestors or descendants).
#' @param meta Metadata including summary.
#' @param type Type of hierarchy ("ancestors" or "descendants").
#' @returns An `omophub_hierarchy` S3 object.
#' @keywords internal
new_omophub_hierarchy <- function(data, meta = list(), type = "hierarchy") {
  if (is.list(data) && !inherits(data, "data.frame")) {
    data <- tryCatch(
      purrr::list_rbind(purrr::map(data, tibble::as_tibble)),
      error = function(e) tibble::tibble(data = data)
    )
  }

  structure(
    list(
      data = tibble::as_tibble(data),
      type = type,
      total = meta$total_items %||% nrow(data),
      summary = meta$summary %||% list()
    ),
    class = c("omophub_hierarchy", "list")
  )
}

#' @export
print.omophub_hierarchy <- function(x, ...) {
  cat("<OMOPHub Hierarchy:", x$type, ">\n")
  cat("  Total:", x$total, "concepts\n\n")
  print(x$data, n = 10)
  invisible(x)
}

#' @export
as.data.frame.omophub_hierarchy <- function(x, ...) {
  as.data.frame(x$data)
}

#' Create OMOPHub Relationships Result
#'
#' @param data Relationship data.
#' @param meta Metadata.
#' @returns An `omophub_relationships` S3 object.
#' @keywords internal
new_omophub_relationships <- function(data, meta = list()) {
  if (is.list(data) && !inherits(data, "data.frame")) {
    data <- tryCatch(
      purrr::list_rbind(purrr::map(data, tibble::as_tibble)),
      error = function(e) tibble::tibble(data = data)
    )
  }

  structure(
    list(
      data = tibble::as_tibble(data),
      total = meta$total_items %||% nrow(data),
      summary = meta$summary %||% list()
    ),
    class = c("omophub_relationships", "list")
  )
}

#' @export
print.omophub_relationships <- function(x, ...) {
  cat("<OMOPHub Relationships>\n")
  cat("  Total:", x$total, "relationships\n\n")
  print(x$data, n = 10)
  invisible(x)
}

#' @export
as.data.frame.omophub_relationships <- function(x, ...) {
  as.data.frame(x$data)
}

#' Create OMOPHub Mappings Result
#'
#' @param data Mapping data.
#' @param meta Metadata.
#' @returns An `omophub_mappings` S3 object.
#' @keywords internal
new_omophub_mappings <- function(data, meta = list()) {
  if (is.list(data) && !inherits(data, "data.frame")) {
    data <- tryCatch(
      purrr::list_rbind(purrr::map(data, tibble::as_tibble)),
      error = function(e) tibble::tibble(data = data)
    )
  }

  structure(
    list(
      data = tibble::as_tibble(data),
      total = meta$total_items %||% nrow(data),
      summary = meta$summary %||% list()
    ),
    class = c("omophub_mappings", "list")
  )
}

#' @export
print.omophub_mappings <- function(x, ...) {
  cat("<OMOPHub Mappings>\n")
  cat("  Total:", x$total, "mappings\n\n")
  print(x$data, n = 10)
  invisible(x)
}

#' @export
as.data.frame.omophub_mappings <- function(x, ...) {
  as.data.frame(x$data)
}

# Tests for S3 response classes (R/response.R)

# ==============================================================================
# omophub_concepts
# ==============================================================================

test_that("new_omophub_concepts creates correct S3 class from list", {
  data <- list(mock_concept(), mock_concept(concept_id = 12345))
  meta <- list(total_items = 100, page = 1, page_size = 20, has_next = TRUE)

  result <- new_omophub_concepts(data, meta)

  expect_s3_class(result, "omophub_concepts")
  expect_s3_class(result, "list")
  expect_s3_class(result$data, "tbl_df")
  expect_equal(nrow(result$data), 2)
  expect_equal(result$total, 100)
  expect_equal(result$page, 1)
  expect_equal(result$page_size, 20)
  expect_true(result$has_next)
})

test_that("new_omophub_concepts handles tibble input", {
  data <- tibble::tibble(
    concept_id = c(201826, 12345),
    concept_name = c("Diabetes", "Hypertension")
  )
  meta <- list(total_items = 2)

  result <- new_omophub_concepts(data, meta)

  expect_s3_class(result, "omophub_concepts")
  expect_equal(nrow(result$data), 2)
  expect_equal(result$total, 2)
})

test_that("new_omophub_concepts handles empty list", {
  result <- new_omophub_concepts(list(), list())

  expect_s3_class(result, "omophub_concepts")
  expect_equal(nrow(result$data), 0)
  expect_equal(result$total, 0)
})

test_that("new_omophub_concepts uses defaults when meta is empty", {
  data <- list(mock_concept())
  result <- new_omophub_concepts(data, list())

  expect_equal(result$page, 1L)
  expect_equal(result$has_next, FALSE)
})

test_that("print.omophub_concepts outputs expected format", {
  data <- list(mock_concept())
  result <- new_omophub_concepts(data, list(total_items = 100, page = 1))

  expect_output(print(result), "<OMOPHub Concept Results>")
  expect_output(print(result), "Total: 100 concepts found")
  expect_output(print(result), "page 1")
})

test_that("print.omophub_concepts returns object invisibly", {
  data <- list(mock_concept())
  result <- new_omophub_concepts(data, list())

  expect_invisible(print(result))
})

test_that("as.data.frame.omophub_concepts returns data.frame", {
  data <- list(mock_concept(), mock_concept(concept_id = 12345))
  result <- new_omophub_concepts(data, list())

  df <- as.data.frame(result)

  expect_s3_class(df, "data.frame")
  expect_equal(nrow(df), 2)
  expect_true("concept_id" %in% names(df))
})

# ==============================================================================
# omophub_vocabularies
# ==============================================================================

test_that("new_omophub_vocabularies creates correct S3 class", {
  data <- list(mock_vocabulary(), mock_vocabulary(vocabulary_id = "ICD10CM"))
  meta <- list(total_items = 50)


  result <- new_omophub_vocabularies(data, meta)

  expect_s3_class(result, "omophub_vocabularies")
  expect_s3_class(result$data, "tbl_df")
  expect_equal(nrow(result$data), 2)
  expect_equal(result$total, 50)
})

test_that("new_omophub_vocabularies handles empty data", {
  result <- new_omophub_vocabularies(list(), list())

  expect_s3_class(result, "omophub_vocabularies")
  expect_equal(nrow(result$data), 0)
})

test_that("print.omophub_vocabularies outputs expected format", {
  data <- list(mock_vocabulary())
  result <- new_omophub_vocabularies(data, list(total_items = 10))

  expect_output(print(result), "<OMOPHub Vocabularies>")
  expect_output(print(result), "Total: 10 vocabularies")
})

test_that("as.data.frame.omophub_vocabularies returns data.frame", {
  data <- list(mock_vocabulary())
  result <- new_omophub_vocabularies(data, list())

  df <- as.data.frame(result)

  expect_s3_class(df, "data.frame")
  expect_true("vocabulary_id" %in% names(df))
})

# ==============================================================================
# omophub_hierarchy
# ==============================================================================

test_that("new_omophub_hierarchy creates correct S3 class", {
  data <- list(mock_concept(), mock_concept(concept_id = 999))
  meta <- list(total_items = 5, summary = list(max_level = 3))

  result <- new_omophub_hierarchy(data, meta, type = "ancestors")

  expect_s3_class(result, "omophub_hierarchy")
  expect_s3_class(result$data, "tbl_df")
  expect_equal(nrow(result$data), 2)
  expect_equal(result$type, "ancestors")
  expect_equal(result$total, 5)
  expect_equal(result$summary$max_level, 3)
})

test_that("new_omophub_hierarchy handles descendants type", {
  result <- new_omophub_hierarchy(list(), list(), type = "descendants")

  expect_equal(result$type, "descendants")
})

test_that("print.omophub_hierarchy outputs expected format", {
  data <- list(mock_concept())
  result <- new_omophub_hierarchy(data, list(total_items = 3), type = "ancestors")

  expect_output(print(result), "<OMOPHub Hierarchy: ancestors")
  expect_output(print(result), "Total: 3 concepts")
})

test_that("as.data.frame.omophub_hierarchy returns data.frame", {
  data <- list(mock_concept())
  result <- new_omophub_hierarchy(data, list())

  df <- as.data.frame(result)

  expect_s3_class(df, "data.frame")
})

# ==============================================================================
# omophub_relationships
# ==============================================================================

test_that("new_omophub_relationships creates correct S3 class", {
  data <- list(
    list(
      relationship_id = "Is a",
      concept_id_1 = 201826,
      concept_id_2 = 999
    )
  )
  meta <- list(total_items = 10)

  result <- new_omophub_relationships(data, meta)

  expect_s3_class(result, "omophub_relationships")
  expect_s3_class(result$data, "tbl_df")
  expect_equal(result$total, 10)
})

test_that("new_omophub_relationships handles empty data", {
  result <- new_omophub_relationships(list(), list())

  expect_s3_class(result, "omophub_relationships")
  expect_equal(nrow(result$data), 0)
})

test_that("print.omophub_relationships outputs expected format", {
  data <- list(list(relationship_id = "Is a", concept_id_1 = 1, concept_id_2 = 2))
  result <- new_omophub_relationships(data, list(total_items = 5))

  expect_output(print(result), "<OMOPHub Relationships>")
  expect_output(print(result), "Total: 5 relationships")
})

test_that("as.data.frame.omophub_relationships returns data.frame", {
  data <- list(list(relationship_id = "Is a", concept_id_1 = 1, concept_id_2 = 2))
  result <- new_omophub_relationships(data, list())

  df <- as.data.frame(result)

  expect_s3_class(df, "data.frame")
})

# ==============================================================================
# omophub_mappings
# ==============================================================================

test_that("new_omophub_mappings creates correct S3 class", {
  data <- list(
    list(
      source_concept_id = 201826,
      target_concept_id = 12345,
      target_vocabulary_id = "ICD10CM"
    )
  )
  meta <- list(total_items = 8)

  result <- new_omophub_mappings(data, meta)

  expect_s3_class(result, "omophub_mappings")
  expect_s3_class(result$data, "tbl_df")
  expect_equal(result$total, 8)
})

test_that("new_omophub_mappings handles empty data", {
  result <- new_omophub_mappings(list(), list())

  expect_s3_class(result, "omophub_mappings")
  expect_equal(nrow(result$data), 0)
})

test_that("print.omophub_mappings outputs expected format", {
  data <- list(list(source_concept_id = 1, target_concept_id = 2, target_vocabulary_id = "X"))
  result <- new_omophub_mappings(data, list(total_items = 3))

  expect_output(print(result), "<OMOPHub Mappings>")
  expect_output(print(result), "Total: 3 mappings")
})

test_that("as.data.frame.omophub_mappings returns data.frame", {
  data <- list(list(source_concept_id = 1, target_concept_id = 2, target_vocabulary_id = "X"))
  result <- new_omophub_mappings(data, list())

  df <- as.data.frame(result)

  expect_s3_class(df, "data.frame")
})

# ==============================================================================
# Edge cases for all types
# ==============================================================================

test_that("response classes handle data with NULL values", {
  data <- list(
    list(concept_id = 201826, concept_name = NULL, vocabulary_id = "SNOMED")
  )

  # Should not error, NULL becomes NA in tibble
  result <- new_omophub_concepts(data, list())

  expect_s3_class(result, "omophub_concepts")
  expect_equal(nrow(result$data), 1)
})

test_that("response classes handle nested pagination meta", {
  data <- list(mock_concept())
  meta <- list(
    pagination = list(
      page = 2,
      page_size = 50,
      total_items = 200,
      has_next = TRUE
    )
  )

  # Using the nested pagination format
  result <- new_omophub_concepts(data, meta$pagination)

  expect_equal(result$page, 2)
  expect_equal(result$page_size, 50)
  expect_equal(result$total, 200)
  expect_true(result$has_next)
})

#' Check if Error is Transient (Retryable)
#'
#' @param resp An httr2 response object.
#' @returns `TRUE` if the error is transient and should be retried.
#' @keywords internal
is_transient_error <- function(resp) {
  status <- httr2::resp_status(resp)
  status %in% c(429L, 500L, 502L, 503L, 504L)
}

#' Extract Error Message from API Response
#'
#' @param resp An httr2 response object.
#' @returns A character string with the error message.
#' @keywords internal
extract_error_message <- function(resp) {
  tryCatch(
    {
      body <- httr2::resp_body_json(resp)
      # Try different error message locations
      msg <- body$error$message %||%
        body$message %||%
        body$detail %||%
        body$error %||%
        "Unknown API error"
      as.character(msg)
    },
    error = function(e) {
      paste("HTTP", httr2::resp_status(resp), "error")
    }
  )
}

#' Abort with OMOPHub API Error
#'
#' @param status HTTP status code.
#' @param message Error message.
#' @param endpoint API endpoint that failed.
#' @param call The calling environment.
#' @keywords internal
abort_api_error <- function(status, message, endpoint, call = rlang::caller_env()) {
  rlang::abort(
    message = c(
      glue::glue("OMOPHub API error ({status})"),
      "x" = message,
      "i" = glue::glue("Endpoint: {endpoint}")
    ),
    class = c("omophub_api_error", "omophub_error"),
    status = status,
    endpoint = endpoint,
    call = call
  )
}

#' Abort with Authentication Error
#'
#' @param message Error message.
#' @param call The calling environment.
#' @keywords internal
abort_auth_error <- function(message, call = rlang::caller_env()) {
  rlang::abort(
    message = c(
      "OMOPHub authentication failed",
      "x" = message,
      "i" = "Check your API key with {.fun get_api_key}"
    ),
    class = c("omophub_auth_error", "omophub_error"),
    call = call
  )
}

#' Abort with Rate Limit Error
#'
#' @param retry_after Seconds until rate limit resets.
#' @param call The calling environment.
#' @keywords internal
abort_rate_limit <- function(retry_after = NULL, call = rlang::caller_env()) {
  msg <- "Rate limit exceeded"
  if (!is.null(retry_after)) {
    msg <- c(msg, "i" = glue::glue("Retry after {retry_after} seconds"))
  }
  rlang::abort(
    message = c("OMOPHub rate limit exceeded", "x" = msg),
    class = c("omophub_rate_limit_error", "omophub_error"),
    retry_after = retry_after,
    call = call
  )
}

#' Abort with Validation Error
#'
#' @param message Error message.
#' @param arg The argument that failed validation.
#' @param call The calling environment.
#' @keywords internal
abort_validation <- function(message, arg = NULL, call = rlang::caller_env()) {
  msg_parts <- "Invalid input"
  if (!is.null(arg)) {
    msg_parts <- c(msg_parts, "x" = glue::glue("Argument: {arg}"))
  }
  msg_parts <- c(msg_parts, "x" = message)

  rlang::abort(
    message = msg_parts,
    class = c("omophub_validation_error", "omophub_error"),
    arg = arg,
    call = call
  )
}

# Null coalescing operator (not exported, no Rd file)
# @noRd
`%||%` <- function(x, y) {
  if (is.null(x)) y else x
}

#' Convert Boolean to API String
#'
#' @param x A logical value.
#' @returns "true" or "false" string, or NULL if x is NULL.
#' @keywords internal
bool_to_str <- function(x) {
  if (is.null(x)) return(NULL)
  if (isTRUE(x)) "true" else "false"
}

#' Join List Elements for Query Parameter
#'
#' @param x A character vector.
#' @param sep Separator (default comma).
#' @returns A single comma-separated string, or NULL if x is NULL/empty.
#' @keywords internal
join_params <- function(x, sep = ",") {
  if (is.null(x) || length(x) == 0) return(NULL)
  paste(x, collapse = sep)
}

#' Validate Concept ID
#'
#' @param concept_id A concept ID to validate.
#' @param arg Argument name for error messages.
#' @param call The calling environment.
#' @keywords internal
validate_concept_id <- function(concept_id, arg = "concept_id", call = rlang::caller_env()) {
  if (!checkmate::test_integerish(concept_id, len = 1, lower = 1)) {
    abort_validation(
      "Must be a positive integer",
      arg = arg,
      call = call
    )
  }
  as.integer(concept_id)
}

#' Validate Page Parameters
#'
#' @param page Page number.
#' @param page_size Page size.
#' @param max_page_size Maximum allowed page size.
#' @param call The calling environment.
#' @keywords internal
validate_pagination <- function(page, page_size, max_page_size = 1000, call = rlang::caller_env()) {
  if (!checkmate::test_integerish(page, len = 1, lower = 1)) {
    abort_validation("Must be a positive integer", arg = "page", call = call)
  }
  if (!checkmate::test_integerish(page_size, len = 1, lower = 1, upper = max_page_size)) {
    abort_validation(
      glue::glue("Must be between 1 and {max_page_size}"),
      arg = "page_size",
      call = call
    )
  }
  list(page = as.integer(page), page_size = as.integer(page_size))
}

test_that("bool_to_str converts booleans correctly", {
  expect_equal(omophub:::bool_to_str(TRUE), "true")
  expect_equal(omophub:::bool_to_str(FALSE), "false")
  expect_null(omophub:::bool_to_str(NULL))
})

test_that("join_params joins vectors correctly", {
  expect_equal(omophub:::join_params(c("a", "b", "c")), "a,b,c")
  expect_equal(omophub:::join_params(c("SNOMED", "ICD10CM")), "SNOMED,ICD10CM")
  expect_null(omophub:::join_params(NULL))
  expect_null(omophub:::join_params(character(0)))
})

test_that("validate_concept_id accepts valid IDs", {
  expect_equal(omophub:::validate_concept_id(201826), 201826L)
  expect_equal(omophub:::validate_concept_id(1L), 1L)
})

test_that("validate_concept_id rejects invalid IDs", {
  expect_error(omophub:::validate_concept_id(0), "positive integer")
  expect_error(omophub:::validate_concept_id(-1), "positive integer")
  expect_error(omophub:::validate_concept_id("abc"), "positive integer")
  expect_error(omophub:::validate_concept_id(c(1, 2)), "positive integer")
})

test_that("validate_pagination accepts valid params", {
  result <- omophub:::validate_pagination(1, 20)
  expect_equal(result$page, 1L)
  expect_equal(result$page_size, 20L)
})

test_that("validate_pagination rejects invalid params", {
  expect_error(omophub:::validate_pagination(0, 20), "positive integer")
  expect_error(omophub:::validate_pagination(1, 0), "between 1 and")
  expect_error(omophub:::validate_pagination(1, 2000), "between 1 and")
})

test_that("null coalescing operator works", {
  # Use rlang's %||% as our internal one is not exported
  # This test verifies the basic behavior
  expect_equal(rlang::`%||%`(NULL, "default"), "default")
  expect_equal(rlang::`%||%`("value", "default"), "value")
  expect_equal(rlang::`%||%`(0, "default"), 0)
  expect_equal(rlang::`%||%`("", "default"), "")
})

# ==============================================================================
# is_transient_error() tests
# ==============================================================================

test_that("is_transient_error returns TRUE for retryable status codes", {
  # Create mock responses with different status codes
  local_mocked_bindings(
    resp_status = function(resp) resp$status_code,
    .package = "httr2"
  )

  mock_resp <- function(status) list(status_code = status)

  expect_true(omophub:::is_transient_error(mock_resp(429L)))  # Rate limit
  expect_true(omophub:::is_transient_error(mock_resp(500L)))  # Internal server error
  expect_true(omophub:::is_transient_error(mock_resp(502L)))  # Bad gateway
  expect_true(omophub:::is_transient_error(mock_resp(503L)))  # Service unavailable
  expect_true(omophub:::is_transient_error(mock_resp(504L)))  # Gateway timeout
})

test_that("is_transient_error returns FALSE for non-retryable status codes", {
  local_mocked_bindings(
    resp_status = function(resp) resp$status_code,
    .package = "httr2"
  )

  mock_resp <- function(status) list(status_code = status)

  expect_false(omophub:::is_transient_error(mock_resp(200L)))  # Success
  expect_false(omophub:::is_transient_error(mock_resp(400L)))  # Bad request
  expect_false(omophub:::is_transient_error(mock_resp(401L)))  # Unauthorized
  expect_false(omophub:::is_transient_error(mock_resp(403L)))  # Forbidden
  expect_false(omophub:::is_transient_error(mock_resp(404L)))  # Not found
  expect_false(omophub:::is_transient_error(mock_resp(422L)))  # Validation error
})

# ==============================================================================
# extract_error_message() tests
# ==============================================================================

test_that("extract_error_message extracts error.message", {
  local_mocked_bindings(
    resp_body_json = function(resp) list(error = list(message = "Detailed error message")),
    resp_status = function(resp) 400L,
    .package = "httr2"
  )

  result <- omophub:::extract_error_message(list())
  expect_equal(result, "Detailed error message")
})

test_that("extract_error_message extracts message field", {
  local_mocked_bindings(
    resp_body_json = function(resp) list(message = "Simple message"),
    resp_status = function(resp) 400L,
    .package = "httr2"
  )

  result <- omophub:::extract_error_message(list())
  expect_equal(result, "Simple message")
})

test_that("extract_error_message extracts detail field", {
  local_mocked_bindings(
    resp_body_json = function(resp) list(detail = "Detail message"),
    resp_status = function(resp) 400L,
    .package = "httr2"
  )

  result <- omophub:::extract_error_message(list())
  expect_equal(result, "Detail message")
})

test_that("extract_error_message handles error as object with message", {
  # When error is an object with a message field
  local_mocked_bindings(
    resp_body_json = function(resp) list(error = list(message = "Nested error message")),
    resp_status = function(resp) 400L,
    .package = "httr2"
  )

  result <- omophub:::extract_error_message(list())
  expect_equal(result, "Nested error message")
})

test_that("extract_error_message returns Unknown API error when no message found", {
  local_mocked_bindings(
    resp_body_json = function(resp) list(status = "failed"),
    resp_status = function(resp) 400L,
    .package = "httr2"
  )

  result <- omophub:::extract_error_message(list())
  expect_equal(result, "Unknown API error")
})

test_that("extract_error_message handles JSON parse failure", {
  local_mocked_bindings(
    resp_body_json = function(resp) stop("JSON parse error"),
    resp_status = function(resp) 500L,
    .package = "httr2"
  )

  result <- omophub:::extract_error_message(list())
  expect_equal(result, "HTTP 500 error")
})

# ==============================================================================
# abort_api_error() tests
# ==============================================================================

test_that("abort_api_error throws error with correct class", {
  err <- rlang::catch_cnd(
    omophub:::abort_api_error(404, "Not found", "/concepts/999")
  )

  expect_s3_class(err, "omophub_api_error")
  expect_s3_class(err, "omophub_error")
})

test_that("abort_api_error includes status and endpoint", {
  err <- rlang::catch_cnd(
    omophub:::abort_api_error(400, "Invalid request", "/search")
  )

  expect_equal(err$status, 400)
  expect_equal(err$endpoint, "/search")
})

test_that("abort_api_error message includes status code", {
  expect_error(
    omophub:::abort_api_error(404, "Resource not found", "/test"),
    "404"
  )
})

# ==============================================================================
# abort_auth_error() tests
# ==============================================================================

test_that("abort_auth_error throws error with correct class", {
  err <- rlang::catch_cnd(
    omophub:::abort_auth_error("Invalid API key")
  )

  expect_s3_class(err, "omophub_auth_error")
  expect_s3_class(err, "omophub_error")
})

test_that("abort_auth_error mentions authentication", {
  expect_error(
    omophub:::abort_auth_error("Invalid key"),
    "authentication"
  )
})

# ==============================================================================
# abort_rate_limit() tests
# ==============================================================================

test_that("abort_rate_limit throws error with correct class", {
  err <- rlang::catch_cnd(
    omophub:::abort_rate_limit(60)
  )

  expect_s3_class(err, "omophub_rate_limit_error")
  expect_s3_class(err, "omophub_error")
})

test_that("abort_rate_limit includes retry_after when provided", {
  err <- rlang::catch_cnd(
    omophub:::abort_rate_limit(60)
  )

  expect_equal(err$retry_after, 60)
})

test_that("abort_rate_limit works without retry_after", {
  err <- rlang::catch_cnd(
    omophub:::abort_rate_limit()
  )

  expect_null(err$retry_after)
  expect_s3_class(err, "omophub_rate_limit_error")
})

# ==============================================================================
# abort_validation() tests
# ==============================================================================

test_that("abort_validation throws error with correct class", {
  err <- rlang::catch_cnd(
    omophub:::abort_validation("Invalid value", arg = "concept_id")
  )

  expect_s3_class(err, "omophub_validation_error")
  expect_s3_class(err, "omophub_error")
})

test_that("abort_validation includes arg when provided", {
  err <- rlang::catch_cnd(
    omophub:::abort_validation("Must be positive", arg = "page")
  )

  expect_equal(err$arg, "page")
})

test_that("abort_validation works without arg", {
  err <- rlang::catch_cnd(
    omophub:::abort_validation("Generic validation error")
  )

  expect_null(err$arg)
  expect_s3_class(err, "omophub_validation_error")
})

#' FHIR-to-OMOP Concept Resolver
#'
#' @description
#' R6 class providing access to the FHIR-to-OMOP Concept Resolver endpoints.
#' Translates FHIR coded values (system URI + code) into OMOP standard
#' concepts, CDM target tables, and optional Phoebe recommendations.
#'
#' @details
#' Access via the `fhir` active binding on an `OMOPHubClient`:
#' ```r
#' client <- OMOPHubClient$new(api_key = "oh_xxx")
#' result <- client$fhir$resolve(
#'   system = "http://snomed.info/sct",
#'   code = "44054006",
#'   resource_type = "Condition"
#' )
#' result$data$resolution$target_table
#' # "condition_occurrence"
#' ```
#'
#' @keywords internal
FhirResource <- R6::R6Class(
  "FhirResource",
  public = list(
    #' @description
    #' Create a new FhirResource.
    #' @param base_req Base httr2 request object.
    initialize = function(base_req) {
      private$.base_req <- base_req
    },

    #' @description
    #' Resolve a single FHIR Coding to an OMOP standard concept.
    #'
    #' Provide at least one of (`system` + `code`), (`vocabulary_id` + `code`),
    #' or `display`.
    #'
    #' @param system FHIR code system URI (e.g. `"http://snomed.info/sct"`).
    #' @param code Code value from the FHIR Coding.
    #' @param display Human-readable text (semantic search fallback).
    #' @param vocabulary_id Direct OMOP vocabulary_id, bypasses URI resolution.
    #' @param resource_type FHIR resource type (e.g. `"Condition"`, `"Observation"`).
    #' @param include_recommendations Logical. Include Phoebe recommendations. Default `FALSE`.
    #' @param recommendations_limit Integer. Max recommendations (1-20). Default `5L`.
    #' @param include_quality Logical. Include mapping quality signal. Default `FALSE`.
    #'
    #' @returns A list with `input` and `resolution` containing source/standard
    #'   concepts, target CDM table, and optional enrichments.
    resolve = function(system = NULL,
                       code = NULL,
                       display = NULL,
                       vocabulary_id = NULL,
                       resource_type = NULL,
                       include_recommendations = FALSE,
                       recommendations_limit = 5L,
                       include_quality = FALSE) {
      body <- compact_list(
        system = system,
        code = code,
        display = display,
        vocabulary_id = vocabulary_id,
        resource_type = resource_type
      )
      if (isTRUE(include_recommendations)) {
        body$include_recommendations <- TRUE
        body$recommendations_limit <- as.integer(recommendations_limit)
      }
      if (isTRUE(include_quality)) {
        body$include_quality <- TRUE
      }

      perform_post(private$.base_req, "fhir/resolve", body = body)
    },

    #' @description
    #' Batch-resolve up to 100 FHIR Codings.
    #'
    #' Failed items are reported inline without failing the batch.
    #'
    #' @param codings A list of coding lists, each with optional elements
    #'   `system`, `code`, `display`, `vocabulary_id`.
    #' @param resource_type FHIR resource type applied to all codings.
    #' @param include_recommendations Logical. Default `FALSE`.
    #' @param recommendations_limit Integer. Default `5L`.
    #' @param include_quality Logical. Default `FALSE`.
    #' @param as_tibble Logical. When `TRUE`, returns a [tibble::tibble]
    #'   with one row per input coding and flat columns for the source
    #'   concept, standard concept, target CDM table, mapping type, and
    #'   resolution status. The batch `summary` (total/resolved/failed)
    #'   is attached as `attr(result, "summary")`. Default `FALSE`
    #'   keeps the legacy list-shaped return.
    #'
    #' @returns When `as_tibble = FALSE` (default), a list with `results`
    #'   (per-item) and `summary` (total/resolved/failed). When
    #'   `as_tibble = TRUE`, a [tibble::tibble] suitable for
    #'   `dplyr`/`tidyr` pipelines.
    resolve_batch = function(codings,
                             resource_type = NULL,
                             include_recommendations = FALSE,
                             recommendations_limit = 5L,
                             include_quality = FALSE,
                             as_tibble = FALSE) {
      stopifnot(is.list(codings), length(codings) >= 1, length(codings) <= 100)
      if (!all(vapply(codings, is.list, logical(1)))) {
        cli::cli_abort(c(
          "{.arg codings} must be a list of coding lists.",
          "i" = "Each element should be a list with {.field system}, {.field code}, etc.",
          "i" = "Example: {.code list(list(system = \"http://snomed.info/sct\", code = \"44054006\"))}"
        ))
      }

      body <- list(codings = codings)
      if (!is.null(resource_type)) body$resource_type <- resource_type
      if (isTRUE(include_recommendations)) {
        body$include_recommendations <- TRUE
        body$recommendations_limit <- as.integer(recommendations_limit)
      }
      if (isTRUE(include_quality)) body$include_quality <- TRUE

      result <- perform_post(private$.base_req, "fhir/resolve/batch", body = body)

      if (isTRUE(as_tibble)) {
        return(fhir_batch_to_tibble(result, codings))
      }
      result
    },

    #' @description
    #' Resolve a FHIR CodeableConcept with vocabulary preference.
    #'
    #' Picks the best match per OHDSI preference order
    #' (SNOMED > RxNorm > LOINC > CVX > ICD-10). Falls back to `text`
    #' via semantic search if no coding resolves.
    #'
    #' @param coding A list of coding lists, each with `system`, `code`,
    #'   and optional `display`.
    #' @param text Optional CodeableConcept.text for semantic fallback.
    #' @param resource_type FHIR resource type.
    #' @param include_recommendations Logical. Default `FALSE`.
    #' @param recommendations_limit Integer. Default `5L`.
    #' @param include_quality Logical. Default `FALSE`.
    #'
    #' @returns A list with `best_match`, `alternatives`, and `unresolved`.
    resolve_codeable_concept = function(coding,
                                        text = NULL,
                                        resource_type = NULL,
                                        include_recommendations = FALSE,
                                        recommendations_limit = 5L,
                                        include_quality = FALSE) {
      stopifnot(is.list(coding), length(coding) >= 1, length(coding) <= 20)
      if (!all(vapply(coding, is.list, logical(1)))) {
        cli::cli_abort(c(
          "{.arg coding} must be a list of coding lists.",
          "i" = "Each element should be a list with {.field system} and {.field code}.",
          "i" = "Example: {.code list(list(system = \"http://snomed.info/sct\", code = \"44054006\"))}"
        ))
      }

      body <- list(coding = coding)
      if (!is.null(text)) body$text <- text
      if (!is.null(resource_type)) body$resource_type <- resource_type
      if (isTRUE(include_recommendations)) {
        body$include_recommendations <- TRUE
        body$recommendations_limit <- as.integer(recommendations_limit)
      }
      if (isTRUE(include_quality)) body$include_quality <- TRUE

      perform_post(private$.base_req, "fhir/resolve/codeable-concept", body = body)
    }
  ),
  private = list(
    .base_req = NULL
  )
)


#' Helper to remove NULL entries from a named list.
#' @param ... Named arguments.
#' @returns A list with NULL entries removed.
#' @keywords internal
compact_list <- function(...) {
  args <- list(...)
  args[!vapply(args, is.null, logical(1))]
}


#' Flatten a batch resolver response into a tibble.
#'
#' Internal helper used by `FhirResource$resolve_batch(as_tibble = TRUE)`.
#' Produces one row per input coding with flat columns for the source
#' concept, standard concept, target CDM table, and status. The original
#' `summary` list is attached to the returned tibble via
#' `attr(x, "summary")`.
#'
#' @param result The raw list result from `perform_post`.
#' @param codings The original input `codings` list, used to align rows
#'   when items fail resolution and no `source_concept` is returned.
#' @returns A [tibble::tibble].
#' @keywords internal
fhir_batch_to_tibble <- function(result, codings) {
  items <- result$results %||% list()
  n <- length(codings)

  # Collapse a possibly-nested error object to a scalar string. The API
  # may return `error` as a plain string, a list like
  # `list(code = "concept_not_found", message = "...")`, or something
  # else. A list-column here would be expanded into extra tibble rows.
  error_to_string <- function(err) {
    if (is.null(err)) return(NA_character_)
    if (is.character(err) && length(err) == 1L) return(err)
    if (is.list(err)) {
      msg <- err$message %||% err$code %||% err$detail
      if (!is.null(msg) && is.character(msg) && length(msg) == 1L) {
        return(msg)
      }
      return(paste(names(err) %||% "", unlist(lapply(err, as.character)),
                   sep = "=", collapse = "; "))
    }
    # Zero-length / non-character fallbacks: single-bracket indexing
    # returns NA on a length-0 vector instead of throwing. Guards
    # against `character(0)`, `integer(0)`, etc.
    as.character(err)[1L]
  }

  make_row <- function(i) {
    input_coding <- codings[[i]]
    item <- if (i <= length(items)) items[[i]] else NULL

    # Failed items: the API returns them without a resolution block.
    resolution <- item$resolution
    if (is.null(resolution)) {
      return(tibble::tibble(
        source_system = input_coding$system %||% NA_character_,
        source_code = input_coding$code %||% NA_character_,
        source_concept_id = NA_integer_,
        source_concept_name = NA_character_,
        standard_concept_id = NA_integer_,
        standard_concept_name = NA_character_,
        standard_vocabulary_id = NA_character_,
        domain_id = NA_character_,
        target_table = NA_character_,
        mapping_type = NA_character_,
        similarity_score = NA_real_,
        status = "failed",
        status_detail = error_to_string(item$error)
      ))
    }

    src <- resolution$source_concept %||% list()
    std <- resolution$standard_concept %||% list()

    tibble::tibble(
      source_system = input_coding$system %||% NA_character_,
      source_code = input_coding$code %||% NA_character_,
      source_concept_id = src$concept_id %||% NA_integer_,
      source_concept_name = src$concept_name %||% NA_character_,
      standard_concept_id = std$concept_id %||% NA_integer_,
      standard_concept_name = std$concept_name %||% NA_character_,
      standard_vocabulary_id = std$vocabulary_id %||% NA_character_,
      domain_id = std$domain_id %||% NA_character_,
      target_table = resolution$target_table %||% NA_character_,
      mapping_type = resolution$mapping_type %||% NA_character_,
      similarity_score = resolution$similarity_score %||% NA_real_,
      status = "resolved",
      status_detail = NA_character_
    )
  }

  rows <- lapply(seq_len(n), make_row)
  tbl <- do.call(rbind, rows)
  attr(tbl, "summary") <- result$summary
  tbl
}


#' Null-coalescing operator.
#' @noRd
`%||%` <- function(a, b) if (is.null(a)) b else a


# =============================================================================
# Standalone wrapper functions
#
# Thin wrappers around the R6 `FhirResource` methods so users can write
# pipe-friendly code without dereferencing the client every call:
#
#   client |> fhir_resolve(system = "http://snomed.info/sct", code = "44054006")
#
# These forward all arguments to the underlying R6 method.
# =============================================================================

#' Resolve a FHIR Coding to an OMOP standard concept
#'
#' Standalone wrapper for `client$fhir$resolve()`. See
#' [FhirResource] for full parameter documentation.
#'
#' @param client An [OMOPHubClient] instance.
#' @param ... Arguments passed to `FhirResource$resolve()`.
#' @returns A list with `input` and `resolution`.
#' @seealso [FhirResource], [fhir_resolve_batch()],
#'   [fhir_resolve_codeable_concept()]
#' @examples
#' \dontrun{
#' client <- OMOPHubClient$new(api_key = Sys.getenv("OMOPHUB_API_KEY"))
#' fhir_resolve(
#'   client,
#'   system = "http://snomed.info/sct",
#'   code = "44054006",
#'   resource_type = "Condition"
#' )
#' }
#' @export
fhir_resolve <- function(client, ...) {
  client$fhir$resolve(...)
}


#' Batch-resolve FHIR Codings
#'
#' Standalone wrapper for `client$fhir$resolve_batch()`. When
#' `as_tibble = TRUE`, the result is a flat `tibble` suitable for
#' `dplyr`/`tidyr` pipelines.
#'
#' @param client An [OMOPHubClient] instance.
#' @param ... Arguments passed to `FhirResource$resolve_batch()`.
#' @returns See `FhirResource$resolve_batch()`.
#' @seealso [FhirResource], [fhir_resolve()],
#'   [fhir_resolve_codeable_concept()]
#' @examples
#' \dontrun{
#' client <- OMOPHubClient$new(api_key = Sys.getenv("OMOPHUB_API_KEY"))
#' tbl <- fhir_resolve_batch(
#'   client,
#'   codings = list(
#'     list(system = "http://snomed.info/sct", code = "44054006"),
#'     list(system = "http://loinc.org", code = "2339-0")
#'   ),
#'   as_tibble = TRUE
#' )
#' dplyr::filter(tbl, status == "resolved")
#' }
#' @export
fhir_resolve_batch <- function(client, ...) {
  client$fhir$resolve_batch(...)
}


#' Resolve a FHIR CodeableConcept with vocabulary preference
#'
#' Standalone wrapper for `client$fhir$resolve_codeable_concept()`.
#'
#' @param client An [OMOPHubClient] instance.
#' @param ... Arguments passed to `FhirResource$resolve_codeable_concept()`.
#' @returns See `FhirResource$resolve_codeable_concept()`.
#' @seealso [FhirResource], [fhir_resolve()], [fhir_resolve_batch()]
#' @examples
#' \dontrun{
#' client <- OMOPHubClient$new(api_key = Sys.getenv("OMOPHUB_API_KEY"))
#' fhir_resolve_codeable_concept(
#'   client,
#'   coding = list(
#'     list(system = "http://snomed.info/sct", code = "44054006"),
#'     list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9")
#'   ),
#'   resource_type = "Condition"
#' )
#' }
#' @export
fhir_resolve_codeable_concept <- function(client, ...) {
  client$fhir$resolve_codeable_concept(...)
}

# Unit tests for FhirResource (R/fhir.R)
# Uses mocked HTTP responses to avoid API calls

# ==============================================================================
# FhirResource initialization
# ==============================================================================

test_that("FhirResource initializes correctly", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- FhirResource$new(base_req)

  expect_s3_class(resource, "FhirResource")
  expect_s3_class(resource, "R6")
})

# ==============================================================================
# Helper: mock FHIR response
# ==============================================================================

# Mocks return what perform_post actually returns: the unwrapped `data`
# field from the API response (perform_post strips the outer {success, data, meta}).
mock_fhir_resolution <- function() {
  list(
    input = list(system = "http://snomed.info/sct", code = "44054006"),
    resolution = list(
      vocabulary_id = "SNOMED",
      source_concept = list(
        concept_id = 201826L,
        concept_name = "Type 2 diabetes mellitus",
        concept_code = "44054006",
        vocabulary_id = "SNOMED",
        domain_id = "Condition",
        concept_class_id = "Clinical Finding",
        standard_concept = "S"
      ),
      standard_concept = list(
        concept_id = 201826L,
        concept_name = "Type 2 diabetes mellitus",
        concept_code = "44054006",
        vocabulary_id = "SNOMED",
        domain_id = "Condition",
        concept_class_id = "Clinical Finding",
        standard_concept = "S"
      ),
      mapping_type = "direct",
      target_table = "condition_occurrence",
      domain_resource_alignment = "aligned"
    )
  )
}

mock_fhir_batch <- function() {
  list(
    results = list(mock_fhir_resolution()),
    summary = list(total = 1L, resolved = 1L, failed = 0L)
  )
}

mock_fhir_codeable_concept <- function() {
  list(
    input = list(
      coding = list(
        list(system = "http://snomed.info/sct", code = "44054006"),
        list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9")
      )
    ),
    best_match = mock_fhir_resolution(),
    alternatives = list(),
    unresolved = list()
  )
}

# ==============================================================================
# resolve()
# ==============================================================================

test_that("fhir$resolve calls correct endpoint with body", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- FhirResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      called_with <<- list(path = path, body = body)
      mock_fhir_resolution()
    }
  )

  result <- resource$resolve(
    system = "http://snomed.info/sct",
    code = "44054006",
    resource_type = "Condition"
  )

  expect_equal(called_with$path, "fhir/resolve")
  expect_equal(called_with$body$system, "http://snomed.info/sct")
  expect_equal(called_with$body$code, "44054006")
  expect_equal(called_with$body$resource_type, "Condition")
  # Optional flags should NOT be in body when FALSE
  expect_null(called_with$body$include_recommendations)
  expect_null(called_with$body$include_quality)
})

test_that("fhir$resolve includes recommendation flags when requested", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- FhirResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      called_with <<- list(body = body)
      mock_fhir_resolution()
    }
  )

  resource$resolve(
    system = "http://snomed.info/sct",
    code = "44054006",
    include_recommendations = TRUE,
    recommendations_limit = 3L,
    include_quality = TRUE
  )

  expect_true(called_with$body$include_recommendations)
  expect_equal(called_with$body$recommendations_limit, 3L)
  expect_true(called_with$body$include_quality)
})

test_that("fhir$resolve returns resolution data", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- FhirResource$new(base_req)

  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      mock_fhir_resolution()
    }
  )

  result <- resource$resolve(system = "http://snomed.info/sct", code = "44054006")

  expect_equal(result$resolution$mapping_type, "direct")
  expect_equal(result$resolution$target_table, "condition_occurrence")
  expect_equal(result$resolution$standard_concept$concept_id, 201826L)
})

# ==============================================================================
# resolve_batch()
# ==============================================================================

test_that("fhir$resolve_batch calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- FhirResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      called_with <<- list(path = path, body = body)
      mock_fhir_batch()
    }
  )

  codings <- list(
    list(system = "http://snomed.info/sct", code = "44054006")
  )
  result <- resource$resolve_batch(codings)

  expect_equal(called_with$path, "fhir/resolve/batch")
  expect_equal(length(called_with$body$codings), 1)
  expect_equal(result$summary$total, 1L)
  expect_equal(result$summary$resolved, 1L)
})

test_that("fhir$resolve_batch passes all optional flags", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- FhirResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      called_with <<- list(body = body)
      mock_fhir_batch()
    }
  )

  resource$resolve_batch(
    list(list(system = "http://snomed.info/sct", code = "44054006")),
    resource_type = "Condition",
    include_recommendations = TRUE,
    recommendations_limit = 3L,
    include_quality = TRUE
  )

  expect_equal(called_with$body$resource_type, "Condition")
  expect_true(called_with$body$include_recommendations)
  expect_equal(called_with$body$recommendations_limit, 3L)
  expect_true(called_with$body$include_quality)
})

test_that("fhir$resolve_batch validates codings length", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- FhirResource$new(base_req)

  expect_error(resource$resolve_batch(list()))
})

test_that("fhir$resolve_batch rejects a single coding (not list-of-lists)", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- FhirResource$new(base_req)

  # A single coding object (not wrapped in an outer list) should fail
  expect_error(
    resource$resolve_batch(list(system = "http://snomed.info/sct", code = "44054006")),
    "list of coding lists"
  )
})

test_that("fhir$resolve_codeable_concept rejects a single coding (not list-of-lists)", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- FhirResource$new(base_req)

  # A single coding object (not wrapped in an outer list) should fail
  expect_error(
    resource$resolve_codeable_concept(list(system = "http://snomed.info/sct", code = "44054006")),
    "list of coding lists"
  )
})

# ==============================================================================
# resolve_codeable_concept()
# ==============================================================================

test_that("fhir$resolve_codeable_concept calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- FhirResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      called_with <<- list(path = path, body = body)
      mock_fhir_codeable_concept()
    }
  )

  coding <- list(
    list(system = "http://snomed.info/sct", code = "44054006"),
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9")
  )
  result <- resource$resolve_codeable_concept(
    coding = coding,
    resource_type = "Condition"
  )

  expect_equal(called_with$path, "fhir/resolve/codeable-concept")
  expect_equal(called_with$body$resource_type, "Condition")
  expect_equal(length(called_with$body$coding), 2)
  expect_false(is.null(result$best_match))
})

test_that("fhir$resolve_codeable_concept passes all optional flags", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- FhirResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      called_with <<- list(body = body)
      mock_fhir_codeable_concept()
    }
  )

  resource$resolve_codeable_concept(
    coding = list(list(system = "http://snomed.info/sct", code = "44054006")),
    text = "diabetes",
    resource_type = "Condition",
    include_recommendations = TRUE,
    recommendations_limit = 3L,
    include_quality = TRUE
  )

  expect_equal(called_with$body$text, "diabetes")
  expect_equal(called_with$body$resource_type, "Condition")
  expect_true(called_with$body$include_recommendations)
  expect_equal(called_with$body$recommendations_limit, 3L)
  expect_true(called_with$body$include_quality)
})

test_that("fhir$resolve_codeable_concept includes text fallback", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- FhirResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      called_with <<- list(body = body)
      mock_fhir_codeable_concept()
    }
  )

  coding <- list(list(system = "http://loinc.org", code = "99999-9"))
  resource$resolve_codeable_concept(coding = coding, text = "Blood Sugar")

  expect_equal(called_with$body$text, "Blood Sugar")
})

# ==============================================================================
# resolve_batch(as_tibble = TRUE)
# ==============================================================================

mock_fhir_batch_mixed <- function() {
  list(
    results = list(
      mock_fhir_resolution(),
      list(error = "unknown code", input = list(
        system = "http://snomed.info/sct", code = "bogus"
      ))
    ),
    summary = list(total = 2L, resolved = 1L, failed = 1L)
  )
}

test_that("resolve_batch returns tibble when as_tibble = TRUE", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- FhirResource$new(base_req)

  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      mock_fhir_batch_mixed()
    }
  )

  codings <- list(
    list(system = "http://snomed.info/sct", code = "44054006"),
    list(system = "http://snomed.info/sct", code = "bogus")
  )
  tbl <- resource$resolve_batch(codings, as_tibble = TRUE)

  expect_s3_class(tbl, "tbl_df")
  expect_equal(nrow(tbl), 2L)
  expect_true(all(c(
    "source_system", "source_code",
    "source_concept_id", "source_concept_name",
    "standard_concept_id", "standard_concept_name",
    "standard_vocabulary_id", "domain_id", "target_table",
    "mapping_type", "similarity_score",
    "status", "status_detail"
  ) %in% names(tbl)))

  expect_equal(tbl$status[1], "resolved")
  expect_equal(tbl$standard_concept_id[1], 201826L)
  expect_equal(tbl$target_table[1], "condition_occurrence")

  expect_equal(tbl$status[2], "failed")
  expect_false(is.na(tbl$status_detail[2]))

  # summary is attached as attribute
  summary <- attr(tbl, "summary")
  expect_equal(summary$total, 2L)
  expect_equal(summary$resolved, 1L)
  expect_equal(summary$failed, 1L)
})

test_that("resolve_batch default return is unchanged (list shape)", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- FhirResource$new(base_req)

  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      mock_fhir_batch()
    }
  )

  result <- resource$resolve_batch(
    list(list(system = "http://snomed.info/sct", code = "44054006"))
  )

  expect_false(inherits(result, "tbl_df"))
  expect_true(is.list(result))
  expect_named(result, c("results", "summary"))
  expect_equal(result$summary$total, 1L)
})

# ==============================================================================
# Standalone wrapper parity
# ==============================================================================

test_that("fhir_resolve standalone matches R6 method", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  client_stub <- list(fhir = FhirResource$new(base_req))

  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      mock_fhir_resolution()
    }
  )

  r6_result <- client_stub$fhir$resolve(
    system = "http://snomed.info/sct",
    code = "44054006"
  )
  wrapper_result <- fhir_resolve(
    client_stub,
    system = "http://snomed.info/sct",
    code = "44054006"
  )

  expect_identical(r6_result, wrapper_result)
})

test_that("fhir_resolve_batch standalone matches R6 method", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  client_stub <- list(fhir = FhirResource$new(base_req))

  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      mock_fhir_batch()
    }
  )

  codings <- list(list(system = "http://snomed.info/sct", code = "44054006"))
  r6_result <- client_stub$fhir$resolve_batch(codings)
  wrapper_result <- fhir_resolve_batch(client_stub, codings)

  expect_identical(r6_result, wrapper_result)
})

test_that("fhir_resolve_codeable_concept standalone matches R6 method", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  client_stub <- list(fhir = FhirResource$new(base_req))

  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      mock_fhir_codeable_concept()
    }
  )

  coding <- list(
    list(system = "http://snomed.info/sct", code = "44054006"),
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9")
  )
  r6_result <- client_stub$fhir$resolve_codeable_concept(coding = coding)
  wrapper_result <- fhir_resolve_codeable_concept(client_stub, coding = coding)

  expect_identical(r6_result, wrapper_result)
})

# ==============================================================================
# resolve_batch(as_tibble = TRUE) with unusual `error` shapes
# ==============================================================================

test_that("resolve_batch(as_tibble = TRUE) handles zero-length error values", {
  # Regression for a bug where `as.character(err)[[1L]]` crashed with
  # "subscript out of bounds" when the API returned an empty character
  # vector (or any other length-0 value) in the `error` field.
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- FhirResource$new(base_req)

  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      list(
        results = list(
          list(error = character(0)),       # empty character vector
          list(error = integer(0)),         # empty integer vector
          list(error = list())              # empty list
        ),
        summary = list(total = 3L, resolved = 0L, failed = 3L)
      )
    }
  )

  codings <- list(
    list(system = "http://snomed.info/sct", code = "a"),
    list(system = "http://snomed.info/sct", code = "b"),
    list(system = "http://snomed.info/sct", code = "c")
  )

  # Must not throw, must return one row per input
  tbl <- resource$resolve_batch(codings, as_tibble = TRUE)
  expect_s3_class(tbl, "tbl_df")
  expect_equal(nrow(tbl), 3L)
  expect_true(all(tbl$status == "failed"))
})

test_that("resolve_batch(as_tibble = TRUE) handles unusual scalar error shapes", {
  # Coverage for the non-character fallback: numeric and logical errors
  # must be coerced via as.character() without throwing.
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- FhirResource$new(base_req)

  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      list(
        results = list(
          list(error = 42L),
          list(error = TRUE)
        ),
        summary = list(total = 2L, resolved = 0L, failed = 2L)
      )
    }
  )

  codings <- list(
    list(system = "http://snomed.info/sct", code = "a"),
    list(system = "http://snomed.info/sct", code = "b")
  )

  tbl <- resource$resolve_batch(codings, as_tibble = TRUE)
  expect_equal(nrow(tbl), 2L)
  expect_equal(tbl$status_detail, c("42", "TRUE"))
})

test_that("resolve_batch(as_tibble = TRUE) handles NULL and missing error", {
  # Coverage for the `is.null(err)` early return: a failed item with no
  # `error` field at all maps to NA_character_ in status_detail.
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- FhirResource$new(base_req)

  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      list(
        results = list(
          list(input = list(system = "http://snomed.info/sct", code = "a")),
          list(error = NULL)
        ),
        summary = list(total = 2L, resolved = 0L, failed = 2L)
      )
    }
  )

  codings <- list(
    list(system = "http://snomed.info/sct", code = "a"),
    list(system = "http://snomed.info/sct", code = "b")
  )

  tbl <- resource$resolve_batch(codings, as_tibble = TRUE)
  expect_equal(nrow(tbl), 2L)
  expect_true(all(tbl$status == "failed"))
  expect_true(all(is.na(tbl$status_detail)))
})

test_that("resolve_batch(as_tibble = TRUE) extracts message/code from list error", {
  # Coverage for the structured-list error branch: picks the first
  # usable of err$message, err$code, err$detail and returns it as
  # status_detail rather than a flattened name=value concatenation.
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- FhirResource$new(base_req)

  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      list(
        results = list(
          list(error = list(
            code = "concept_not_found",
            message = "No matching OMOP concept"
          )),
          list(error = list(code = "vocabulary_restricted")),  # message missing
          list(error = list(detail = "just a detail field"))   # only detail
        ),
        summary = list(total = 3L, resolved = 0L, failed = 3L)
      )
    }
  )

  codings <- list(
    list(system = "http://snomed.info/sct", code = "a"),
    list(system = "http://snomed.info/sct", code = "b"),
    list(system = "http://snomed.info/sct", code = "c")
  )

  tbl <- resource$resolve_batch(codings, as_tibble = TRUE)
  expect_equal(nrow(tbl), 3L)
  expect_equal(tbl$status_detail[1], "No matching OMOP concept")  # message wins
  expect_equal(tbl$status_detail[2], "vocabulary_restricted")     # code fallback
  expect_equal(tbl$status_detail[3], "just a detail field")       # detail fallback
})

# Integration tests for FhirResource against the live OMOPHub API
# Uses the same skip_if_no_integration_key() pattern as other integration tests.

test_that("fhir$resolve resolves SNOMED 44054006 live", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$fhir$resolve(
    system = "http://snomed.info/sct",
    code = "44054006",
    resource_type = "Condition"
  )

  res <- result$resolution
  expect_equal(res$mapping_type, "direct")
  expect_equal(res$target_table, "condition_occurrence")
  expect_equal(res$standard_concept$vocabulary_id, "SNOMED")
  expect_equal(res$domain_resource_alignment, "aligned")
})

test_that("fhir$resolve resolves ICD-10-CM E11.9 live", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$fhir$resolve(
    system = "http://hl7.org/fhir/sid/icd-10-cm",
    code = "E11.9"
  )

  res <- result$resolution
  expect_equal(res$vocabulary_id, "ICD10CM")
  expect_equal(res$source_concept$vocabulary_id, "ICD10CM")
  expect_equal(res$standard_concept$standard_concept, "S")
  expect_equal(res$target_table, "condition_occurrence")
})

test_that("fhir$resolve_batch resolves 3 codings live", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$fhir$resolve_batch(list(
    list(system = "http://snomed.info/sct", code = "44054006"),
    list(system = "http://loinc.org", code = "2339-0"),
    list(system = "http://www.nlm.nih.gov/research/umls/rxnorm", code = "197696")
  ))

  expect_equal(result$summary$total, 3L)
  expect_equal(
    result$summary$resolved + result$summary$failed,
    result$summary$total
  )
  expect_equal(length(result$results), 3L)
})

test_that("fhir$resolve_codeable_concept picks SNOMED over ICD-10-CM live", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$fhir$resolve_codeable_concept(
    coding = list(
      list(system = "http://snomed.info/sct", code = "44054006"),
      list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9")
    ),
    resource_type = "Condition"
  )

  expect_false(is.null(result$best_match))
  best <- result$best_match$resolution
  expect_equal(best$source_concept$vocabulary_id, "SNOMED")
  expect_equal(best$target_table, "condition_occurrence")
})

# ==============================================================================
# resolve_batch(as_tibble = TRUE) against the live API
# ==============================================================================

test_that("fhir$resolve_batch(as_tibble = TRUE) returns a shaped tibble live", {
  skip_if_no_integration_key()
  client <- integration_client()

  codings <- list(
    list(system = "http://snomed.info/sct", code = "44054006"),
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9"),
    list(system = "http://loinc.org", code = "2339-0")
  )
  tbl <- client$fhir$resolve_batch(codings, as_tibble = TRUE)

  expect_s3_class(tbl, "tbl_df")
  expect_equal(nrow(tbl), 3L)
  expect_true(all(c(
    "source_system", "source_code",
    "source_concept_id", "source_concept_name",
    "standard_concept_id", "standard_concept_name",
    "standard_vocabulary_id", "domain_id", "target_table",
    "mapping_type", "similarity_score", "status", "status_detail"
  ) %in% names(tbl)))

  # At least one row should have resolved, and resolved rows need a
  # populated standard_concept_id + target_table.
  resolved <- tbl[tbl$status == "resolved", ]
  expect_gte(nrow(resolved), 1L)
  expect_true(all(!is.na(resolved$standard_concept_id)))
  expect_true(all(!is.na(resolved$target_table)))

  # Summary attribute is aligned with the flat tibble
  summary <- attr(tbl, "summary")
  expect_equal(summary$total, 3L)
  expect_equal(summary$resolved, nrow(resolved))
})

# ==============================================================================
# Standalone wrapper smoke check
# ==============================================================================

test_that("fhir_resolve standalone wrapper works live", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- fhir_resolve(
    client,
    system = "http://snomed.info/sct",
    code = "44054006",
    resource_type = "Condition"
  )
  expect_equal(result$resolution$standard_concept$concept_id, 201826L)
  expect_equal(result$resolution$target_table, "condition_occurrence")
})

# Mock helpers for testing

#' Create a mock response for testing
#'
#' @param data Response data
#' @param status HTTP status code
#' @param meta Pagination metadata
#' @keywords internal
mock_response <- function(data = list(), status = 200, meta = list()) {
  list(
    data = data,
    meta = meta,
    status = status
  )
}

#' Create mock concept data
#' @keywords internal
mock_concept <- function(
    concept_id = 201826,
    concept_name = "Type 2 diabetes mellitus",
    vocabulary_id = "SNOMED",
    domain_id = "Condition",
    concept_class_id = "Clinical Finding",
    standard_concept = "S",
    concept_code = "44054006"
) {
  list(
    concept_id = concept_id,
    concept_name = concept_name,
    vocabulary_id = vocabulary_id,
    domain_id = domain_id,
    concept_class_id = concept_class_id,
    standard_concept = standard_concept,
    concept_code = concept_code
  )
}

#' Create mock vocabulary data
#' @keywords internal
mock_vocabulary <- function(
    vocabulary_id = "SNOMED",
    vocabulary_name = "Systematic Nomenclature of Medicine - Clinical Terms",
    vocabulary_version = "2024-07-01",
    vocabulary_reference = "http://www.snomed.org/"
) {
  list(
    vocabulary_id = vocabulary_id,
    vocabulary_name = vocabulary_name,
    vocabulary_version = vocabulary_version,
    vocabulary_reference = vocabulary_reference
  )
}

#' Create mock pagination metadata
#' @keywords internal
mock_pagination <- function(
    page = 1,
    page_size = 20,
    total_items = 100,
    has_next = TRUE,
    has_previous = FALSE
) {
  list(
    pagination = list(
      page = page,
      page_size = page_size,
      total_items = total_items,
      total_pages = ceiling(total_items / page_size),
      has_next = has_next,
      has_previous = has_previous
    )
  )
}

# FHIR client interop helpers for the OMOPHub FHIR Terminology Service.
#
# These helpers make it easy to configure an external FHIR client (like
# `httr2`, `fhircrackr`, or a custom Spring Security OAuth2 client) to
# talk directly to OMOPHub's FHIR endpoint. They are intentionally thin
# and have no client dependency so users can use them even without an
# `OMOPHubClient` instance.


#' OMOPHub FHIR Terminology Service URL
#'
#' Convenience helper for constructing the OMOPHub FHIR Terminology
#' Service base URL for a given FHIR version. Use it when configuring
#' an external FHIR client library (`httr2`, `fhircrackr`, etc.) to
#' talk to OMOPHub's FHIR endpoint directly.
#'
#' @param version FHIR version prefix. One of `"r4"` (default),
#'   `"r4b"`, `"r5"`, or `"r6"`.
#' @returns A character scalar with the full FHIR base URL, e.g.
#'   `"https://fhir.omophub.com/fhir/r4"`.
#' @examples
#' omophub_fhir_url()
#' omophub_fhir_url("r5")
#'
#' \dontrun{
#' # Use with httr2 to call the $lookup operation directly
#' library(httr2)
#' req <- request(omophub_fhir_url()) |>
#'   req_url_path_append("CodeSystem/$lookup") |>
#'   req_url_query(
#'     system = "http://snomed.info/sct",
#'     code = "44054006"
#'   ) |>
#'   req_headers(Authorization = paste("Bearer", Sys.getenv("OMOPHUB_API_KEY")))
#' resp <- req_perform(req)
#' }
#' @export
omophub_fhir_url <- function(version = c("r4", "r4b", "r5", "r6")) {
  version <- match.arg(version)
  paste0("https://fhir.omophub.com/fhir/", version)
}

# Unit tests for omophub_fhir_url() (R/fhir_interop.R)

test_that("omophub_fhir_url() defaults to r4", {
  expect_equal(omophub_fhir_url(), "https://fhir.omophub.com/fhir/r4")
})

test_that("omophub_fhir_url() supports all four FHIR versions", {
  expect_equal(omophub_fhir_url("r4"), "https://fhir.omophub.com/fhir/r4")
  expect_equal(omophub_fhir_url("r4b"), "https://fhir.omophub.com/fhir/r4b")
  expect_equal(omophub_fhir_url("r5"), "https://fhir.omophub.com/fhir/r5")
  expect_equal(omophub_fhir_url("r6"), "https://fhir.omophub.com/fhir/r6")
})

test_that("omophub_fhir_url() rejects invalid versions", {
  expect_error(omophub_fhir_url("stu3"))
  expect_error(omophub_fhir_url("invalid"))
})

#' Vocabularies Resource
#'
#' @description
#' R6 class providing access to vocabulary operations.
#'
#' @keywords internal
VocabulariesResource <- R6::R6Class(
  "VocabulariesResource",
  public = list(
    #' @description
    #' Create a new VocabulariesResource.
    #' @param base_req Base httr2 request object.
    initialize = function(base_req) {
      private$.base_req <- base_req
    },

    #' @description
    #' List all vocabularies.
    #'
    #' @param include_stats Include vocabulary statistics. Default `FALSE`.
    #' @param include_inactive Include inactive vocabularies. Default `FALSE`.
    #' @param sort_by Sort field ("name", "priority", "updated"). Default "name".
    #' @param sort_order Sort order ("asc" or "desc"). Default "asc".
    #' @param page Page number. Default 1.
    #' @param page_size Results per page. Default 100.
    #'
    #' @returns Paginated vocabulary list.
    list = function(include_stats = FALSE,
                    include_inactive = FALSE,
                    sort_by = "name",
                    sort_order = "asc",
                    page = 1,
                    page_size = 100) {
      pag <- validate_pagination(page, page_size)

      params <- list(
        sort_by = sort_by,
        sort_order = sort_order,
        page = pag$page,
        page_size = pag$page_size
      )

      if (isTRUE(include_stats)) {
        params$include_stats <- "true"
      }
      if (isTRUE(include_inactive)) {
        params$include_inactive <- "true"
      }

      perform_get(private$.base_req, "vocabularies", query = params)
    },

    #' @description
    #' Get vocabulary details.
    #'
    #' @param vocabulary_id The vocabulary ID.
    #'
    #' @returns Vocabulary details including vocabulary_id, vocabulary_name,
    #'   vocabulary_reference, vocabulary_version, vocabulary_concept_id.
    get = function(vocabulary_id) {
      checkmate::assert_string(vocabulary_id, min.chars = 1)

      perform_get(
        private$.base_req,
        paste0("vocabularies/", vocabulary_id)
      )
    },

    #' @description
    #' Get vocabulary statistics.
    #'
    #' @param vocabulary_id The vocabulary ID.
    #'
    #' @returns Vocabulary statistics.
    stats = function(vocabulary_id) {
      checkmate::assert_string(vocabulary_id, min.chars = 1)

      perform_get(
        private$.base_req,
        paste0("vocabularies/", vocabulary_id, "/stats")
      )
    },

    #' @description
    #' Get statistics for a specific domain within a vocabulary.
    #'
    #' @param vocabulary_id The vocabulary ID (e.g., "SNOMED", "ICD10CM").
    #' @param domain_id The domain ID (e.g., "Condition", "Drug", "Procedure").
    #'
    #' @returns Domain statistics including concept counts and class breakdown.
    domain_stats = function(vocabulary_id, domain_id) {
      checkmate::assert_string(vocabulary_id, min.chars = 1)
      checkmate::assert_string(domain_id, min.chars = 1)

      perform_get(
        private$.base_req,
        paste0("vocabularies/", vocabulary_id, "/stats/domains/", domain_id)
      )
    },

    #' @description
    #' Get all standard OHDSI domains.
    #'
    #' @returns List of all available domains with domain_id, domain_name, and description.
    domains = function() {
      perform_get(private$.base_req, "vocabularies/domains")
    },

    #' @description
    #' Get all concept classes.
    #'
    #' @returns List of all available concept classes with concept_class_id,
    #'   concept_class_name, and concept_class_concept_id.
    concept_classes = function() {
      perform_get(private$.base_req, "vocabularies/concept-classes")
    },

    #' @description
    #' Get concepts in a vocabulary.
    #'
    #' @param vocabulary_id The vocabulary ID.
    #' @param search Search term to filter concepts by name or code.
    #' @param standard_concept Filter by standard concept status ('S', 'C', 'all'). Default "all".
    #' @param include_invalid Include invalid or deprecated concepts. Default `FALSE`.
    #' @param include_relationships Include concept relationships. Default `FALSE`.
    #' @param include_synonyms Include concept synonyms. Default `FALSE`.
    #' @param sort_by Sort field ('name', 'concept_id', 'concept_code'). Default "name".
    #' @param sort_order Sort order ('asc' or 'desc'). Default "asc".
    #' @param page Page number. Default 1.
    #' @param page_size Results per page (max 1000). Default 20.
    #'
    #' @returns Paginated concepts.
    concepts = function(vocabulary_id,
                        search = NULL,
                        standard_concept = "all",
                        include_invalid = FALSE,
                        include_relationships = FALSE,
                        include_synonyms = FALSE,
                        sort_by = "name",
                        sort_order = "asc",
                        page = 1,
                        page_size = 20) {
      checkmate::assert_string(vocabulary_id, min.chars = 1)
      pag <- validate_pagination(page, page_size)

      params <- list(
        page = pag$page,
        page_size = pag$page_size,
        standard_concept = standard_concept,
        sort_by = sort_by,
        sort_order = sort_order
      )

      if (!is.null(search)) {
        params$search <- search
      }
      if (isTRUE(include_invalid)) {
        params$include_invalid <- "true"
      }
      if (isTRUE(include_relationships)) {
        params$include_relationships <- "true"
      }
      if (isTRUE(include_synonyms)) {
        params$include_synonyms <- "true"
      }

      perform_get(
        private$.base_req,
        paste0("vocabularies/", vocabulary_id, "/concepts"),
        query = params
      )
    },

    #' @description
    #' Print resource information.
    print = function() {
      cat("<OMOPHub VocabulariesResource>\n")
      cat("  Methods: list, get, stats, domain_stats, domains, concept_classes, concepts\n")
      invisible(self)
    }
  ),
  private = list(
    .base_req = NULL
  )
)

# Integration tests for vocabularies resource
# Run with: testthat::test_file("tests/testthat/test-vocabularies-integration.R")

test_that("list vocabularies works", {
  skip_if_no_integration_key()
  client <- integration_client()

  # Use larger page_size to ensure common vocabularies are included
  result <- client$vocabularies$list(page_size = 200)

  vocabs <- extract_data(result, "vocabularies")
  expect_gt(length(vocabs), 0)

  # Check for common vocabularies
  vocab_ids <- vapply(vocabs, function(v) v$vocabulary_id %||% "", character(1))
  expect_true("SNOMED" %in% vocab_ids)
  expect_true("ICD10CM" %in% vocab_ids)
})

test_that("list vocabularies with stats works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$vocabularies$list(
    include_stats = TRUE,
    page_size = 50
  )

  vocabs <- extract_data(result, "vocabularies")
  expect_gt(length(vocabs), 0)
  # Verify vocabularies are returned with expected structure
  for (vocab in vocabs) {
    expect_true("vocabulary_id" %in% names(vocab))
  }
})

test_that("get vocabulary works", {
  skip_if_no_integration_key()
  client <- integration_client()

  vocab <- client$vocabularies$get("SNOMED")

  expect_equal(vocab$vocabulary_id, "SNOMED")
  expect_true("vocabulary_name" %in% names(vocab))
})

test_that("get vocabulary with options works", {
  skip_if_no_integration_key()
  client <- integration_client()

  vocab <- client$vocabularies$get("SNOMED")

  expect_equal(vocab$vocabulary_id, "SNOMED")
  expect_true("vocabulary_name" %in% names(vocab))
})

test_that("get vocabulary stats works", {
  skip_if_no_integration_key()
  client <- integration_client()

  stats <- client$vocabularies$stats("SNOMED")

  expect_equal(stats$vocabulary_id, "SNOMED")
  # Should have concept counts
  expect_true(
    "total_concepts" %in% names(stats) ||
      "concept_count" %in% names(stats) ||
      "standard_concepts" %in% names(stats)
  )
})

test_that("get vocabulary domains works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$vocabularies$domains()

  domains <- extract_data(result, "domains")
  expect_true(is.list(domains))
})

test_that("get vocabulary concepts works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$vocabularies$concepts(
    "SNOMED",
    search = "diabetes",
    standard_concept = "S",
    page_size = 10
  )

  concepts <- extract_data(result, "concepts")
  expect_true(is.list(concepts))
})

# Unit tests for VocabulariesResource (R/vocabularies.R)
# Uses mocked HTTP responses to avoid API calls

# ==============================================================================
# VocabulariesResource initialization
# ==============================================================================

test_that("VocabulariesResource initializes correctly", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- VocabulariesResource$new(base_req)

  expect_s3_class(resource, "VocabulariesResource")
  expect_s3_class(resource, "R6")
})

test_that("VocabulariesResource print method works", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- VocabulariesResource$new(base_req)

  expect_output(print(resource), "<OMOPHub VocabulariesResource>")
  expect_output(print(resource), "list, get, stats")
})

# ==============================================================================
# list() method
# ==============================================================================

test_that("vocabularies$list calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- VocabulariesResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      list(vocabularies = list())
    }
  )

  resource$list()

  expect_equal(called_with$path, "vocabularies")
  expect_equal(called_with$query$sort_by, "name")
  expect_equal(called_with$query$sort_order, "asc")
})

test_that("vocabularies$list includes optional params", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- VocabulariesResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(vocabularies = list())
    }
  )

  resource$list(
    include_stats = TRUE,
    include_inactive = TRUE,
    sort_by = "priority",
    sort_order = "desc",
    page = 2,
    page_size = 50
  )

  expect_equal(called_with$query$include_stats, "true")
  expect_equal(called_with$query$include_inactive, "true")
  expect_equal(called_with$query$sort_by, "priority")
  expect_equal(called_with$query$sort_order, "desc")
  expect_equal(called_with$query$page, 2L)
  expect_equal(called_with$query$page_size, 50L)
})

# ==============================================================================
# get() method
# ==============================================================================

test_that("vocabularies$get validates vocabulary_id", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- VocabulariesResource$new(base_req)

  expect_error(resource$get(""))
  expect_error(resource$get(123))
})

test_that("vocabularies$get calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- VocabulariesResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      mock_vocabulary()
    }
  )

  resource$get("SNOMED")

  expect_equal(called_with$path, "vocabularies/SNOMED")
  expect_null(called_with$query)
})

test_that("vocabularies$get only accepts vocabulary_id", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- VocabulariesResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      mock_vocabulary()
    }
  )

  resource$get("SNOMED")

  expect_equal(called_with$path, "vocabularies/SNOMED")
  expect_null(called_with$query)
})

# ==============================================================================
# stats() method
# ==============================================================================

test_that("vocabularies$stats validates vocabulary_id", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- VocabulariesResource$new(base_req)

  expect_error(resource$stats(""))
})

test_that("vocabularies$stats calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- VocabulariesResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path)
      list(total_concepts = 1000)
    }
  )

  resource$stats("SNOMED")

  expect_equal(called_with$path, "vocabularies/SNOMED/stats")
})

# ==============================================================================
# domains() method
# ==============================================================================

test_that("vocabularies$domains calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- VocabulariesResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      list(domains = list())
    }
  )

  resource$domains()

  expect_equal(called_with$path, "vocabularies/domains")
  expect_null(called_with$query)
})

# ==============================================================================
# concepts() method
# ==============================================================================

test_that("vocabularies$concepts validates vocabulary_id", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- VocabulariesResource$new(base_req)

  expect_error(resource$concepts(""))
})

test_that("vocabularies$concepts calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- VocabulariesResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      list(concepts = list())
    }
  )

  resource$concepts("SNOMED", page = 1, page_size = 20)

  expect_equal(called_with$path, "vocabularies/SNOMED/concepts")
  expect_equal(called_with$query$page, 1L)
  expect_equal(called_with$query$page_size, 20L)
  expect_equal(called_with$query$standard_concept, "all")
  expect_equal(called_with$query$sort_by, "name")
  expect_equal(called_with$query$sort_order, "asc")
})

test_that("vocabularies$concepts includes optional filters", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- VocabulariesResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(concepts = list())
    }
  )

  resource$concepts(
    "SNOMED",
    search = "diabetes",
    standard_concept = "S",
    include_invalid = TRUE,
    include_relationships = TRUE,
    include_synonyms = TRUE,
    sort_by = "concept_id",
    sort_order = "desc"
  )

  expect_equal(called_with$query$search, "diabetes")
  expect_equal(called_with$query$standard_concept, "S")
  expect_equal(called_with$query$include_invalid, "true")
  expect_equal(called_with$query$include_relationships, "true")
  expect_equal(called_with$query$include_synonyms, "true")
  expect_equal(called_with$query$sort_by, "concept_id")
  expect_equal(called_with$query$sort_order, "desc")
})

# Unit tests for MappingsResource (R/mappings.R)
# Uses mocked HTTP responses to avoid API calls

# ==============================================================================
# MappingsResource initialization
# ==============================================================================

test_that("MappingsResource initializes correctly", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- MappingsResource$new(base_req)

  expect_s3_class(resource, "MappingsResource")
  expect_s3_class(resource, "R6")
})

test_that("MappingsResource print method works", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- MappingsResource$new(base_req)

  expect_output(print(resource), "<OMOPHub MappingsResource>")
  expect_output(print(resource), "get, map")
})

# ==============================================================================
# get() method
# ==============================================================================

test_that("mappings$get validates concept_id", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- MappingsResource$new(base_req)

  expect_error(resource$get("invalid"))
  expect_error(resource$get(-1))
})

test_that("mappings$get calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- MappingsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      list(mappings = list())
    }
  )

  resource$get(201826)

  expect_equal(called_with$path, "concepts/201826/mappings")
})

test_that("mappings$get includes target vocabulary filter", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- MappingsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(mappings = list())
    }
  )

  resource$get(201826, target_vocabulary = "ICD10CM")

  expect_equal(called_with$query$target_vocabulary, "ICD10CM")
})

test_that("mappings$get includes include_invalid option", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- MappingsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(mappings = list())
    }
  )

  resource$get(201826, include_invalid = TRUE)

  expect_equal(called_with$query$include_invalid, "true")
})

test_that("mappings$get includes vocab_release option", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- MappingsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(mappings = list())
    }
  )

  resource$get(201826, vocab_release = "2025.1")

  expect_equal(called_with$query$vocab_release, "2025.1")
})

# ==============================================================================
# map() method
# ==============================================================================

test_that("mappings$map validates target_vocabulary", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- MappingsResource$new(base_req)

  expect_error(resource$map("", source_concepts = c(201826)))
  expect_error(resource$map(123, source_concepts = c(201826)))
})

test_that("mappings$map calls correct endpoint with body", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- MappingsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      called_with <<- list(path = path, body = body, query = query)
      list(mappings = list(), summary = list())
    }
  )

  resource$map("ICD10CM", source_concepts = c(201826, 12345))

  expect_equal(called_with$path, "concepts/map")
  expect_equal(called_with$body$source_concepts, c(201826L, 12345L))
  expect_equal(called_with$body$target_vocabulary, "ICD10CM")
})

test_that("mappings$map includes optional parameters", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- MappingsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      called_with <<- list(body = body, query = query)
      list(mappings = list())
    }
  )

  resource$map(
    "ICD10CM",
    source_concepts = c(201826),
    mapping_type = "equivalent",
    include_invalid = TRUE
  )

  expect_equal(called_with$body$mapping_type, "equivalent")
  expect_true(called_with$body$include_invalid)
})

# ==============================================================================
# map() with source_codes
# ==============================================================================

test_that("mappings$map works with source_codes", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- MappingsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      called_with <<- list(path = path, body = body)
      list(mappings = list())
    }
  )

  resource$map(
    target_vocabulary = "RxNorm",
    source_codes = list(
      list(vocabulary_id = "SNOMED", concept_code = "387517004"),
      list(vocabulary_id = "SNOMED", concept_code = "108774000")
    )
  )

  expect_equal(called_with$path, "concepts/map")
  expect_equal(called_with$body$target_vocabulary, "RxNorm")
  expect_true("source_codes" %in% names(called_with$body))
  expect_equal(length(called_with$body$source_codes), 2)
  expect_equal(called_with$body$source_codes[[1]]$vocabulary_id, "SNOMED")
})

test_that("mappings$map requires either source_concepts or source_codes", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- MappingsResource$new(base_req)

  expect_error(
    resource$map(target_vocabulary = "ICD10CM"),
    "Either source_concepts or source_codes is required"
  )
})

test_that("mappings$map rejects both source_concepts and source_codes", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- MappingsResource$new(base_req)

  expect_error(
    resource$map(
      target_vocabulary = "ICD10CM",
      source_concepts = c(201826),
      source_codes = list(list(vocabulary_id = "SNOMED", concept_code = "44054006"))
    ),
    "Cannot use both source_concepts and source_codes"
  )
})

test_that("mappings$map validates source_codes structure", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- MappingsResource$new(base_req)

  # Missing vocabulary_id
  expect_error(
    resource$map(
      target_vocabulary = "ICD10CM",
      source_codes = list(list(concept_code = "44054006"))
    ),
    "must have 'vocabulary_id' and 'concept_code'"
  )

  # Missing concept_code
  expect_error(
    resource$map(
      target_vocabulary = "ICD10CM",
      source_codes = list(list(vocabulary_id = "SNOMED"))
    ),
    "must have 'vocabulary_id' and 'concept_code'"
  )
})

#' Build Base OMOPHub Request
#'
#' @description
#' Creates an httr2 request object with authentication, rate limiting,
#' and retry logic configured.
#'
#' @param base_url API base URL.
#' @param api_key API key for authentication.
#' @param timeout Request timeout in seconds.
#' @param max_retries Maximum retry attempts.
#' @param vocab_version Optional vocabulary version.
#'
#' @returns An httr2 request object.
#' @keywords internal
build_request <- function(base_url, api_key, timeout = 30, max_retries = 3,
                          vocab_version = NULL) {
  req <- httr2::request(base_url) |>
    httr2::req_headers(
      "Authorization" = paste("Bearer", api_key),
      "Content-Type" = "application/json",
      "Accept" = "application/json"
    ) |>
    httr2::req_user_agent(paste0("OMOPHub-SDK-R/", utils::packageVersion("omophub"))) |>
    httr2::req_timeout(timeout) |>
    httr2::req_throttle(
      rate = .omophub_env$rate_limit_capacity / .omophub_env$rate_limit_fill_time
    ) |>
    httr2::req_retry(
      max_tries = max_retries,
      is_transient = is_transient_error
    ) |>
    httr2::req_error(body = extract_error_message)

  # Add vocabulary version header if specified
  if (!is.null(vocab_version)) {
    req <- httr2::req_headers(req, "X-Vocabulary-Version" = vocab_version)
  }

  req
}

#' Perform GET Request
#'
#' @param base_req Base request object.
#' @param endpoint API endpoint path.
#' @param query Named list of query parameters.
#'
#' @returns Parsed JSON response. For paginated endpoints, returns a list with
#'   `data` (the results) and `meta` (pagination info). For single-item endpoints,
#'   returns the unwrapped data directly.
#' @keywords internal
perform_get <- function(base_req, endpoint, query = NULL) {
  req <- base_req |>
    httr2::req_url_path_append(endpoint)

  # Add query parameters, removing NULLs
  if (!is.null(query)) {
    query <- Filter(Negate(is.null), query)
    if (length(query) > 0) {
      req <- httr2::req_url_query(req, !!!query)
    }
  }

  resp <- httr2::req_perform(req)
  body <- httr2::resp_body_json(resp)

  # Check if response has pagination info - if so, preserve structure for pagination
  if (is.list(body) && "meta" %in% names(body) && "data" %in% names(body)) {
    meta <- body$meta
    has_pagination <- !is.null(meta$pagination) ||
                      !is.null(meta$page) ||
                      !is.null(meta$total_pages) ||
                      !is.null(meta$has_next)

    if (has_pagination) {
      # Return with pagination structure preserved
      return(list(
        data = body$data,
        meta = meta$pagination %||% meta
      ))
    }
    # No pagination - unwrap to just data
    return(body$data)
  }

  body
}

#' Perform POST Request
#'
#' @param base_req Base request object.
#' @param endpoint API endpoint path.
#' @param body Named list for JSON body.
#' @param query Named list of query parameters.
#'
#' @returns Parsed JSON response (unwrapped from `data` field if present).
#' @keywords internal
perform_post <- function(base_req, endpoint, body = NULL, query = NULL) {
  req <- base_req |>
    httr2::req_url_path_append(endpoint) |>
    httr2::req_method("POST")

  # Add query parameters, removing NULLs
  if (!is.null(query)) {
    query <- Filter(Negate(is.null), query)
    if (length(query) > 0) {
      req <- httr2::req_url_query(req, !!!query)
    }
  }

  # Add JSON body
  if (!is.null(body)) {
    body <- Filter(Negate(is.null), body)
    req <- httr2::req_body_json(req, body)
  }

  resp <- httr2::req_perform(req)
  resp_body <- httr2::resp_body_json(resp)

  # Unwrap data field if present (matching Python SDK behavior)
  if (is.list(resp_body) && "data" %in% names(resp_body)) {
    return(resp_body$data)
  }
  resp_body
}

# Unit tests for request.R
# Tests HTTP request building and execution functions

# ==============================================================================
# build_request() tests
# ==============================================================================

test_that("build_request creates request with correct headers", {
  req <- omophub:::build_request(
    base_url = "https://api.omophub.com/v1",
    api_key = "test_api_key"
  )

  # Headers are stored in httr2_headers object
  # Convert to list to check values
  headers_list <- as.list(req$headers)

  # Check Authorization header (httr2 redacts it, so check it exists)
  expect_true("Authorization" %in% names(headers_list))

  # Check Content-Type header
  expect_equal(headers_list$`Content-Type`, "application/json")

  # Check Accept header
  expect_equal(headers_list$Accept, "application/json")
})

test_that("build_request sets User-Agent header", {
  req <- omophub:::build_request(
    base_url = "https://api.omophub.com/v1",
    api_key = "test_api_key"
  )

  # User-Agent is set via req_user_agent, stored in options$useragent
  expect_true("useragent" %in% names(req$options))
  expect_true(grepl("OMOPHub-SDK-R", req$options$useragent))
})

test_that("build_request adds vocab_version header when specified", {
  req <- omophub:::build_request(
    base_url = "https://api.omophub.com/v1",
    api_key = "test_api_key",
    vocab_version = "2025.1"
  )

  headers_list <- as.list(req$headers)
  expect_equal(headers_list$`X-Vocabulary-Version`, "2025.1")
})

test_that("build_request does not add vocab_version header when NULL", {
  req <- omophub:::build_request(
    base_url = "https://api.omophub.com/v1",
    api_key = "test_api_key",
    vocab_version = NULL
  )

  headers_list <- as.list(req$headers)
  expect_null(headers_list$`X-Vocabulary-Version`)
})

test_that("build_request configures timeout", {
  req <- omophub:::build_request(
    base_url = "https://api.omophub.com/v1",
    api_key = "test_api_key",
    timeout = 60
  )

  # Timeout is configured via req_timeout
  expect_true(req$options$timeout_ms == 60000 || is.null(req$options$timeout_ms))
})

# ==============================================================================
# perform_get() tests
# ==============================================================================

test_that("perform_get constructs correct endpoint path", {
  base_req <- httr2::request("https://api.omophub.com/v1")

  called_url <- NULL
  local_mocked_bindings(
    req_perform = function(req) {
      called_url <<- req$url
      structure(
        list(status_code = 200, body = charToRaw('{"data": {"test": true}}')),
        class = "httr2_response"
      )
    },
    resp_body_json = function(resp) list(data = list(test = TRUE)),
    .package = "httr2"
  )

  omophub:::perform_get(base_req, "concepts/201826")

  expect_true(grepl("concepts/201826", called_url))
})

test_that("perform_get filters NULL query parameters", {
  base_req <- httr2::request("https://api.omophub.com/v1")

  called_url <- NULL
  local_mocked_bindings(
    req_perform = function(req) {
      called_url <<- req$url
      structure(
        list(status_code = 200, body = charToRaw('{"test": true}')),
        class = "httr2_response"
      )
    },
    resp_body_json = function(resp) list(test = TRUE),
    .package = "httr2"
  )

  omophub:::perform_get(base_req, "test", query = list(a = "value", b = NULL, c = "other"))

  expect_true(grepl("a=value", called_url))
  expect_true(grepl("c=other", called_url))
  expect_false(grepl("b=", called_url))
})

test_that("perform_get handles empty query", {
  base_req <- httr2::request("https://api.omophub.com/v1")

  called_url <- NULL
  local_mocked_bindings(
    req_perform = function(req) {
      called_url <<- req$url
      structure(list(status_code = 200), class = "httr2_response")
    },
    resp_body_json = function(resp) list(result = TRUE),
    .package = "httr2"
  )

  omophub:::perform_get(base_req, "test", query = NULL)

  expect_false(grepl("\\?", called_url))
})

test_that("perform_get returns paginated response with meta", {
  base_req <- httr2::request("https://api.omophub.com/v1")

  local_mocked_bindings(
    req_perform = function(req) structure(list(status_code = 200), class = "httr2_response"),
    resp_body_json = function(resp) list(
      data = list(list(id = 1), list(id = 2)),
      meta = list(
        pagination = list(page = 1, page_size = 20, total_items = 100, has_next = TRUE)
      )
    ),
    .package = "httr2"
  )

  result <- omophub:::perform_get(base_req, "test")

  expect_true("data" %in% names(result))
  expect_true("meta" %in% names(result))
  expect_equal(result$meta$page, 1)
  expect_equal(result$meta$has_next, TRUE)
})

test_that("perform_get unwraps non-paginated data response", {
  base_req <- httr2::request("https://api.omophub.com/v1")

  local_mocked_bindings(
    req_perform = function(req) structure(list(status_code = 200), class = "httr2_response"),
    resp_body_json = function(resp) list(
      data = list(concept_id = 201826, concept_name = "Test"),
      meta = list(request_id = "abc123")
    ),
    .package = "httr2"
  )

  result <- omophub:::perform_get(base_req, "concepts/201826")

  # Should unwrap to just data since no pagination
  expect_equal(result$concept_id, 201826)
  expect_equal(result$concept_name, "Test")
})

test_that("perform_get returns body as-is when no data/meta structure", {
  base_req <- httr2::request("https://api.omophub.com/v1")

  local_mocked_bindings(
    req_perform = function(req) structure(list(status_code = 200), class = "httr2_response"),
    resp_body_json = function(resp) list(status = "ok", version = "1.0"),
    .package = "httr2"
  )

  result <- omophub:::perform_get(base_req, "health")

  expect_equal(result$status, "ok")
  expect_equal(result$version, "1.0")
})

# ==============================================================================
# perform_post() tests
# ==============================================================================

test_that("perform_post uses POST method", {
  base_req <- httr2::request("https://api.omophub.com/v1")

  called_method <- NULL
  local_mocked_bindings(
    req_perform = function(req) {
      called_method <<- req$method
      structure(list(status_code = 200), class = "httr2_response")
    },
    resp_body_json = function(resp) list(success = TRUE),
    .package = "httr2"
  )

  omophub:::perform_post(base_req, "concepts/batch", body = list(concept_ids = c(1, 2)))

  expect_equal(called_method, "POST")
})

test_that("perform_post constructs correct endpoint path", {
  base_req <- httr2::request("https://api.omophub.com/v1")

  called_url <- NULL
  local_mocked_bindings(
    req_perform = function(req) {
      called_url <<- req$url
      structure(list(status_code = 200), class = "httr2_response")
    },
    resp_body_json = function(resp) list(data = list()),
    .package = "httr2"
  )

  omophub:::perform_post(base_req, "concepts/batch")

  expect_true(grepl("concepts/batch", called_url))
})

test_that("perform_post handles NULL body", {
  base_req <- httr2::request("https://api.omophub.com/v1")

  local_mocked_bindings(
    req_perform = function(req) structure(list(status_code = 200), class = "httr2_response"),
    resp_body_json = function(resp) list(result = TRUE),
    .package = "httr2"
  )

  # Should not error
  result <- omophub:::perform_post(base_req, "test", body = NULL)
  expect_equal(result$result, TRUE)
})

test_that("perform_post filters NULL body fields", {
  base_req <- httr2::request("https://api.omophub.com/v1")

  captured_body <- NULL
  local_mocked_bindings(
    req_perform = function(req) {
      captured_body <<- req$body$data
      structure(list(status_code = 200), class = "httr2_response")
    },
    resp_body_json = function(resp) list(data = list()),
    .package = "httr2"
  )

  omophub:::perform_post(
    base_req,
    "test",
    body = list(a = "value", b = NULL, c = 123)
  )

  # Body should have a and c, but not b
  expect_true(!is.null(captured_body))
})

test_that("perform_post handles query parameters", {
  base_req <- httr2::request("https://api.omophub.com/v1")

  called_url <- NULL
  local_mocked_bindings(
    req_perform = function(req) {
      called_url <<- req$url
      structure(list(status_code = 200), class = "httr2_response")
    },
    resp_body_json = function(resp) list(data = list()),
    .package = "httr2"
  )

  omophub:::perform_post(
    base_req,
    "test",
    body = list(ids = c(1, 2)),
    query = list(format = "json")
  )

  expect_true(grepl("format=json", called_url))
})

test_that("perform_post unwraps data field from response", {
  base_req <- httr2::request("https://api.omophub.com/v1")

  local_mocked_bindings(
    req_perform = function(req) structure(list(status_code = 200), class = "httr2_response"),
    resp_body_json = function(resp) list(
      data = list(concepts = list(list(id = 1), list(id = 2)))
    ),
    .package = "httr2"
  )

  result <- omophub:::perform_post(base_req, "concepts/batch")

  expect_true("concepts" %in% names(result))
  expect_equal(length(result$concepts), 2)
})

test_that("perform_post returns body as-is when no data field", {
  base_req <- httr2::request("https://api.omophub.com/v1")

  local_mocked_bindings(
    req_perform = function(req) structure(list(status_code = 200), class = "httr2_response"),
    resp_body_json = function(resp) list(status = "created", id = 123),
    .package = "httr2"
  )

  result <- omophub:::perform_post(base_req, "test")

  expect_equal(result$status, "created")
  expect_equal(result$id, 123)
})

#' Hierarchy Resource
#'
#' @description
#' R6 class providing access to hierarchy operations (ancestors and descendants).
#'
#' @keywords internal
HierarchyResource <- R6::R6Class(
  "HierarchyResource",
  public = list(
    #' @description
    #' Create a new HierarchyResource.
    #' @param base_req Base httr2 request object.
    initialize = function(base_req) {
      private$.base_req <- base_req
    },

    #' @description
    #' Get complete concept hierarchy (ancestors and descendants).
    #'
    #' @param concept_id The concept ID.
    #' @param format Response format - "flat" (default) or "graph" for visualization.
    #' @param vocabulary_ids Filter to specific vocabularies (character vector).
    #' @param domain_ids Filter to specific domains (character vector).
    #' @param max_levels Maximum hierarchy levels to traverse in both directions (default 10).
    #' @param max_results Maximum results per direction for performance optimization.
    #' @param relationship_types Relationship types to follow (default: "Is a").
    #' @param include_invalid Include deprecated/invalid concepts. Default `FALSE`.
    #'
    #' @returns For flat format: ancestors, descendants arrays with level/total counts.
    #'          For graph format: nodes and edges arrays for visualization.
    get = function(concept_id,
                   format = "flat",
                   vocabulary_ids = NULL,
                   domain_ids = NULL,
                   max_levels = 10,
                   max_results = NULL,
                   relationship_types = NULL,
                   include_invalid = FALSE) {
      concept_id <- validate_concept_id(concept_id)

      params <- list(
        format = format,
        max_levels = min(as.integer(max_levels), 20L)
      )

      if (!is.null(vocabulary_ids)) {
        params$vocabulary_ids <- join_params(vocabulary_ids)
      }
      if (!is.null(domain_ids)) {
        params$domain_ids <- join_params(domain_ids)
      }
      if (!is.null(max_results)) {
        params$max_results <- as.integer(max_results)
      }
      if (!is.null(relationship_types)) {
        params$relationship_types <- join_params(relationship_types)
      }
      if (isTRUE(include_invalid)) {
        params$include_invalid <- "true"
      }

      perform_get(
        private$.base_req,
        paste0("concepts/", concept_id, "/hierarchy"),
        query = params
      )
    },

    #' @description
    #' Get concept ancestors.
    #'
    #' @param concept_id The concept ID.
    #' @param vocabulary_ids Filter to specific vocabularies (character vector).
    #' @param max_levels Maximum hierarchy levels to traverse.
    #' @param relationship_types Relationship types to follow (default: "Is a").
    #' @param include_paths Include path information. Default `FALSE`.
    #' @param include_distance Include distance from source. Default `TRUE`.
    #' @param include_invalid Include deprecated/invalid concepts. Default `FALSE`.
    #' @param page Page number. Default 1.
    #' @param page_size Results per page. Default 100.
    #'
    #' @returns Ancestors with hierarchy summary.
    ancestors = function(concept_id,
                         vocabulary_ids = NULL,
                         max_levels = NULL,
                         relationship_types = NULL,
                         include_paths = FALSE,
                         include_distance = TRUE,
                         include_invalid = FALSE,
                         page = 1,
                         page_size = 100) {
      concept_id <- validate_concept_id(concept_id)
      pag <- validate_pagination(page, page_size)

      params <- list(
        page = pag$page,
        page_size = pag$page_size
      )

      if (!is.null(vocabulary_ids)) {
        params$vocabulary_ids <- join_params(vocabulary_ids)
      }
      if (!is.null(max_levels)) {
        params$max_levels <- as.integer(max_levels)
      }
      if (!is.null(relationship_types)) {
        params$relationship_types <- join_params(relationship_types)
      }
      if (isTRUE(include_paths)) {
        params$include_paths <- "true"
      }
      if (isTRUE(include_distance)) {
        params$include_distance <- "true"
      }
      if (isTRUE(include_invalid)) {
        params$include_invalid <- "true"
      }

      perform_get(
        private$.base_req,
        paste0("concepts/", concept_id, "/ancestors"),
        query = params
      )
    },

    #' @description
    #' Get concept descendants.
    #'
    #' @param concept_id The concept ID.
    #' @param vocabulary_ids Filter to specific vocabularies (character vector).
    #' @param max_levels Maximum hierarchy levels (default 10, max 20).
    #' @param relationship_types Relationship types to follow.
    #' @param include_distance Include distance from source. Default `TRUE`.
    #' @param include_paths Include path information. Default `FALSE`.
    #' @param include_invalid Include deprecated/invalid concepts. Default `FALSE`.
    #' @param domain_ids Filter by domains.
    #' @param page Page number. Default 1.
    #' @param page_size Results per page. Default 100.
    #'
    #' @returns Descendants with hierarchy summary.
    descendants = function(concept_id,
                           vocabulary_ids = NULL,
                           max_levels = 10,
                           relationship_types = NULL,
                           include_distance = TRUE,
                           include_paths = FALSE,
                           include_invalid = FALSE,
                           domain_ids = NULL,
                           page = 1,
                           page_size = 100) {
      concept_id <- validate_concept_id(concept_id)
      pag <- validate_pagination(page, page_size)

      params <- list(
        max_levels = min(as.integer(max_levels), 20L),
        page = pag$page,
        page_size = pag$page_size
      )

      if (!is.null(vocabulary_ids)) {
        params$vocabulary_ids <- join_params(vocabulary_ids)
      }
      if (!is.null(relationship_types)) {
        params$relationship_types <- join_params(relationship_types)
      }
      if (isTRUE(include_distance)) {
        params$include_distance <- "true"
      }
      if (isTRUE(include_paths)) {
        params$include_paths <- "true"
      }
      if (isTRUE(include_invalid)) {
        params$include_invalid <- "true"
      }
      if (!is.null(domain_ids)) {
        params$domain_ids <- join_params(domain_ids)
      }

      perform_get(
        private$.base_req,
        paste0("concepts/", concept_id, "/descendants"),
        query = params
      )
    },

    #' @description
    #' Print resource information.
    print = function() {
      cat("<OMOPHub HierarchyResource>\n")
      cat("  Methods: get, ancestors, descendants\n")
      invisible(self)
    }
  ),
  private = list(
    .base_req = NULL
  )
)

# Unit tests for HierarchyResource (R/hierarchy.R)
# Uses mocked HTTP responses to avoid API calls

# ==============================================================================
# HierarchyResource initialization
# ==============================================================================

test_that("HierarchyResource initializes correctly", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  expect_s3_class(resource, "HierarchyResource")
  expect_s3_class(resource, "R6")
})

test_that("HierarchyResource print method works", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  expect_output(print(resource), "<OMOPHub HierarchyResource>")
  expect_output(print(resource), "ancestors, descendants")
})

# ==============================================================================
# ancestors() method
# ==============================================================================

test_that("hierarchy$ancestors validates concept_id", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  expect_error(resource$ancestors("invalid"))
  expect_error(resource$ancestors(-1))
})

test_that("hierarchy$ancestors calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      list(ancestors = list())
    }
  )

  resource$ancestors(201826, page = 1, page_size = 100)

  expect_equal(called_with$path, "concepts/201826/ancestors")
  expect_equal(called_with$query$page, 1L)
  expect_equal(called_with$query$page_size, 100L)
})

test_that("hierarchy$ancestors includes optional params", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(ancestors = list())
    }
  )

  resource$ancestors(
    201826,
    vocabulary_ids = c("SNOMED"),
    max_levels = 5,
    relationship_types = c("Is a", "Subsumes"),
    include_paths = TRUE,
    include_distance = TRUE,
    include_invalid = TRUE
  )

  expect_equal(called_with$query$vocabulary_ids, "SNOMED")
  expect_equal(called_with$query$max_levels, 5L)
  expect_equal(called_with$query$relationship_types, "Is a,Subsumes")
  expect_equal(called_with$query$include_paths, "true")
  expect_equal(called_with$query$include_distance, "true")
  expect_equal(called_with$query$include_invalid, "true")
})

# ==============================================================================
# descendants() method
# ==============================================================================

test_that("hierarchy$descendants validates concept_id", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  expect_error(resource$descendants("invalid"))
  expect_error(resource$descendants(0))
})

test_that("hierarchy$descendants calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      list(descendants = list())
    }
  )

  resource$descendants(201826, page = 1, page_size = 100)

  expect_equal(called_with$path, "concepts/201826/descendants")
  expect_equal(called_with$query$page, 1L)
  expect_equal(called_with$query$page_size, 100L)
})

test_that("hierarchy$descendants limits max_levels to 20", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(descendants = list())
    }
  )

  resource$descendants(201826, max_levels = 50)  # Request 50

  expect_equal(called_with$query$max_levels, 20L)  # Capped at 20
})

test_that("hierarchy$descendants includes optional params", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(descendants = list())
    }
  )

  resource$descendants(
    201826,
    vocabulary_ids = c("SNOMED"),
    max_levels = 5,
    relationship_types = c("Is a"),
    include_distance = TRUE,
    include_paths = TRUE,
    include_invalid = TRUE,
    domain_ids = c("Condition")
  )

  expect_equal(called_with$query$vocabulary_ids, "SNOMED")
  expect_equal(called_with$query$max_levels, 5L)
  expect_equal(called_with$query$relationship_types, "Is a")
  expect_equal(called_with$query$include_distance, "true")
  expect_equal(called_with$query$include_paths, "true")
  expect_equal(called_with$query$include_invalid, "true")
  expect_equal(called_with$query$domain_ids, "Condition")
})

# ==============================================================================
# get() method
# ==============================================================================

test_that("hierarchy$get validates concept_id", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  expect_error(resource$get("invalid"))
  expect_error(resource$get(-1))
  expect_error(resource$get(0))
})

test_that("hierarchy$get calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      list(ancestors = list(), descendants = list())
    }
  )

  resource$get(201826)

  expect_equal(called_with$path, "concepts/201826/hierarchy")
})

test_that("hierarchy$get includes format parameter", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list()
    }
  )

  resource$get(201826, format = "graph")

  expect_equal(called_with$query$format, "graph")
})

test_that("hierarchy$get defaults format to flat", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list()
    }
  )

  resource$get(201826)

  expect_equal(called_with$query$format, "flat")
})

test_that("hierarchy$get joins vocabulary_ids correctly", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list()
    }
  )

  resource$get(201826, vocabulary_ids = c("SNOMED", "ICD10CM"))

  expect_equal(called_with$query$vocabulary_ids, "SNOMED,ICD10CM")
})

test_that("hierarchy$get joins domain_ids correctly", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list()
    }
  )

  resource$get(201826, domain_ids = c("Condition", "Drug"))

  expect_equal(called_with$query$domain_ids, "Condition,Drug")
})

test_that("hierarchy$get caps max_levels at 20", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list()
    }
  )

  resource$get(201826, max_levels = 50)  # Request 50

  expect_equal(called_with$query$max_levels, 20L)  # Capped at 20
})

test_that("hierarchy$get includes max_results parameter", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list()
    }
  )

  resource$get(201826, max_results = 100)

  expect_equal(called_with$query$max_results, 100L)
})

test_that("hierarchy$get joins relationship_types correctly", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list()
    }
  )

  resource$get(201826, relationship_types = c("Is a", "Subsumes"))

  expect_equal(called_with$query$relationship_types, "Is a,Subsumes")
})

test_that("hierarchy$get adds include_invalid when TRUE", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list()
    }
  )

  resource$get(201826, include_invalid = TRUE)

  expect_equal(called_with$query$include_invalid, "true")
})

test_that("hierarchy$get does not add include_invalid when FALSE", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- HierarchyResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list()
    }
  )

  resource$get(201826, include_invalid = FALSE)

  expect_null(called_with$query$include_invalid)
})

#' Concepts Resource
#'
#' @description
#' R6 class providing access to concept operations.
#'
#' @keywords internal
ConceptsResource <- R6::R6Class(
  "ConceptsResource",
  public = list(
    #' @description
    #' Create a new ConceptsResource.
    #' @param base_req Base httr2 request object.
    initialize = function(base_req) {
      private$.base_req <- base_req
    },

    #' @description
    #' Get a concept by ID.
    #'
    #' @param concept_id The OMOP concept ID.
    #' @param include_relationships Include related concepts (parents/children). Default `FALSE`.
    #' @param include_synonyms Include concept synonyms. Default `FALSE`.
    #' @param include_hierarchy Include hierarchy information. Default `FALSE`.
    #' @param vocab_release Specific vocabulary release (e.g., "2025.2"). Default `NULL`.
    #'
    #' @returns A list containing the concept data.
    get = function(concept_id,
                   include_relationships = FALSE,
                   include_synonyms = FALSE,
                   include_hierarchy = FALSE,
                   vocab_release = NULL) {
      concept_id <- validate_concept_id(concept_id)

      query <- list()
      if (isTRUE(include_relationships)) {
        query$include_relationships <- "true"
      }
      if (isTRUE(include_synonyms)) {
        query$include_synonyms <- "true"
      }
      if (isTRUE(include_hierarchy)) {
        query$include_hierarchy <- "true"
      }
      if (!is.null(vocab_release)) {
        query$vocab_release <- vocab_release
      }

      perform_get(
        private$.base_req,
        paste0("concepts/", concept_id),
        query = if (length(query) > 0) query else NULL
      )
    },

    #' @description
    #' Get a concept by vocabulary and code.
    #'
    #' @param vocabulary_id The vocabulary ID (e.g., "SNOMED", "ICD10CM").
    #' @param concept_code The concept code within the vocabulary.
    #' @param include_relationships Include related concepts (parents/children). Default `FALSE`.
    #' @param include_synonyms Include concept synonyms. Default `FALSE`.
    #' @param include_hierarchy Include hierarchy information. Default `FALSE`.
    #' @param vocab_release Specific vocabulary release (e.g., "2025.2"). Default `NULL`.
    #'
    #' @returns A list containing the concept data with optional relationships and synonyms.
    get_by_code = function(vocabulary_id,
                           concept_code,
                           include_relationships = FALSE,
                           include_synonyms = FALSE,
                           include_hierarchy = FALSE,
                           vocab_release = NULL) {
      checkmate::assert_string(vocabulary_id, min.chars = 1)
      checkmate::assert_string(concept_code, min.chars = 1)

      query <- list()
      if (isTRUE(include_relationships)) {
        query$include_relationships <- "true"
      }
      if (isTRUE(include_synonyms)) {
        query$include_synonyms <- "true"
      }
      if (isTRUE(include_hierarchy)) {
        query$include_hierarchy <- "true"
      }
      if (!is.null(vocab_release)) {
        query$vocab_release <- vocab_release
      }

      perform_get(
        private$.base_req,
        paste0("concepts/by-code/", vocabulary_id, "/", concept_code),
        query = if (length(query) > 0) query else NULL
      )
    },

    #' @description
    #' Get multiple concepts by IDs.
    #'
    #' @param concept_ids Vector of concept IDs (max 100).
    #' @param include_relationships Include related concepts. Default `FALSE`.
    #' @param include_synonyms Include concept synonyms. Default `FALSE`.
    #' @param include_mappings Include concept mappings. Default `FALSE`.
    #' @param vocabulary_filter Filter results to specific vocabularies.
    #' @param standard_only Only return standard concepts. Default `TRUE`.
    #'
    #' @returns A list with `concepts` and any `failures`.
    batch = function(concept_ids,
                     include_relationships = FALSE,
                     include_synonyms = FALSE,
                     include_mappings = FALSE,
                     vocabulary_filter = NULL,
                     standard_only = TRUE) {
      checkmate::assert_integerish(concept_ids, min.len = 1, max.len = 100)

      body <- list(concept_ids = as.integer(concept_ids))

      if (isTRUE(include_relationships)) {
        body$include_relationships <- TRUE
      }
      if (isTRUE(include_synonyms)) {
        body$include_synonyms <- TRUE
      }
      if (isTRUE(include_mappings)) {
        body$include_mappings <- TRUE
      }
      if (!is.null(vocabulary_filter)) {
        body$vocabulary_filter <- as.character(vocabulary_filter)
      }
      if (isTRUE(standard_only)) {
        body$standard_only <- TRUE
      }

      perform_post(private$.base_req, "concepts/batch", body = body)
    },

    #' @description
    #' Get concept suggestions (autocomplete).
    #'
    #' @param query Search query (min 2 characters, max 100 characters).
    #' @param page Page number (default 1).
    #' @param page_size Number of suggestions per page (default 10, max 100).
    #' @param vocabulary_ids Filter to specific vocabularies (character vector).
    #' @param domain_ids Filter to specific domains (character vector).
    #' @param vocab_release Specific vocabulary release (e.g., "2025.2").
    #'
    #' @returns A list with suggestions and pagination metadata.
    suggest = function(query,
                       page = 1,
                       page_size = 10,
                       vocabulary_ids = NULL,
                       domain_ids = NULL,
                       vocab_release = NULL) {
      checkmate::assert_string(query, min.chars = 2, max.chars = 100)
      checkmate::assert_integerish(page, len = 1, lower = 1)
      checkmate::assert_integerish(page_size, len = 1, lower = 1, upper = 100)

      params <- list(
        query = query,
        page = as.integer(page),
        page_size = as.integer(page_size)
      )
      if (!is.null(vocabulary_ids)) {
        params$vocabulary_ids <- join_params(vocabulary_ids)
      }
      if (!is.null(domain_ids)) {
        params$domain_ids <- join_params(domain_ids)
      }
      if (!is.null(vocab_release)) {
        params$vocab_release <- vocab_release
      }

      perform_get(private$.base_req, "concepts/suggest", query = params)
    },

    #' @description
    #' Get related concepts.
    #'
    #' @param concept_id The source concept ID.
    #' @param relationship_types Filter by relationship types (e.g., c("Is a", "Maps to")).
    #' @param min_score Minimum relationship score (0.0-1.0).
    #' @param page_size Maximum number of results (default 20, max 100).
    #' @param vocab_release Specific vocabulary release (e.g., "2025.1").
    #'
    #' @returns Related concepts with relationship scores.
    related = function(concept_id,
                       relationship_types = NULL,
                       min_score = NULL,
                       page_size = 20,
                       vocab_release = NULL) {
      concept_id <- validate_concept_id(concept_id)
      checkmate::assert_integerish(page_size, len = 1, lower = 1, upper = 100)

      params <- list(page_size = as.integer(page_size))

      if (!is.null(relationship_types)) {
        params$relationship_types <- join_params(relationship_types)
      }
      if (!is.null(min_score)) {
        checkmate::assert_number(min_score, lower = 0, upper = 1)
        params$min_score <- min_score
      }
      if (!is.null(vocab_release)) {
        params$vocab_release <- vocab_release
      }

      perform_get(
        private$.base_req,
        paste0("concepts/", concept_id, "/related"),
        query = params
      )
    },

    #' @description
    #' Get concept relationships.
    #'
    #' @param concept_id The concept ID.
    #' @param relationship_ids Filter by relationship type IDs (character vector or comma-separated string).
    #' @param vocabulary_ids Filter by target vocabulary IDs (character vector or comma-separated string).
    #' @param domain_ids Filter by target domain IDs (character vector or comma-separated string).
    #' @param include_invalid Include relationships to invalid concepts. Default `FALSE`.
    #' @param standard_only Only include relationships to standard concepts. Default `FALSE`.
    #' @param include_reverse Include reverse relationships. Default `FALSE`.
    #' @param vocab_release Specific vocabulary release version. Default `NULL`.
    #'
    #' @returns Relationships data.
    relationships = function(concept_id,
                             relationship_ids = NULL,
                             vocabulary_ids = NULL,
                             domain_ids = NULL,
                             include_invalid = FALSE,
                             standard_only = FALSE,
                             include_reverse = FALSE,
                             vocab_release = NULL) {
      concept_id <- validate_concept_id(concept_id)

      params <- list()

      if (!is.null(relationship_ids)) {
        params$relationship_ids <- if (length(relationship_ids) > 1) {
          paste(relationship_ids, collapse = ",")
        } else {
          relationship_ids
        }
      }
      if (!is.null(vocabulary_ids)) {
        params$vocabulary_ids <- if (length(vocabulary_ids) > 1) {
          paste(vocabulary_ids, collapse = ",")
        } else {
          vocabulary_ids
        }
      }
      if (!is.null(domain_ids)) {
        params$domain_ids <- if (length(domain_ids) > 1) {
          paste(domain_ids, collapse = ",")
        } else {
          domain_ids
        }
      }
      if (isTRUE(include_invalid)) {
        params$include_invalid <- "true"
      }
      if (isTRUE(standard_only)) {
        params$standard_only <- "true"
      }
      if (isTRUE(include_reverse)) {
        params$include_reverse <- "true"
      }
      if (!is.null(vocab_release)) {
        params$vocab_release <- vocab_release
      }

      perform_get(
        private$.base_req,
        paste0("concepts/", concept_id, "/relationships"),
        query = if (length(params) > 0) params else NULL
      )
    },

    #' @description
    #' Get recommended concepts using OHDSI Phoebe algorithm.
    #'
    #' @param concept_ids Vector of source concept IDs (1-100).
    #' @param relationship_types Filter by relationship types (max 20).
    #' @param vocabulary_ids Filter to specific vocabularies (max 50).
    #' @param domain_ids Filter to specific domains (max 50).
    #' @param standard_only Only return standard concepts. Default `TRUE`.
    #' @param include_invalid Include invalid/deprecated concepts. Default `FALSE`.
    #' @param page Page number. Default 1.
    #' @param page_size Results per page (default 100, max 1000).
    #'
    #' @returns Recommendations grouped by source concept ID with pagination metadata.
    recommended = function(concept_ids,
                           relationship_types = NULL,
                           vocabulary_ids = NULL,
                           domain_ids = NULL,
                           standard_only = TRUE,
                           include_invalid = FALSE,
                           page = 1,
                           page_size = 100) {
      checkmate::assert_integerish(concept_ids, min.len = 1, max.len = 100)
      checkmate::assert_integerish(page, len = 1, lower = 1)
      checkmate::assert_integerish(page_size, len = 1, lower = 1, upper = 1000)

      body <- list(concept_ids = as.integer(concept_ids))

      if (!is.null(relationship_types)) {
        checkmate::assert_character(relationship_types, max.len = 20)
        body$relationship_types <- as.character(relationship_types)
      }
      if (!is.null(vocabulary_ids)) {
        checkmate::assert_character(vocabulary_ids, max.len = 50)
        body$vocabulary_ids <- as.character(vocabulary_ids)
      }
      if (!is.null(domain_ids)) {
        checkmate::assert_character(domain_ids, max.len = 50)
        body$domain_ids <- as.character(domain_ids)
      }

      body$standard_only <- isTRUE(standard_only)
      body$include_invalid <- isTRUE(include_invalid)
      body$page <- as.integer(page)
      body$page_size <- as.integer(page_size)

      perform_post(private$.base_req, "concepts/recommended", body = body)
    },

    #' @description
    #' Print resource information.
    print = function() {
      cat("<OMOPHub ConceptsResource>\n")
      cat("  Methods: get, get_by_code, batch, suggest, related, relationships, recommended\n")
      invisible(self)
    }
  ),
  private = list(
    .base_req = NULL
  )
)

# Integration tests for concepts resource
# Run with: testthat::test_file("tests/testthat/test-concepts-integration.R")

test_that("get real concept works", {
  skip_if_no_integration_key()
  client <- integration_client()

  concept <- client$concepts$get(DIABETES_CONCEPT_ID)

  expect_equal(concept$concept_id, DIABETES_CONCEPT_ID)
  expect_match(concept$concept_name, "Type 2 diabetes", ignore.case = TRUE)
  expect_equal(concept$vocabulary_id, "SNOMED")
  expect_equal(concept$domain_id, "Condition")
  expect_equal(concept$standard_concept, "S")
})

test_that("get concept with synonyms works", {
  skip_if_no_integration_key()
  client <- integration_client()

  concept <- client$concepts$get(
    DIABETES_CONCEPT_ID,
    include_synonyms = TRUE
  )

  expect_equal(concept$concept_id, DIABETES_CONCEPT_ID)
})

test_that("get concept by code works", {
  skip_if_no_integration_key()
  client <- integration_client()

  # SNOMED code for Type 2 diabetes mellitus
  concept <- client$concepts$get_by_code("SNOMED", "44054006")

  expect_equal(concept$concept_id, DIABETES_CONCEPT_ID)
  expect_equal(concept$concept_code, "44054006")
})

test_that("batch concepts works", {
  skip_if_no_integration_key()
  client <- integration_client()

  concept_ids <- c(DIABETES_CONCEPT_ID, ASPIRIN_CONCEPT_ID, MI_CONCEPT_ID)
  result <- client$concepts$batch(concept_ids)

  # Extract concepts from result
  concepts <- extract_data(result, "concepts")
  expect_true(is.list(concepts) || is.data.frame(concepts))
  expect_gte(length(concepts), 1)
})

test_that("suggest concepts works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$concepts$suggest("diabetes", page_size = 5)

  # Extract suggestions
  suggestions <- extract_data(result, "suggestions")
  expect_true(is.list(suggestions) || is.data.frame(suggestions))
  expect_gt(length(suggestions), 0)

  # Check at least one suggestion contains 'diabetes'
  suggestion_texts <- vapply(suggestions, function(s) {
    if (is.list(s)) {
      text <- s$concept_name %||% s$suggestion %||% ""
      tolower(as.character(text))
    } else {
      tolower(as.character(s))
    }
  }, character(1))
  expect_true(any(grepl("diabetes", suggestion_texts, ignore.case = TRUE)))
})

test_that("suggest concepts with filters works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$concepts$suggest(
    "aspirin",
    vocabulary_ids = "RxNorm",
    domain_ids = "Drug",
    page_size = 10
  )

  suggestions <- extract_data(result, "suggestions")
  expect_true(is.list(suggestions))
})

test_that("get related concepts works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$concepts$related(
    DIABETES_CONCEPT_ID,
    page_size = 10
  )

  related <- extract_data(result, "related_concepts")
  expect_true(is.list(related))
})

test_that("get concept relationships works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$concepts$relationships(DIABETES_CONCEPT_ID)

  relationships <- extract_data(result, "relationships")
  expect_true(is.list(relationships))
})

# Unit tests for ConceptsResource (R/concepts.R)
# Uses mocked HTTP responses to avoid API calls

# ==============================================================================
# ConceptsResource initialization
# ==============================================================================

test_that("ConceptsResource initializes correctly", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  expect_s3_class(resource, "ConceptsResource")
  expect_s3_class(resource, "R6")
})

test_that("ConceptsResource print method works", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  expect_output(print(resource), "<OMOPHub ConceptsResource>")
  expect_output(print(resource), "get, get_by_code, batch")
})

# ==============================================================================
# get() method
# ==============================================================================

test_that("concepts$get validates concept_id", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  expect_error(resource$get("not_a_number"), class = "omophub_validation_error")
  expect_error(resource$get(-1), class = "omophub_validation_error")
  expect_error(resource$get(0), class = "omophub_validation_error")
})

test_that("concepts$get calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  # Mock the perform_get function
  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      mock_concept()
    }
  )

  resource$get(201826)

  expect_equal(called_with$path, "concepts/201826")
  expect_null(called_with$query)
})

test_that("concepts$get adds query params for options", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      mock_concept()
    }
  )

  resource$get(201826, include_relationships = TRUE, include_synonyms = TRUE)

  expect_equal(called_with$query$include_relationships, "true")
  expect_equal(called_with$query$include_synonyms, "true")
})

test_that("concepts$get passes include_hierarchy and vocab_release", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      mock_concept()
    }
  )

  resource$get(201826, include_hierarchy = TRUE, vocab_release = "2025.2")

  expect_equal(called_with$query$include_hierarchy, "true")
  expect_equal(called_with$query$vocab_release, "2025.2")
})

# ==============================================================================
# get_by_code() method
# ==============================================================================

test_that("concepts$get_by_code validates arguments", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  expect_error(resource$get_by_code("", "12345"))
  expect_error(resource$get_by_code("SNOMED", ""))
  expect_error(resource$get_by_code(123, "12345"))
})

test_that("concepts$get_by_code calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path)
      mock_concept()
    }
  )

  resource$get_by_code("SNOMED", "44054006")

  expect_equal(called_with$path, "concepts/by-code/SNOMED/44054006")
})

test_that("concepts$get_by_code passes include_hierarchy and vocab_release", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      mock_concept()
    }
  )

  resource$get_by_code("SNOMED", "44054006",
    include_hierarchy = TRUE,
    vocab_release = "2025.2"
  )

  expect_equal(called_with$query$include_hierarchy, "true")
  expect_equal(called_with$query$vocab_release, "2025.2")
})

# ==============================================================================
# batch() method
# ==============================================================================

test_that("concepts$batch validates concept_ids", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  expect_error(resource$batch(c()))  # Empty
  expect_error(resource$batch("not_numbers"))  # Not integers
})

test_that("concepts$batch calls correct endpoint with body", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(path = path, body = body)
      list(concepts = list(mock_concept()))
    }
  )

  resource$batch(c(201826, 12345))

  expect_equal(called_with$path, "concepts/batch")
  expect_equal(called_with$body$concept_ids, c(201826L, 12345L))
})

test_that("concepts$batch includes optional parameters", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(body = body)
      list(concepts = list())
    }
  )

  resource$batch(
    c(201826),
    include_relationships = TRUE,
    include_synonyms = TRUE,
    include_mappings = TRUE,
    vocabulary_filter = c("SNOMED", "ICD10CM"),
    standard_only = TRUE
  )

  expect_true(called_with$body$include_relationships)
  expect_true(called_with$body$include_synonyms)
  expect_true(called_with$body$include_mappings)
  expect_equal(called_with$body$vocabulary_filter, c("SNOMED", "ICD10CM"))
  expect_true(called_with$body$standard_only)
})

# ==============================================================================
# suggest() method
# ==============================================================================

test_that("concepts$suggest validates query length", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  expect_error(resource$suggest("a"))  # Too short
  expect_error(resource$suggest(""))
})

test_that("concepts$suggest validates limit range", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) list()
  )

  expect_error(resource$suggest("diabetes", limit = 0))
  expect_error(resource$suggest("diabetes", limit = 100))
})

test_that("concepts$suggest calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      list(suggestions = list())
    }
  )

  resource$suggest("diab", vocabulary_ids = "SNOMED", domain_ids = "Condition", page_size = 5)

  expect_equal(called_with$path, "concepts/suggest")
  expect_equal(called_with$query$query, "diab")
  expect_equal(called_with$query$page_size, 5L)
  expect_equal(called_with$query$vocabulary_ids, "SNOMED")
  expect_equal(called_with$query$domain_ids, "Condition")
})

# ==============================================================================
# related() method
# ==============================================================================

test_that("concepts$related validates concept_id", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  expect_error(resource$related("invalid"))
})

test_that("concepts$related calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      list(related_concepts = list())
    }
  )

  resource$related(201826, page_size = 10)

  expect_equal(called_with$path, "concepts/201826/related")
  expect_equal(called_with$query$page_size, 10L)
})

test_that("concepts$related includes optional filters", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list()
    }
  )

  resource$related(
    201826,
    relationship_types = c("Is a", "Maps to"),
    min_score = 0.5,
    page_size = 50
  )

  expect_equal(called_with$query$relationship_types, "Is a,Maps to")
  expect_equal(called_with$query$min_score, 0.5)
  expect_equal(called_with$query$page_size, 50L)
})

# ==============================================================================
# relationships() method
# ==============================================================================

test_that("concepts$relationships validates concept_id", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  expect_error(resource$relationships("invalid"))
})

test_that("concepts$relationships calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      list(relationships = list())
    }
  )

  resource$relationships(201826)

  expect_equal(called_with$path, "concepts/201826/relationships")
})

test_that("concepts$relationships includes optional filters", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list()
    }
  )

  resource$relationships(
    201826,
    relationship_ids = "Is a",
    vocabulary_ids = "SNOMED",
    include_invalid = TRUE
  )

  expect_equal(called_with$query$relationship_ids, "Is a")
  expect_equal(called_with$query$vocabulary_ids, "SNOMED")
  expect_equal(called_with$query$include_invalid, "true")
})

test_that("concepts$suggest passes vocab_release", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list()
    }
  )

  resource$suggest("diabetes", vocab_release = "2025.2")

  expect_equal(called_with$query$vocab_release, "2025.2")
})

test_that("concepts$related passes vocab_release", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list()
    }
  )

  resource$related(201826, vocab_release = "2025.2")

  expect_equal(called_with$query$vocab_release, "2025.2")
})

test_that("concepts$relationships passes multi-element vectors and all flags", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list()
    }
  )

  resource$relationships(
    201826,
    relationship_ids = c("Is a", "Maps to"),
    vocabulary_ids = c("SNOMED", "ICD10CM"),
    domain_ids = c("Condition", "Drug"),
    standard_only = TRUE,
    include_reverse = TRUE,
    vocab_release = "2025.2"
  )

  expect_equal(called_with$query$relationship_ids, "Is a,Maps to")
  expect_equal(called_with$query$vocabulary_ids, "SNOMED,ICD10CM")
  expect_equal(called_with$query$domain_ids, "Condition,Drug")
  expect_equal(called_with$query$standard_only, "true")
  expect_equal(called_with$query$include_reverse, "true")
  expect_equal(called_with$query$vocab_release, "2025.2")
})

# ==============================================================================
# recommended() method
# ==============================================================================

test_that("concepts$recommended calls correct endpoint with body", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      called_with <<- list(path = path, body = body)
      list()
    }
  )

  resource$recommended(c(201826L, 4329847L))

  expect_equal(called_with$path, "concepts/recommended")
  expect_equal(called_with$body$concept_ids, c(201826L, 4329847L))
  expect_true(called_with$body$standard_only)
  expect_false(called_with$body$include_invalid)
  expect_equal(called_with$body$page, 1L)
  expect_equal(called_with$body$page_size, 100L)
})

test_that("concepts$recommended passes all optional filters", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- ConceptsResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL, query = NULL) {
      called_with <<- list(body = body)
      list()
    }
  )

  resource$recommended(
    c(201826L),
    relationship_types = c("Has finding", "Associated finding"),
    vocabulary_ids = c("SNOMED", "LOINC"),
    domain_ids = c("Condition"),
    standard_only = FALSE,
    include_invalid = TRUE,
    page = 2L,
    page_size = 50L
  )

  expect_equal(called_with$body$relationship_types, c("Has finding", "Associated finding"))
  expect_equal(called_with$body$vocabulary_ids, c("SNOMED", "LOINC"))
  expect_equal(called_with$body$domain_ids, c("Condition"))
  expect_false(called_with$body$standard_only)
  expect_true(called_with$body$include_invalid)
  expect_equal(called_with$body$page, 2L)
  expect_equal(called_with$body$page_size, 50L)
})

#' Mappings Resource
#'
#' @description
#' R6 class providing access to mapping operations.
#'
#' @keywords internal
MappingsResource <- R6::R6Class(
  "MappingsResource",
  public = list(
    #' @description
    #' Create a new MappingsResource.
    #' @param base_req Base httr2 request object.
    initialize = function(base_req) {
      private$.base_req <- base_req
    },

    #' @description
    #' Get mappings for a concept.
    #'
    #' @param concept_id The concept ID.
    #' @param target_vocabulary Filter to a specific target vocabulary (e.g., "ICD10CM").
    #' @param include_invalid Include invalid/deprecated mappings. Default `FALSE`.
    #' @param vocab_release Specific vocabulary release version (e.g., "2025.1"). Default `NULL`.
    #'
    #' @returns Mappings for the concept.
    get = function(concept_id,
                   target_vocabulary = NULL,
                   include_invalid = FALSE,
                   vocab_release = NULL) {
      concept_id <- validate_concept_id(concept_id)

      params <- list()

      if (!is.null(target_vocabulary)) {
        checkmate::assert_string(target_vocabulary, min.chars = 1)
        params$target_vocabulary <- target_vocabulary
      }
      if (isTRUE(include_invalid)) {
        params$include_invalid <- "true"
      }
      if (!is.null(vocab_release)) {
        checkmate::assert_string(vocab_release, min.chars = 1)
        params$vocab_release <- vocab_release
      }

      perform_get(
        private$.base_req,
        paste0("concepts/", concept_id, "/mappings"),
        query = if (length(params) > 0) params else NULL
      )
    },

    #' @description
    #' Map concepts to a target vocabulary.
    #'
    #' @param target_vocabulary Target vocabulary ID (e.g., "ICD10CM", "SNOMED", "RxNorm").
    #' @param source_concepts Vector of OMOP concept IDs to map. Use this OR source_codes, not both.
    #' @param source_codes List of vocabulary/code pairs to map. Each element should be a list
    #'   with `vocabulary_id` and `concept_code`. Use this OR source_concepts, not both.
    #' @param mapping_type Mapping type filter (direct, equivalent, broader, narrower).
    #' @param include_invalid Include invalid mappings. Default `FALSE`.
    #' @param vocab_release Specific vocabulary release version (e.g., "2025.1"). Default `NULL`.
    #'
    #' @returns Mapping results with summary.
    map = function(target_vocabulary,
                   source_concepts = NULL,
                   source_codes = NULL,
                   mapping_type = NULL,
                   include_invalid = FALSE,
                   vocab_release = NULL) {
      checkmate::assert_string(target_vocabulary, min.chars = 1)

      # Validate: exactly one of source_concepts or source_codes required
      has_concepts <- !is.null(source_concepts) && length(source_concepts) > 0
      has_codes <- !is.null(source_codes) && length(source_codes) > 0

      if (!has_concepts && !has_codes) {
        abort_validation("Either source_concepts or source_codes is required")
      }
      if (has_concepts && has_codes) {
        abort_validation("Cannot use both source_concepts and source_codes")
      }

      body <- list(target_vocabulary = target_vocabulary)

      if (has_concepts) {
        checkmate::assert_integerish(source_concepts, min.len = 1)
        body$source_concepts <- as.integer(source_concepts)
      }

      if (has_codes) {
        checkmate::assert_list(source_codes, min.len = 1)
        # Validate each code entry has required fields
        for (i in seq_along(source_codes)) {
          if (!all(c("vocabulary_id", "concept_code") %in% names(source_codes[[i]]))) {
            abort_validation(
              sprintf("source_codes[%d] must have 'vocabulary_id' and 'concept_code'", i)
            )
          }
        }
        body$source_codes <- source_codes
      }

      if (!is.null(mapping_type)) {
        body$mapping_type <- mapping_type
      }
      if (isTRUE(include_invalid)) {
        body$include_invalid <- TRUE
      }

      query <- list()
      if (!is.null(vocab_release)) {
        query$vocab_release <- vocab_release
      }

      perform_post(private$.base_req, "concepts/map", body = body, query = if (length(query) > 0) query else NULL)
    },

    #' @description
    #' Print resource information.
    print = function() {
      cat("<OMOPHub MappingsResource>\n")
      cat("  Methods: get, map\n")
      invisible(self)
    }
  ),
  private = list(
    .base_req = NULL
  )
)

# Test setup for omophub package

# Only set mock API key if no real key is present
# This allows integration tests to run when OMOPHUB_API_KEY is set externally
if (Sys.getenv("OMOPHUB_API_KEY") == "" && Sys.getenv("TEST_API_KEY") == "") {
  Sys.setenv(OMOPHUB_API_KEY = "test_api_key_for_mocking")
}

# Helper to skip tests that require real API
skip_if_no_auth <- function() {
  real_key <- Sys.getenv("OMOPHUB_API_KEY_REAL", unset = "")
  if (real_key == "" || real_key == "test_api_key_for_mocking") {
    testthat::skip("Real OMOPHUB_API_KEY not available")
  }
}

# Skip on CRAN and CI for integration tests
skip_on_cran_and_ci <- function() {
  testthat::skip_on_cran()
  if (Sys.getenv("CI") != "") {
    testthat::skip("Skipping on CI")
  }
}

#' Fetch All Paginated Results
#'
#' @description
#' Fetches all pages of results and combines them into a single tibble.
#'
#' @param fetch_fn A function that takes `page` and `size` arguments and returns
#'   a list with `data` (the results) and `meta` (pagination metadata).
#' @param page_size Number of items per page. Default 100.
#' @param max_pages Maximum number of pages to fetch. Default `Inf`.
#' @param progress Show progress bar. Default `TRUE`.
#'
#' @returns A tibble containing all fetched results.
#'
#' @keywords internal
paginate_all <- function(fetch_fn, page_size = 100, max_pages = Inf, progress = TRUE) {
  all_data <- list()
  page <- 1
  has_more <- TRUE

  # Progress bar setup
  pb <- NULL
  if (progress && interactive()) {
    cli::cli_progress_bar(
      "Fetching pages",
      total = NA,
      format = "{cli::pb_spin} Fetching page {page}... ({length(all_data)} items)"
    )
  }

  while (has_more && page <= max_pages) {
    result <- fetch_fn(page, page_size)

    data <- result$data
    meta <- result$meta

    if (length(data) == 0) {
      break
    }

    all_data <- c(all_data, data)

    if (progress && interactive()) {
      cli::cli_progress_update()
    }

    # Check if there are more pages
    if (!is.null(meta$has_next)) {
      has_more <- isTRUE(meta$has_next)
    } else if (!is.null(meta$total_pages)) {
      has_more <- page < meta$total_pages
    } else {
      # If no pagination info, check if we got fewer results than requested
      has_more <- length(data) >= page_size
    }

    page <- page + 1
  }

  if (progress && interactive()) {
    cli::cli_progress_done()
  }

  # Convert list of lists to tibble
  if (length(all_data) == 0) {
    return(tibble::tibble())
  }

  # Try to bind rows - all_data is a list of concept lists
  tryCatch(
    {
      # Convert each concept to a tibble row and bind them
      tibble_list <- purrr::map(all_data, function(item) {
        if (is.data.frame(item)) {
          tibble::as_tibble(item)
        } else if (is.list(item) && !is.null(names(item))) {
          # Named list - convert NULL values to NA for proper tibble conversion
          item <- purrr::map(item, ~ if (is.null(.x)) NA else .x)
          tibble::as_tibble(item)
        } else {
          tibble::tibble(value = list(item))
        }
      })
      purrr::list_rbind(tibble_list)
    },
    error = function(e) {
      # If list_rbind fails, return list as-is with warning
      warning("Could not convert results to tibble: ", e$message)
      all_data
    }
  )
}

#' Create Lazy Pagination Iterator
#'
#' @description
#' Creates an iterator that fetches pages on demand.
#'
#' @param fetch_fn A function that takes `page` and `size` arguments.
#' @param page_size Number of items per page. Default 100.
#'
#' @returns An iterator object with `next_page()` and `has_more()` methods.
#'
#' @keywords internal
paginate_lazy <- function(fetch_fn, page_size = 100) {
  env <- new.env(parent = emptyenv())
  env$current_page <- 0
  env$has_more <- TRUE
  env$last_result <- NULL

  list(
    #' Get next page of results
    next_page = function() {
      if (!env$has_more) {
        return(NULL)
      }

      env$current_page <- env$current_page + 1
      result <- fetch_fn(env$current_page, page_size)

      env$last_result <- result

      # Check if there are more pages
      meta <- result$meta
      if (!is.null(meta$has_next)) {
        env$has_more <- isTRUE(meta$has_next)
      } else if (!is.null(meta$total_pages)) {
        env$has_more <- env$current_page < meta$total_pages
      } else {
        env$has_more <- length(result$data) >= page_size
      }

      result$data
    },

    #' Check if more pages are available
    has_more = function() {
      env$has_more
    },

    #' Get current page number
    current_page = function() {
      env$current_page
    },

    #' Reset iterator to beginning
    reset = function() {
      env$current_page <- 0
      env$has_more <- TRUE
      env$last_result <- NULL
      invisible(NULL)
    }
  )
}

#' Extract Pagination Metadata
#'
#' @description
#' Extracts pagination metadata from an API response.
#'
#' @param response API response object.
#'
#' @returns A list with pagination fields.
#'
#' @keywords internal
extract_pagination <- function(response) {
  meta <- response$meta$pagination %||% response$meta %||% list()

  list(
    page = meta$page %||% 1L,
    page_size = meta$page_size %||% meta$per_page %||% 20L,
    total_items = meta$total_items %||% meta$total %||% NA_integer_,
    total_pages = meta$total_pages %||% NA_integer_,
    has_next = meta$has_next %||% NA,
    has_previous = meta$has_previous %||% NA
  )
}

# Tests for pagination utilities (R/pagination.R)

# ==============================================================================
# extract_pagination
# ==============================================================================

test_that("extract_pagination extracts nested pagination metadata", {
  response <- list(
    meta = list(
      pagination = list(
        page = 2,
        page_size = 50,
        total_items = 200,
        total_pages = 4,
        has_next = TRUE,
        has_previous = TRUE
      )
    )
  )

  result <- extract_pagination(response)

  expect_equal(result$page, 2)
  expect_equal(result$page_size, 50)

  expect_equal(result$total_items, 200)
  expect_equal(result$total_pages, 4)
  expect_true(result$has_next)
  expect_true(result$has_previous)
})

test_that("extract_pagination extracts flat meta format", {
  response <- list(
    meta = list(
      page = 1,
      page_size = 20,
      total_items = 50,
      has_next = FALSE
    )
  )

  result <- extract_pagination(response)

  expect_equal(result$page, 1)
  expect_equal(result$page_size, 20)
  expect_equal(result$total_items, 50)
  expect_false(result$has_next)
})

test_that("extract_pagination uses defaults for missing fields", {
  response <- list(meta = list())

  result <- extract_pagination(response)

  expect_equal(result$page, 1L)
  expect_equal(result$page_size, 20L)
  expect_true(is.na(result$total_items))
  expect_true(is.na(result$total_pages))
  expect_true(is.na(result$has_next))
  expect_true(is.na(result$has_previous))
})

test_that("extract_pagination handles NULL response", {
  result <- extract_pagination(list())

  expect_equal(result$page, 1L)
  expect_equal(result$page_size, 20L)
})

test_that("extract_pagination handles per_page alias", {
  response <- list(
    meta = list(
      page = 1,
      per_page = 25
    )
  )

  result <- extract_pagination(response)

  expect_equal(result$page_size, 25)
})

test_that("extract_pagination handles total alias", {
  response <- list(
    meta = list(
      total = 150
    )
  )

  result <- extract_pagination(response)

  expect_equal(result$total_items, 150)
})

# ==============================================================================
# paginate_lazy
# ==============================================================================

test_that("paginate_lazy creates iterator with correct methods", {
  fetch_fn <- function(page, size) {
    list(
      data = list(list(id = page)),
      meta = list(has_next = page < 3)
    )
  }

  iterator <- paginate_lazy(fetch_fn, page_size = 10)

  expect_type(iterator, "list")
  expect_true("next_page" %in% names(iterator))
  expect_true("has_more" %in% names(iterator))
  expect_true("current_page" %in% names(iterator))
  expect_true("reset" %in% names(iterator))
})

test_that("paginate_lazy starts at page 0", {
  fetch_fn <- function(page, size) list(data = list(), meta = list(has_next = FALSE))

  iterator <- paginate_lazy(fetch_fn)

  expect_equal(iterator$current_page(), 0)
  expect_true(iterator$has_more())
})

test_that("paginate_lazy next_page fetches sequential pages", {
  fetch_fn <- function(page, size) {
    list(
      data = list(list(id = page)),
      meta = list(has_next = page < 3)
    )
  }

  iterator <- paginate_lazy(fetch_fn, page_size = 10)

  result1 <- iterator$next_page()
  expect_equal(iterator$current_page(), 1)
  expect_equal(result1[[1]]$id, 1)
  expect_true(iterator$has_more())

  result2 <- iterator$next_page()
  expect_equal(iterator$current_page(), 2)
  expect_equal(result2[[1]]$id, 2)
  expect_true(iterator$has_more())

  result3 <- iterator$next_page()
  expect_equal(iterator$current_page(), 3)
  expect_equal(result3[[1]]$id, 3)
  expect_false(iterator$has_more())
})

test_that("paginate_lazy returns NULL when no more pages", {
  fetch_fn <- function(page, size) {
    list(
      data = list(list(id = page)),
      meta = list(has_next = FALSE)
    )
  }

  iterator <- paginate_lazy(fetch_fn)

  # First page
  iterator$next_page()
  expect_false(iterator$has_more())

  # Should return NULL when no more pages
  result <- iterator$next_page()
  expect_null(result)
})

test_that("paginate_lazy reset works correctly", {
  call_count <- 0
  fetch_fn <- function(page, size) {
    call_count <<- call_count + 1
    list(
      data = list(list(id = page)),
      meta = list(has_next = page < 2)
    )
  }

  iterator <- paginate_lazy(fetch_fn)

  # Fetch first two pages
  iterator$next_page()
  iterator$next_page()
  expect_equal(iterator$current_page(), 2)
  expect_false(iterator$has_more())

  # Reset
  iterator$reset()
  expect_equal(iterator$current_page(), 0)
  expect_true(iterator$has_more())

  # Fetch again
  result <- iterator$next_page()
  expect_equal(result[[1]]$id, 1)
})

test_that("paginate_lazy uses total_pages when has_next not available", {
  fetch_fn <- function(page, size) {
    list(
      data = list(list(id = page)),
      meta = list(total_pages = 2)
    )
  }

  iterator <- paginate_lazy(fetch_fn)

  iterator$next_page()
  expect_true(iterator$has_more())

  iterator$next_page()
  expect_false(iterator$has_more())
})

test_that("paginate_lazy uses data length when no pagination info", {
  fetch_fn <- function(page, size) {
    # Return fewer items than page_size to signal end
    if (page == 1) {
      list(data = as.list(1:10), meta = list())
    } else {
      list(data = as.list(1:5), meta = list())  # Less than page_size
    }
  }

  iterator <- paginate_lazy(fetch_fn, page_size = 10)

  iterator$next_page()
  expect_true(iterator$has_more())  # Got 10 items = page_size

  iterator$next_page()
  expect_false(iterator$has_more())  # Got 5 items < page_size
})

# ==============================================================================
# paginate_all
# ==============================================================================

test_that("paginate_all fetches all pages and combines to tibble", {
  fetch_fn <- function(page, size) {
    if (page <= 3) {
      list(
        data = list(
          list(id = (page - 1) * 2 + 1, name = paste0("Item", (page - 1) * 2 + 1)),
          list(id = (page - 1) * 2 + 2, name = paste0("Item", (page - 1) * 2 + 2))
        ),
        meta = list(has_next = page < 3)
      )
    } else {
      list(data = list(), meta = list(has_next = FALSE))
    }
  }

  result <- paginate_all(fetch_fn, page_size = 2, progress = FALSE)

  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 6)  # 3 pages * 2 items
  expect_true("id" %in% names(result))
  expect_true("name" %in% names(result))
})

test_that("paginate_all respects max_pages limit", {
  fetch_fn <- function(page, size) {
    list(
      data = list(list(id = page)),
      meta = list(has_next = TRUE)  # Always has more
    )
  }

  result <- paginate_all(fetch_fn, page_size = 1, max_pages = 2, progress = FALSE)

  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 2)  # Limited to 2 pages
})

test_that("paginate_all returns empty tibble for no results", {
  fetch_fn <- function(page, size) {
    list(data = list(), meta = list(has_next = FALSE))
  }

  result <- paginate_all(fetch_fn, progress = FALSE)

  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 0)
})

test_that("paginate_all handles single page result", {
  fetch_fn <- function(page, size) {
    list(
      data = list(list(id = 1, name = "Only item")),
      meta = list(has_next = FALSE)
    )
  }

  result <- paginate_all(fetch_fn, progress = FALSE)

  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 1)
})

test_that("paginate_all stops when data is empty", {
  call_count <- 0
  fetch_fn <- function(page, size) {
    call_count <<- call_count + 1
    if (page == 1) {
      list(
        data = list(list(id = 1)),
        meta = list(has_next = TRUE)  # Says there's more
      )
    } else {
      list(data = list(), meta = list(has_next = TRUE))  # But returns empty
    }
  }

  result <- paginate_all(fetch_fn, progress = FALSE)

  expect_equal(nrow(result), 1)
  expect_equal(call_count, 2)  # Called twice, stopped on empty
})

test_that("paginate_all handles data frames in results", {
  fetch_fn <- function(page, size) {
    list(
      data = list(
        data.frame(id = page, name = paste0("Item", page))
      ),
      meta = list(has_next = page < 2)
    )
  }

  result <- paginate_all(fetch_fn, progress = FALSE)

  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 2)
})

test_that("paginate_all uses total_pages when has_next not available", {
  fetch_fn <- function(page, size) {
    list(
      data = list(list(id = page)),
      meta = list(total_pages = 3)
    )
  }

  result <- paginate_all(fetch_fn, progress = FALSE)

  expect_equal(nrow(result), 3)
})

test_that("paginate_all handles NULL values in data", {
  fetch_fn <- function(page, size) {
    list(
      data = list(list(id = page, name = NULL, value = "test")),
      meta = list(has_next = page < 2)
    )
  }

  result <- paginate_all(fetch_fn, progress = FALSE)

  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 2)
  # NULL values should become NA
  expect_true(all(is.na(result$name)))
})

# Integration test helpers and constants for omophub package
# These are loaded automatically by testthat before tests run

# Import null coalescing operator from rlang for test use
`%||%` <- rlang::`%||%`

# Well-known test concept IDs for integration tests
DIABETES_CONCEPT_ID <- 201826L    # Type 2 diabetes mellitus (SNOMED)
ASPIRIN_CONCEPT_ID <- 1112807L    # Aspirin (RxNorm)
MI_CONCEPT_ID <- 4329847L         # Myocardial infarction (SNOMED)
HYPERTENSION_CONCEPT_ID <- 316866L  # Hypertensive disorder (SNOMED)
DIABETES_PARENT_ID <- 201820L     # Diabetes mellitus (parent of Type 2)

#' Skip if integration API key is not available
#'
#' Checks for OMOPHUB_API_KEY or TEST_API_KEY in environment.
#' Skips the test if neither is set.
#'
#' @keywords internal
skip_if_no_integration_key <- function() {
  key <- get_integration_api_key()
  if (is.null(key) || key == "" || key == "test_api_key_for_mocking") {
    testthat::skip("Integration API key not available (set OMOPHUB_API_KEY or TEST_API_KEY)")
  }
}

#' Get API key for integration tests
#'
#' Checks TEST_API_KEY first, then OMOPHUB_API_KEY.
#'
#' @returns API key string or NULL if not set
#' @keywords internal
get_integration_api_key <- function() {
  # Check TEST_API_KEY first (explicit test key), then OMOPHUB_API_KEY
  key <- Sys.getenv("TEST_API_KEY", unset = "")
  if (key != "") return(key)

  key <- Sys.getenv("OMOPHUB_API_KEY", unset = "")
  if (key != "" && key != "test_api_key_for_mocking") return(key)

  NULL
}

#' Create an integration test client
#'
#' Creates an OMOPHubClient configured for integration tests.
#' Skips the test if no API key is available.
#'
#' @returns An OMOPHubClient object
#' @keywords internal
integration_client <- function() {
  skip_if_no_integration_key()
  key <- get_integration_api_key()
  Sys.sleep(1)  # Rate limit delay between tests
  OMOPHubClient$new(api_key = key)
}

#' Safely extract data from API response
#'
#' Handles both old format (direct array) and new format (wrapped in named key).
#'
#' @param result API response, either a list or named list with data
#' @param key The key to extract (e.g., "concepts", "vocabularies", "suggestions")
#' @returns The extracted list of items, always a list
#' @keywords internal
extract_data <- function(result, key) {
  if (is.null(result)) return(list())

  # If result is already a data.frame/tibble, return it as-is
  if (is.data.frame(result)) return(result)

  # If it's a list with the key, extract it
  if (is.list(result) && !is.data.frame(result)) {
    # Check for the requested key
    if (key %in% names(result)) {
      return(result[[key]])
    }

    # Fallback: check 'results' key for batch endpoint backward compatibility
    if (key == "concepts" && "results" %in% names(result)) {
      return(result[["results"]])
    }

    # If it's an unnamed list (array), return as-is
    if (is.null(names(result)) || all(names(result) == "")) {
      return(result)
    }

    # Check if the result has a 'data' wrapper
    if ("data" %in% names(result)) {
      data <- result[["data"]]
      if (is.list(data) && key %in% names(data)) {
        return(data[[key]])
      }
      return(data)
    }
  }

  # Return empty list if extraction failed
  list()
}

#' Load .env file if it exists
#'
#' Attempts to load environment variables from .env file.
#'
#' @keywords internal
load_env_file <- function() {
  env_paths <- c(
    ".env",
    "../.env",
    "../../.env",
    file.path(testthat::test_path(), "..", "..", ".env")
  )

  for (path in env_paths) {
    if (file.exists(path)) {
      lines <- readLines(path, warn = FALSE)
      for (line in lines) {
        # Skip comments and empty lines
        if (grepl("^\\s*#", line) || grepl("^\\s*$", line)) next
        # Parse KEY=VALUE
        parts <- strsplit(line, "=", fixed = TRUE)[[1]]
        if (length(parts) >= 2) {
          key <- trimws(parts[1])
          value <- trimws(paste(parts[-1], collapse = "="))
          # Remove quotes if present
          value <- gsub("^['\"]|['\"]$", "", value)
          if (Sys.getenv(key) == "") {
            Sys.setenv(key = value)
            do.call(Sys.setenv, stats::setNames(list(value), key))
          }
        }
      }
      break
    }
  }
}

# Load .env file on helper load
load_env_file()

# Integration tests for mappings resource
# Run with: testthat::test_file("tests/testthat/test-mappings-integration.R")

test_that("get mappings works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$mappings$get(DIABETES_CONCEPT_ID)

  mappings <- extract_data(result, "mappings")
  expect_true(is.list(mappings))
})

test_that("get mappings to ICD-10 works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$mappings$get(
    DIABETES_CONCEPT_ID,
    target_vocabulary = "ICD10CM"
  )

  mappings <- extract_data(result, "mappings")
  expect_true(is.list(mappings))
})

test_that("get mappings with options works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$mappings$get(
    DIABETES_CONCEPT_ID,
    include_invalid = TRUE
  )

  mappings <- extract_data(result, "mappings")
  expect_true(is.list(mappings))
})

test_that("map concepts batch works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$mappings$map(
    target_vocabulary = "ICD10CM",
    source_concepts = c(DIABETES_CONCEPT_ID, MI_CONCEPT_ID)
  )

  # Should get some result structure
  mappings <- extract_data(result, "mappings")
  expect_true(is.list(mappings))
})

# ==============================================================================
# map() with source_codes integration tests
# ==============================================================================

test_that("map concepts with source_codes works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$mappings$map(
    target_vocabulary = "RxNorm",
    source_codes = list(
      list(vocabulary_id = "SNOMED", concept_code = "387517004"),
      list(vocabulary_id = "SNOMED", concept_code = "108774000")
    )
  )

  mappings <- extract_data(result, "mappings")
  expect_true(is.list(mappings))
})

test_that("map concepts with source_codes and mapping_type works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$mappings$map(
    target_vocabulary = "RxNorm",
    source_codes = list(
      list(vocabulary_id = "SNOMED", concept_code = "387517004")
    ),
    mapping_type = "equivalent"
  )

  mappings <- extract_data(result, "mappings")
  expect_true(is.list(mappings))
})

test_that("map concepts with invalid source_codes returns empty", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$mappings$map(
    target_vocabulary = "RxNorm",
    source_codes = list(
      list(vocabulary_id = "SNOMED", concept_code = "INVALID_CODE_12345")
    )
  )

  mappings <- extract_data(result, "mappings")
  expect_true(is.list(mappings))
  expect_equal(length(mappings), 0)
})

# Integration tests for domains resource
# Run with: testthat::test_file("tests/testthat/test-domains-integration.R")

test_that("list domains works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$domains$list()

  domains <- extract_data(result, "domains")
  expect_gt(length(domains), 0)

  # Check for common domains
  domain_ids <- vapply(domains, function(d) d$domain_id %||% "", character(1))
  expect_true("Condition" %in% domain_ids)
  expect_true("Drug" %in% domain_ids)
  expect_true("Procedure" %in% domain_ids)
})

test_that("list domains with stats works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$domains$list(include_stats = TRUE)

  domains <- extract_data(result, "domains")
  expect_gt(length(domains), 0)
  # Verify domains are returned with expected structure including stats
  for (domain in domains) {
    expect_true("domain_id" %in% names(domain))
    expect_true("concept_count" %in% names(domain))
    expect_true("standard_concept_count" %in% names(domain))
  }
})

test_that("get domain concepts works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$domains$concepts(
    "Condition",
    vocabulary_ids = "SNOMED",
    standard_only = TRUE,
    page_size = 20
  )

  concepts <- extract_data(result, "concepts")
  expect_true(is.list(concepts))
  # All concepts should be in Condition domain
  for (concept in concepts) {
    expect_equal(concept$domain_id, "Condition")
  }
})

test_that("get drug domain concepts works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$domains$concepts(
    "Drug",
    vocabulary_ids = "RxNorm",
    standard_only = TRUE,
    page_size = 10
  )

  concepts <- extract_data(result, "concepts")
  expect_true(is.list(concepts))
  # Verify all concepts are in Drug domain
  for (concept in concepts) {
    expect_equal(concept$domain_id, "Drug")
  }
})

# Integration tests for relationships resource
# Run with: testthat::test_file("tests/testthat/test-relationships-integration.R")

test_that("get relationships works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$relationships$get(DIABETES_CONCEPT_ID)

  relationships <- extract_data(result, "relationships")
  expect_true(is.list(relationships))
})

test_that("get relationships with type filter works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$relationships$get(
    DIABETES_CONCEPT_ID,
    relationship_ids = "Is a",
    page_size = 50
  )

  relationships <- extract_data(result, "relationships")
  expect_true(is.list(relationships))
})

test_that("get relationships with vocabulary filter works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$relationships$get(
    DIABETES_CONCEPT_ID,
    vocabulary_ids = "SNOMED",
    page_size = 100
  )

  relationships <- extract_data(result, "relationships")
  expect_true(is.list(relationships))
})

test_that("get relationship types works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$relationships$types()

  types <- extract_data(result, "relationship_types")
  expect_true(is.list(types))
  expect_gt(length(types), 0)
})

test_that("get relationship types with filters works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$relationships$types(
    page_size = 50
  )

  types <- extract_data(result, "relationship_types")
  expect_true(is.list(types))
})

test_that("get relationship types by category works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$relationships$types()

  types <- extract_data(result, "relationship_types")
  expect_true(is.list(types))
})

# Integration tests for hierarchy resource
# Run with: testthat::test_file("tests/testthat/test-hierarchy-integration.R")

test_that("get ancestors works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$hierarchy$ancestors(
    DIABETES_CONCEPT_ID,
    max_levels = 3
  )

  ancestors <- extract_data(result, "ancestors")
  expect_true(is.list(ancestors))
  # Type 2 diabetes in SNOMED should have ancestors (parent concepts)
  # Empty list is acceptable if concept has no hierarchy
})

test_that("get ancestors with options works", {

  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$hierarchy$ancestors(
    DIABETES_CONCEPT_ID,
    vocabulary_ids = c("SNOMED"),
    max_levels = 5,
    include_distance = TRUE,
    page_size = 50
  )

  ancestors <- extract_data(result, "ancestors")
  expect_true(is.list(ancestors))

  # Verify hierarchy_summary structure if present
  if ("hierarchy_summary" %in% names(result)) {
    expect_true(
      "max_hierarchy_depth" %in% names(result$hierarchy_summary) ||
      "total_ancestors" %in% names(result$hierarchy_summary)
    )
  }
})

test_that("get descendants works", {
  skip_if_no_integration_key()
  client <- integration_client()

  # 201820 is Diabetes mellitus (parent of Type 2)
  result <- client$hierarchy$descendants(
    DIABETES_PARENT_ID,
    max_levels = 2
  )

  descendants <- extract_data(result, "descendants")
  expect_true(is.list(descendants))
})

test_that("get descendants with filters works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$hierarchy$descendants(
    DIABETES_PARENT_ID,
    vocabulary_ids = c("SNOMED"),
    max_levels = 3,
    include_distance = TRUE,
    include_invalid = FALSE,
    page_size = 100
  )

  descendants <- extract_data(result, "descendants")
  expect_true(is.list(descendants))

  # Verify hierarchy_summary structure if present
  if ("hierarchy_summary" %in% names(result)) {
    expect_true(
      "max_hierarchy_depth" %in% names(result$hierarchy_summary) ||
      "total_descendants" %in% names(result$hierarchy_summary)
    )
  }
})

#' Search Resource
#'
#' @description
#' R6 class providing access to search operations.
#'
#' @keywords internal
SearchResource <- R6::R6Class(
  "SearchResource",
  public = list(
    #' @description
    #' Create a new SearchResource.
    #' @param base_req Base httr2 request object.
    initialize = function(base_req) {
      private$.base_req <- base_req
    },

    #' @description
    #' Basic concept search.
    #'
    #' @param query Search query string.
    #' @param vocabulary_ids Filter by vocabulary IDs.
    #' @param domain_ids Filter by domain IDs.
    #' @param concept_class_ids Filter by concept class IDs.
    #' @param standard_concept Filter by standard concept ("S", "C", or NULL).
    #' @param include_synonyms Search in synonyms. Default `FALSE`.
    #' @param include_invalid Include invalid concepts. Default `FALSE`.
    #' @param min_score Minimum relevance score.
    #' @param exact_match Require exact match. Default `FALSE`.
    #' @param page Page number (1-based). Default 1.
    #' @param page_size Results per page. Default 20.
    #' @param sort_by Sort field.
    #' @param sort_order Sort order ("asc" or "desc").
    #'
    #' @returns Search results with pagination.
    basic = function(query,
                     vocabulary_ids = NULL,
                     domain_ids = NULL,
                     concept_class_ids = NULL,
                     standard_concept = NULL,
                     include_synonyms = FALSE,
                     include_invalid = FALSE,
                     min_score = NULL,
                     exact_match = FALSE,
                     page = 1,
                     page_size = 20,
                     sort_by = NULL,
                     sort_order = NULL) {
      checkmate::assert_string(query, min.chars = 1)
      pag <- validate_pagination(page, page_size)

      params <- list(
        query = query,
        page = pag$page,
        page_size = pag$page_size
      )

      if (!is.null(vocabulary_ids)) {
        params$vocabulary_ids <- join_params(vocabulary_ids)
      }
      if (!is.null(domain_ids)) {
        params$domain_ids <- join_params(domain_ids)
      }
      if (!is.null(concept_class_ids)) {
        params$concept_class_ids <- join_params(concept_class_ids)
      }
      if (!is.null(standard_concept)) {
        params$standard_concept <- standard_concept
      }
      if (isTRUE(include_synonyms)) {
        params$include_synonyms <- "true"
      }
      if (isTRUE(include_invalid)) {
        params$include_invalid <- "true"
      }
      if (!is.null(min_score)) {
        params$min_score <- min_score
      }
      if (isTRUE(exact_match)) {
        params$exact_match <- "true"
      }
      if (!is.null(sort_by)) {
        params$sort_by <- sort_by
      }
      if (!is.null(sort_order)) {
        params$sort_order <- sort_order
      }

      perform_get(private$.base_req, "search/concepts", query = params)
    },

    #' @description
    #' Fetch all search results with automatic pagination.
    #'
    #' @param query Search query string.
    #' @param vocabulary_ids Filter by vocabulary IDs.
    #' @param domain_ids Filter by domain IDs.
    #' @param page_size Results per page. Default 100.
    #' @param max_pages Maximum pages to fetch. Default Inf.
    #' @param progress Show progress bar. Default `TRUE`.
    #' @param ... Additional search parameters.
    #'
    #' @returns A tibble of all matching concepts.
    basic_all = function(query,
                         vocabulary_ids = NULL,
                         domain_ids = NULL,
                         page_size = 100,
                         max_pages = Inf,
                         progress = TRUE,
                         ...) {
      fetch_fn <- function(page, size) {
        result <- self$basic(
          query,
          vocabulary_ids = vocabulary_ids,
          domain_ids = domain_ids,
          page = page,
          page_size = size,
          ...
        )
        # Result has pagination structure: list(data=concepts, meta=pagination)
        # or it's directly the concepts array (for non-paginated responses)
        if (is.list(result) && "data" %in% names(result) && "meta" %in% names(result)) {
          # Paginated response - return data and meta
          list(data = result$data, meta = result$meta)
        } else if (is.list(result) && !is.null(result$concepts)) {
          # Wrapped in concepts key
          data <- result$concepts
          meta <- result$meta$pagination %||% result$meta %||% list()
          list(data = data, meta = meta)
        } else if (is.list(result) && (is.null(names(result)) || !("data" %in% names(result)))) {
          # Unnamed list = array of concepts directly
          list(data = result, meta = list())
        } else {
          list(data = result %||% list(), meta = list())
        }
      }

      paginate_all(fetch_fn, page_size = page_size, max_pages = max_pages, progress = progress)
    },

    #' @description
    #' Advanced concept search with facets.
    #'
    #' @param query Search query string.
    #' @param vocabulary_ids Filter by vocabulary IDs.
    #' @param domain_ids Filter by domain IDs.
    #' @param concept_class_ids Filter by concept class IDs.
    #' @param standard_concepts_only Only return standard concepts. Default `FALSE`.
    #' @param include_invalid Include invalid concepts. Default `FALSE`.
    #' @param relationship_filters Relationship-based filters.
    #' @param page Page number (1-based). Default 1.
    #' @param page_size Results per page. Default 20.
    #'
    #' @returns Search results with facets and metadata.
    advanced = function(query,
                        vocabulary_ids = NULL,
                        domain_ids = NULL,
                        concept_class_ids = NULL,
                        standard_concepts_only = FALSE,
                        include_invalid = FALSE,
                        relationship_filters = NULL,
                        page = 1,
                        page_size = 20) {
      checkmate::assert_string(query, min.chars = 1)
      checkmate::assert_integerish(page, lower = 1, len = 1, any.missing = FALSE)
      checkmate::assert_integerish(page_size, lower = 1, upper = 1000, len = 1, any.missing = FALSE)

      body <- list(query = query)

      if (!is.null(vocabulary_ids)) {
        body$vocabulary_ids <- as.character(vocabulary_ids)
      }
      if (!is.null(domain_ids)) {
        body$domain_ids <- as.character(domain_ids)
      }
      if (!is.null(concept_class_ids)) {
        body$concept_class_ids <- as.character(concept_class_ids)
      }
      if (isTRUE(standard_concepts_only)) {
        body$standard_concepts_only <- TRUE
      }
      if (isTRUE(include_invalid)) {
        body$include_invalid <- TRUE
      }
      if (!is.null(relationship_filters)) {
        body$relationship_filters <- relationship_filters
      }
      if (page != 1) {
        body$page <- as.integer(page)
      }
      if (page_size != 20) {
        body$page_size <- as.integer(page_size)
      }

      perform_post(private$.base_req, "search/advanced", body = body)
    },

    #' @description
    #' Get autocomplete suggestions.
    #'
    #' @param query Partial query string.
    #' @param vocabulary_ids Filter by vocabulary IDs.
    #' @param domains Filter by domains.
    #' @param max_suggestions Maximum suggestions. Default 10.
    #'
    #' @returns Autocomplete suggestions.
    autocomplete = function(query,
                            vocabulary_ids = NULL,
                            domains = NULL,
                            max_suggestions = 10) {
      checkmate::assert_string(query, min.chars = 1)
      checkmate::assert_integerish(max_suggestions, lower = 1, len = 1, any.missing = FALSE)

      params <- list(
        query = query,
        max_suggestions = as.integer(max_suggestions)
      )

      if (!is.null(vocabulary_ids)) {
        params$vocabulary_ids <- join_params(vocabulary_ids)
      }
      if (!is.null(domains)) {
        params$domains <- join_params(domains)
      }

      perform_get(private$.base_req, "search/suggest", query = params)
    },

    #' @description
    #' Semantic concept search using neural embeddings.
    #'
    #' @param query Natural language search query (required).
    #' @param vocabulary_ids Filter by vocabulary IDs.
    #' @param domain_ids Filter by domain IDs.
    #' @param standard_concept Filter by standard concept ('S' or 'C').
    #' @param concept_class_id Filter by concept class ID.
    #' @param threshold Minimum similarity threshold (0.0-1.0, default 0.5).
    #' @param page Page number (1-based). Default 1.
    #' @param page_size Results per page (max 100). Default 20.
    #'
    #' @returns List with results and pagination metadata.
    semantic = function(query,
                        vocabulary_ids = NULL,
                        domain_ids = NULL,
                        standard_concept = NULL,
                        concept_class_id = NULL,
                        threshold = NULL,
                        page = 1,
                        page_size = 20) {
      checkmate::assert_string(query, min.chars = 1)
      pag <- validate_pagination(page, page_size, max_page_size = 100)

      params <- list(
        query = query,
        page = pag$page,
        page_size = pag$page_size
      )

      if (!is.null(vocabulary_ids)) {
        params$vocabulary_ids <- join_params(vocabulary_ids)
      }
      if (!is.null(domain_ids)) {
        params$domain_ids <- join_params(domain_ids)
      }
      if (!is.null(standard_concept)) {
        checkmate::assert_choice(standard_concept, c("S", "C"))
        params$standard_concept <- standard_concept
      }
      if (!is.null(concept_class_id)) {
        params$concept_class_id <- concept_class_id
      }
      if (!is.null(threshold)) {
        checkmate::assert_number(threshold, lower = 0, upper = 1)
        params$threshold <- threshold
      }

      perform_get(private$.base_req, "concepts/semantic-search", query = params)
    },

    #' @description
    #' Fetch all semantic search results with automatic pagination.
    #'
    #' @param query Natural language search query (required).
    #' @param vocabulary_ids Filter by vocabulary IDs.
    #' @param domain_ids Filter by domain IDs.
    #' @param standard_concept Filter by standard concept ('S' or 'C').
    #' @param concept_class_id Filter by concept class ID.
    #' @param threshold Minimum similarity threshold (0.0-1.0).
    #' @param page_size Results per page. Default 100.
    #' @param max_pages Maximum pages to fetch. Default Inf.
    #' @param progress Show progress bar. Default `TRUE`.
    #'
    #' @returns A tibble of all matching concepts with similarity scores.
    semantic_all = function(query,
                            vocabulary_ids = NULL,
                            domain_ids = NULL,
                            standard_concept = NULL,
                            concept_class_id = NULL,
                            threshold = NULL,
                            page_size = 100,
                            max_pages = Inf,
                            progress = TRUE) {
      fetch_fn <- function(page, size) {
        result <- self$semantic(
          query = query,
          vocabulary_ids = vocabulary_ids,
          domain_ids = domain_ids,
          standard_concept = standard_concept,
          concept_class_id = concept_class_id,
          threshold = threshold,
          page = page,
          page_size = size
        )
        # Handle different response structures
        if (is.list(result) && "data" %in% names(result) && "meta" %in% names(result)) {
          data <- result$data
          if (is.list(data) && "results" %in% names(data)) {
            data <- data$results
          }
          list(data = data, meta = result$meta)
        } else if (is.list(result) && "results" %in% names(result)) {
          list(data = result$results, meta = result$meta %||% list())
        } else {
          list(data = result %||% list(), meta = list())
        }
      }

      paginate_all(fetch_fn, page_size = page_size, max_pages = max_pages, progress = progress)
    },

    #' @description
    #' Execute multiple lexical searches in a single request (max 50).
    #'
    #' @param searches List of search inputs. Each element is a named list with:
    #'   - `search_id` (required): Unique ID to match results.
    #'   - `query` (required): Search query string.
    #'   - `vocabulary_ids`, `domain_ids`, `concept_class_ids`: Optional filters.
    #'   - `standard_concept`, `include_invalid`, `page_size`: Optional params.
    #' @param defaults Named list of default filters applied to all searches.
    #'   Individual search-level values override defaults.
    #'
    #' @returns List with `results` (per-search), `total_searches`,
    #'   `completed_searches`, `failed_searches`.
    bulk_basic = function(searches, defaults = NULL) {
      checkmate::assert_list(searches, min.len = 1, max.len = 50)
      for (s in searches) {
        checkmate::assert_list(s)
        checkmate::assert_string(s$search_id, min.chars = 1)
        checkmate::assert_string(s$query, min.chars = 1)
      }

      body <- list(searches = searches)
      if (!is.null(defaults)) {
        checkmate::assert_list(defaults)
        body$defaults <- defaults
      }

      perform_post(private$.base_req, "search/bulk", body = body)
    },

    #' @description
    #' Execute multiple semantic searches in a single request (max 25).
    #'
    #' @param searches List of search inputs. Each element is a named list with:
    #'   - `search_id` (required): Unique ID to match results.
    #'   - `query` (required): Natural language query (1-500 chars).
    #'   - `threshold`: Per-search similarity threshold (0-1).
    #'   - `page_size`: Per-search result limit (1-50).
    #'   - `vocabulary_ids`, `domain_ids`, `standard_concept`: Optional filters.
    #' @param defaults Named list of default filters applied to all searches.
    #'
    #' @returns List with `results` (per-search), `total_searches`,
    #'   `completed_count`, `failed_count`, `total_duration`.
    bulk_semantic = function(searches, defaults = NULL) {
      checkmate::assert_list(searches, min.len = 1, max.len = 25)
      for (s in searches) {
        checkmate::assert_list(s)
        checkmate::assert_string(s$search_id, min.chars = 1)
        checkmate::assert_string(s$query, min.chars = 1, max.chars = 500)
      }

      body <- list(searches = searches)
      if (!is.null(defaults)) {
        checkmate::assert_list(defaults)
        body$defaults <- defaults
      }

      perform_post(private$.base_req, "search/semantic-bulk", body = body)
    },

    #' @description
    #' Find concepts similar to a reference concept or query.
    #'
    #' Must provide exactly one of: concept_id, concept_name, or query.
    #'
    #' @param concept_id Concept ID to find similar concepts for.
    #' @param concept_name Concept name to find similar concepts for.
    #' @param query Natural language query for semantic similarity.
    #' @param algorithm One of 'semantic', 'lexical', or 'hybrid' (default).
    #' @param similarity_threshold Minimum similarity (0.0-1.0). Default 0.7.
    #' @param page_size Max results (max 1000). Default 20.
    #' @param vocabulary_ids Filter by vocabulary IDs.
    #' @param domain_ids Filter by domain IDs.
    #' @param standard_concept Filter by standard concept flag ('S', 'C', or 'N').
    #' @param include_invalid Include invalid/deprecated concepts.
    #' @param include_scores Include detailed similarity scores.
    #' @param include_explanations Include similarity explanations.
    #'
    #' @returns List with similar_concepts and search_metadata.
    #'
    #' @note When algorithm='semantic', only single vocabulary/domain filter supported.
    similar = function(concept_id = NULL,
                       concept_name = NULL,
                       query = NULL,
                       algorithm = "hybrid",
                       similarity_threshold = 0.7,
                       page_size = 20,
                       vocabulary_ids = NULL,
                       domain_ids = NULL,
                       standard_concept = NULL,
                       include_invalid = NULL,
                       include_scores = NULL,
                       include_explanations = NULL) {
      # Validate exactly one of concept_id, concept_name, or query provided
      provided <- sum(!is.null(concept_id), !is.null(concept_name), !is.null(query))
      if (provided != 1) {
        cli::cli_abort(
          "Exactly one of {.arg concept_id}, {.arg concept_name}, or {.arg query} must be provided"
        )
      }

      checkmate::assert_choice(algorithm, c("semantic", "lexical", "hybrid"))
      checkmate::assert_number(similarity_threshold, lower = 0, upper = 1)
      checkmate::assert_integerish(page_size, lower = 1, upper = 1000)
      if (!is.null(concept_id)) {
        checkmate::assert_integerish(concept_id, len = 1, any.missing = FALSE)
      }

      body <- list(
        algorithm = algorithm,
        similarity_threshold = similarity_threshold
      )

      if (!is.null(concept_id)) {
        body$concept_id <- as.integer(concept_id)
      }
      if (!is.null(concept_name)) {
        body$concept_name <- concept_name
      }
      if (!is.null(query)) {
        body$query <- query
      }
      if (page_size != 20) {
        body$page_size <- as.integer(page_size)
      }
      if (!is.null(vocabulary_ids)) {
        body$vocabulary_ids <- as.list(vocabulary_ids)
      }
      if (!is.null(domain_ids)) {
        body$domain_ids <- as.list(domain_ids)
      }
      if (!is.null(standard_concept)) {
        checkmate::assert_choice(standard_concept, c("S", "C", "N"))
        body$standard_concept <- standard_concept
      }
      if (!is.null(include_invalid)) {
        body$include_invalid <- include_invalid
      }
      if (!is.null(include_scores)) {
        body$include_scores <- include_scores
      }
      if (!is.null(include_explanations)) {
        body$include_explanations <- include_explanations
      }

      perform_post(private$.base_req, "search/similar", body = body)
    },

    #' @description
    #' Print resource information.
    print = function() {
      cat("<OMOPHub SearchResource>\n")
      cat("  Methods: basic, basic_all, advanced, autocomplete, semantic, semantic_all, bulk_basic, bulk_semantic, similar\n")
      invisible(self)
    }
  ),
  private = list(
    .base_req = NULL
  )
)

# Unit tests for SearchResource (R/search.R)
# Uses mocked HTTP responses to avoid API calls

# ==============================================================================
# SearchResource initialization
# ==============================================================================

test_that("SearchResource initializes correctly", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  expect_s3_class(resource, "SearchResource")
  expect_s3_class(resource, "R6")
})

test_that("SearchResource print method works", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  expect_output(print(resource), "<OMOPHub SearchResource>")
  expect_output(print(resource), "basic, basic_all, advanced, autocomplete")
})

# ==============================================================================
# basic() method
# ==============================================================================

test_that("search$basic validates query", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  expect_error(resource$basic(""))  # Empty query
})

test_that("search$basic calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      list(data = list(), meta = list())
    }
  )

  resource$basic("diabetes", page = 1, page_size = 20)

  expect_equal(called_with$path, "search/concepts")
  expect_equal(called_with$query$query, "diabetes")
  expect_equal(called_with$query$page, 1L)
  expect_equal(called_with$query$page_size, 20L)
})

test_that("search$basic includes vocabulary filter", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(data = list(), meta = list())
    }
  )

  resource$basic("diabetes", vocabulary_ids = c("SNOMED", "ICD10CM"))

  expect_equal(called_with$query$vocabulary_ids, "SNOMED,ICD10CM")
})

test_that("search$basic includes domain filter", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(data = list(), meta = list())
    }
  )

  resource$basic("aspirin", domain_ids = c("Drug", "Ingredient"))

  expect_equal(called_with$query$domain_ids, "Drug,Ingredient")
})

test_that("search$basic includes concept_class filter", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(data = list(), meta = list())
    }
  )

  resource$basic("diabetes", concept_class_ids = "Clinical Finding")

  expect_equal(called_with$query$concept_class_ids, "Clinical Finding")
})

test_that("search$basic includes standard_concept filter", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(data = list(), meta = list())
    }
  )

  resource$basic("diabetes", standard_concept = "S")

  expect_equal(called_with$query$standard_concept, "S")
})

test_that("search$basic includes boolean options", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(data = list(), meta = list())
    }
  )

  resource$basic(
    "diabetes",
    include_synonyms = TRUE,
    include_invalid = TRUE,
    exact_match = TRUE
  )

  expect_equal(called_with$query$include_synonyms, "true")
  expect_equal(called_with$query$include_invalid, "true")
  expect_equal(called_with$query$exact_match, "true")
})

test_that("search$basic includes scoring and sorting", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(data = list(), meta = list())
    }
  )

  resource$basic(
    "diabetes",
    min_score = 0.5,
    sort_by = "concept_name",
    sort_order = "asc"
  )

  expect_equal(called_with$query$min_score, 0.5)
  expect_equal(called_with$query$sort_by, "concept_name")
  expect_equal(called_with$query$sort_order, "asc")
})

# ==============================================================================
# basic_all() method
# ==============================================================================

test_that("search$basic_all fetches multiple pages", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  page_calls <- 0
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      page_calls <<- page_calls + 1
      list(
        data = list(mock_concept(concept_id = page_calls)),
        meta = list(
          page = query$page,
          page_size = query$page_size,
          has_next = page_calls < 3
        )
      )
    }
  )

  result <- resource$basic_all("diabetes", page_size = 1, progress = FALSE)

  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 3)  # 3 pages
  expect_equal(page_calls, 3)
})

test_that("search$basic_all respects max_pages", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  page_calls <- 0
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      page_calls <<- page_calls + 1
      list(
        data = list(mock_concept()),
        meta = list(has_next = TRUE)
      )
    }
  )

  result <- resource$basic_all("diabetes", max_pages = 2, progress = FALSE)

  expect_equal(page_calls, 2)
})

test_that("search$basic_all handles concepts key in response", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      list(
        concepts = list(mock_concept()),
        meta = list(has_next = FALSE)
      )
    }
  )

  result <- resource$basic_all("diabetes", progress = FALSE)

  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 1)
})

# ==============================================================================
# advanced() method
# ==============================================================================

test_that("search$advanced validates query", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  expect_error(resource$advanced(""))
})

test_that("search$advanced calls correct endpoint with body", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(path = path, body = body)
      list(results = list(), facets = list())
    }
  )

  resource$advanced("diabetes")

  expect_equal(called_with$path, "search/advanced")
  expect_equal(called_with$body$query, "diabetes")
})

test_that("search$advanced includes filters", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(body = body)
      list(results = list())
    }
  )

  resource$advanced(
    "diabetes",
    vocabulary_ids = c("SNOMED", "ICD10CM"),
    domain_ids = c("Condition"),
    concept_class_ids = c("Clinical Finding"),
    standard_concepts_only = TRUE,
    include_invalid = TRUE
  )

  expect_equal(called_with$body$vocabulary_ids, c("SNOMED", "ICD10CM"))
  expect_equal(called_with$body$domain_ids, "Condition")
  expect_equal(called_with$body$concept_class_ids, "Clinical Finding")
  expect_true(called_with$body$standard_concepts_only)
  expect_true(called_with$body$include_invalid)
})

test_that("search$advanced includes pagination params", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(body = body)
      list(results = list())
    }
  )

  resource$advanced("diabetes", page = 3, page_size = 50)

  expect_equal(called_with$body$page, 3L)
  expect_equal(called_with$body$page_size, 50L)
})

test_that("search$advanced omits default page_size", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(body = body)
      list(results = list())
    }
  )

  resource$advanced("diabetes", page_size = 20)  # Default

  expect_null(called_with$body$page_size)
})

# ==============================================================================
# autocomplete() method
# ==============================================================================

test_that("search$autocomplete validates query", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  expect_error(resource$autocomplete(""))
})

test_that("search$autocomplete calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      list(suggestions = list())
    }
  )

  resource$autocomplete("diab", max_suggestions = 5)

  expect_equal(called_with$path, "search/suggest")
  expect_equal(called_with$query$query, "diab")
  expect_equal(called_with$query$max_suggestions, 5L)
})

test_that("search$autocomplete includes filters", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(suggestions = list())
    }
  )

  resource$autocomplete(
    "diab",
    vocabulary_ids = c("SNOMED"),
    domains = c("Condition", "Drug")
  )

  expect_equal(called_with$query$vocabulary_ids, "SNOMED")
  expect_equal(called_with$query$domains, "Condition,Drug")
})

# ==============================================================================
# semantic() method
# ==============================================================================

test_that("search$semantic validates query", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  expect_error(resource$semantic(""))  # Empty query
})

test_that("search$semantic calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(path = path, query = query)
      list(data = list(), meta = list())
    }
  )

  resource$semantic("heart attack", page = 1, page_size = 20)

  expect_equal(called_with$path, "concepts/semantic-search")
  expect_equal(called_with$query$query, "heart attack")
  expect_equal(called_with$query$page, 1L)
  expect_equal(called_with$query$page_size, 20L)
})

test_that("search$semantic includes vocabulary filter", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(data = list(), meta = list())
    }
  )

  resource$semantic("heart attack", vocabulary_ids = c("SNOMED", "ICD10CM"))

  expect_equal(called_with$query$vocabulary_ids, "SNOMED,ICD10CM")
})

test_that("search$semantic includes domain filter", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(data = list(), meta = list())
    }
  )

  resource$semantic("diabetes", domain_ids = c("Condition", "Observation"))

  expect_equal(called_with$query$domain_ids, "Condition,Observation")
})

test_that("search$semantic includes standard_concept filter", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(data = list(), meta = list())
    }
  )

  resource$semantic("diabetes", standard_concept = "S")

  expect_equal(called_with$query$standard_concept, "S")
})

test_that("search$semantic includes threshold", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(data = list(), meta = list())
    }
  )

  resource$semantic("diabetes", threshold = 0.5)

  expect_equal(called_with$query$threshold, 0.5)
})

test_that("search$semantic includes concept_class_id", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(data = list(), meta = list())
    }
  )

  resource$semantic("diabetes", concept_class_id = "Clinical Finding")

  expect_equal(called_with$query$concept_class_id, "Clinical Finding")
})

test_that("search$semantic validates threshold range", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  expect_error(resource$semantic("diabetes", threshold = 1.5))
  expect_error(resource$semantic("diabetes", threshold = -0.1))
})

test_that("search$semantic validates standard_concept choices", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  expect_error(resource$semantic("diabetes", standard_concept = "X"))
})

test_that("search$semantic rejects page_size over 100", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  # semantic uses max_page_size = 100
  expect_error(resource$semantic("diabetes", page_size = 101))
})

# ==============================================================================
# semantic_all() method
# ==============================================================================

test_that("search$semantic_all fetches multiple pages", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  page_calls <- 0
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      page_calls <<- page_calls + 1
      list(
        data = list(
          results = list(
            list(concept_id = page_calls, similarity_score = 0.9)
          )
        ),
        meta = list(
          page = query$page,
          page_size = query$page_size,
          has_next = page_calls < 3
        )
      )
    }
  )

  result <- resource$semantic_all("heart attack", page_size = 1, progress = FALSE)

  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 3)  # 3 pages
  expect_equal(page_calls, 3)
})

test_that("search$semantic_all respects max_pages", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  page_calls <- 0
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      page_calls <<- page_calls + 1
      list(
        data = list(results = list(mock_concept())),
        meta = list(has_next = TRUE)
      )
    }
  )

  result <- resource$semantic_all("diabetes", max_pages = 2, progress = FALSE)

  expect_equal(page_calls, 2)
})

# ==============================================================================
# similar() method
# ==============================================================================

test_that("search$similar requires exactly one search parameter", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  # No parameters
  expect_error(resource$similar(), "Exactly one")

  # Multiple parameters
  expect_error(
    resource$similar(concept_id = 123, concept_name = "test"),
    "Exactly one"
  )
  expect_error(
    resource$similar(concept_id = 123, query = "test"),
    "Exactly one"
  )
})

test_that("search$similar by concept_id calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(path = path, body = body)
      list(similar_concepts = list(), search_metadata = list())
    }
  )

  resource$similar(concept_id = 4329847)

  expect_equal(called_with$path, "search/similar")
  expect_equal(called_with$body$concept_id, 4329847L)
  expect_equal(called_with$body$algorithm, "hybrid")
  expect_equal(called_with$body$similarity_threshold, 0.7)
})

test_that("search$similar by concept_name works", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(body = body)
      list(similar_concepts = list(), search_metadata = list())
    }
  )

  resource$similar(concept_name = "Type 2 diabetes mellitus")

  expect_equal(called_with$body$concept_name, "Type 2 diabetes mellitus")
})

test_that("search$similar by query works", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(body = body)
      list(similar_concepts = list(), search_metadata = list())
    }
  )

  resource$similar(query = "high blood sugar condition")

  expect_equal(called_with$body$query, "high blood sugar condition")
})

test_that("search$similar includes algorithm option", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(body = body)
      list(similar_concepts = list(), search_metadata = list())
    }
  )

  resource$similar(concept_id = 123, algorithm = "semantic")

  expect_equal(called_with$body$algorithm, "semantic")
})

test_that("search$similar includes similarity_threshold", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(body = body)
      list(similar_concepts = list(), search_metadata = list())
    }
  )

  resource$similar(concept_id = 123, similarity_threshold = 0.85)

  expect_equal(called_with$body$similarity_threshold, 0.85)
})

test_that("search$similar includes vocabulary_ids as list", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(body = body)
      list(similar_concepts = list(), search_metadata = list())
    }
  )

  resource$similar(concept_id = 123, vocabulary_ids = c("SNOMED", "ICD10CM"))

  # POST body uses array format
  expect_equal(called_with$body$vocabulary_ids, list("SNOMED", "ICD10CM"))
})

test_that("search$similar includes domain_ids as list", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(body = body)
      list(similar_concepts = list(), search_metadata = list())
    }
  )

  resource$similar(concept_id = 123, domain_ids = c("Condition", "Observation"))

  expect_equal(called_with$body$domain_ids, list("Condition", "Observation"))
})

test_that("search$similar includes boolean options", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(body = body)
      list(similar_concepts = list(), search_metadata = list())
    }
  )

  resource$similar(
    concept_id = 123,
    include_invalid = TRUE,
    include_scores = TRUE,
    include_explanations = TRUE
  )

  expect_true(called_with$body$include_invalid)
  expect_true(called_with$body$include_scores)
  expect_true(called_with$body$include_explanations)
})

test_that("search$similar validates algorithm choices", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  expect_error(resource$similar(concept_id = 123, algorithm = "invalid"))
})

test_that("search$similar validates similarity_threshold range", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  expect_error(resource$similar(concept_id = 123, similarity_threshold = 1.5))
  expect_error(resource$similar(concept_id = 123, similarity_threshold = -0.1))
})

test_that("search$similar validates standard_concept choices", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  expect_error(resource$similar(concept_id = 123, standard_concept = "X"))
})

test_that("search$similar omits default page_size", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(body = body)
      list(similar_concepts = list(), search_metadata = list())
    }
  )

  resource$similar(concept_id = 123, page_size = 20)  # Default

  expect_null(called_with$body$page_size)
})

test_that("search$similar includes non-default page_size", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(body = body)
      list(similar_concepts = list(), search_metadata = list())
    }
  )

  resource$similar(concept_id = 123, page_size = 50)

  expect_equal(called_with$body$page_size, 50L)
})

test_that("search$similar rejects concept_name and query together", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  # Test the third mutual exclusion pair
  expect_error(
    resource$similar(concept_name = "diabetes", query = "high blood sugar"),
    "Exactly one"
  )
})

test_that("search$similar rejects page_size zero", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  expect_error(resource$similar(concept_id = 123, page_size = 0))
})

test_that("search$similar rejects page_size over 1000", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  expect_error(resource$similar(concept_id = 123, page_size = 1001))
})

test_that("search$similar accepts standard_concept N", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(body = body)
      list(similar_concepts = list(), search_metadata = list())
    }
  )

  # "N" is a valid value for similar() but wasn't tested
  resource$similar(concept_id = 123, standard_concept = "N")

  expect_equal(called_with$body$standard_concept, "N")
})

test_that("search$semantic accepts standard_concept C", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      called_with <<- list(query = query)
      list(data = list(), meta = list())
    }
  )

  # "C" is a valid value for semantic() - test it explicitly
  resource$semantic("diabetes", standard_concept = "C")

  expect_equal(called_with$query$standard_concept, "C")
})

test_that("search$semantic_all returns empty tibble for no results", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      list(
        data = list(results = list()),
        meta = list(has_next = FALSE)
      )
    }
  )

  result <- resource$semantic_all("nonexistent query xyz", progress = FALSE)

  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 0)
})

test_that("search$semantic_all handles top-level results key", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  local_mocked_bindings(
    perform_get = function(req, path, query = NULL) {
      # Response structure with top-level 'results' key
      list(
        results = list(
          list(concept_id = 1, similarity_score = 0.9),
          list(concept_id = 2, similarity_score = 0.8)
        ),
        meta = list(has_next = FALSE)
      )
    }
  )

  result <- resource$semantic_all("test", progress = FALSE)

  expect_s3_class(result, "tbl_df")
  expect_equal(nrow(result), 2)
})

# ==============================================================================
# print() method update
# ==============================================================================

test_that("SearchResource print method includes new methods", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  expect_output(print(resource), "semantic")
  expect_output(print(resource), "semantic_all")
  expect_output(print(resource), "similar")
})

# ==============================================================================
# bulk_basic() method
# ==============================================================================

test_that("search$bulk_basic validates searches", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  expect_error(resource$bulk_basic(list()))  # Empty list
  expect_error(resource$bulk_basic(list(list(search_id = "", query = "test"))))  # Empty search_id
  expect_error(resource$bulk_basic(list(list(search_id = "q1", query = ""))))  # Empty query
})

test_that("search$bulk_basic calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(path = path, body = body)
      list(results = list(), total_searches = 0, completed_searches = 0, failed_searches = 0)
    }
  )

  resource$bulk_basic(list(
    list(search_id = "q1", query = "diabetes"),
    list(search_id = "q2", query = "hypertension")
  ))

  expect_equal(called_with$path, "search/bulk")
  expect_length(called_with$body$searches, 2)
  expect_equal(called_with$body$searches[[1]]$search_id, "q1")
  expect_equal(called_with$body$searches[[2]]$query, "hypertension")
})

test_that("search$bulk_basic passes defaults", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(path = path, body = body)
      list(results = list(), total_searches = 1, completed_searches = 1, failed_searches = 0)
    }
  )

  resource$bulk_basic(
    list(list(search_id = "q1", query = "diabetes")),
    defaults = list(vocabulary_ids = list("SNOMED"), page_size = 5)
  )

  expect_equal(called_with$body$defaults$vocabulary_ids, list("SNOMED"))
  expect_equal(called_with$body$defaults$page_size, 5)
})

# ==============================================================================
# bulk_semantic() method
# ==============================================================================

test_that("search$bulk_semantic validates searches", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  expect_error(resource$bulk_semantic(list()))  # Empty list
  expect_error(resource$bulk_semantic(list(list(search_id = "", query = "test"))))  # Empty search_id
  expect_error(resource$bulk_semantic(list(list(search_id = "s1", query = ""))))  # Empty query
})

test_that("search$bulk_semantic calls correct endpoint", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(path = path, body = body)
      list(results = list(), total_searches = 0, completed_count = 0, failed_count = 0)
    }
  )

  resource$bulk_semantic(list(
    list(search_id = "s1", query = "heart failure treatment")
  ))

  expect_equal(called_with$path, "search/semantic-bulk")
  expect_length(called_with$body$searches, 1)
  expect_equal(called_with$body$searches[[1]]$search_id, "s1")
})

test_that("search$bulk_semantic passes defaults", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  called_with <- NULL
  local_mocked_bindings(
    perform_post = function(req, path, body = NULL) {
      called_with <<- list(path = path, body = body)
      list(results = list(), total_searches = 1, completed_count = 1, failed_count = 0)
    }
  )

  resource$bulk_semantic(
    list(list(search_id = "s1", query = "diabetes medications")),
    defaults = list(threshold = 0.8, page_size = 10)
  )

  expect_equal(called_with$body$defaults$threshold, 0.8)
  expect_equal(called_with$body$defaults$page_size, 10)
})

test_that("SearchResource print includes bulk methods", {
  base_req <- httr2::request("https://api.omophub.com/v1")
  resource <- SearchResource$new(base_req)

  expect_output(print(resource), "bulk_basic")
  expect_output(print(resource), "bulk_semantic")
})

# Integration tests for search resource
# Run with: testthat::test_file("tests/testthat/test-search-integration.R")

test_that("basic search works", {
  skip_if_no_integration_key()
  client <- integration_client()

  results <- client$search$basic("diabetes mellitus", page_size = 10)

  concepts <- extract_data(results, "concepts")
  expect_gt(length(concepts), 0)
})

test_that("search with vocabulary filter works", {
  skip_if_no_integration_key()
  client <- integration_client()

  results <- client$search$basic(
    "myocardial infarction",
    vocabulary_ids = "SNOMED",
    page_size = 20
  )

  concepts <- extract_data(results, "concepts")
  # If results exist, all should be SNOMED
  for (concept in concepts) {
    expect_equal(concept$vocabulary_id, "SNOMED")
  }
})

test_that("search with domain filter works", {
  skip_if_no_integration_key()
  client <- integration_client()

  results <- client$search$basic(
    "aspirin",
    domain_ids = "Drug",
    page_size = 10
  )

  concepts <- extract_data(results, "concepts")
  # If results exist, all should be in Drug domain
  for (concept in concepts) {
    expect_equal(concept$domain_id, "Drug")
  }
})

test_that("search with standard_concept filter works", {
  skip_if_no_integration_key()
  client <- integration_client()

  results <- client$search$basic(
    "diabetes",
    standard_concept = "S",
    page_size = 20
  )

  concepts <- extract_data(results, "concepts")
  expect_gt(length(concepts), 0)

  # All results should be standard concepts
  for (concept in concepts) {
    expect_equal(concept$standard_concept, "S")
  }
})

test_that("search with multiple filters and pagination works", {
  skip_if_no_integration_key()
  client <- integration_client()

  # This combination triggers the COUNT query with all parameters
  results <- client$search$basic(
    "diabetes",
    vocabulary_ids = "SNOMED",
    domain_ids = "Condition",
    standard_concept = "S",
    page_size = 10
  )

  concepts <- extract_data(results, "concepts")
  expect_gt(length(concepts), 0)

  # Verify all filters applied correctly
  for (concept in concepts) {
    expect_equal(concept$vocabulary_id, "SNOMED")
    expect_equal(concept$domain_id, "Condition")
    expect_equal(concept$standard_concept, "S")
  }

  # Verify pagination metadata exists (proves COUNT query worked)
  expect_true(!is.null(results$meta))
  expect_true(!is.null(results$meta$total_items))
})

test_that("autocomplete works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$search$autocomplete(
    "diab",
    max_suggestions = 10
  )

  suggestions <- extract_data(result, "suggestions")
  expect_gt(length(suggestions), 0)

  # Suggestions should contain the query
  for (suggestion in suggestions) {
    if (is.list(suggestion)) {
      text <- suggestion$suggestion %||% suggestion$concept_name %||% ""
      if (is.list(text)) {
        text <- text$concept_name %||% ""
      }
      text <- tolower(as.character(text))
    } else {
      text <- tolower(as.character(suggestion))
    }
    expect_match(text, "diab", ignore.case = TRUE)
  }
})

test_that("autocomplete with filters works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$search$autocomplete(
    "hyper",
    vocabulary_ids = "SNOMED",
    domains = "Condition",
    max_suggestions = 5
  )

  suggestions <- extract_data(result, "suggestions")
  expect_true(is.list(suggestions))
})

test_that("basic_all pagination works", {
  skip_if_no_integration_key()
  client <- integration_client()

  # Fetch multiple pages - returns a tibble
  results <- client$search$basic_all(
    "diabetes",
    page_size = 5,
    max_pages = 2
  )

  # Results should be a tibble with concept_id column
  expect_true(is.data.frame(results))
  expect_gt(nrow(results), 0)
  expect_true("concept_id" %in% names(results))
})

# ==============================================================================
# semantic() integration tests
# ==============================================================================

test_that("semantic search returns results", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$search$semantic("myocardial infarction", page_size = 5)

  # Extract results from response
  data <- extract_data(result, "results")
  expect_true(is.list(data) || is.data.frame(data))
})

test_that("semantic search respects threshold", {
  skip_if_no_integration_key()
  client <- integration_client()

  # Higher threshold should return fewer or equal results
  high <- client$search$semantic("heart attack", threshold = 0.8, page_size = 20)
  low <- client$search$semantic("heart attack", threshold = 0.3, page_size = 20)

  high_data <- extract_data(high, "results")
  low_data <- extract_data(low, "results")

  high_count <- if (is.data.frame(high_data)) nrow(high_data) else length(high_data)
  low_count <- if (is.data.frame(low_data)) nrow(low_data) else length(low_data)

  expect_lte(high_count, low_count)
})

test_that("semantic search filters by vocabulary", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$search$semantic(
    "diabetes",
    vocabulary_ids = "SNOMED",
    page_size = 10
  )

  data <- extract_data(result, "results")
  # Verify all results have vocabulary_id = SNOMED
  if (is.list(data) && length(data) > 0) {
    for (item in data) {
      if (!is.null(item$vocabulary_id)) {
        expect_equal(item$vocabulary_id, "SNOMED")
      }
    }
  }
})

test_that("semantic search filters by domain", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$search$semantic(
    "heart attack",
    domain_ids = "Condition",
    page_size = 10
  )

  data <- extract_data(result, "results")
  # Verify all results have domain_id = Condition
  if (is.list(data) && length(data) > 0) {
    for (item in data) {
      if (!is.null(item$domain_id)) {
        expect_equal(item$domain_id, "Condition")
      }
    }
  }
})

# ==============================================================================
# semantic_all() integration tests
# ==============================================================================

test_that("semantic_all returns tibble with pagination", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$search$semantic_all(
    "diabetes",
    page_size = 5,
    max_pages = 2,
    progress = FALSE
  )

  expect_s3_class(result, "tbl_df")
  expect_true("concept_id" %in% names(result))
  # Should have fetched some results
  expect_gt(nrow(result), 0)
})

# ==============================================================================
# similar() integration tests
# ==============================================================================

test_that("similar by concept_id returns results", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$search$similar(concept_id = MI_CONCEPT_ID, page_size = 5)

  # Result should be a list with similar_concepts or be directly the list
  expect_true(is.list(result))
})

test_that("similar by query returns results", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$search$similar(query = "elevated blood glucose", page_size = 5)

  expect_true(is.list(result))
})

test_that("similar with semantic algorithm works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$search$similar(
    concept_id = MI_CONCEPT_ID,
    algorithm = "semantic",
    similarity_threshold = 0.6,
    page_size = 5
  )

  expect_true(is.list(result))
})

test_that("similar by concept_name returns results", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$search$similar(
    concept_name = "Type 2 diabetes mellitus",
    page_size = 5
  )

  expect_true(is.list(result))
})

test_that("similar with vocabulary filter works", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$search$similar(
    concept_id = DIABETES_CONCEPT_ID,
    vocabulary_ids = "SNOMED",
    page_size = 10
  )

  # Verify results, if any, are from SNOMED
  similar <- result$similar_concepts %||% result
  if (is.list(similar) && length(similar) > 0) {
    for (concept in similar) {
      if (!is.null(concept$vocabulary_id)) {
        expect_equal(concept$vocabulary_id, "SNOMED")
      }
    }
  }
})

# ==============================================================================
# Bulk lexical search integration tests
# ==============================================================================

test_that("bulk_basic search works with multiple queries", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$search$bulk_basic(list(
    list(search_id = "q1", query = "diabetes mellitus"),
    list(search_id = "q2", query = "hypertension"),
    list(search_id = "q3", query = "aspirin")
  ), defaults = list(page_size = 5))

  results <- extract_data(result, "results")
  expect_length(results, 3)

  for (item in results) {
    expect_true(item$search_id %in% c("q1", "q2", "q3"))
    expect_equal(item$status, "completed")
    expect_gt(length(item$results), 0)
  }
})

test_that("bulk_basic search works with vocabulary filter", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$search$bulk_basic(
    list(list(search_id = "snomed1", query = "diabetes")),
    defaults = list(vocabulary_ids = list("SNOMED"), page_size = 3)
  )

  results <- extract_data(result, "results")
  expect_length(results, 1)
  expect_equal(results[[1]]$status, "completed")
})

# ==============================================================================
# Bulk semantic search integration tests
# ==============================================================================

test_that("bulk_semantic search works with multiple queries", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$search$bulk_semantic(list(
    list(search_id = "s1", query = "heart failure treatment options"),
    list(search_id = "s2", query = "type 2 diabetes medication")
  ), defaults = list(threshold = 0.5, page_size = 5))

  results <- extract_data(result, "results")
  expect_length(results, 2)

  for (item in results) {
    expect_true(item$search_id %in% c("s1", "s2"))
    expect_equal(item$status, "completed")
  }
})

test_that("bulk_semantic search works with single query", {
  skip_if_no_integration_key()
  client <- integration_client()

  result <- client$search$bulk_semantic(
    list(list(search_id = "one", query = "elevated blood pressure", threshold = 0.5))
  )

  results <- extract_data(result, "results")
  expect_length(results, 1)
  expect_equal(results[[1]]$search_id, "one")
})

---
title: "Getting Started with omophub"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with omophub}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

## Introduction

The omophub package provides an R interface to the OMOPHub API for accessing
OHDSI ATHENA standardized medical vocabularies. This vignette demonstrates
basic usage patterns.

## Installation

Install from GitHub:

```{r install}
# install.packages("devtools")
devtools::install_github("omophub/omophub-R")
```

## Authentication

The package requires an API key from [OMOPHub](https://dashboard.omophub.com).

Set your API key as an environment variable:

```{r auth-env}
Sys.setenv(OMOPHUB_API_KEY = "your_api_key_here")
```

Or add it to your `.Renviron` file for persistence:

```
OMOPHUB_API_KEY=your_api_key_here
```

## Creating a Client

```{r client}
library(omophub)

# Create client (reads API key from environment)
client <- OMOPHubClient$new()

# Or provide API key explicitly
client <- OMOPHubClient$new(api_key = "your_api_key")

# With additional options
client <- OMOPHubClient$new(
  api_key = "your_api_key",
  timeout = 30,
  max_retries = 3,
  vocab_version = "2025.1"
)
```

## Getting Concepts

Retrieve a concept by its OMOP concept ID:

```{r get-concept}
concept <- client$concepts$get(201826)
print(concept$concept_name)
# [1] "Type 2 diabetes mellitus"
```

Get a concept by vocabulary-specific code:

```{r get-by-code}
concept <- client$concepts$get_by_code("SNOMED", "44054006")
print(concept$concept_name)
# [1] "Type 2 diabetes mellitus"
```

## Batch Operations

Retrieve multiple concepts in a single request:

```{r batch}
result <- client$concepts$batch(c(201826, 4329847, 1112807))
for (concept in result$concepts) {
  cat(sprintf("%s: %s\n", concept$concept_id, concept$concept_name))
}
```

## Searching Concepts

Basic search:

```{r search-basic}
results <- client$search$basic("diabetes mellitus", page_size = 10)
for (concept in results$data) {
  cat(sprintf("%s: %s\n", concept$concept_id, concept$concept_name))
}
```

Search with filters:

```{r search-filters}
results <- client$search$basic(
  "heart attack",
  vocabulary_ids = "SNOMED",
  domain_ids = "Condition",
  page_size = 20
)
```

## Semantic Search

Search using natural language queries powered by neural embeddings:

```{r semantic-search}
# Natural language search - understands clinical intent
results <- client$search$semantic("high blood sugar levels")
for (r in results$data$results) {
  cat(sprintf("%s (similarity: %.2f)\n", r$concept_name, r$similarity_score))
}
```

Filter semantic search results:

```{r semantic-filtered}
results <- client$search$semantic(
  "heart attack",
  vocabulary_ids = "SNOMED",
  domain_ids = "Condition",
  threshold = 0.5
)
```

Fetch all semantic search results with automatic pagination:

```{r semantic-all}
all_results <- client$search$semantic_all(
  "chronic kidney disease",
  page_size = 50,
  max_pages = 5,
  progress = TRUE
)
print(nrow(all_results))
```

## Similarity Search

Find concepts similar to a reference concept:

```{r similar-by-id}
# Find concepts similar to Type 2 diabetes mellitus
similar <- client$search$similar(concept_id = 201826)
for (s in similar$similar_concepts) {
  cat(sprintf("%s (score: %.2f)\n", s$concept_name, s$similarity_score))
}
```

Search by natural language query with different algorithms:

```{r similar-by-query}
# Semantic similarity (neural embeddings)
similar <- client$search$similar(
  query = "high blood pressure",
  algorithm = "semantic"
)

# Lexical similarity (string matching)
similar <- client$search$similar(
  query = "high blood pressure",
  algorithm = "lexical"
)

# Hybrid (combined - default)
similar <- client$search$similar(
  query = "high blood pressure",
  algorithm = "hybrid",
  include_scores = TRUE,
  include_explanations = TRUE
)
```

## Bulk Search

Search for multiple queries in a single API call — much faster than individual requests when you have many terms to look up.

### Bulk Lexical Search

Execute up to 50 keyword searches at once:

```{r bulk-basic}
results <- client$search$bulk_basic(list(
  list(search_id = "q1", query = "diabetes mellitus"),
  list(search_id = "q2", query = "hypertension"),
  list(search_id = "q3", query = "aspirin")
), defaults = list(vocabulary_ids = list("SNOMED"), page_size = 5))

# Each result is matched by search_id
for (item in results$results) {
  cat(sprintf("%s: %d results\n", item$search_id, length(item$results)))
}
```

### Bulk Semantic Search

Execute up to 25 natural-language searches using neural embeddings:

```{r bulk-semantic}
results <- client$search$bulk_semantic(list(
  list(search_id = "s1", query = "heart failure treatment options"),
  list(search_id = "s2", query = "type 2 diabetes medication")
), defaults = list(threshold = 0.5, page_size = 10))

for (item in results$results) {
  cat(sprintf("%s: %d results\n", item$search_id,
              item$result_count %||% length(item$results)))
}
```

Defaults apply to all searches; per-search values override them:

```{r bulk-overrides}
# Different domains per query, shared vocabulary filter
results <- client$search$bulk_basic(list(
  list(search_id = "cond", query = "diabetes", domain_ids = list("Condition")),
  list(search_id = "drug", query = "metformin", domain_ids = list("Drug"))
), defaults = list(vocabulary_ids = list("SNOMED", "RxNorm"), page_size = 5))
```

## Autocomplete

Get suggestions for autocomplete:

```{r autocomplete}
suggestions <- client$concepts$suggest("diab", page_size = 5)
for (s in suggestions$suggestions) {
  print(s$suggestion)
}
```

## Pagination

### Manual Pagination

```{r pagination-manual}
# First page
results <- client$search$basic("diabetes", page = 1, page_size = 50)

# Check pagination info
print(results$meta$total_items)
print(results$meta$has_next)

# Get next page if available
if (isTRUE(results$meta$has_next)) {
  page2 <- client$search$basic("diabetes", page = 2, page_size = 50)
}
```

### Automatic Pagination

Fetch all results as a tibble:

```{r pagination-auto}
all_results <- client$search$basic_all(
  "diabetes",
  page_size = 100,
  max_pages = 5,
  progress = TRUE
)

# Results are a tibble
print(nrow(all_results))
print(names(all_results))
```

## Hierarchy Navigation

Get ancestors (parent concepts):

```{r ancestors}
result <- client$hierarchy$ancestors(201826, max_levels = 3)
for (ancestor in result$ancestors) {
  print(ancestor$concept_name)
}
```

Get descendants (child concepts):

```{r descendants}
result <- client$hierarchy$descendants(201826, max_levels = 2)
for (descendant in result$descendants) {
  print(descendant$concept_name)
}
```

## Concept Mappings

Find how a concept maps to other vocabularies:

```{r mappings}
result <- client$mappings$get(201826)
for (mapping in result$mappings) {
  cat(sprintf("%s: %s\n",
              mapping$target_vocabulary_id,
              mapping$target_concept_name))
}
```

Map to specific vocabularies:

```{r mappings-filter}
result <- client$mappings$get(
  201826,
  target_vocabulary = "ICD10CM"
)
```

## Error Handling

Use `tryCatch` to handle errors:

```{r error-handling}
tryCatch(
  {
    concept <- client$concepts$get(999999999)
  },
  omophub_not_found = function(e) {
    message("Concept not found: ", e$message)
  },
  omophub_api_error = function(e) {
    message("API error: ", e$message)
  }
)
```

## FHIR-to-OMOP Resolution

The FHIR resolver translates FHIR coded values to OMOP standard concepts in a
single call --- handling URI mapping, code lookup, `Maps to` traversal, and CDM
target table assignment automatically.

### Single Coding

```{r fhir-resolve}
result <- client$fhir$resolve(
  system = "http://snomed.info/sct",
  code = "44054006",
  resource_type = "Condition"
)
cat(result$resolution$standard_concept$concept_name)
cat(result$resolution$target_table)   # "condition_occurrence"
cat(result$resolution$mapping_type)   # "direct"
```

### Non-Standard Code (Automatic Maps-to Traversal)

```{r fhir-mapped}
result <- client$fhir$resolve(
  system = "http://hl7.org/fhir/sid/icd-10-cm",
  code = "E11.9"
)
cat(result$resolution$mapping_type)                    # "mapped"
cat(result$resolution$standard_concept$vocabulary_id)  # "SNOMED"
```

### Batch Resolution

```{r fhir-batch}
batch <- client$fhir$resolve_batch(list(
  list(system = "http://snomed.info/sct", code = "44054006"),
  list(system = "http://loinc.org", code = "2339-0")
))
cat(sprintf("Resolved: %d/%d\n", batch$summary$resolved, batch$summary$total))
```

### CodeableConcept with Vocabulary Preference

```{r fhir-codeable}
result <- client$fhir$resolve_codeable_concept(
  coding = list(
    list(system = "http://snomed.info/sct", code = "44054006"),
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9")
  ),
  resource_type = "Condition"
)
# SNOMED wins over ICD-10-CM per OHDSI preference
cat(result$best_match$resolution$source_concept$vocabulary_id)  # "SNOMED"
```

To override the vocabulary preference, mark a coding with `user_selected = TRUE`
(mirroring FHIR `Coding.userSelected`); that coding then wins `best_match`
regardless of its vocabulary:

```r
client$fhir$resolve_codeable_concept(
  coding = list(
    list(
      system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9",
      user_selected = TRUE
    ),
    list(system = "http://snomed.info/sct", code = "44054006")
  ),
  resource_type = "Condition"
)
```

### Tibble Output for Batch Resolution

For `dplyr` / `tidyr` workflows, pass `as_tibble = TRUE` to get a flat
tibble with one row per input coding and columns for the source and
standard concepts, target CDM table, mapping type, and resolution status.
Composite concepts decomposed via `Maps to value` (the HL7 FHIR-to-OMOP IG
Value-as-Concept pattern) also populate `value_as_concept_id` /
`value_as_concept_name`. This is the most ergonomic shape for ETL pipelines
processing many codes:

```{r fhir-batch-tibble}
library(dplyr)

tbl <- client$fhir$resolve_batch(
  list(
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9"),
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "I10"),
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "J45.909")
  ),
  as_tibble = TRUE
)

tbl |>
  filter(status == "resolved") |>
  select(source_code, standard_concept_name, target_table)
```

Failed rows are kept in-place with `status = "failed"` and the error
text in `status_detail` --- you can filter them out rather than silently
dropping them. The batch summary (`total` / `resolved` / `failed`) is
attached as an attribute:

```{r fhir-batch-summary}
attr(tbl, "summary")
```

The default `as_tibble = FALSE` still returns the list-shaped
`list(results, summary)`, so existing code keeps working unchanged.

### Standalone Wrapper Functions

For pipe-friendly workflows, `omophub` also exports standalone wrapper
functions that take the client as their first argument. Both forms are
fully supported --- pick whichever reads better for the surrounding
code:

```{r fhir-standalone-wrappers}
# Equivalent to client$fhir$resolve()
client |>
  fhir_resolve(
    system = "http://snomed.info/sct",
    code = "44054006",
    resource_type = "Condition"
  )

# Tibble-shaped batch in a pipe
tbl <- client |>
  fhir_resolve_batch(
    codings = list(
      list(system = "http://snomed.info/sct", code = "44054006"),
      list(system = "http://loinc.org", code = "2339-0")
    ),
    as_tibble = TRUE
  )

client |>
  fhir_resolve_codeable_concept(
    coding = list(
      list(system = "http://snomed.info/sct", code = "44054006"),
      list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9")
    ),
    resource_type = "Condition"
  )
```

### FHIR Client Interop with `omophub_fhir_url()`

When you need raw FHIR `Parameters` / `Bundle` responses instead of the
Concept Resolver envelope, `omophub_fhir_url()` returns the OMOPHub FHIR
Terminology Service base URL so you can talk to it directly with
`httr2` or `fhircrackr`. Supports FHIR versions `"r4"` (default),
`"r4b"`, `"r5"`, and `"r6"`:

```{r fhir-url-helper}
omophub_fhir_url()
#> "https://fhir.omophub.com/fhir/r4"

omophub_fhir_url("r5")
#> "https://fhir.omophub.com/fhir/r5"
```

Example: call `CodeSystem/$lookup` directly with `httr2`:

```{r fhir-httr2}
library(httr2)

resp <- request(omophub_fhir_url()) |>
  req_url_path_append("CodeSystem/$lookup") |>
  req_url_query(
    system = "http://snomed.info/sct",
    code = "44054006"
  ) |>
  req_headers(Authorization = paste("Bearer", Sys.getenv("OMOPHUB_API_KEY"))) |>
  req_perform()

params <- resp_body_json(resp)
# Raw FHIR Parameters resource with the concept display and designations.
```

Use `client$fhir$resolve()` (or `fhir_resolve()`) when you want
OMOP-enriched answers (standard concept, CDM target table, mapping
quality). Use `omophub_fhir_url()` + `httr2` when you need FHIR-native
responses for FHIR-native tooling.

## Further Resources

- [OMOPHub API Documentation](https://docs.omophub.com)
- [Package Documentation](https://omophub.github.io/omophub-R/)
- [GitHub Repository](https://github.com/omophub/omophub-R)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pagination.R
\name{extract_pagination}
\alias{extract_pagination}
\title{Extract Pagination Metadata}
\usage{
extract_pagination(response)
}
\arguments{
\item{response}{API response object.}
}
\value{
A list with pagination fields.
}
\description{
Extracts pagination metadata from an API response.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fhir.R
\name{FhirResource}
\alias{FhirResource}
\title{FHIR-to-OMOP Concept Resolver}
\value{
A list with \code{input} and \code{resolution} containing source/standard
concepts, target CDM table, and optional enrichments.

When \code{as_tibble = FALSE} (default), a list with \code{results}
(per-item) and \code{summary} (total/resolved/failed). When
\code{as_tibble = TRUE}, a \link[tibble:tibble]{tibble::tibble} suitable for
\code{dplyr}/\code{tidyr} pipelines.

A list with \code{best_match}, \code{alternatives}, and \code{unresolved}.
}
\description{
R6 class providing access to the FHIR-to-OMOP Concept Resolver endpoints.
Translates FHIR coded values (system URI + code) into OMOP standard
concepts, CDM target tables, and optional Phoebe recommendations.
}
\details{
Access via the \code{fhir} active binding on an \code{OMOPHubClient}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{client <- OMOPHubClient$new(api_key = "oh_xxx")
result <- client$fhir$resolve(
  system = "http://snomed.info/sct",
  code = "44054006",
  resource_type = "Condition"
)
result$data$resolution$target_table
# "condition_occurrence"
}\if{html}{\out{</div>}}
}
\keyword{internal}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-FhirResource-new}{\code{FhirResource$new()}}
\item \href{#method-FhirResource-resolve}{\code{FhirResource$resolve()}}
\item \href{#method-FhirResource-resolve_batch}{\code{FhirResource$resolve_batch()}}
\item \href{#method-FhirResource-resolve_codeable_concept}{\code{FhirResource$resolve_codeable_concept()}}
\item \href{#method-FhirResource-clone}{\code{FhirResource$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-FhirResource-new"></a>}}
\if{latex}{\out{\hypertarget{method-FhirResource-new}{}}}
\subsection{Method \code{new()}}{
Create a new FhirResource.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{FhirResource$new(base_req)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{base_req}}{Base httr2 request object.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-FhirResource-resolve"></a>}}
\if{latex}{\out{\hypertarget{method-FhirResource-resolve}{}}}
\subsection{Method \code{resolve()}}{
Resolve a single FHIR Coding to an OMOP standard concept.

Provide at least one of (\code{system} + \code{code}), (\code{vocabulary_id} + \code{code}),
or \code{display}.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{FhirResource$resolve(
  system = NULL,
  code = NULL,
  display = NULL,
  vocabulary_id = NULL,
  resource_type = NULL,
  include_recommendations = FALSE,
  recommendations_limit = 5L,
  include_quality = FALSE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{system}}{FHIR code system URI (e.g. \code{"http://snomed.info/sct"}).}

\item{\code{code}}{Code value from the FHIR Coding.}

\item{\code{display}}{Human-readable text (semantic search fallback).}

\item{\code{vocabulary_id}}{Direct OMOP vocabulary_id, bypasses URI resolution.}

\item{\code{resource_type}}{FHIR resource type (e.g. \code{"Condition"}, \code{"Observation"}).}

\item{\code{include_recommendations}}{Logical. Include Phoebe recommendations. Default \code{FALSE}.}

\item{\code{recommendations_limit}}{Integer. Max recommendations (1-20). Default \code{5L}.}

\item{\code{include_quality}}{Logical. Include mapping quality signal. Default \code{FALSE}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-FhirResource-resolve_batch"></a>}}
\if{latex}{\out{\hypertarget{method-FhirResource-resolve_batch}{}}}
\subsection{Method \code{resolve_batch()}}{
Batch-resolve up to 100 FHIR Codings.

Failed items are reported inline without failing the batch.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{FhirResource$resolve_batch(
  codings,
  resource_type = NULL,
  include_recommendations = FALSE,
  recommendations_limit = 5L,
  include_quality = FALSE,
  as_tibble = FALSE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{codings}}{A list of coding lists, each with optional elements
\code{system}, \code{code}, \code{display}, \code{vocabulary_id}.}

\item{\code{resource_type}}{FHIR resource type applied to all codings.}

\item{\code{include_recommendations}}{Logical. Default \code{FALSE}.}

\item{\code{recommendations_limit}}{Integer. Default \code{5L}.}

\item{\code{include_quality}}{Logical. Default \code{FALSE}.}

\item{\code{as_tibble}}{Logical. When \code{TRUE}, returns a \link[tibble:tibble]{tibble::tibble}
with one row per input coding and flat columns for the source
concept, standard concept, target CDM table, mapping type, and
resolution status. The batch \code{summary} (total/resolved/failed)
is attached as \code{attr(result, "summary")}. Default \code{FALSE}
keeps the legacy list-shaped return.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-FhirResource-resolve_codeable_concept"></a>}}
\if{latex}{\out{\hypertarget{method-FhirResource-resolve_codeable_concept}{}}}
\subsection{Method \code{resolve_codeable_concept()}}{
Resolve a FHIR CodeableConcept with vocabulary preference.

Picks the best match per OHDSI preference order
(SNOMED > RxNorm > LOINC > CVX > ICD-10). Falls back to \code{text}
via semantic search if no coding resolves.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{FhirResource$resolve_codeable_concept(
  coding,
  text = NULL,
  resource_type = NULL,
  include_recommendations = FALSE,
  recommendations_limit = 5L,
  include_quality = FALSE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{coding}}{A list of coding lists, each with \code{system}, \code{code},
and optional \code{display}.}

\item{\code{text}}{Optional CodeableConcept.text for semantic fallback.}

\item{\code{resource_type}}{FHIR resource type.}

\item{\code{include_recommendations}}{Logical. Default \code{FALSE}.}

\item{\code{recommendations_limit}}{Integer. Default \code{5L}.}

\item{\code{include_quality}}{Logical. Default \code{FALSE}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-FhirResource-clone"></a>}}
\if{latex}{\out{\hypertarget{method-FhirResource-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{FhirResource$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{extract_error_message}
\alias{extract_error_message}
\title{Extract Error Message from API Response}
\usage{
extract_error_message(resp)
}
\arguments{
\item{resp}{An httr2 response object.}
}
\value{
A character string with the error message.
}
\description{
Extract Error Message from API Response
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pagination.R
\name{paginate_all}
\alias{paginate_all}
\title{Fetch All Paginated Results}
\usage{
paginate_all(fetch_fn, page_size = 100, max_pages = Inf, progress = TRUE)
}
\arguments{
\item{fetch_fn}{A function that takes \code{page} and \code{size} arguments and returns
a list with \code{data} (the results) and \code{meta} (pagination metadata).}

\item{page_size}{Number of items per page. Default 100.}

\item{max_pages}{Maximum number of pages to fetch. Default \code{Inf}.}

\item{progress}{Show progress bar. Default \code{TRUE}.}
}
\value{
A tibble containing all fetched results.
}
\description{
Fetches all pages of results and combines them into a single tibble.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{abort_rate_limit}
\alias{abort_rate_limit}
\title{Abort with Rate Limit Error}
\usage{
abort_rate_limit(retry_after = NULL, call = rlang::caller_env())
}
\arguments{
\item{retry_after}{Seconds until rate limit resets.}

\item{call}{The calling environment.}
}
\description{
Abort with Rate Limit Error
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/relationships.R
\name{RelationshipsResource}
\alias{RelationshipsResource}
\title{Relationships Resource}
\value{
Relationships data with pagination metadata.

Relationship types with pagination metadata.
}
\description{
R6 class providing access to relationship operations.
}
\keyword{internal}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-RelationshipsResource-new}{\code{RelationshipsResource$new()}}
\item \href{#method-RelationshipsResource-get}{\code{RelationshipsResource$get()}}
\item \href{#method-RelationshipsResource-types}{\code{RelationshipsResource$types()}}
\item \href{#method-RelationshipsResource-print}{\code{RelationshipsResource$print()}}
\item \href{#method-RelationshipsResource-clone}{\code{RelationshipsResource$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RelationshipsResource-new"></a>}}
\if{latex}{\out{\hypertarget{method-RelationshipsResource-new}{}}}
\subsection{Method \code{new()}}{
Create a new RelationshipsResource.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RelationshipsResource$new(base_req)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{base_req}}{Base httr2 request object.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RelationshipsResource-get"></a>}}
\if{latex}{\out{\hypertarget{method-RelationshipsResource-get}{}}}
\subsection{Method \code{get()}}{
Get relationships for a concept.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RelationshipsResource$get(
  concept_id,
  relationship_ids = NULL,
  vocabulary_ids = NULL,
  domain_ids = NULL,
  include_invalid = FALSE,
  standard_only = FALSE,
  include_reverse = FALSE,
  page = 1,
  page_size = 100,
  vocab_release = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{concept_id}}{The concept ID.}

\item{\code{relationship_ids}}{Filter by relationship type IDs (character vector or comma-separated string).}

\item{\code{vocabulary_ids}}{Filter by target vocabulary IDs (character vector or comma-separated string).}

\item{\code{domain_ids}}{Filter by target domain IDs (character vector or comma-separated string).}

\item{\code{include_invalid}}{Include relationships to invalid concepts. Default \code{FALSE}.}

\item{\code{standard_only}}{Only include relationships to standard concepts. Default \code{FALSE}.}

\item{\code{include_reverse}}{Include reverse relationships. Default \code{FALSE}.}

\item{\code{page}}{Page number. Default 1.}

\item{\code{page_size}}{Results per page (max 1000). Default 100.}

\item{\code{vocab_release}}{Specific vocabulary release version. Default \code{NULL}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RelationshipsResource-types"></a>}}
\if{latex}{\out{\hypertarget{method-RelationshipsResource-types}{}}}
\subsection{Method \code{types()}}{
Get available relationship types from the OMOP CDM.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RelationshipsResource$types(page = 1, page_size = 100)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{page}}{Page number. Default 1.}

\item{\code{page_size}}{Results per page (max 500). Default 100.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RelationshipsResource-print"></a>}}
\if{latex}{\out{\hypertarget{method-RelationshipsResource-print}{}}}
\subsection{Method \code{print()}}{
Print resource information.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RelationshipsResource$print()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RelationshipsResource-clone"></a>}}
\if{latex}{\out{\hypertarget{method-RelationshipsResource-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RelationshipsResource$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fhir.R
\name{fhir_resolve_batch}
\alias{fhir_resolve_batch}
\title{Batch-resolve FHIR Codings}
\usage{
fhir_resolve_batch(client, ...)
}
\arguments{
\item{client}{An \link{OMOPHubClient} instance.}

\item{...}{Arguments passed to \code{FhirResource$resolve_batch()}.}
}
\value{
See \code{FhirResource$resolve_batch()}.
}
\description{
Standalone wrapper for \code{client$fhir$resolve_batch()}. When
\code{as_tibble = TRUE}, the result is a flat \code{tibble} suitable for
\code{dplyr}/\code{tidyr} pipelines.
}
\examples{
\dontrun{
client <- OMOPHubClient$new(api_key = Sys.getenv("OMOPHUB_API_KEY"))
tbl <- fhir_resolve_batch(
  client,
  codings = list(
    list(system = "http://snomed.info/sct", code = "44054006"),
    list(system = "http://loinc.org", code = "2339-0")
  ),
  as_tibble = TRUE
)
dplyr::filter(tbl, status == "resolved")
}
}
\seealso{
\link{FhirResource}, \code{\link[=fhir_resolve]{fhir_resolve()}},
\code{\link[=fhir_resolve_codeable_concept]{fhir_resolve_codeable_concept()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/response.R
\name{new_omophub_vocabularies}
\alias{new_omophub_vocabularies}
\title{Create OMOPHub Vocabularies Result}
\usage{
new_omophub_vocabularies(data, meta = list())
}
\arguments{
\item{data}{Vocabulary data.}

\item{meta}{Pagination metadata.}
}
\value{
An \code{omophub_vocabularies} S3 object.
}
\description{
Create OMOPHub Vocabularies Result
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auth.R
\name{get_api_key}
\alias{get_api_key}
\title{Get OMOPHub API Key}
\usage{
get_api_key(key = NULL)
}
\arguments{
\item{key}{Optional explicit API key. If provided, this takes precedence.}
}
\value{
A character string containing the API key.
}
\description{
Retrieves the OMOPHub API key from multiple sources in priority order:
\enumerate{
\item Explicit argument
\item \code{OMOPHUB_API_KEY} environment variable
\item System keyring (if \code{keyring} package is installed)
}
}
\examples{
\dontrun{
# From environment variable
Sys.setenv(OMOPHUB_API_KEY = "your_api_key")
key <- get_api_key()

# Explicit key
key <- get_api_key("your_api_key")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hierarchy.R
\name{HierarchyResource}
\alias{HierarchyResource}
\title{Hierarchy Resource}
\value{
For flat format: ancestors, descendants arrays with level/total counts.
For graph format: nodes and edges arrays for visualization.

Ancestors with hierarchy summary.

Descendants with hierarchy summary.
}
\description{
R6 class providing access to hierarchy operations (ancestors and descendants).
}
\keyword{internal}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-HierarchyResource-new}{\code{HierarchyResource$new()}}
\item \href{#method-HierarchyResource-get}{\code{HierarchyResource$get()}}
\item \href{#method-HierarchyResource-ancestors}{\code{HierarchyResource$ancestors()}}
\item \href{#method-HierarchyResource-descendants}{\code{HierarchyResource$descendants()}}
\item \href{#method-HierarchyResource-print}{\code{HierarchyResource$print()}}
\item \href{#method-HierarchyResource-clone}{\code{HierarchyResource$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-HierarchyResource-new"></a>}}
\if{latex}{\out{\hypertarget{method-HierarchyResource-new}{}}}
\subsection{Method \code{new()}}{
Create a new HierarchyResource.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{HierarchyResource$new(base_req)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{base_req}}{Base httr2 request object.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-HierarchyResource-get"></a>}}
\if{latex}{\out{\hypertarget{method-HierarchyResource-get}{}}}
\subsection{Method \code{get()}}{
Get complete concept hierarchy (ancestors and descendants).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{HierarchyResource$get(
  concept_id,
  format = "flat",
  vocabulary_ids = NULL,
  domain_ids = NULL,
  max_levels = 10,
  max_results = NULL,
  relationship_types = NULL,
  include_invalid = FALSE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{concept_id}}{The concept ID.}

\item{\code{format}}{Response format - "flat" (default) or "graph" for visualization.}

\item{\code{vocabulary_ids}}{Filter to specific vocabularies (character vector).}

\item{\code{domain_ids}}{Filter to specific domains (character vector).}

\item{\code{max_levels}}{Maximum hierarchy levels to traverse in both directions (default 10).}

\item{\code{max_results}}{Maximum results per direction for performance optimization.}

\item{\code{relationship_types}}{Relationship types to follow (default: "Is a").}

\item{\code{include_invalid}}{Include deprecated/invalid concepts. Default \code{FALSE}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-HierarchyResource-ancestors"></a>}}
\if{latex}{\out{\hypertarget{method-HierarchyResource-ancestors}{}}}
\subsection{Method \code{ancestors()}}{
Get concept ancestors.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{HierarchyResource$ancestors(
  concept_id,
  vocabulary_ids = NULL,
  max_levels = NULL,
  relationship_types = NULL,
  include_paths = FALSE,
  include_distance = TRUE,
  include_invalid = FALSE,
  page = 1,
  page_size = 100
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{concept_id}}{The concept ID.}

\item{\code{vocabulary_ids}}{Filter to specific vocabularies (character vector).}

\item{\code{max_levels}}{Maximum hierarchy levels to traverse.}

\item{\code{relationship_types}}{Relationship types to follow (default: "Is a").}

\item{\code{include_paths}}{Include path information. Default \code{FALSE}.}

\item{\code{include_distance}}{Include distance from source. Default \code{TRUE}.}

\item{\code{include_invalid}}{Include deprecated/invalid concepts. Default \code{FALSE}.}

\item{\code{page}}{Page number. Default 1.}

\item{\code{page_size}}{Results per page. Default 100.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-HierarchyResource-descendants"></a>}}
\if{latex}{\out{\hypertarget{method-HierarchyResource-descendants}{}}}
\subsection{Method \code{descendants()}}{
Get concept descendants.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{HierarchyResource$descendants(
  concept_id,
  vocabulary_ids = NULL,
  max_levels = 10,
  relationship_types = NULL,
  include_distance = TRUE,
  include_paths = FALSE,
  include_invalid = FALSE,
  domain_ids = NULL,
  page = 1,
  page_size = 100
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{concept_id}}{The concept ID.}

\item{\code{vocabulary_ids}}{Filter to specific vocabularies (character vector).}

\item{\code{max_levels}}{Maximum hierarchy levels (default 10, max 20).}

\item{\code{relationship_types}}{Relationship types to follow.}

\item{\code{include_distance}}{Include distance from source. Default \code{TRUE}.}

\item{\code{include_paths}}{Include path information. Default \code{FALSE}.}

\item{\code{include_invalid}}{Include deprecated/invalid concepts. Default \code{FALSE}.}

\item{\code{domain_ids}}{Filter by domains.}

\item{\code{page}}{Page number. Default 1.}

\item{\code{page_size}}{Results per page. Default 100.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-HierarchyResource-print"></a>}}
\if{latex}{\out{\hypertarget{method-HierarchyResource-print}{}}}
\subsection{Method \code{print()}}{
Print resource information.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{HierarchyResource$print()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-HierarchyResource-clone"></a>}}
\if{latex}{\out{\hypertarget{method-HierarchyResource-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{HierarchyResource$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{join_params}
\alias{join_params}
\title{Join List Elements for Query Parameter}
\usage{
join_params(x, sep = ",")
}
\arguments{
\item{x}{A character vector.}

\item{sep}{Separator (default comma).}
}
\value{
A single comma-separated string, or NULL if x is NULL/empty.
}
\description{
Join List Elements for Query Parameter
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{abort_auth_error}
\alias{abort_auth_error}
\title{Abort with Authentication Error}
\usage{
abort_auth_error(message, call = rlang::caller_env())
}
\arguments{
\item{message}{Error message.}

\item{call}{The calling environment.}
}
\description{
Abort with Authentication Error
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/omophub-package.R
\docType{package}
\name{omophub-package}
\alias{omophub-package}
\alias{omophub}
\title{omophub: R Client for the 'OMOPHub' Medical Vocabulary API}
\description{
Provides an R interface to the 'OMOPHub' API for accessing 'OHDSI ATHENA' standardized medical vocabularies. Supports concept search, semantic search using neural embeddings, concept similarity, vocabulary exploration, hierarchy navigation, relationship queries, concept mappings, and FHIR-to-OMOP concept resolution with automatic pagination.

Provides an R interface to the OMOPHub API for accessing OHDSI ATHENA
standardized medical vocabularies. Supports concept search, vocabulary
exploration, hierarchy navigation, relationship queries, and concept
mappings with automatic pagination and rate limiting.
}
\section{Main Client}{

The main entry point is the \link{OMOPHubClient} R6 class which provides
access to all API resources:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{library(omophub)
client <- OMOPHubClient$new()

# Search for concepts
results <- client$search$basic("diabetes")

# Get a specific concept
concept <- client$concepts$get(201826)
}\if{html}{\out{</div>}}
}

\section{Authentication}{

Set your API key using one of these methods:
\itemize{
\item Environment variable: \code{OMOPHUB_API_KEY}
\item Explicit argument: \code{OMOPHubClient$new(api_key = "your_key")}
\item Keyring: \code{set_api_key("your_key", store = "keyring")}
}
}

\section{Resources}{

The client provides access to these resources:
\itemize{
\item \code{concepts}: Concept lookup and batch operations
\item \code{search}: Basic and advanced concept search
\item \code{vocabularies}: Vocabulary listing and details
\item \code{domains}: Domain listing and concept filtering
\item \code{hierarchy}: Ancestor and descendant navigation
\item \code{relationships}: Concept relationships
\item \code{mappings}: Concept mappings between vocabularies
}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/omopHub/omophub-R}
  \item \url{https://docs.omophub.com}
  \item \url{https://omophub.github.io/omophub-R/}
  \item \url{https://omophub.com}
  \item Report bugs at \url{https://github.com/omopHub/omophub-R/issues}
}

}
\author{
\strong{Maintainer}: Alex Chen \email{alex@omophub.com} [copyright holder]

Other contributors:
\itemize{
  \item Observational Health Data Science and Informatics [copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{.onAttach}
\alias{.onAttach}
\title{Package Attach Hook}
\usage{
.onAttach(libname, pkgname)
}
\arguments{
\item{libname}{Library name.}

\item{pkgname}{Package name.}
}
\description{
Package Attach Hook
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/domains.R
\name{DomainsResource}
\alias{DomainsResource}
\title{Domains Resource}
\value{
Domain list.

Paginated concepts.
}
\description{
R6 class providing access to domain operations.
}
\keyword{internal}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-DomainsResource-new}{\code{DomainsResource$new()}}
\item \href{#method-DomainsResource-list}{\code{DomainsResource$list()}}
\item \href{#method-DomainsResource-concepts}{\code{DomainsResource$concepts()}}
\item \href{#method-DomainsResource-print}{\code{DomainsResource$print()}}
\item \href{#method-DomainsResource-clone}{\code{DomainsResource$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DomainsResource-new"></a>}}
\if{latex}{\out{\hypertarget{method-DomainsResource-new}{}}}
\subsection{Method \code{new()}}{
Create a new DomainsResource.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DomainsResource$new(base_req)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{base_req}}{Base httr2 request object.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DomainsResource-list"></a>}}
\if{latex}{\out{\hypertarget{method-DomainsResource-list}{}}}
\subsection{Method \code{list()}}{
List all domains.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DomainsResource$list(include_stats = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{include_stats}}{Include concept counts and vocabulary coverage. Default \code{FALSE}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DomainsResource-concepts"></a>}}
\if{latex}{\out{\hypertarget{method-DomainsResource-concepts}{}}}
\subsection{Method \code{concepts()}}{
Get concepts in a domain.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DomainsResource$concepts(
  domain_id,
  vocabulary_ids = NULL,
  standard_only = FALSE,
  include_invalid = FALSE,
  page = 1,
  page_size = 50
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{domain_id}}{The domain ID.}

\item{\code{vocabulary_ids}}{Filter by vocabularies.}

\item{\code{standard_only}}{Only standard concepts. Default \code{FALSE}.}

\item{\code{include_invalid}}{Include invalid/deprecated concepts. Default \code{FALSE}.}

\item{\code{page}}{Page number. Default 1.}

\item{\code{page_size}}{Results per page. Default 50.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DomainsResource-print"></a>}}
\if{latex}{\out{\hypertarget{method-DomainsResource-print}{}}}
\subsection{Method \code{print()}}{
Print resource information.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DomainsResource$print()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DomainsResource-clone"></a>}}
\if{latex}{\out{\hypertarget{method-DomainsResource-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DomainsResource$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auth.R
\name{has_api_key}
\alias{has_api_key}
\title{Check if API Key is Available}
\usage{
has_api_key()
}
\value{
\code{TRUE} if an API key is available, \code{FALSE} otherwise.
}
\description{
Checks whether an OMOPHub API key is available without throwing an error.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{abort_validation}
\alias{abort_validation}
\title{Abort with Validation Error}
\usage{
abort_validation(message, arg = NULL, call = rlang::caller_env())
}
\arguments{
\item{message}{Error message.}

\item{arg}{The argument that failed validation.}

\item{call}{The calling environment.}
}
\description{
Abort with Validation Error
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fhir_interop.R
\name{omophub_fhir_url}
\alias{omophub_fhir_url}
\title{OMOPHub FHIR Terminology Service URL}
\usage{
omophub_fhir_url(version = c("r4", "r4b", "r5", "r6"))
}
\arguments{
\item{version}{FHIR version prefix. One of \code{"r4"} (default),
\code{"r4b"}, \code{"r5"}, or \code{"r6"}.}
}
\value{
A character scalar with the full FHIR base URL, e.g.
\code{"https://fhir.omophub.com/fhir/r4"}.
}
\description{
Convenience helper for constructing the OMOPHub FHIR Terminology
Service base URL for a given FHIR version. Use it when configuring
an external FHIR client library (\code{httr2}, \code{fhircrackr}, etc.) to
talk to OMOPHub's FHIR endpoint directly.
}
\examples{
omophub_fhir_url()
omophub_fhir_url("r5")

\dontrun{
# Use with httr2 to call the $lookup operation directly
library(httr2)
req <- request(omophub_fhir_url()) |>
  req_url_path_append("CodeSystem/$lookup") |>
  req_url_query(
    system = "http://snomed.info/sct",
    code = "44054006"
  ) |>
  req_headers(Authorization = paste("Bearer", Sys.getenv("OMOPHUB_API_KEY")))
resp <- req_perform(req)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fhir.R
\name{fhir_resolve}
\alias{fhir_resolve}
\title{Resolve a FHIR Coding to an OMOP standard concept}
\usage{
fhir_resolve(client, ...)
}
\arguments{
\item{client}{An \link{OMOPHubClient} instance.}

\item{...}{Arguments passed to \code{FhirResource$resolve()}.}
}
\value{
A list with \code{input} and \code{resolution}.
}
\description{
Standalone wrapper for \code{client$fhir$resolve()}. See
\link{FhirResource} for full parameter documentation.
}
\examples{
\dontrun{
client <- OMOPHubClient$new(api_key = Sys.getenv("OMOPHUB_API_KEY"))
fhir_resolve(
  client,
  system = "http://snomed.info/sct",
  code = "44054006",
  resource_type = "Condition"
)
}
}
\seealso{
\link{FhirResource}, \code{\link[=fhir_resolve_batch]{fhir_resolve_batch()}},
\code{\link[=fhir_resolve_codeable_concept]{fhir_resolve_codeable_concept()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/response.R
\name{new_omophub_hierarchy}
\alias{new_omophub_hierarchy}
\title{Create OMOPHub Hierarchy Result}
\usage{
new_omophub_hierarchy(data, meta = list(), type = "hierarchy")
}
\arguments{
\item{data}{Hierarchy data (ancestors or descendants).}

\item{meta}{Metadata including summary.}

\item{type}{Type of hierarchy ("ancestors" or "descendants").}
}
\value{
An \code{omophub_hierarchy} S3 object.
}
\description{
Create OMOPHub Hierarchy Result
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/request.R
\name{build_request}
\alias{build_request}
\title{Build Base OMOPHub Request}
\usage{
build_request(
  base_url,
  api_key,
  timeout = 30,
  max_retries = 3,
  vocab_version = NULL
)
}
\arguments{
\item{base_url}{API base URL.}

\item{api_key}{API key for authentication.}

\item{timeout}{Request timeout in seconds.}

\item{max_retries}{Maximum retry attempts.}

\item{vocab_version}{Optional vocabulary version.}
}
\value{
An httr2 request object.
}
\description{
Creates an httr2 request object with authentication, rate limiting,
and retry logic configured.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pagination.R
\name{paginate_lazy}
\alias{paginate_lazy}
\title{Create Lazy Pagination Iterator}
\usage{
paginate_lazy(fetch_fn, page_size = 100)
}
\arguments{
\item{fetch_fn}{A function that takes \code{page} and \code{size} arguments.}

\item{page_size}{Number of items per page. Default 100.}
}
\value{
An iterator object with \code{next_page()} and \code{has_more()} methods.
}
\description{
Creates an iterator that fetches pages on demand.
}
\keyword{Check}
\keyword{Get}
\keyword{Reset}
\keyword{are}
\keyword{available}
\keyword{beginning}
\keyword{current}
\keyword{if}
\keyword{internal}
\keyword{iterator}
\keyword{more}
\keyword{next}
\keyword{number}
\keyword{of}
\keyword{page}
\keyword{pages}
\keyword{results}
\keyword{to}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{abort_api_error}
\alias{abort_api_error}
\title{Abort with OMOPHub API Error}
\usage{
abort_api_error(status, message, endpoint, call = rlang::caller_env())
}
\arguments{
\item{status}{HTTP status code.}

\item{message}{Error message.}

\item{endpoint}{API endpoint that failed.}

\item{call}{The calling environment.}
}
\description{
Abort with OMOPHub API Error
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{bool_to_str}
\alias{bool_to_str}
\title{Convert Boolean to API String}
\usage{
bool_to_str(x)
}
\arguments{
\item{x}{A logical value.}
}
\value{
"true" or "false" string, or NULL if x is NULL.
}
\description{
Convert Boolean to API String
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fhir.R
\name{compact_list}
\alias{compact_list}
\title{Helper to remove NULL entries from a named list.}
\usage{
compact_list(...)
}
\arguments{
\item{...}{Named arguments.}
}
\value{
A list with NULL entries removed.
}
\description{
Helper to remove NULL entries from a named list.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{validate_concept_id}
\alias{validate_concept_id}
\title{Validate Concept ID}
\usage{
validate_concept_id(concept_id, arg = "concept_id", call = rlang::caller_env())
}
\arguments{
\item{concept_id}{A concept ID to validate.}

\item{arg}{Argument name for error messages.}

\item{call}{The calling environment.}
}
\description{
Validate Concept ID
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{is_transient_error}
\alias{is_transient_error}
\title{Check if Error is Transient (Retryable)}
\usage{
is_transient_error(resp)
}
\arguments{
\item{resp}{An httr2 response object.}
}
\value{
\code{TRUE} if the error is transient and should be retried.
}
\description{
Check if Error is Transient (Retryable)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vocabularies.R
\name{VocabulariesResource}
\alias{VocabulariesResource}
\title{Vocabularies Resource}
\value{
Paginated vocabulary list.

Vocabulary details including vocabulary_id, vocabulary_name,
vocabulary_reference, vocabulary_version, vocabulary_concept_id.

Vocabulary statistics.

Domain statistics including concept counts and class breakdown.

List of all available domains with domain_id, domain_name, and description.

List of all available concept classes with concept_class_id,
concept_class_name, and concept_class_concept_id.

Paginated concepts.
}
\description{
R6 class providing access to vocabulary operations.
}
\keyword{internal}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-VocabulariesResource-new}{\code{VocabulariesResource$new()}}
\item \href{#method-VocabulariesResource-list}{\code{VocabulariesResource$list()}}
\item \href{#method-VocabulariesResource-get}{\code{VocabulariesResource$get()}}
\item \href{#method-VocabulariesResource-stats}{\code{VocabulariesResource$stats()}}
\item \href{#method-VocabulariesResource-domain_stats}{\code{VocabulariesResource$domain_stats()}}
\item \href{#method-VocabulariesResource-domains}{\code{VocabulariesResource$domains()}}
\item \href{#method-VocabulariesResource-concept_classes}{\code{VocabulariesResource$concept_classes()}}
\item \href{#method-VocabulariesResource-concepts}{\code{VocabulariesResource$concepts()}}
\item \href{#method-VocabulariesResource-print}{\code{VocabulariesResource$print()}}
\item \href{#method-VocabulariesResource-clone}{\code{VocabulariesResource$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VocabulariesResource-new"></a>}}
\if{latex}{\out{\hypertarget{method-VocabulariesResource-new}{}}}
\subsection{Method \code{new()}}{
Create a new VocabulariesResource.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VocabulariesResource$new(base_req)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{base_req}}{Base httr2 request object.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VocabulariesResource-list"></a>}}
\if{latex}{\out{\hypertarget{method-VocabulariesResource-list}{}}}
\subsection{Method \code{list()}}{
List all vocabularies.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VocabulariesResource$list(
  include_stats = FALSE,
  include_inactive = FALSE,
  sort_by = "name",
  sort_order = "asc",
  page = 1,
  page_size = 100
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{include_stats}}{Include vocabulary statistics. Default \code{FALSE}.}

\item{\code{include_inactive}}{Include inactive vocabularies. Default \code{FALSE}.}

\item{\code{sort_by}}{Sort field ("name", "priority", "updated"). Default "name".}

\item{\code{sort_order}}{Sort order ("asc" or "desc"). Default "asc".}

\item{\code{page}}{Page number. Default 1.}

\item{\code{page_size}}{Results per page. Default 100.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VocabulariesResource-get"></a>}}
\if{latex}{\out{\hypertarget{method-VocabulariesResource-get}{}}}
\subsection{Method \code{get()}}{
Get vocabulary details.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VocabulariesResource$get(vocabulary_id)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{vocabulary_id}}{The vocabulary ID.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VocabulariesResource-stats"></a>}}
\if{latex}{\out{\hypertarget{method-VocabulariesResource-stats}{}}}
\subsection{Method \code{stats()}}{
Get vocabulary statistics.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VocabulariesResource$stats(vocabulary_id)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{vocabulary_id}}{The vocabulary ID.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VocabulariesResource-domain_stats"></a>}}
\if{latex}{\out{\hypertarget{method-VocabulariesResource-domain_stats}{}}}
\subsection{Method \code{domain_stats()}}{
Get statistics for a specific domain within a vocabulary.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VocabulariesResource$domain_stats(vocabulary_id, domain_id)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{vocabulary_id}}{The vocabulary ID (e.g., "SNOMED", "ICD10CM").}

\item{\code{domain_id}}{The domain ID (e.g., "Condition", "Drug", "Procedure").}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VocabulariesResource-domains"></a>}}
\if{latex}{\out{\hypertarget{method-VocabulariesResource-domains}{}}}
\subsection{Method \code{domains()}}{
Get all standard OHDSI domains.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VocabulariesResource$domains()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VocabulariesResource-concept_classes"></a>}}
\if{latex}{\out{\hypertarget{method-VocabulariesResource-concept_classes}{}}}
\subsection{Method \code{concept_classes()}}{
Get all concept classes.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VocabulariesResource$concept_classes()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VocabulariesResource-concepts"></a>}}
\if{latex}{\out{\hypertarget{method-VocabulariesResource-concepts}{}}}
\subsection{Method \code{concepts()}}{
Get concepts in a vocabulary.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VocabulariesResource$concepts(
  vocabulary_id,
  search = NULL,
  standard_concept = "all",
  include_invalid = FALSE,
  include_relationships = FALSE,
  include_synonyms = FALSE,
  sort_by = "name",
  sort_order = "asc",
  page = 1,
  page_size = 20
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{vocabulary_id}}{The vocabulary ID.}

\item{\code{search}}{Search term to filter concepts by name or code.}

\item{\code{standard_concept}}{Filter by standard concept status ('S', 'C', 'all'). Default "all".}

\item{\code{include_invalid}}{Include invalid or deprecated concepts. Default \code{FALSE}.}

\item{\code{include_relationships}}{Include concept relationships. Default \code{FALSE}.}

\item{\code{include_synonyms}}{Include concept synonyms. Default \code{FALSE}.}

\item{\code{sort_by}}{Sort field ('name', 'concept_id', 'concept_code'). Default "name".}

\item{\code{sort_order}}{Sort order ('asc' or 'desc'). Default "asc".}

\item{\code{page}}{Page number. Default 1.}

\item{\code{page_size}}{Results per page (max 1000). Default 20.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VocabulariesResource-print"></a>}}
\if{latex}{\out{\hypertarget{method-VocabulariesResource-print}{}}}
\subsection{Method \code{print()}}{
Print resource information.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VocabulariesResource$print()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VocabulariesResource-clone"></a>}}
\if{latex}{\out{\hypertarget{method-VocabulariesResource-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VocabulariesResource$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/response.R
\name{new_omophub_relationships}
\alias{new_omophub_relationships}
\title{Create OMOPHub Relationships Result}
\usage{
new_omophub_relationships(data, meta = list())
}
\arguments{
\item{data}{Relationship data.}

\item{meta}{Metadata.}
}
\value{
An \code{omophub_relationships} S3 object.
}
\description{
Create OMOPHub Relationships Result
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fhir.R
\name{fhir_batch_to_tibble}
\alias{fhir_batch_to_tibble}
\title{Flatten a batch resolver response into a tibble.}
\usage{
fhir_batch_to_tibble(result, codings)
}
\arguments{
\item{result}{The raw list result from \code{perform_post}.}

\item{codings}{The original input \code{codings} list, used to align rows
when items fail resolution and no \code{source_concept} is returned.}
}
\value{
A \link[tibble:tibble]{tibble::tibble}.
}
\description{
Internal helper used by \code{FhirResource$resolve_batch(as_tibble = TRUE)}.
Produces one row per input coding with flat columns for the source
concept, standard concept, target CDM table, and status. The original
\code{summary} list is attached to the returned tibble via
\code{attr(x, "summary")}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/search.R
\name{SearchResource}
\alias{SearchResource}
\title{Search Resource}
\value{
Search results with pagination.

A tibble of all matching concepts.

Search results with facets and metadata.

Autocomplete suggestions.

List with results and pagination metadata.

A tibble of all matching concepts with similarity scores.

List with \code{results} (per-search), \code{total_searches},
\code{completed_searches}, \code{failed_searches}.

List with \code{results} (per-search), \code{total_searches},
\code{completed_count}, \code{failed_count}, \code{total_duration}.

List with similar_concepts and search_metadata.
}
\description{
R6 class providing access to search operations.
}
\note{
When algorithm='semantic', only single vocabulary/domain filter supported.
}
\keyword{internal}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-SearchResource-new}{\code{SearchResource$new()}}
\item \href{#method-SearchResource-basic}{\code{SearchResource$basic()}}
\item \href{#method-SearchResource-basic_all}{\code{SearchResource$basic_all()}}
\item \href{#method-SearchResource-advanced}{\code{SearchResource$advanced()}}
\item \href{#method-SearchResource-autocomplete}{\code{SearchResource$autocomplete()}}
\item \href{#method-SearchResource-semantic}{\code{SearchResource$semantic()}}
\item \href{#method-SearchResource-semantic_all}{\code{SearchResource$semantic_all()}}
\item \href{#method-SearchResource-bulk_basic}{\code{SearchResource$bulk_basic()}}
\item \href{#method-SearchResource-bulk_semantic}{\code{SearchResource$bulk_semantic()}}
\item \href{#method-SearchResource-similar}{\code{SearchResource$similar()}}
\item \href{#method-SearchResource-print}{\code{SearchResource$print()}}
\item \href{#method-SearchResource-clone}{\code{SearchResource$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-SearchResource-new"></a>}}
\if{latex}{\out{\hypertarget{method-SearchResource-new}{}}}
\subsection{Method \code{new()}}{
Create a new SearchResource.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SearchResource$new(base_req)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{base_req}}{Base httr2 request object.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-SearchResource-basic"></a>}}
\if{latex}{\out{\hypertarget{method-SearchResource-basic}{}}}
\subsection{Method \code{basic()}}{
Basic concept search.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SearchResource$basic(
  query,
  vocabulary_ids = NULL,
  domain_ids = NULL,
  concept_class_ids = NULL,
  standard_concept = NULL,
  include_synonyms = FALSE,
  include_invalid = FALSE,
  min_score = NULL,
  exact_match = FALSE,
  page = 1,
  page_size = 20,
  sort_by = NULL,
  sort_order = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{query}}{Search query string.}

\item{\code{vocabulary_ids}}{Filter by vocabulary IDs.}

\item{\code{domain_ids}}{Filter by domain IDs.}

\item{\code{concept_class_ids}}{Filter by concept class IDs.}

\item{\code{standard_concept}}{Filter by standard concept ("S", "C", or NULL).}

\item{\code{include_synonyms}}{Search in synonyms. Default \code{FALSE}.}

\item{\code{include_invalid}}{Include invalid concepts. Default \code{FALSE}.}

\item{\code{min_score}}{Minimum relevance score.}

\item{\code{exact_match}}{Require exact match. Default \code{FALSE}.}

\item{\code{page}}{Page number (1-based). Default 1.}

\item{\code{page_size}}{Results per page. Default 20.}

\item{\code{sort_by}}{Sort field.}

\item{\code{sort_order}}{Sort order ("asc" or "desc").}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-SearchResource-basic_all"></a>}}
\if{latex}{\out{\hypertarget{method-SearchResource-basic_all}{}}}
\subsection{Method \code{basic_all()}}{
Fetch all search results with automatic pagination.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SearchResource$basic_all(
  query,
  vocabulary_ids = NULL,
  domain_ids = NULL,
  page_size = 100,
  max_pages = Inf,
  progress = TRUE,
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{query}}{Search query string.}

\item{\code{vocabulary_ids}}{Filter by vocabulary IDs.}

\item{\code{domain_ids}}{Filter by domain IDs.}

\item{\code{page_size}}{Results per page. Default 100.}

\item{\code{max_pages}}{Maximum pages to fetch. Default Inf.}

\item{\code{progress}}{Show progress bar. Default \code{TRUE}.}

\item{\code{...}}{Additional search parameters.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-SearchResource-advanced"></a>}}
\if{latex}{\out{\hypertarget{method-SearchResource-advanced}{}}}
\subsection{Method \code{advanced()}}{
Advanced concept search with facets.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SearchResource$advanced(
  query,
  vocabulary_ids = NULL,
  domain_ids = NULL,
  concept_class_ids = NULL,
  standard_concepts_only = FALSE,
  include_invalid = FALSE,
  relationship_filters = NULL,
  page = 1,
  page_size = 20
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{query}}{Search query string.}

\item{\code{vocabulary_ids}}{Filter by vocabulary IDs.}

\item{\code{domain_ids}}{Filter by domain IDs.}

\item{\code{concept_class_ids}}{Filter by concept class IDs.}

\item{\code{standard_concepts_only}}{Only return standard concepts. Default \code{FALSE}.}

\item{\code{include_invalid}}{Include invalid concepts. Default \code{FALSE}.}

\item{\code{relationship_filters}}{Relationship-based filters.}

\item{\code{page}}{Page number (1-based). Default 1.}

\item{\code{page_size}}{Results per page. Default 20.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-SearchResource-autocomplete"></a>}}
\if{latex}{\out{\hypertarget{method-SearchResource-autocomplete}{}}}
\subsection{Method \code{autocomplete()}}{
Get autocomplete suggestions.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SearchResource$autocomplete(
  query,
  vocabulary_ids = NULL,
  domains = NULL,
  max_suggestions = 10
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{query}}{Partial query string.}

\item{\code{vocabulary_ids}}{Filter by vocabulary IDs.}

\item{\code{domains}}{Filter by domains.}

\item{\code{max_suggestions}}{Maximum suggestions. Default 10.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-SearchResource-semantic"></a>}}
\if{latex}{\out{\hypertarget{method-SearchResource-semantic}{}}}
\subsection{Method \code{semantic()}}{
Semantic concept search using neural embeddings.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SearchResource$semantic(
  query,
  vocabulary_ids = NULL,
  domain_ids = NULL,
  standard_concept = NULL,
  concept_class_id = NULL,
  threshold = NULL,
  page = 1,
  page_size = 20
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{query}}{Natural language search query (required).}

\item{\code{vocabulary_ids}}{Filter by vocabulary IDs.}

\item{\code{domain_ids}}{Filter by domain IDs.}

\item{\code{standard_concept}}{Filter by standard concept ('S' or 'C').}

\item{\code{concept_class_id}}{Filter by concept class ID.}

\item{\code{threshold}}{Minimum similarity threshold (0.0-1.0, default 0.5).}

\item{\code{page}}{Page number (1-based). Default 1.}

\item{\code{page_size}}{Results per page (max 100). Default 20.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-SearchResource-semantic_all"></a>}}
\if{latex}{\out{\hypertarget{method-SearchResource-semantic_all}{}}}
\subsection{Method \code{semantic_all()}}{
Fetch all semantic search results with automatic pagination.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SearchResource$semantic_all(
  query,
  vocabulary_ids = NULL,
  domain_ids = NULL,
  standard_concept = NULL,
  concept_class_id = NULL,
  threshold = NULL,
  page_size = 100,
  max_pages = Inf,
  progress = TRUE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{query}}{Natural language search query (required).}

\item{\code{vocabulary_ids}}{Filter by vocabulary IDs.}

\item{\code{domain_ids}}{Filter by domain IDs.}

\item{\code{standard_concept}}{Filter by standard concept ('S' or 'C').}

\item{\code{concept_class_id}}{Filter by concept class ID.}

\item{\code{threshold}}{Minimum similarity threshold (0.0-1.0).}

\item{\code{page_size}}{Results per page. Default 100.}

\item{\code{max_pages}}{Maximum pages to fetch. Default Inf.}

\item{\code{progress}}{Show progress bar. Default \code{TRUE}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-SearchResource-bulk_basic"></a>}}
\if{latex}{\out{\hypertarget{method-SearchResource-bulk_basic}{}}}
\subsection{Method \code{bulk_basic()}}{
Execute multiple lexical searches in a single request (max 50).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SearchResource$bulk_basic(searches, defaults = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{searches}}{List of search inputs. Each element is a named list with:
\itemize{
\item \code{search_id} (required): Unique ID to match results.
\item \code{query} (required): Search query string.
\item \code{vocabulary_ids}, \code{domain_ids}, \code{concept_class_ids}: Optional filters.
\item \code{standard_concept}, \code{include_invalid}, \code{page_size}: Optional params.
}}

\item{\code{defaults}}{Named list of default filters applied to all searches.
Individual search-level values override defaults.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-SearchResource-bulk_semantic"></a>}}
\if{latex}{\out{\hypertarget{method-SearchResource-bulk_semantic}{}}}
\subsection{Method \code{bulk_semantic()}}{
Execute multiple semantic searches in a single request (max 25).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SearchResource$bulk_semantic(searches, defaults = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{searches}}{List of search inputs. Each element is a named list with:
\itemize{
\item \code{search_id} (required): Unique ID to match results.
\item \code{query} (required): Natural language query (1-500 chars).
\item \code{threshold}: Per-search similarity threshold (0-1).
\item \code{page_size}: Per-search result limit (1-50).
\item \code{vocabulary_ids}, \code{domain_ids}, \code{standard_concept}: Optional filters.
}}

\item{\code{defaults}}{Named list of default filters applied to all searches.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-SearchResource-similar"></a>}}
\if{latex}{\out{\hypertarget{method-SearchResource-similar}{}}}
\subsection{Method \code{similar()}}{
Find concepts similar to a reference concept or query.

Must provide exactly one of: concept_id, concept_name, or query.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SearchResource$similar(
  concept_id = NULL,
  concept_name = NULL,
  query = NULL,
  algorithm = "hybrid",
  similarity_threshold = 0.7,
  page_size = 20,
  vocabulary_ids = NULL,
  domain_ids = NULL,
  standard_concept = NULL,
  include_invalid = NULL,
  include_scores = NULL,
  include_explanations = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{concept_id}}{Concept ID to find similar concepts for.}

\item{\code{concept_name}}{Concept name to find similar concepts for.}

\item{\code{query}}{Natural language query for semantic similarity.}

\item{\code{algorithm}}{One of 'semantic', 'lexical', or 'hybrid' (default).}

\item{\code{similarity_threshold}}{Minimum similarity (0.0-1.0). Default 0.7.}

\item{\code{page_size}}{Max results (max 1000). Default 20.}

\item{\code{vocabulary_ids}}{Filter by vocabulary IDs.}

\item{\code{domain_ids}}{Filter by domain IDs.}

\item{\code{standard_concept}}{Filter by standard concept flag ('S', 'C', or 'N').}

\item{\code{include_invalid}}{Include invalid/deprecated concepts.}

\item{\code{include_scores}}{Include detailed similarity scores.}

\item{\code{include_explanations}}{Include similarity explanations.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-SearchResource-print"></a>}}
\if{latex}{\out{\hypertarget{method-SearchResource-print}{}}}
\subsection{Method \code{print()}}{
Print resource information.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SearchResource$print()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-SearchResource-clone"></a>}}
\if{latex}{\out{\hypertarget{method-SearchResource-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SearchResource$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auth.R
\name{set_api_key}
\alias{set_api_key}
\title{Set OMOPHub API Key}
\usage{
set_api_key(key, store = c("env", "keyring"))
}
\arguments{
\item{key}{The API key to store.}

\item{store}{Where to store the key. One of:
\itemize{
\item \code{"env"}: Set as environment variable for current session (default)
\item \code{"keyring"}: Store securely in system keyring (requires \code{keyring} package)
}}
}
\value{
Invisibly returns \code{TRUE} on success.
}
\description{
Stores the OMOPHub API key in the specified location.
}
\examples{
\dontrun{
# Store in environment (current session only)
set_api_key("your_api_key")

# Store securely in keyring (persistent)
set_api_key("your_api_key", store = "keyring")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/request.R
\name{perform_post}
\alias{perform_post}
\title{Perform POST Request}
\usage{
perform_post(base_req, endpoint, body = NULL, query = NULL)
}
\arguments{
\item{base_req}{Base request object.}

\item{endpoint}{API endpoint path.}

\item{body}{Named list for JSON body.}

\item{query}{Named list of query parameters.}
}
\value{
Parsed JSON response (unwrapped from \code{data} field if present).
}
\description{
Perform POST Request
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{.onLoad}
\alias{.onLoad}
\title{Package Load Hook}
\usage{
.onLoad(libname, pkgname)
}
\arguments{
\item{libname}{Library name.}

\item{pkgname}{Package name.}
}
\description{
Package Load Hook
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fhir.R
\name{fhir_resolve_codeable_concept}
\alias{fhir_resolve_codeable_concept}
\title{Resolve a FHIR CodeableConcept with vocabulary preference}
\usage{
fhir_resolve_codeable_concept(client, ...)
}
\arguments{
\item{client}{An \link{OMOPHubClient} instance.}

\item{...}{Arguments passed to \code{FhirResource$resolve_codeable_concept()}.}
}
\value{
See \code{FhirResource$resolve_codeable_concept()}.
}
\description{
Standalone wrapper for \code{client$fhir$resolve_codeable_concept()}.
}
\examples{
\dontrun{
client <- OMOPHubClient$new(api_key = Sys.getenv("OMOPHUB_API_KEY"))
fhir_resolve_codeable_concept(
  client,
  coding = list(
    list(system = "http://snomed.info/sct", code = "44054006"),
    list(system = "http://hl7.org/fhir/sid/icd-10-cm", code = "E11.9")
  ),
  resource_type = "Condition"
)
}
}
\seealso{
\link{FhirResource}, \code{\link[=fhir_resolve]{fhir_resolve()}}, \code{\link[=fhir_resolve_batch]{fhir_resolve_batch()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mappings.R
\name{MappingsResource}
\alias{MappingsResource}
\title{Mappings Resource}
\value{
Mappings for the concept.

Mapping results with summary.
}
\description{
R6 class providing access to mapping operations.
}
\keyword{internal}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-MappingsResource-new}{\code{MappingsResource$new()}}
\item \href{#method-MappingsResource-get}{\code{MappingsResource$get()}}
\item \href{#method-MappingsResource-map}{\code{MappingsResource$map()}}
\item \href{#method-MappingsResource-print}{\code{MappingsResource$print()}}
\item \href{#method-MappingsResource-clone}{\code{MappingsResource$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-MappingsResource-new"></a>}}
\if{latex}{\out{\hypertarget{method-MappingsResource-new}{}}}
\subsection{Method \code{new()}}{
Create a new MappingsResource.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MappingsResource$new(base_req)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{base_req}}{Base httr2 request object.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-MappingsResource-get"></a>}}
\if{latex}{\out{\hypertarget{method-MappingsResource-get}{}}}
\subsection{Method \code{get()}}{
Get mappings for a concept.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MappingsResource$get(
  concept_id,
  target_vocabulary = NULL,
  include_invalid = FALSE,
  vocab_release = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{concept_id}}{The concept ID.}

\item{\code{target_vocabulary}}{Filter to a specific target vocabulary (e.g., "ICD10CM").}

\item{\code{include_invalid}}{Include invalid/deprecated mappings. Default \code{FALSE}.}

\item{\code{vocab_release}}{Specific vocabulary release version (e.g., "2025.1"). Default \code{NULL}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-MappingsResource-map"></a>}}
\if{latex}{\out{\hypertarget{method-MappingsResource-map}{}}}
\subsection{Method \code{map()}}{
Map concepts to a target vocabulary.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MappingsResource$map(
  target_vocabulary,
  source_concepts = NULL,
  source_codes = NULL,
  mapping_type = NULL,
  include_invalid = FALSE,
  vocab_release = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{target_vocabulary}}{Target vocabulary ID (e.g., "ICD10CM", "SNOMED", "RxNorm").}

\item{\code{source_concepts}}{Vector of OMOP concept IDs to map. Use this OR source_codes, not both.}

\item{\code{source_codes}}{List of vocabulary/code pairs to map. Each element should be a list
with \code{vocabulary_id} and \code{concept_code}. Use this OR source_concepts, not both.}

\item{\code{mapping_type}}{Mapping type filter (direct, equivalent, broader, narrower).}

\item{\code{include_invalid}}{Include invalid mappings. Default \code{FALSE}.}

\item{\code{vocab_release}}{Specific vocabulary release version (e.g., "2025.1"). Default \code{NULL}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-MappingsResource-print"></a>}}
\if{latex}{\out{\hypertarget{method-MappingsResource-print}{}}}
\subsection{Method \code{print()}}{
Print resource information.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MappingsResource$print()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-MappingsResource-clone"></a>}}
\if{latex}{\out{\hypertarget{method-MappingsResource-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MappingsResource$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{validate_pagination}
\alias{validate_pagination}
\title{Validate Page Parameters}
\usage{
validate_pagination(
  page,
  page_size,
  max_page_size = 1000,
  call = rlang::caller_env()
)
}
\arguments{
\item{page}{Page number.}

\item{page_size}{Page size.}

\item{max_page_size}{Maximum allowed page size.}

\item{call}{The calling environment.}
}
\description{
Validate Page Parameters
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/concepts.R
\name{ConceptsResource}
\alias{ConceptsResource}
\title{Concepts Resource}
\value{
A list containing the concept data.

A list containing the concept data with optional relationships and synonyms.

A list with \code{concepts} and any \code{failures}.

A list with suggestions and pagination metadata.

Related concepts with relationship scores.

Relationships data.

Recommendations grouped by source concept ID with pagination metadata.
}
\description{
R6 class providing access to concept operations.
}
\keyword{internal}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-ConceptsResource-new}{\code{ConceptsResource$new()}}
\item \href{#method-ConceptsResource-get}{\code{ConceptsResource$get()}}
\item \href{#method-ConceptsResource-get_by_code}{\code{ConceptsResource$get_by_code()}}
\item \href{#method-ConceptsResource-batch}{\code{ConceptsResource$batch()}}
\item \href{#method-ConceptsResource-suggest}{\code{ConceptsResource$suggest()}}
\item \href{#method-ConceptsResource-related}{\code{ConceptsResource$related()}}
\item \href{#method-ConceptsResource-relationships}{\code{ConceptsResource$relationships()}}
\item \href{#method-ConceptsResource-recommended}{\code{ConceptsResource$recommended()}}
\item \href{#method-ConceptsResource-print}{\code{ConceptsResource$print()}}
\item \href{#method-ConceptsResource-clone}{\code{ConceptsResource$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ConceptsResource-new"></a>}}
\if{latex}{\out{\hypertarget{method-ConceptsResource-new}{}}}
\subsection{Method \code{new()}}{
Create a new ConceptsResource.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ConceptsResource$new(base_req)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{base_req}}{Base httr2 request object.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ConceptsResource-get"></a>}}
\if{latex}{\out{\hypertarget{method-ConceptsResource-get}{}}}
\subsection{Method \code{get()}}{
Get a concept by ID.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ConceptsResource$get(
  concept_id,
  include_relationships = FALSE,
  include_synonyms = FALSE,
  include_hierarchy = FALSE,
  vocab_release = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{concept_id}}{The OMOP concept ID.}

\item{\code{include_relationships}}{Include related concepts (parents/children). Default \code{FALSE}.}

\item{\code{include_synonyms}}{Include concept synonyms. Default \code{FALSE}.}

\item{\code{include_hierarchy}}{Include hierarchy information. Default \code{FALSE}.}

\item{\code{vocab_release}}{Specific vocabulary release (e.g., "2025.2"). Default \code{NULL}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ConceptsResource-get_by_code"></a>}}
\if{latex}{\out{\hypertarget{method-ConceptsResource-get_by_code}{}}}
\subsection{Method \code{get_by_code()}}{
Get a concept by vocabulary and code.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ConceptsResource$get_by_code(
  vocabulary_id,
  concept_code,
  include_relationships = FALSE,
  include_synonyms = FALSE,
  include_hierarchy = FALSE,
  vocab_release = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{vocabulary_id}}{The vocabulary ID (e.g., "SNOMED", "ICD10CM").}

\item{\code{concept_code}}{The concept code within the vocabulary.}

\item{\code{include_relationships}}{Include related concepts (parents/children). Default \code{FALSE}.}

\item{\code{include_synonyms}}{Include concept synonyms. Default \code{FALSE}.}

\item{\code{include_hierarchy}}{Include hierarchy information. Default \code{FALSE}.}

\item{\code{vocab_release}}{Specific vocabulary release (e.g., "2025.2"). Default \code{NULL}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ConceptsResource-batch"></a>}}
\if{latex}{\out{\hypertarget{method-ConceptsResource-batch}{}}}
\subsection{Method \code{batch()}}{
Get multiple concepts by IDs.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ConceptsResource$batch(
  concept_ids,
  include_relationships = FALSE,
  include_synonyms = FALSE,
  include_mappings = FALSE,
  vocabulary_filter = NULL,
  standard_only = TRUE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{concept_ids}}{Vector of concept IDs (max 100).}

\item{\code{include_relationships}}{Include related concepts. Default \code{FALSE}.}

\item{\code{include_synonyms}}{Include concept synonyms. Default \code{FALSE}.}

\item{\code{include_mappings}}{Include concept mappings. Default \code{FALSE}.}

\item{\code{vocabulary_filter}}{Filter results to specific vocabularies.}

\item{\code{standard_only}}{Only return standard concepts. Default \code{TRUE}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ConceptsResource-suggest"></a>}}
\if{latex}{\out{\hypertarget{method-ConceptsResource-suggest}{}}}
\subsection{Method \code{suggest()}}{
Get concept suggestions (autocomplete).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ConceptsResource$suggest(
  query,
  page = 1,
  page_size = 10,
  vocabulary_ids = NULL,
  domain_ids = NULL,
  vocab_release = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{query}}{Search query (min 2 characters, max 100 characters).}

\item{\code{page}}{Page number (default 1).}

\item{\code{page_size}}{Number of suggestions per page (default 10, max 100).}

\item{\code{vocabulary_ids}}{Filter to specific vocabularies (character vector).}

\item{\code{domain_ids}}{Filter to specific domains (character vector).}

\item{\code{vocab_release}}{Specific vocabulary release (e.g., "2025.2").}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ConceptsResource-related"></a>}}
\if{latex}{\out{\hypertarget{method-ConceptsResource-related}{}}}
\subsection{Method \code{related()}}{
Get related concepts.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ConceptsResource$related(
  concept_id,
  relationship_types = NULL,
  min_score = NULL,
  page_size = 20,
  vocab_release = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{concept_id}}{The source concept ID.}

\item{\code{relationship_types}}{Filter by relationship types (e.g., c("Is a", "Maps to")).}

\item{\code{min_score}}{Minimum relationship score (0.0-1.0).}

\item{\code{page_size}}{Maximum number of results (default 20, max 100).}

\item{\code{vocab_release}}{Specific vocabulary release (e.g., "2025.1").}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ConceptsResource-relationships"></a>}}
\if{latex}{\out{\hypertarget{method-ConceptsResource-relationships}{}}}
\subsection{Method \code{relationships()}}{
Get concept relationships.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ConceptsResource$relationships(
  concept_id,
  relationship_ids = NULL,
  vocabulary_ids = NULL,
  domain_ids = NULL,
  include_invalid = FALSE,
  standard_only = FALSE,
  include_reverse = FALSE,
  vocab_release = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{concept_id}}{The concept ID.}

\item{\code{relationship_ids}}{Filter by relationship type IDs (character vector or comma-separated string).}

\item{\code{vocabulary_ids}}{Filter by target vocabulary IDs (character vector or comma-separated string).}

\item{\code{domain_ids}}{Filter by target domain IDs (character vector or comma-separated string).}

\item{\code{include_invalid}}{Include relationships to invalid concepts. Default \code{FALSE}.}

\item{\code{standard_only}}{Only include relationships to standard concepts. Default \code{FALSE}.}

\item{\code{include_reverse}}{Include reverse relationships. Default \code{FALSE}.}

\item{\code{vocab_release}}{Specific vocabulary release version. Default \code{NULL}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ConceptsResource-recommended"></a>}}
\if{latex}{\out{\hypertarget{method-ConceptsResource-recommended}{}}}
\subsection{Method \code{recommended()}}{
Get recommended concepts using OHDSI Phoebe algorithm.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ConceptsResource$recommended(
  concept_ids,
  relationship_types = NULL,
  vocabulary_ids = NULL,
  domain_ids = NULL,
  standard_only = TRUE,
  include_invalid = FALSE,
  page = 1,
  page_size = 100
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{concept_ids}}{Vector of source concept IDs (1-100).}

\item{\code{relationship_types}}{Filter by relationship types (max 20).}

\item{\code{vocabulary_ids}}{Filter to specific vocabularies (max 50).}

\item{\code{domain_ids}}{Filter to specific domains (max 50).}

\item{\code{standard_only}}{Only return standard concepts. Default \code{TRUE}.}

\item{\code{include_invalid}}{Include invalid/deprecated concepts. Default \code{FALSE}.}

\item{\code{page}}{Page number. Default 1.}

\item{\code{page_size}}{Results per page (default 100, max 1000).}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ConceptsResource-print"></a>}}
\if{latex}{\out{\hypertarget{method-ConceptsResource-print}{}}}
\subsection{Method \code{print()}}{
Print resource information.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ConceptsResource$print()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ConceptsResource-clone"></a>}}
\if{latex}{\out{\hypertarget{method-ConceptsResource-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ConceptsResource$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/client.R
\name{OMOPHubClient}
\alias{OMOPHubClient}
\title{OMOPHub API Client}
\value{
A new \code{OMOPHubClient} object.
}
\description{
R6 class for interacting with the OMOPHub vocabulary service.
Provides access to OHDSI ATHENA standardized medical vocabularies.
}
\details{
The client provides access to these resources:
\itemize{
\item \code{concepts}: Concept lookup and batch operations
\item \code{search}: Basic and advanced concept search
\item \code{vocabularies}: Vocabulary listing and details
\item \code{domains}: Domain listing and concept filtering
\item \code{hierarchy}: Ancestor and descendant navigation
\item \code{relationships}: Concept relationships
\item \code{mappings}: Concept mappings between vocabularies
}
}
\examples{
\dontrun{
# Create client (uses OMOPHUB_API_KEY env var)
client <- OMOPHubClient$new()

# Or with explicit API key
client <- OMOPHubClient$new(api_key = "your_api_key")

# Search for concepts
results <- client$search$basic("diabetes")

# Get a specific concept
concept <- client$concepts$get(201826)

# Use specific vocabulary version
client <- OMOPHubClient$new(vocab_version = "2025.1")
}
}
\section{Active bindings}{
\if{html}{\out{<div class="r6-active-bindings">}}
\describe{
\item{\code{concepts}}{Access to concept operations.}

\item{\code{search}}{Access to search operations.}

\item{\code{vocabularies}}{Access to vocabulary operations.}

\item{\code{domains}}{Access to domain operations.}

\item{\code{hierarchy}}{Access to hierarchy operations.}

\item{\code{relationships}}{Access to relationship operations.}

\item{\code{mappings}}{Access to mapping operations.}

\item{\code{fhir}}{Access to FHIR-to-OMOP Concept Resolver operations.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-OMOPHubClient-new}{\code{OMOPHubClient$new()}}
\item \href{#method-OMOPHubClient-print}{\code{OMOPHubClient$print()}}
\item \href{#method-OMOPHubClient-clone}{\code{OMOPHubClient$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-OMOPHubClient-new"></a>}}
\if{latex}{\out{\hypertarget{method-OMOPHubClient-new}{}}}
\subsection{Method \code{new()}}{
Create a new OMOPHub client.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{OMOPHubClient$new(
  api_key = NULL,
  base_url = NULL,
  timeout = 30,
  max_retries = 3,
  vocab_version = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{api_key}}{API key for authentication. If not provided, reads from
\code{OMOPHUB_API_KEY} environment variable or system keyring.}

\item{\code{base_url}}{API base URL. Defaults to \verb{https://api.omophub.com/v1}.}

\item{\code{timeout}}{Request timeout in seconds. Defaults to 30.}

\item{\code{max_retries}}{Maximum retry attempts for failed requests. Defaults to 3.}

\item{\code{vocab_version}}{Optional vocabulary version (e.g., "2025.1").
If not specified, uses the latest version.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-OMOPHubClient-print"></a>}}
\if{latex}{\out{\hypertarget{method-OMOPHubClient-print}{}}}
\subsection{Method \code{print()}}{
Print client information.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{OMOPHubClient$print()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-OMOPHubClient-clone"></a>}}
\if{latex}{\out{\hypertarget{method-OMOPHubClient-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{OMOPHubClient$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/response.R
\name{new_omophub_concepts}
\alias{new_omophub_concepts}
\title{Create OMOPHub Concepts Result}
\usage{
new_omophub_concepts(data, meta = list())
}
\arguments{
\item{data}{Concept data (list or tibble).}

\item{meta}{Pagination metadata.}
}
\value{
An \code{omophub_concepts} S3 object.
}
\description{
Creates an S3 object for concept search results.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/request.R
\name{perform_get}
\alias{perform_get}
\title{Perform GET Request}
\usage{
perform_get(base_req, endpoint, query = NULL)
}
\arguments{
\item{base_req}{Base request object.}

\item{endpoint}{API endpoint path.}

\item{query}{Named list of query parameters.}
}
\value{
Parsed JSON response. For paginated endpoints, returns a list with
\code{data} (the results) and \code{meta} (pagination info). For single-item endpoints,
returns the unwrapped data directly.
}
\description{
Perform GET Request
}
\keyword{internal}
